@@ -1,26 +1,26 @@
 /* -*- Mode: C++; tab-width: 2; indent-tabs-mode: nil; c-basic-offset: 2 -*- */
+/* vim: set ts=2 et sw=2 tw=80: */
 /* This Source Code Form is subject to the terms of the Mozilla Public
  * License, v. 2.0. If a copy of the MPL was not distributed with this
  * file, You can obtain one at http://mozilla.org/MPL/2.0/. */
 
 #include "AccEvent.h"
 
-#include "ApplicationAccessibleWrap.h"
-#include "nsAccessibilityService.h"
 #include "nsAccUtils.h"
 #include "DocAccessible.h"
-#include "nsIAccessibleText.h"
-#include "nsAccEvent.h"
+#include "xpcAccEvents.h"
 #include "States.h"
 
 #include "nsEventStateManager.h"
-#include "nsIServiceManager.h"
-#ifdef MOZ_XUL
-#include "nsIDOMXULMultSelectCntrlEl.h"
-#endif
+#include "mozilla/Selection.h"
 
+using namespace mozilla;
 using namespace mozilla::a11y;
 
+static_assert(static_cast<bool>(eNoUserInput) == false &&
+              static_cast<bool>(eFromUserInput) == true,
+              "EIsFromUserInput cannot be casted to bool");
+
 ////////////////////////////////////////////////////////////////////////////////
 // AccEvent
 ////////////////////////////////////////////////////////////////////////////////
@@ -32,177 +32,21 @@ AccEvent::AccEvent(uint32_t aEventType, Accessible* aAccessible,
                    EIsFromUserInput aIsFromUserInput, EEventRule aEventRule) :
   mEventType(aEventType), mEventRule(aEventRule), mAccessible(aAccessible)
 {
-  CaptureIsFromUserInput(aIsFromUserInput);
-}
-
-AccEvent::AccEvent(uint32_t aEventType, nsINode* aNode,
-                   EIsFromUserInput aIsFromUserInput, EEventRule aEventRule) :
-  mEventType(aEventType), mEventRule(aEventRule), mNode(aNode)
-{
-  CaptureIsFromUserInput(aIsFromUserInput);
-}
-
-////////////////////////////////////////////////////////////////////////////////
-// AccEvent public methods
-
-Accessible* 
-AccEvent::GetAccessible()
-{
-  if (!mAccessible)
-    mAccessible = GetAccessibleForNode();
-
-  return mAccessible;
-}
-
-nsINode*
-AccEvent::GetNode()
-{
-  if (!mNode && mAccessible)
-    mNode = mAccessible->GetNode();
-
-  return mNode;
-}
-
-DocAccessible*
-AccEvent::GetDocAccessible()
-{
-  if (mAccessible)
-    return mAccessible->Document();
-
-  nsINode* node = GetNode();
-  if (node)
-    return GetAccService()->GetDocAccessible(node->OwnerDoc());
-
-  return nullptr;
-}
-
-already_AddRefed<nsAccEvent>
-AccEvent::CreateXPCOMObject()
-{
-  nsAccEvent* event = new nsAccEvent(this);
-  NS_IF_ADDREF(event);
-  return event;
+  if (aIsFromUserInput == eAutoDetect)
+    mIsFromUserInput = nsEventStateManager::IsHandlingUserInput();
+  else
+    mIsFromUserInput = aIsFromUserInput == eFromUserInput ? true : false;
 }
 
 ////////////////////////////////////////////////////////////////////////////////
 // AccEvent cycle collection
 
-NS_IMPL_CYCLE_COLLECTION_NATIVE_CLASS(AccEvent)
-
-NS_IMPL_CYCLE_COLLECTION_UNLINK_BEGIN_NATIVE(AccEvent)
-  NS_IMPL_CYCLE_COLLECTION_UNLINK(mAccessible)
-NS_IMPL_CYCLE_COLLECTION_UNLINK_END
-
-NS_IMPL_CYCLE_COLLECTION_TRAVERSE_NATIVE_BEGIN(AccEvent)
-  NS_CYCLE_COLLECTION_NOTE_EDGE_NAME(cb, "mAccessible");
-  cb.NoteXPCOMChild(static_cast<nsIAccessible*>(tmp->mAccessible));
-NS_IMPL_CYCLE_COLLECTION_TRAVERSE_END
+NS_IMPL_CYCLE_COLLECTION_1(AccEvent, mAccessible)
 
 NS_IMPL_CYCLE_COLLECTION_ROOT_NATIVE(AccEvent, AddRef)
 NS_IMPL_CYCLE_COLLECTION_UNROOT_NATIVE(AccEvent, Release)
 
 ////////////////////////////////////////////////////////////////////////////////
-// AccEvent protected methods
-
-Accessible*
-AccEvent::GetAccessibleForNode() const
-{
-  if (mNode) {
-    DocAccessible* document =
-      GetAccService()->GetDocAccessible(mNode->OwnerDoc());
-    if (document)
-      return document->GetAccessible(mNode);
-  }
-
-  return nullptr;
-}
-
-void
-AccEvent::CaptureIsFromUserInput(EIsFromUserInput aIsFromUserInput)
-{
-  nsINode *targetNode = GetNode();
-
-#ifdef DEBUG
-  if (!targetNode) {
-    // XXX: remove this hack during reorganization of 506907. Meanwhile we
-    // want to get rid an assertion for application accessible events which
-    // don't have DOM node (see bug 506206).
-
-    if (mAccessible != static_cast<nsIAccessible*>(ApplicationAcc()))
-      NS_ASSERTION(targetNode, "There should always be a DOM node for an event");
-  }
-#endif
-
-  if (aIsFromUserInput != eAutoDetect) {
-    mIsFromUserInput = aIsFromUserInput == eFromUserInput ? true : false;
-    return;
-  }
-
-  if (!targetNode)
-    return;
-
-  nsIPresShell *presShell = nsCoreUtils::GetPresShellFor(targetNode);
-  if (!presShell) {
-    NS_NOTREACHED("Threre should always be an pres shell for an event");
-    return;
-  }
-
-  nsEventStateManager *esm = presShell->GetPresContext()->EventStateManager();
-  if (!esm) {
-    NS_NOTREACHED("There should always be an ESM for an event");
-    return;
-  }
-
-  mIsFromUserInput = esm->IsHandlingUserInputExternal();
-}
-
-
-////////////////////////////////////////////////////////////////////////////////
-// AccStateChangeEvent
-////////////////////////////////////////////////////////////////////////////////
-
-// Note: we pass in eAllowDupes to the base class because we don't currently
-// support correct state change coalescence (XXX Bug 569356). Also we need to
-// decide how to coalesce events created via accessible (instead of node).
-AccStateChangeEvent::
-  AccStateChangeEvent(Accessible* aAccessible, uint64_t aState,
-                      bool aIsEnabled, EIsFromUserInput aIsFromUserInput):
-  AccEvent(nsIAccessibleEvent::EVENT_STATE_CHANGE, aAccessible,
-           aIsFromUserInput, eAllowDupes),
-  mState(aState), mIsEnabled(aIsEnabled)
-{
-}
-
-AccStateChangeEvent::
-  AccStateChangeEvent(nsINode* aNode, uint64_t aState, bool aIsEnabled):
-  AccEvent(::nsIAccessibleEvent::EVENT_STATE_CHANGE, aNode,
-           eAutoDetect, eAllowDupes),
-  mState(aState), mIsEnabled(aIsEnabled)
-{
-}
-
-AccStateChangeEvent::
-  AccStateChangeEvent(nsINode* aNode, uint64_t aState) :
-  AccEvent(::nsIAccessibleEvent::EVENT_STATE_CHANGE, aNode,
-           eAutoDetect, eAllowDupes),
-  mState(aState)
-{
-  // Use GetAccessibleForNode() because we do not want to store an accessible
-  // since it leads to problems with delayed events in the case when
-  // an accessible gets reorder event before delayed event is processed.
-  Accessible* accessible = GetAccessibleForNode();
-  mIsEnabled = accessible && ((accessible->State() & mState) != 0);
-}
-
-already_AddRefed<nsAccEvent>
-AccStateChangeEvent::CreateXPCOMObject()
-{
-  nsAccEvent* event = new nsAccStateChangeEvent(this);
-  NS_IF_ADDREF(event);
-  return event;
-}
-
-
 ////////////////////////////////////////////////////////////////////////////////
 // AccTextChangeEvent
 ////////////////////////////////////////////////////////////////////////////////
@@ -232,14 +76,6 @@ AccTextChangeEvent::
     (states::FOCUSED | states::EDITABLE);
 }
 
-already_AddRefed<nsAccEvent>
-AccTextChangeEvent::CreateXPCOMObject()
-{
-  nsAccEvent* event = new nsAccTextChangeEvent(this);
-  NS_IF_ADDREF(event);
-  return event;
-}
-
 
 ////////////////////////////////////////////////////////////////////////////////
 // AccReorderEvent
@@ -250,10 +86,12 @@ AccReorderEvent::IsShowHideEventTarget(const Accessible* aTarget) const
 {
   uint32_t count = mDependentEvents.Length();
   for (uint32_t index = count - 1; index < count; index--) {
-    if (mDependentEvents[index]->mAccessible == aTarget &&
-        mDependentEvents[index]->mEventType == nsIAccessibleEvent::EVENT_SHOW ||
-        mDependentEvents[index]->mEventType == nsIAccessibleEvent::EVENT_HIDE) {
-      return mDependentEvents[index]->mEventType;
+    if (mDependentEvents[index]->mAccessible == aTarget) {
+      uint32_t eventType = mDependentEvents[index]->mEventType;
+      if (eventType == nsIAccessibleEvent::EVENT_SHOW ||
+          eventType == nsIAccessibleEvent::EVENT_HIDE) {
+        return mDependentEvents[index]->mEventType;
+      }
     }
   }
 
@@ -272,14 +110,6 @@ AccHideEvent::
   mPrevSibling = mAccessible->PrevSibling();
 }
 
-already_AddRefed<nsAccEvent>
-AccHideEvent::CreateXPCOMObject()
-{
-  nsAccEvent* event = new nsAccHideEvent(this);
-  NS_ADDREF(event);
-  return event;
-}
-
 
 ////////////////////////////////////////////////////////////////////////////////
 // AccShowEvent
@@ -293,32 +123,26 @@ AccShowEvent::
 
 
 ////////////////////////////////////////////////////////////////////////////////
-// AccCaretMoveEvent
+// AccTextSelChangeEvent
 ////////////////////////////////////////////////////////////////////////////////
 
-AccCaretMoveEvent::
-  AccCaretMoveEvent(Accessible* aAccessible, int32_t aCaretOffset) :
-  AccEvent(::nsIAccessibleEvent::EVENT_TEXT_CARET_MOVED, aAccessible),
-  mCaretOffset(aCaretOffset)
-{
-}
+AccTextSelChangeEvent::AccTextSelChangeEvent(HyperTextAccessible* aTarget,
+                                             Selection* aSelection,
+                                             int32_t aReason) :
+  AccEvent(nsIAccessibleEvent::EVENT_TEXT_SELECTION_CHANGED, aTarget,
+           eAutoDetect, eCoalesceTextSelChange),
+  mSel(aSelection), mReason(aReason) {}
 
-AccCaretMoveEvent::
-  AccCaretMoveEvent(nsINode* aNode) :
-  AccEvent(::nsIAccessibleEvent::EVENT_TEXT_CARET_MOVED, aNode),
-  mCaretOffset(-1)
-{
-}
+AccTextSelChangeEvent::~AccTextSelChangeEvent() { }
 
-already_AddRefed<nsAccEvent>
-AccCaretMoveEvent::CreateXPCOMObject()
+bool
+AccTextSelChangeEvent::IsCaretMoveOnly() const
 {
-  nsAccEvent* event = new nsAccCaretMoveEvent(this);
-  NS_IF_ADDREF(event);
-  return event;
+  return mSel->GetRangeCount() == 1 && mSel->IsCollapsed() &&
+    ((mReason & (nsISelectionListener::COLLAPSETOSTART_REASON |
+                 nsISelectionListener::COLLAPSETOEND_REASON)) == 0);
 }
 
-
 ////////////////////////////////////////////////////////////////////////////////
 // AccSelChangeEvent
 ////////////////////////////////////////////////////////////////////////////////
@@ -353,14 +177,6 @@ AccTableChangeEvent::
 {
 }
 
-already_AddRefed<nsAccEvent>
-AccTableChangeEvent::CreateXPCOMObject()
-{
-  nsAccEvent* event = new nsAccTableChangeEvent(this);
-  NS_IF_ADDREF(event);
-  return event;
-}
-
 
 ////////////////////////////////////////////////////////////////////////////////
 // AccVCChangeEvent
@@ -377,10 +193,63 @@ AccVCChangeEvent::
 {
 }
 
-already_AddRefed<nsAccEvent>
-AccVCChangeEvent::CreateXPCOMObject()
+already_AddRefed<nsIAccessibleEvent>
+a11y::MakeXPCEvent(AccEvent* aEvent)
 {
-  nsAccEvent* event = new nsAccVirtualCursorChangeEvent(this);
-  NS_ADDREF(event);
-  return event;
-}
+  DocAccessible* doc = aEvent->GetDocAccessible();
+  Accessible* acc = aEvent->GetAccessible();
+  nsINode* node = acc->GetNode();
+  nsIDOMNode* domNode = node ? node->AsDOMNode() : nullptr;
+  bool fromUser = aEvent->IsFromUserInput();
+  uint32_t type = aEvent->GetEventType();
+  uint32_t eventGroup = aEvent->GetEventGroups();
+  nsCOMPtr<nsIAccessibleEvent> xpEvent;
+
+  if (eventGroup & (1 << AccEvent::eStateChangeEvent)) {
+    AccStateChangeEvent* sc = downcast_accEvent(aEvent);
+    bool extra = false;
+    uint32_t state = nsAccUtils::To32States(sc->GetState(), &extra);
+    xpEvent = new xpcAccStateChangeEvent(type, acc, doc, domNode, fromUser,
+                                         state, extra, sc->IsStateEnabled());
+    return xpEvent.forget();
+  }
+
+  if (eventGroup & (1 << AccEvent::eTextChangeEvent)) {
+    AccTextChangeEvent* tc = downcast_accEvent(aEvent);
+    nsString text;
+    tc->GetModifiedText(text);
+    xpEvent = new xpcAccTextChangeEvent(type, acc, doc, domNode, fromUser,
+                                        tc->GetStartOffset(), tc->GetLength(),
+                                        tc->IsTextInserted(), text);
+    return xpEvent.forget();
+  }
+
+  if (eventGroup & (1 << AccEvent::eHideEvent)) {
+    AccHideEvent* hideEvent = downcast_accEvent(aEvent);
+    xpEvent = new xpcAccHideEvent(type, acc, doc, domNode, fromUser,
+                                  hideEvent->TargetParent(),
+                                  hideEvent->TargetNextSibling(),
+                                  hideEvent->TargetPrevSibling());
+    return xpEvent.forget();
+  }
+
+  if (eventGroup & (1 << AccEvent::eCaretMoveEvent)) {
+    AccCaretMoveEvent* cm = downcast_accEvent(aEvent);
+    xpEvent = new xpcAccCaretMoveEvent(type, acc, doc, domNode, fromUser,
+                                       cm->GetCaretOffset());
+    return xpEvent.forget();
+  }
+
+  if (eventGroup & (1 << AccEvent::eVirtualCursorChangeEvent)) {
+    AccVCChangeEvent* vcc = downcast_accEvent(aEvent);
+    xpEvent = new xpcAccVirtualCursorChangeEvent(type, acc, doc, domNode, fromUser,
+                                                 vcc->OldAccessible(),
+                                                 vcc->OldStartOffset(),
+                                                 vcc->OldEndOffset(),
+                                                 vcc->Reason());
+    return xpEvent.forget();
+  }
+
+  xpEvent = new xpcAccEvent(type, acc, doc, domNode, fromUser);
+  return xpEvent.forget();
+  }
