@@ -7,9 +7,8 @@
 #ifndef nsDOMMutationObserver_h
 #define nsDOMMutationObserver_h
 
-#include "nsIDOMMutationObserver.h"
+#include "mozilla/Attributes.h"
 #include "nsCycleCollectionParticipant.h"
-#include "nsIJSNativeInitializer.h"
 #include "nsPIDOMWindow.h"
 #include "nsIScriptContext.h"
 #include "nsStubMutationObserver.h"
@@ -21,32 +20,92 @@
 #include "mozilla/dom/Element.h"
 #include "nsClassHashtable.h"
 #include "nsNodeUtils.h"
+#include "nsIDOMMutationEvent.h"
+#include "nsWrapperCache.h"
+#include "mozilla/dom/MutationObserverBinding.h"
+#include "nsIDocument.h"
 
 class nsDOMMutationObserver;
+using mozilla::dom::MutationObservingInfo;
 
-class nsDOMMutationRecord : public nsIDOMMutationRecord
+class nsDOMMutationRecord : public nsISupports,
+                            public nsWrapperCache
 {
 public:
-  nsDOMMutationRecord(const nsAString& aType) : mType(aType)
+  nsDOMMutationRecord(nsIAtom* aType, nsISupports* aOwner)
+  : mType(aType), mOwner(aOwner)
   {
-    mAttrName.SetIsVoid(PR_TRUE);
     mAttrNamespace.SetIsVoid(PR_TRUE);
     mPrevValue.SetIsVoid(PR_TRUE);
+    SetIsDOMBinding();
   }
   virtual ~nsDOMMutationRecord() {}
+
+  nsISupports* GetParentObject() const
+  {
+    return mOwner;
+  }
+
+  virtual JSObject* WrapObject(JSContext* aCx,
+                               JS::Handle<JSObject*> aScope) MOZ_OVERRIDE
+  {
+    return mozilla::dom::MutationRecordBinding::Wrap(aCx, aScope, this);
+  }
+
   NS_DECL_CYCLE_COLLECTING_ISUPPORTS
-  NS_DECL_CYCLE_COLLECTION_CLASS(nsDOMMutationRecord)
-  NS_DECL_NSIDOMMUTATIONRECORD
+  NS_DECL_CYCLE_COLLECTION_SCRIPT_HOLDER_CLASS(nsDOMMutationRecord)
+
+  void GetType(mozilla::dom::DOMString& aRetVal) const
+  {
+    aRetVal.SetOwnedAtom(mType, mozilla::dom::DOMString::eNullNotExpected);
+  }
+
+  nsINode* GetTarget() const
+  {
+    return mTarget;
+  }
+
+  nsINodeList* AddedNodes();
+
+  nsINodeList* RemovedNodes();
+
+  nsINode* GetPreviousSibling() const
+  {
+    return mPreviousSibling;
+  }
+
+  nsINode* GetNextSibling() const
+  {
+    return mNextSibling;
+  }
+
+  void GetAttributeName(mozilla::dom::DOMString& aRetVal) const
+  {
+    aRetVal.SetOwnedAtom(mAttrName, mozilla::dom::DOMString::eTreatNullAsNull);
+  }
+
+  void GetAttributeNamespace(mozilla::dom::DOMString& aRetVal) const
+  {
+    aRetVal.SetOwnedString(mAttrNamespace);
+  }
+
+  void GetOldValue(mozilla::dom::DOMString& aRetVal) const
+  {
+    aRetVal.SetOwnedString(mPrevValue);
+  }
 
   nsCOMPtr<nsINode>             mTarget;
-  nsString                      mType;
-  nsString                      mAttrName;
+  nsCOMPtr<nsIAtom>             mType;
+  nsCOMPtr<nsIAtom>             mAttrName;
   nsString                      mAttrNamespace;
   nsString                      mPrevValue;
   nsRefPtr<nsSimpleContentList> mAddedNodes;
   nsRefPtr<nsSimpleContentList> mRemovedNodes;
   nsCOMPtr<nsINode>             mPreviousSibling;
   nsCOMPtr<nsINode>             mNextSibling;
+
+  nsRefPtr<nsDOMMutationRecord> mNext;
+  nsCOMPtr<nsISupports>         mOwner;
 };
  
 // Base class just prevents direct access to
@@ -141,7 +200,7 @@ public:
   }
   
 protected:
-  nsMutationReceiverBase(nsINode* aTarget, nsIDOMMutationObserver* aObserver)
+  nsMutationReceiverBase(nsINode* aTarget, nsDOMMutationObserver* aObserver)
   : mTarget(aTarget), mObserver(aObserver), mRegisterTarget(aTarget)
   {
     mRegisterTarget->AddMutationObserver(this);
@@ -151,7 +210,7 @@ protected:
 
   nsMutationReceiverBase(nsINode* aRegisterTarget,
                          nsMutationReceiverBase* aParent)
-  : mTarget(nsnull), mObserver(nsnull), mParent(aParent),
+  : mTarget(nullptr), mObserver(nullptr), mParent(aParent),
     mRegisterTarget(aRegisterTarget), mKungFuDeathGrip(aParent->Target())
   {
     NS_ASSERTION(mParent->Subtree(), "Should clone a non-subtree observer!");
@@ -161,7 +220,7 @@ protected:
   }
 
   bool ObservesAttr(mozilla::dom::Element* aElement,
-                    PRInt32 aNameSpaceID,
+                    int32_t aNameSpaceID,
                     nsIAtom* aAttr)
   {
     if (mParent) {
@@ -179,7 +238,7 @@ protected:
     }
 
     nsCOMArray<nsIAtom>& filters = AttributeFilter();
-    for (PRInt32 i = 0; i < filters.Count(); ++i) {         
+    for (int32_t i = 0; i < filters.Count(); ++i) {
       if (filters[i] == aAttr) {
         return true;
       }
@@ -189,7 +248,7 @@ protected:
 
   // The target for the MutationObserver.observe() method.
   nsINode*                           mTarget;
-  nsIDOMMutationObserver*            mObserver;
+  nsDOMMutationObserver*             mObserver;
   nsRefPtr<nsMutationReceiverBase>   mParent; // Cleared after microtask.
   // The node to which Gecko-internal nsIMutationObserver was registered to.
   // This is different than mTarget when dealing with transient observers.
@@ -211,18 +270,10 @@ private:
 };
 
 
-#define NS_MUTATION_OBSERVER_IID \
-{ 0xe628f313, 0x8129, 0x4f90, \
-  { 0x8e, 0xc3, 0x85, 0xe8, 0x28, 0x22, 0xe7, 0xab } }
-
 class nsMutationReceiver : public nsMutationReceiverBase
 {
 public:
-  nsMutationReceiver(nsINode* aTarget, nsIDOMMutationObserver* aObserver)
-  : nsMutationReceiverBase(aTarget, aObserver)
-  {
-    mTarget->BindObject(aObserver);
-  }
+  nsMutationReceiver(nsINode* aTarget, nsDOMMutationObserver* aObserver);
 
   nsMutationReceiver(nsINode* aRegisterTarget, nsMutationReceiverBase* aParent)
   : nsMutationReceiverBase(aRegisterTarget, aParent)
@@ -241,7 +292,7 @@ public:
 
   void RemoveClones()
   {
-    for (PRInt32 i = 0; i < mTransientReceivers.Count(); ++i) {
+    for (int32_t i = 0; i < mTransientReceivers.Count(); ++i) {
       nsMutationReceiver* r =
         static_cast<nsMutationReceiver*>(mTransientReceivers[i]);
       r->DisconnectTransientReceiver();
@@ -253,17 +304,16 @@ public:
   {
     if (mRegisterTarget) {
       mRegisterTarget->RemoveMutationObserver(this);
-      mRegisterTarget = nsnull;
+      mRegisterTarget = nullptr;
     }
 
-    mParent = nsnull;
+    mParent = nullptr;
     NS_ASSERTION(!mTarget, "Should not have mTarget");
     NS_ASSERTION(!mObserver, "Should not have mObserver");
   }
 
   void Disconnect(bool aRemoveFromObserver);
 
-  NS_DECLARE_STATIC_IID_ACCESSOR(NS_IMUTATION_OBSERVER_IID)
   NS_DECL_AND_IMPL_ZEROING_OPERATOR_NEW
   NS_DECL_ISUPPORTS
 
@@ -273,29 +323,91 @@ public:
   NS_DECL_NSIMUTATIONOBSERVER_CONTENTINSERTED
   NS_DECL_NSIMUTATIONOBSERVER_CONTENTREMOVED
   NS_DECL_NSIMUTATIONOBSERVER_NODEWILLBEDESTROYED
+
+  virtual void AttributeSetToCurrentValue(nsIDocument* aDocument,
+                                          mozilla::dom::Element* aElement,
+                                          int32_t aNameSpaceID,
+                                          nsIAtom* aAttribute) MOZ_OVERRIDE
+  {
+    // We can reuse AttributeWillChange implementation.
+    AttributeWillChange(aDocument, aElement, aNameSpaceID, aAttribute,
+                        nsIDOMMutationEvent::MODIFICATION);
+  }
 };
 
-NS_DEFINE_STATIC_IID_ACCESSOR(nsMutationReceiver, NS_MUTATION_OBSERVER_IID)
+#define NS_DOM_MUTATION_OBSERVER_IID \
+{ 0x0c3b91f8, 0xcc3b, 0x4b08, \
+  { 0x9e, 0xab, 0x07, 0x47, 0xa9, 0xe4, 0x65, 0xb4 } }
 
-class nsDOMMutationObserver : public nsIDOMMutationObserver,
-                              public nsIJSNativeInitializer
+class nsDOMMutationObserver : public nsISupports,
+                              public nsWrapperCache
 {
 public:
-  nsDOMMutationObserver() : mWaitingForRun(false), mId(++sCount)
+  nsDOMMutationObserver(already_AddRefed<nsPIDOMWindow>&& aOwner,
+                        mozilla::dom::MutationCallback& aCb)
+  : mOwner(aOwner), mLastPendingMutation(nullptr), mPendingMutationCount(0),
+    mCallback(&aCb), mWaitingForRun(false), mId(++sCount)
   {
-    mTransientReceivers.Init();
+    SetIsDOMBinding();
   }
   virtual ~nsDOMMutationObserver();
   NS_DECL_CYCLE_COLLECTING_ISUPPORTS
-  NS_DECL_CYCLE_COLLECTION_CLASS_AMBIGUOUS(nsDOMMutationObserver,
-                                           nsIDOMMutationObserver)
-  NS_DECL_NSIDOMMUTATIONOBSERVER
+  NS_DECL_CYCLE_COLLECTION_SCRIPT_HOLDER_CLASS(nsDOMMutationObserver)
+  NS_DECLARE_STATIC_IID_ACCESSOR(NS_DOM_MUTATION_OBSERVER_IID)
 
-  NS_IMETHOD Initialize(nsISupports* aOwner, JSContext* cx, JSObject* obj,
-                        PRUint32 argc, jsval* argv);
+  static already_AddRefed<nsDOMMutationObserver>
+  Constructor(const mozilla::dom::GlobalObject& aGlobal,
+              mozilla::dom::MutationCallback& aCb,
+              mozilla::ErrorResult& aRv);
+
+  virtual JSObject* WrapObject(JSContext* aCx,
+                               JS::Handle<JSObject*> aScope) MOZ_OVERRIDE
+  {
+    return mozilla::dom::MutationObserverBinding::Wrap(aCx, aScope, this);
+  }
+
+  nsISupports* GetParentObject() const
+  {
+    return mOwner;
+  }
+
+  void Observe(nsINode& aTarget,
+               const mozilla::dom::MutationObserverInit& aOptions,
+               mozilla::ErrorResult& aRv);
+
+  void Disconnect();
+
+  void TakeRecords(nsTArray<nsRefPtr<nsDOMMutationRecord> >& aRetVal);
 
   void HandleMutation();
 
+  void GetObservingInfo(nsTArray<Nullable<MutationObservingInfo> >& aResult);
+
+  mozilla::dom::MutationCallback* MutationCallback() { return mCallback; }
+
+  void AppendMutationRecord(already_AddRefed<nsDOMMutationRecord> aRecord)
+  {
+    nsRefPtr<nsDOMMutationRecord> record = aRecord;
+    MOZ_ASSERT(record);
+    if (!mLastPendingMutation) {
+      MOZ_ASSERT(!mFirstPendingMutation);
+      mFirstPendingMutation = record.forget();
+      mLastPendingMutation = mFirstPendingMutation;
+    } else {
+      MOZ_ASSERT(mFirstPendingMutation);
+      mLastPendingMutation->mNext = record.forget();
+      mLastPendingMutation = mLastPendingMutation->mNext;
+    }
+    ++mPendingMutationCount;
+  }
+
+  void ClearPendingRecords()
+  {
+    mFirstPendingMutation = nullptr;
+    mLastPendingMutation = nullptr;
+    mPendingMutationCount = 0;
+  }
+
   // static methods
   static void HandleMutations()
   {
@@ -307,11 +419,6 @@ public:
   static void EnterMutationHandling();
   static void LeaveMutationHandling();
 
-  static nsIDOMMutationObserver* CurrentObserver()
-  {
-    return sCurrentObserver;
-  }
-
   static void Shutdown();
 protected:
   friend class nsMutationReceiver;
@@ -326,13 +433,13 @@ protected:
   void ScheduleForRun();
   void RescheduleForRun();
 
-  nsDOMMutationRecord* CurrentRecord(const nsAString& aType);
+  nsDOMMutationRecord* CurrentRecord(nsIAtom* aType);
   bool HasCurrentRecord(const nsAString& aType);
 
   bool Suppressed()
   {
     if (mOwner) {
-      nsCOMPtr<nsIDocument> d = do_QueryInterface(mOwner->GetExtantDocument());
+      nsCOMPtr<nsIDocument> d = mOwner->GetExtantDoc();
       return d && d->IsInSyncOperation();
     }
     return false;
@@ -342,44 +449,48 @@ protected:
 
   static void AddCurrentlyHandlingObserver(nsDOMMutationObserver* aObserver);
 
-  nsCOMPtr<nsIScriptContext>                         mScriptContext;
   nsCOMPtr<nsPIDOMWindow>                            mOwner;
 
   nsCOMArray<nsMutationReceiver>                     mReceivers;
   nsClassHashtable<nsISupportsHashKey,
-                   nsCOMArray<nsMutationReceiver> >  mTransientReceivers;  
+                   nsCOMArray<nsMutationReceiver> >  mTransientReceivers;
   // MutationRecords which are being constructed.
   nsAutoTArray<nsDOMMutationRecord*, 4>              mCurrentMutations;
   // MutationRecords which will be handed to the callback at the end of
   // the microtask.
-  nsCOMArray<nsDOMMutationRecord>                    mPendingMutations;
-  nsCOMPtr<nsIMutationObserverCallback>              mCallback;
+  nsRefPtr<nsDOMMutationRecord>                      mFirstPendingMutation;
+  nsDOMMutationRecord*                               mLastPendingMutation;
+  uint32_t                                           mPendingMutationCount;
+
+  nsRefPtr<mozilla::dom::MutationCallback>           mCallback;
 
   bool                                               mWaitingForRun;
 
-  PRUint64                                           mId;
+  uint64_t                                           mId;
 
-  static PRUint64                                    sCount;
-  static nsCOMArray<nsIDOMMutationObserver>*         sScheduledMutationObservers;
-  static nsIDOMMutationObserver*                     sCurrentObserver;
+  static uint64_t                                    sCount;
+  static nsAutoTArray<nsRefPtr<nsDOMMutationObserver>, 4>* sScheduledMutationObservers;
+  static nsDOMMutationObserver*                      sCurrentObserver;
 
-  static PRUint32                                    sMutationLevel;
-  static nsAutoTArray<nsCOMArray<nsIDOMMutationObserver>, 4>*
+  static uint32_t                                    sMutationLevel;
+  static nsAutoTArray<nsAutoTArray<nsRefPtr<nsDOMMutationObserver>, 4>, 4>*
                                                      sCurrentlyHandlingObservers;
 };
 
+NS_DEFINE_STATIC_IID_ACCESSOR(nsDOMMutationObserver, NS_DOM_MUTATION_OBSERVER_IID)
+
 class nsAutoMutationBatch
 {
 public:
   nsAutoMutationBatch()
-  : mPreviousBatch(nsnull), mBatchTarget(nsnull), mRemovalDone(false),
+  : mPreviousBatch(nullptr), mBatchTarget(nullptr), mRemovalDone(false),
     mFromFirstToLast(false), mAllowNestedBatches(false)    
   {
   }
 
   nsAutoMutationBatch(nsINode* aTarget, bool aFromFirstToLast,
                       bool aAllowNestedBatches)
-  : mPreviousBatch(nsnull), mBatchTarget(nsnull), mRemovalDone(false),
+  : mPreviousBatch(nullptr), mBatchTarget(nullptr), mRemovalDone(false),
     mFromFirstToLast(false), mAllowNestedBatches(false)
   {
     Init(aTarget, aFromFirstToLast, aAllowNestedBatches);
@@ -423,8 +534,8 @@ public:
   static void UpdateObserver(nsDOMMutationObserver* aObserver,
                              bool aWantsChildList)
   {
-    PRUint32 l = sCurrentBatch->mObservers.Length();
-    for (PRUint32 i = 0; i < l; ++i) {
+    uint32_t l = sCurrentBatch->mObservers.Length();
+    for (uint32_t i = 0; i < l; ++i) {
       if (sCurrentBatch->mObservers[i].mObserver == aObserver) {
         if (aWantsChildList) {
           sCurrentBatch->mObservers[i].mWantsChildList = aWantsChildList;
@@ -444,7 +555,7 @@ public:
   static void NodeRemoved(nsIContent* aChild)
   {
     if (IsBatching() && !sCurrentBatch->mRemovalDone) {
-      PRUint32 len = sCurrentBatch->mRemovedNodes.Length();
+      uint32_t len = sCurrentBatch->mRemovedNodes.Length();
       if (!len ||
           sCurrentBatch->mRemovedNodes[len - 1] != aChild) {
         sCurrentBatch->mRemovedNodes.AppendElement(aChild);
@@ -496,12 +607,4 @@ nsMutationReceiverBase::Observer()
     mParent->Observer() : static_cast<nsDOMMutationObserver*>(mObserver);
 }
 
-#define NS_DOMMUTATIONOBSERVER_CID           \
- { /* b66b9490-52f7-4f2a-b998-dbb1d59bc13e */ \
-  0xb66b9490, 0x52f7, 0x4f2a,                 \
-  { 0xb9, 0x98, 0xdb, 0xb1, 0xd5, 0x9b, 0xc1, 0x3e } }
-
-#define NS_DOMMUTATIONOBSERVER_CONTRACTID \
-  "@mozilla.org/dommutationobserver;1"
-
 #endif
