@@ -7,11 +7,10 @@
 #ifndef HTMLPropertiesCollection_h_
 #define HTMLPropertiesCollection_h_
 
-#include "nsDOMLists.h"
+#include "mozilla/Attributes.h"
+#include "mozilla/dom/DOMStringList.h"
 #include "nsCycleCollectionParticipant.h"
 #include "nsAutoPtr.h"
-#include "nsIDOMHTMLPropertiesCollection.h"
-#include "nsIDOMPropertyNodeList.h"
 #include "nsCOMArray.h"
 #include "nsIMutationObserver.h"
 #include "nsStubMutationObserver.h"
@@ -20,9 +19,8 @@
 #include "nsIHTMLCollection.h"
 #include "nsHashKeys.h"
 #include "nsRefPtrHashtable.h"
+#include "nsGenericHTMLElement.h"
 
-class nsGenericHTMLElement;
-class nsXPCClassInfo;
 class nsIDocument;
 class nsINode;
 
@@ -33,22 +31,22 @@ class HTMLPropertiesCollection;
 class PropertyNodeList;
 class Element;
 
-class PropertyStringList : public nsDOMStringList
+class PropertyStringList : public DOMStringList
 {
 public:
   PropertyStringList(HTMLPropertiesCollection* aCollection);
-  NS_DECL_CYCLE_COLLECTING_ISUPPORTS
-  NS_DECL_CYCLE_COLLECTION_CLASS(PropertyStringList)
-  NS_DECL_NSIDOMDOMSTRINGLIST
+  NS_DECL_ISUPPORTS_INHERITED
+  NS_DECL_CYCLE_COLLECTION_CLASS_INHERITED(PropertyStringList, DOMStringList)
 
   bool ContainsInternal(const nsAString& aString);
 
 protected:
+  virtual void EnsureFresh() MOZ_OVERRIDE;
+
   nsRefPtr<HTMLPropertiesCollection> mCollection;
 };
 
 class HTMLPropertiesCollection : public nsIHTMLCollection,
-                                 public nsIDOMHTMLPropertiesCollection,
                                  public nsStubMutationObserver,
                                  public nsWrapperCache
 {
@@ -58,32 +56,47 @@ public:
   HTMLPropertiesCollection(nsGenericHTMLElement* aRoot);
   virtual ~HTMLPropertiesCollection();
 
-  using nsWrapperCache::GetWrapper;
-  virtual JSObject* WrapObject(JSContext *cx, JSObject *scope,
-                               bool *triedToWrap);
+  // nsWrapperCache
+  using nsWrapperCache::GetWrapperPreserveColor;
+  virtual JSObject* WrapObject(JSContext* aCx,
+                               JS::Handle<JSObject*> aScope) MOZ_OVERRIDE;
+protected:
+  virtual JSObject* GetWrapperPreserveColorInternal() MOZ_OVERRIDE
+  {
+    return nsWrapperCache::GetWrapperPreserveColor();
+  }
+public:
 
   virtual Element* GetElementAt(uint32_t aIndex);
 
-  NS_IMETHOD NamedItem(const nsAString& aName, nsIDOMNode** aResult);
   void SetDocument(nsIDocument* aDocument);
-  nsINode* GetParentObject();
-  virtual JSObject* NamedItem(JSContext* cx, const nsAString& name,
-                              mozilla::ErrorResult& error);
+  nsINode* GetParentObject() MOZ_OVERRIDE;
+
+  virtual Element*
+  GetFirstNamedElement(const nsAString& aName, bool& aFound) MOZ_OVERRIDE
+  {
+    // HTMLPropertiesCollection.namedItem and the named getter call the
+    // NamedItem that returns a PropertyNodeList, calling
+    // HTMLCollection.namedItem doesn't make sense so this returns null.
+    aFound = false;
+    return nullptr;
+  }
   PropertyNodeList* NamedItem(const nsAString& aName);
   PropertyNodeList* NamedGetter(const nsAString& aName, bool& aFound)
   {
     aFound = IsSupportedNamedProperty(aName);
     return aFound ? NamedItem(aName) : nullptr;
   }
-  nsDOMStringList* Names()
+  DOMStringList* Names()
   {
     EnsureFresh();
     return mNames;
   }
-  virtual void GetSupportedNames(nsTArray<nsString>& aNames);
+  virtual void GetSupportedNames(nsTArray<nsString>& aNames) MOZ_OVERRIDE;
+
+  NS_DECL_NSIDOMHTMLCOLLECTION
 
   NS_DECL_CYCLE_COLLECTING_ISUPPORTS
-  NS_DECL_NSIDOMHTMLPROPERTIESCOLLECTION
 
   NS_DECL_NSIMUTATIONOBSERVER_ATTRIBUTECHANGED
   NS_DECL_NSIMUTATIONOBSERVER_CONTENTAPPENDED
@@ -93,12 +106,10 @@ public:
   NS_DECL_CYCLE_COLLECTION_SCRIPT_HOLDER_CLASS_AMBIGUOUS(HTMLPropertiesCollection,
                                                          nsIHTMLCollection)
 
-  nsXPCClassInfo* GetClassInfo();
-
 protected:
   // Make sure this collection is up to date, in case the DOM has been mutated.
   void EnsureFresh();
-  
+
   // Crawl the properties of mRoot, following any itemRefs it may have
   void CrawlProperties();
 
@@ -112,26 +123,25 @@ protected:
   }
 
   // the items that make up this collection
-  nsTArray<nsRefPtr<nsGenericHTMLElement> > mProperties; 
-  
+  nsTArray<nsRefPtr<nsGenericHTMLElement> > mProperties;
+
   // the itemprop attribute of the properties
-  nsRefPtr<PropertyStringList> mNames; 
- 
-  // The cached PropertyNodeLists that are NamedItems of this collection 
+  nsRefPtr<PropertyStringList> mNames;
+
+  // The cached PropertyNodeLists that are NamedItems of this collection
   nsRefPtrHashtable<nsStringHashKey, PropertyNodeList> mNamedItemEntries;
-  
+
   // The element this collection is rooted at
-  nsCOMPtr<nsGenericHTMLElement> mRoot;
-  
+  nsRefPtr<nsGenericHTMLElement> mRoot;
+
   // The document mRoot is in, if any
   nsCOMPtr<nsIDocument> mDoc;
-  
+
   // True if there have been DOM modifications since the last EnsureFresh call.
   bool mIsDirty;
 };
 
 class PropertyNodeList : public nsINodeList,
-                         public nsIDOMPropertyNodeList,
                          public nsStubMutationObserver
 {
 public:
@@ -139,21 +149,21 @@ public:
                    nsIContent* aRoot, const nsAString& aName);
   virtual ~PropertyNodeList();
 
-  virtual JSObject* WrapObject(JSContext *cx, JSObject *scope,
-                               bool *triedToWrap);
+  virtual JSObject* WrapObject(JSContext *cx,
+                               JS::Handle<JSObject*> scope) MOZ_OVERRIDE;
 
   void SetDocument(nsIDocument* aDocument);
 
   void GetValues(JSContext* aCx, nsTArray<JS::Value >& aResult,
                  ErrorResult& aError);
 
-  virtual nsIContent* Item(uint32_t aIndex);
-  NS_DECL_NSIDOMPROPERTYNODELIST
+  virtual nsIContent* Item(uint32_t aIndex) MOZ_OVERRIDE;
 
   NS_DECL_CYCLE_COLLECTING_ISUPPORTS
 
   NS_DECL_CYCLE_COLLECTION_SCRIPT_HOLDER_CLASS_AMBIGUOUS(PropertyNodeList,
                                                          nsINodeList)
+  NS_DECL_NSIDOMNODELIST
 
   NS_DECL_NSIMUTATIONOBSERVER_ATTRIBUTECHANGED
   NS_DECL_NSIMUTATIONOBSERVER_CONTENTAPPENDED
@@ -161,26 +171,26 @@ public:
   NS_DECL_NSIMUTATIONOBSERVER_CONTENTREMOVED
 
   // nsINodeList interface
-  virtual int32_t IndexOf(nsIContent* aContent);
-  virtual nsINode* GetParentObject();
-  
+  virtual int32_t IndexOf(nsIContent* aContent) MOZ_OVERRIDE;
+  virtual nsINode* GetParentObject() MOZ_OVERRIDE;
+
   void AppendElement(nsGenericHTMLElement* aElement)
   {
     mElements.AppendElement(aElement);
   }
-  
+
   void Clear()
   {
     mElements.Clear();
   }
-  
+
   void SetDirty() { mIsDirty = true; }
- 
+
 protected:
   // Make sure this list is up to date, in case the DOM has been mutated.
   void EnsureFresh();
 
-  // the the name that this list corresponds to 
+  // the the name that this list corresponds to
   nsString mName;
 
   // the document mParent is in, if any
@@ -201,4 +211,4 @@ protected:
 
 } // namespace dom
 } // namespace mozilla
-#endif // HTMLPropertiesCollection_h_ 
+#endif // HTMLPropertiesCollection_h_
