@@ -3,10 +3,10 @@
  * License, v. 2.0. If a copy of the MPL was not distributed with this
  * file, You can obtain one at http://mozilla.org/MPL/2.0/. */
 
-#include "mozilla/Util.h"
 
 #include "nsMathMLElement.h"
-#include "nsDOMClassInfoID.h" // for eDOMClassInfo_MathElement_id.
+#include "base/compiler_specific.h"
+#include "mozilla/ArrayUtils.h"
 #include "nsGkAtoms.h"
 #include "nsCRT.h"
 #include "nsRuleData.h"
@@ -18,6 +18,11 @@
 #include "nsIPresShell.h"
 #include "nsPresContext.h"
 #include "mozAutoDocUpdate.h"
+#include "nsIScriptError.h"
+#include "nsContentUtils.h"
+#include "nsIURI.h"
+
+#include "mozilla/dom/ElementBinding.h"
 
 using namespace mozilla;
 using namespace mozilla::dom;
@@ -25,21 +30,60 @@ using namespace mozilla::dom;
 //----------------------------------------------------------------------
 // nsISupports methods:
 
-DOMCI_NODE_DATA(MathMLElement, nsMathMLElement)
+NS_IMPL_ISUPPORTS_INHERITED3(nsMathMLElement, nsMathMLElementBase,
+                             nsIDOMElement, nsIDOMNode, Link)
 
-NS_INTERFACE_TABLE_HEAD(nsMathMLElement)
-  NS_NODE_OFFSET_AND_INTERFACE_TABLE_BEGIN(nsMathMLElement)
-    NS_INTERFACE_TABLE_ENTRY(nsMathMLElement, nsIDOMNode)
-    NS_INTERFACE_TABLE_ENTRY(nsMathMLElement, nsIDOMElement)
-    NS_INTERFACE_TABLE_ENTRY(nsMathMLElement, nsILink)
-    NS_INTERFACE_TABLE_ENTRY(nsMathMLElement, Link)
-  NS_OFFSET_AND_INTERFACE_TABLE_END
-  NS_ELEMENT_INTERFACE_TABLE_TO_MAP_SEGUE
-  NS_DOM_INTERFACE_MAP_ENTRY_CLASSINFO(MathMLElement)
-NS_ELEMENT_INTERFACE_MAP_END
+static nsresult
+WarnDeprecated(const char16_t* aDeprecatedAttribute, 
+               const char16_t* aFavoredAttribute, nsIDocument* aDocument)
+{
+  const char16_t *argv[] = 
+    { aDeprecatedAttribute, aFavoredAttribute };
+  return nsContentUtils::
+          ReportToConsole(nsIScriptError::warningFlag,
+                          NS_LITERAL_CSTRING("MathML"), aDocument,
+                          nsContentUtils::eMATHML_PROPERTIES,
+                          "DeprecatedSupersededBy", argv, 2);
+}
 
-NS_IMPL_ADDREF_INHERITED(nsMathMLElement, nsMathMLElementBase)
-NS_IMPL_RELEASE_INHERITED(nsMathMLElement, nsMathMLElementBase)
+static nsresult 
+ReportLengthParseError(const nsString& aValue, nsIDocument* aDocument)
+{
+  const char16_t *arg = aValue.get();
+  return nsContentUtils::
+         ReportToConsole(nsIScriptError::errorFlag,
+                         NS_LITERAL_CSTRING("MathML"), aDocument,
+                         nsContentUtils::eMATHML_PROPERTIES,
+                         "LengthParsingError", &arg, 1);
+}
+
+static nsresult
+ReportParseErrorNoTag(const nsString& aValue, 
+                      nsIAtom*        aAtom,
+                      nsIDocument*    aDocument)
+{
+  const char16_t *argv[] = 
+    { aValue.get(), aAtom->GetUTF16String() };
+  return nsContentUtils::
+         ReportToConsole(nsIScriptError::errorFlag,
+                         NS_LITERAL_CSTRING("MathML"), aDocument,
+                         nsContentUtils::eMATHML_PROPERTIES,
+                         "AttributeParsingErrorNoTag", argv, 2);
+}
+
+nsMathMLElement::nsMathMLElement(already_AddRefed<nsINodeInfo>& aNodeInfo)
+: nsMathMLElementBase(aNodeInfo),
+  ALLOW_THIS_IN_INITIALIZER_LIST(Link(this)),
+  mIncrementScriptLevel(false)
+{
+}
+
+nsMathMLElement::nsMathMLElement(already_AddRefed<nsINodeInfo>&& aNodeInfo)
+: nsMathMLElementBase(aNodeInfo),
+  ALLOW_THIS_IN_INITIALIZER_LIST(Link(this)),
+  mIncrementScriptLevel(false)
+{
+}
 
 nsresult
 nsMathMLElement::BindToTree(nsIDocument* aDocument, nsIContent* aParent,
@@ -48,7 +92,7 @@ nsMathMLElement::BindToTree(nsIDocument* aDocument, nsIContent* aParent,
 {
   static const char kMathMLStyleSheetURI[] = "resource://gre-resources/mathml.css";
 
-  Link::ResetLinkState(false);
+  Link::ResetLinkState(false, Link::ElementHasHref());
 
   nsresult rv = nsMathMLElementBase::BindToTree(aDocument, aParent,
                                                 aBindingParent,
@@ -83,7 +127,7 @@ nsMathMLElement::UnbindFromTree(bool aDeep, bool aNullParent)
 {
   // If this link is ever reinserted into a document, it might
   // be under a different xml:base, so forget the cached state now.
-  Link::ResetLinkState(false);
+  Link::ResetLinkState(false, Link::ElementHasHref());
   
   nsIDocument* doc = GetCurrentDoc();
   if (doc) {
@@ -100,6 +144,14 @@ nsMathMLElement::ParseAttribute(int32_t aNamespaceID,
                                 nsAttrValue& aResult)
 {
   if (aNamespaceID == kNameSpaceID_None) {
+    if (Tag() == nsGkAtoms::math && aAttribute == nsGkAtoms::mode) {
+      WarnDeprecated(nsGkAtoms::mode->GetUTF16String(),
+                     nsGkAtoms::display->GetUTF16String(), OwnerDoc());
+    }
+    if (aAttribute == nsGkAtoms::color) {
+      WarnDeprecated(nsGkAtoms::color->GetUTF16String(),
+                     nsGkAtoms::mathcolor_->GetUTF16String(), OwnerDoc());
+    }
     if (aAttribute == nsGkAtoms::color ||
         aAttribute == nsGkAtoms::mathcolor_ ||
         aAttribute == nsGkAtoms::background ||
@@ -122,6 +174,9 @@ static Element::MappedAttributeEntry sTokenStyles[] = {
   { &nsGkAtoms::fontsize_ },
   { &nsGkAtoms::color },
   { &nsGkAtoms::fontfamily_ },
+  { &nsGkAtoms::fontstyle_ },
+  { &nsGkAtoms::fontweight_ },
+  { &nsGkAtoms::mathvariant_},
   { nullptr }
 };
 
@@ -139,6 +194,11 @@ static Element::MappedAttributeEntry sCommonPresStyles[] = {
   { nullptr }
 };
 
+static Element::MappedAttributeEntry sDirStyles[] = {
+  { &nsGkAtoms::dir },
+  { nullptr }
+};
+
 bool
 nsMathMLElement::IsAttributeMapped(const nsIAtom* aAttribute) const
 {
@@ -148,17 +208,23 @@ nsMathMLElement::IsAttributeMapped(const nsIAtom* aAttribute) const
   };
   static const MappedAttributeEntry* const tokenMap[] = {
     sTokenStyles,
-    sCommonPresStyles
+    sCommonPresStyles,
+    sDirStyles
   };
   static const MappedAttributeEntry* const mstyleMap[] = {
     sTokenStyles,
     sEnvironmentStyles,
-    sCommonPresStyles
+    sCommonPresStyles,
+    sDirStyles
   };
   static const MappedAttributeEntry* const commonPresMap[] = {
     sCommonPresStyles
   };
-  
+  static const MappedAttributeEntry* const mrowMap[] = {
+    sCommonPresStyles,
+    sDirStyles
+  };
+
   // We don't support mglyph (yet).
   nsIAtom* tag = Tag();
   if (tag == nsGkAtoms::ms_ || tag == nsGkAtoms::mi_ ||
@@ -172,6 +238,9 @@ nsMathMLElement::IsAttributeMapped(const nsIAtom* aAttribute) const
   if (tag == nsGkAtoms::mtable_)
     return FindAttributeDependence(aAttribute, mtableMap);
 
+  if (tag == nsGkAtoms::mrow_)
+    return FindAttributeDependence(aAttribute, mrowMap);
+
   if (tag == nsGkAtoms::maction_ ||
       tag == nsGkAtoms::maligngroup_ ||
       tag == nsGkAtoms::malignmark_ ||
@@ -184,7 +253,6 @@ nsMathMLElement::IsAttributeMapped(const nsIAtom* aAttribute) const
       tag == nsGkAtoms::mphantom_ ||
       tag == nsGkAtoms::mprescripts_ ||
       tag == nsGkAtoms::mroot_ ||
-      tag == nsGkAtoms::mrow_ ||
       tag == nsGkAtoms::msqrt_ ||
       tag == nsGkAtoms::msub_ ||
       tag == nsGkAtoms::msubsup_ ||
@@ -294,14 +362,19 @@ nsMathMLElement::ParseNamedSpaceValue(const nsString& aString,
 /* static */ bool
 nsMathMLElement::ParseNumericValue(const nsString& aString,
                                    nsCSSValue&     aCSSValue,
-                                   uint32_t        aFlags)
+                                   uint32_t        aFlags,
+                                   nsIDocument*    aDocument)
 {
   nsAutoString str(aString);
   str.CompressWhitespace(); // aString is const in this code...
 
   int32_t stringLength = str.Length();
-  if (!stringLength)
+  if (!stringLength) {
+    if (!(aFlags & PARSE_SUPPRESS_WARNINGS)) {
+      ReportLengthParseError(aString, aDocument);
+    }
     return false;
+  }
 
   if (ParseNamedSpaceValue(aString, aCSSValue, aFlags)) {
     return true;
@@ -311,7 +384,7 @@ nsMathMLElement::ParseNumericValue(const nsString& aString,
 
   // see if the negative sign is there
   int32_t i = 0;
-  PRUnichar c = str[0];
+  char16_t c = str[0];
   if (c == '-') {
     number.Append(c);
     i++;
@@ -321,8 +394,12 @@ nsMathMLElement::ParseNumericValue(const nsString& aString,
   bool gotDot = false;
   for ( ; i < stringLength; i++) {
     c = str[i];
-    if (gotDot && c == '.')
+    if (gotDot && c == '.') {
+      if (!(aFlags & PARSE_SUPPRESS_WARNINGS)) {
+        ReportLengthParseError(aString, aDocument);
+      }
       return false;  // two dots encountered
+    }
     else if (c == '.')
       gotDot = true;
     else if (!nsCRT::IsAsciiDigit(c)) {
@@ -337,22 +414,45 @@ nsMathMLElement::ParseNumericValue(const nsString& aString,
   // Convert number to floating point
   nsresult errorCode;
   float floatValue = number.ToFloat(&errorCode);
-  if (NS_FAILED(errorCode))
+  if (NS_FAILED(errorCode)) {
+    if (!(aFlags & PARSE_SUPPRESS_WARNINGS)) {
+      ReportLengthParseError(aString, aDocument);
+    }
     return false;
-  if (floatValue < 0 && !(aFlags & PARSE_ALLOW_NEGATIVE))
+  }
+  if (floatValue < 0 && !(aFlags & PARSE_ALLOW_NEGATIVE)) {
+    if (!(aFlags & PARSE_SUPPRESS_WARNINGS)) {
+      ReportLengthParseError(aString, aDocument);
+    }
     return false;
+  }
 
   nsCSSUnit cssUnit;
   if (unit.IsEmpty()) {
     if (aFlags & PARSE_ALLOW_UNITLESS) {
       // no explicit unit, this is a number that will act as a multiplier
-      cssUnit = eCSSUnit_Number;
+      if (!(aFlags & PARSE_SUPPRESS_WARNINGS)) {
+        nsContentUtils::ReportToConsole(nsIScriptError::warningFlag,
+                                        NS_LITERAL_CSTRING("MathML"), aDocument,
+                                        nsContentUtils::eMATHML_PROPERTIES,
+                                        "UnitlessValuesAreDeprecated");
+      }
+      if (aFlags & CONVERT_UNITLESS_TO_PERCENT) {
+        aCSSValue.SetPercentValue(floatValue);
+        return true;
+      }
+      else
+        cssUnit = eCSSUnit_Number;
     } else {
       // We are supposed to have a unit, but there isn't one.
       // If the value is 0 we can just call it "pixels" otherwise
       // this is illegal.
-      if (floatValue != 0.0)
+      if (floatValue != 0.0) {
+        if (!(aFlags & PARSE_SUPPRESS_WARNINGS)) {
+          ReportLengthParseError(aString, aDocument);
+        }
         return false;
+      }
       cssUnit = eCSSUnit_Pixel;
     }
   }
@@ -368,8 +468,12 @@ nsMathMLElement::ParseNumericValue(const nsString& aString,
   else if (unit.EqualsLiteral("mm")) cssUnit = eCSSUnit_Millimeter;
   else if (unit.EqualsLiteral("pt")) cssUnit = eCSSUnit_Point;
   else if (unit.EqualsLiteral("pc")) cssUnit = eCSSUnit_Pica;
-  else // unexpected unit
+  else { // unexpected unit
+    if (!(aFlags & PARSE_SUPPRESS_WARNINGS)) {
+      ReportLengthParseError(aString, aDocument);
+    }
     return false;
+  }
 
   aCSSValue.SetFloatValue(floatValue, cssUnit);
   return true;
@@ -403,6 +507,10 @@ nsMathMLElement::MapMathMLAttributesInto(const nsMappedAttributes* aAttributes,
         // Negative scriptsizemultipliers are not parsed
         if (NS_SUCCEEDED(errorCode) && floatValue >= 0.0f) {
           scriptSizeMultiplier->SetFloatValue(floatValue, eCSSUnit_Number);
+        } else {
+          ReportParseErrorNoTag(str,
+                                nsGkAtoms::scriptsizemultiplier_,
+                                aData->mPresContext->Document());
         }
       }
     }
@@ -416,14 +524,20 @@ nsMathMLElement::MapMathMLAttributesInto(const nsMappedAttributes* aAttributes,
     // default: 8pt
     //
     // We don't allow negative values.
-    // XXXfredw Should we allow unitless values? (bug 411227)
-    // XXXfredw Does a relative unit give a multiple of the default value?
+    // Unitless and percent values give a multiple of the default value.
     //
     value = aAttributes->GetAttr(nsGkAtoms::scriptminsize_);
     nsCSSValue* scriptMinSize = aData->ValueForScriptMinSize();
     if (value && value->Type() == nsAttrValue::eString &&
         scriptMinSize->GetUnit() == eCSSUnit_Null) {
-      ParseNumericValue(value->GetStringValue(), *scriptMinSize, 0);
+      ParseNumericValue(value->GetStringValue(), *scriptMinSize,
+                        PARSE_ALLOW_UNITLESS | CONVERT_UNITLESS_TO_PERCENT,
+                        aData->mPresContext->Document());
+
+      if (scriptMinSize->GetUnit() == eCSSUnit_Percent) {
+        scriptMinSize->SetFloatValue(8.0 * scriptMinSize->GetPercentValue(),
+                                     eCSSUnit_Point);
+      }
     }
 
     // scriptlevel
@@ -451,12 +565,16 @@ nsMathMLElement::MapMathMLAttributesInto(const nsMappedAttributes* aAttributes,
           // then it's a relative value and we store the nsCSSValue as an
           // Integer to indicate that. Otherwise we store it as a Number
           // to indicate that the scriptlevel is absolute.
-          PRUnichar ch = str.CharAt(0);
+          char16_t ch = str.CharAt(0);
           if (ch == '+' || ch == '-') {
             scriptLevel->SetIntValue(intValue, eCSSUnit_Integer);
           } else {
             scriptLevel->SetFloatValue(intValue, eCSSUnit_Number);
           }
+        } else {
+          ReportParseErrorNoTag(str,
+                                nsGkAtoms::scriptlevel_,
+                                aData->mPresContext->Document());
         }
       }
     }
@@ -480,23 +598,29 @@ nsMathMLElement::MapMathMLAttributesInto(const nsMappedAttributes* aAttributes,
     // default: inherited
     //
     // In both cases, we don't allow negative values.
-    // XXXfredw Should we allow unitless values? (bug 411227)
-    // XXXfredw Does a relative unit give a multiple of the default value?
+    // Unitless values give a multiple of the default value.
     //  
     bool parseSizeKeywords = true;
     value = aAttributes->GetAttr(nsGkAtoms::mathsize_);
     if (!value) {
       parseSizeKeywords = false;
       value = aAttributes->GetAttr(nsGkAtoms::fontsize_);
+      if (value) {
+        WarnDeprecated(nsGkAtoms::fontsize_->GetUTF16String(),
+                       nsGkAtoms::mathsize_->GetUTF16String(),
+                       aData->mPresContext->Document());
+      }
     }
     nsCSSValue* fontSize = aData->ValueForFontSize();
     if (value && value->Type() == nsAttrValue::eString &&
         fontSize->GetUnit() == eCSSUnit_Null) {
       nsAutoString str(value->GetStringValue());
-      if (!ParseNumericValue(str, *fontSize, 0) &&
-          parseSizeKeywords) {
+      if (!ParseNumericValue(str, *fontSize, PARSE_SUPPRESS_WARNINGS |
+                             PARSE_ALLOW_UNITLESS | CONVERT_UNITLESS_TO_PERCENT,
+                             nullptr)
+          && parseSizeKeywords) {
         static const char sizes[3][7] = { "small", "normal", "big" };
-        static const int32_t values[NS_ARRAY_LENGTH(sizes)] = {
+        static const int32_t values[MOZ_ARRAY_LENGTH(sizes)] = {
           NS_STYLE_FONT_SIZE_SMALL, NS_STYLE_FONT_SIZE_MEDIUM,
           NS_STYLE_FONT_SIZE_LARGE
         };
@@ -520,10 +644,119 @@ nsMathMLElement::MapMathMLAttributesInto(const nsMappedAttributes* aAttributes,
     // 
     value = aAttributes->GetAttr(nsGkAtoms::fontfamily_);
     nsCSSValue* fontFamily = aData->ValueForFontFamily();
+    if (value) {
+      WarnDeprecated(nsGkAtoms::fontfamily_->GetUTF16String(),
+                     nsGkAtoms::mathvariant_->GetUTF16String(),
+                     aData->mPresContext->Document());
+    }
     if (value && value->Type() == nsAttrValue::eString &&
         fontFamily->GetUnit() == eCSSUnit_Null) {
       fontFamily->SetStringValue(value->GetStringValue(), eCSSUnit_Families);
     }
+
+    // fontstyle
+    //
+    // "Specified the font style to use for the token. Deprecated in favor of
+    //  mathvariant."
+    //
+    // values: "normal" | "italic"
+    // default:	normal (except on <mi>)
+    //
+    // Note that the font-style property is reset in layout/style/ when
+    // -moz-math-variant is specified.
+    nsCSSValue* fontStyle = aData->ValueForFontStyle();
+    value = aAttributes->GetAttr(nsGkAtoms::fontstyle_);
+    if (value) {
+      WarnDeprecated(nsGkAtoms::fontstyle_->GetUTF16String(),
+                       nsGkAtoms::mathvariant_->GetUTF16String(),
+                       aData->mPresContext->Document());
+      if (value->Type() == nsAttrValue::eString &&
+          fontStyle->GetUnit() == eCSSUnit_Null) {
+        nsAutoString str(value->GetStringValue());
+        str.CompressWhitespace();
+        if (str.EqualsASCII("normal")) {
+          fontStyle->SetIntValue(NS_STYLE_FONT_STYLE_NORMAL,
+                                eCSSUnit_Enumerated);
+        } else if (str.EqualsASCII("italic")) {
+          fontStyle->SetIntValue(NS_STYLE_FONT_STYLE_ITALIC,
+                                eCSSUnit_Enumerated);
+        }
+      }
+    }
+
+    // fontweight
+    //
+    // "Specified the font weight for the token. Deprecated in favor of
+    // mathvariant."
+    //
+    // values: "normal" | "bold"
+    // default: normal
+    //
+    // Note that the font-weight property is reset in layout/style/ when
+    // -moz-math-variant is specified.
+    nsCSSValue* fontWeight = aData->ValueForFontWeight();
+    value = aAttributes->GetAttr(nsGkAtoms::fontweight_);
+    if (value) {
+      WarnDeprecated(nsGkAtoms::fontweight_->GetUTF16String(),
+                       nsGkAtoms::mathvariant_->GetUTF16String(),
+                       aData->mPresContext->Document());
+      if (value->Type() == nsAttrValue::eString &&
+          fontWeight->GetUnit() == eCSSUnit_Null) {
+        nsAutoString str(value->GetStringValue());
+        str.CompressWhitespace();
+        if (str.EqualsASCII("normal")) {
+          fontWeight->SetIntValue(NS_STYLE_FONT_WEIGHT_NORMAL,
+                                 eCSSUnit_Enumerated);
+        } else if (str.EqualsASCII("bold")) {
+          fontWeight->SetIntValue(NS_STYLE_FONT_WEIGHT_BOLD,
+                                  eCSSUnit_Enumerated);
+        }
+      }
+    }
+
+    // mathvariant
+    //
+    // "Specifies the logical class of the token. Note that this class is more
+    // than styling, it typically conveys semantic intent;"
+    //
+    // values: "normal" | "bold" | "italic" | "bold-italic" | "double-struck" |
+    // "bold-fraktur" | "script" | "bold-script" | "fraktur" | "sans-serif" |
+    // "bold-sans-serif" | "sans-serif-italic" | "sans-serif-bold-italic" |
+    // "monospace" | "initial" | "tailed" | "looped" | "stretched"
+    // default: normal (except on <mi>)
+    //
+    nsCSSValue* mathVariant = aData->ValueForMathVariant();
+    value = aAttributes->GetAttr(nsGkAtoms::mathvariant_);
+    if (value && value->Type() == nsAttrValue::eString &&
+        mathVariant->GetUnit() == eCSSUnit_Null) {
+      nsAutoString str(value->GetStringValue());
+      str.CompressWhitespace();
+      static const char sizes[19][23] = {
+        "normal", "bold", "italic", "bold-italic", "script", "bold-script",
+        "fraktur", "double-struck", "bold-fraktur", "sans-serif",
+        "bold-sans-serif", "sans-serif-italic", "sans-serif-bold-italic",
+        "monospace", "initial", "tailed", "looped", "stretched"
+      };
+      static const int32_t values[MOZ_ARRAY_LENGTH(sizes)] = {
+        NS_MATHML_MATHVARIANT_NORMAL, NS_MATHML_MATHVARIANT_BOLD,
+        NS_MATHML_MATHVARIANT_ITALIC, NS_MATHML_MATHVARIANT_BOLD_ITALIC,
+        NS_MATHML_MATHVARIANT_SCRIPT, NS_MATHML_MATHVARIANT_BOLD_SCRIPT,
+        NS_MATHML_MATHVARIANT_FRAKTUR, NS_MATHML_MATHVARIANT_DOUBLE_STRUCK,
+        NS_MATHML_MATHVARIANT_BOLD_FRAKTUR, NS_MATHML_MATHVARIANT_SANS_SERIF,
+        NS_MATHML_MATHVARIANT_BOLD_SANS_SERIF,
+        NS_MATHML_MATHVARIANT_SANS_SERIF_ITALIC,
+        NS_MATHML_MATHVARIANT_SANS_SERIF_BOLD_ITALIC,
+        NS_MATHML_MATHVARIANT_MONOSPACE, NS_MATHML_MATHVARIANT_INITIAL,
+        NS_MATHML_MATHVARIANT_TAILED, NS_MATHML_MATHVARIANT_LOOPED,
+        NS_MATHML_MATHVARIANT_STRETCHED
+      };
+      for (uint32_t i = 0; i < ArrayLength(sizes); ++i) {
+        if (str.EqualsASCII(sizes[i])) {
+          mathVariant->SetIntValue(values[i], eCSSUnit_Enumerated);
+          break;
+        }
+      }
+    }
   }
 
   // mathbackground
@@ -549,6 +782,11 @@ nsMathMLElement::MapMathMLAttributesInto(const nsMappedAttributes* aAttributes,
       aAttributes->GetAttr(nsGkAtoms::mathbackground_);
     if (!value) {
       value = aAttributes->GetAttr(nsGkAtoms::background);
+      if (value) {
+        WarnDeprecated(nsGkAtoms::background->GetUTF16String(),
+                       nsGkAtoms::mathbackground_->GetUTF16String(),
+                       aData->mPresContext->Document());
+      }
     }
     nsCSSValue* backgroundColor = aData->ValueForBackgroundColor();
     if (value && backgroundColor->GetUnit() == eCSSUnit_Null) {
@@ -580,6 +818,11 @@ nsMathMLElement::MapMathMLAttributesInto(const nsMappedAttributes* aAttributes,
     const nsAttrValue* value = aAttributes->GetAttr(nsGkAtoms::mathcolor_);
     if (!value) {
       value = aAttributes->GetAttr(nsGkAtoms::color);
+      if (value) {
+        WarnDeprecated(nsGkAtoms::color->GetUTF16String(),
+                       nsGkAtoms::mathcolor_->GetUTF16String(),
+                       aData->mPresContext->Document());
+      }
     }
     nscolor color;
     nsCSSValue* colorValue = aData->ValueForColor();
@@ -590,17 +833,67 @@ nsMathMLElement::MapMathMLAttributesInto(const nsMappedAttributes* aAttributes,
   }
 
   if (aData->mSIDs & NS_STYLE_INHERIT_BIT(Position)) {
-    // width: value
+    // width
+    //
+    // "Specifies the desired width of the entire table and is intended for
+    // visual user agents. When the value is a percentage value, the value is
+    // relative to the horizontal space a MathML renderer has available for the
+    // math element. When the value is "auto", the MathML renderer should
+    // calculate the table width from its contents using whatever layout
+    // algorithm it chooses. "
+    //
+    // values: "auto" | length
+    // default: auto
+    //
     nsCSSValue* width = aData->ValueForWidth();
     if (width->GetUnit() == eCSSUnit_Null) {
       const nsAttrValue* value = aAttributes->GetAttr(nsGkAtoms::width);
       // This does not handle auto and unitless values
       if (value && value->Type() == nsAttrValue::eString) {
-        ParseNumericValue(value->GetStringValue(), *width, 0);
+        ParseNumericValue(value->GetStringValue(), *width, 0,
+                          aData->mPresContext->Document());
       }
     }
   }
 
+  // dir
+  //
+  // Overall Directionality of Mathematics Formulas:
+  // "The overall directionality for a formula, basically the direction of the
+  // Layout Schemata, is specified by the dir attribute on the containing math
+  // element (see Section 2.2 The Top-Level math Element). The default is ltr.
+  // [...] The overall directionality is usually set on the math, but may also 
+  // be switched for individual subformula by using the dir attribute on mrow
+  // or mstyle elements." 
+  //
+  // Bidirectional Layout in Token Elements:
+  // "Specifies the initial directionality for text within the token:
+  // ltr (Left To Right) or rtl (Right To Left). This attribute should only be
+  // needed in rare cases involving weak or neutral characters;
+  // see Section 3.1.5.1 Overall Directionality of Mathematics Formulas for
+  // further discussion. It has no effect on mspace."
+  //
+  // values: "ltr" | "rtl"
+  // default: inherited
+  //
+  if (aData->mSIDs & NS_STYLE_INHERIT_BIT(Visibility)) {
+    const nsAttrValue* value = aAttributes->GetAttr(nsGkAtoms::dir);
+    nsCSSValue* direction = aData->ValueForDirection();
+    if (value && value->Type() == nsAttrValue::eString &&
+        direction->GetUnit() == eCSSUnit_Null) {
+      nsAutoString str(value->GetStringValue());
+      static const char dirs[][4] = { "ltr", "rtl" };
+      static const int32_t dirValues[MOZ_ARRAY_LENGTH(dirs)] = {
+        NS_STYLE_DIRECTION_LTR, NS_STYLE_DIRECTION_RTL
+      };
+      for (uint32_t i = 0; i < ArrayLength(dirs); ++i) {
+        if (str.EqualsASCII(dirs[i])) {
+          direction->SetIntValue(dirValues[i], eCSSUnit_Enumerated);
+          break;
+        }
+      }
+    }
+  }
 }
 
 nsresult
@@ -647,7 +940,7 @@ nsMathMLElement::SetIncrementScriptLevel(bool aIncrementScriptLevel,
 }
 
 bool
-nsMathMLElement::IsFocusable(int32_t *aTabIndex, bool aWithMouse)
+nsMathMLElement::IsFocusableInternal(int32_t* aTabIndex, bool aWithMouse)
 {
   nsCOMPtr<nsIURI> uri;
   if (IsLink(getter_AddRefs(uri))) {
@@ -764,12 +1057,6 @@ nsMathMLElement::GetLinkTarget(nsAString& aTarget)
   }
 }
 
-nsLinkState
-nsMathMLElement::GetLinkState() const
-{
-  return Link::GetLinkState();
-}
-
 already_AddRefed<nsIURI>
 nsMathMLElement::GetHrefURI() const
 {
@@ -793,7 +1080,11 @@ nsMathMLElement::SetAttr(int32_t aNameSpaceID, nsIAtom* aName,
   if (aName == nsGkAtoms::href &&
       (aNameSpaceID == kNameSpaceID_None ||
        aNameSpaceID == kNameSpaceID_XLink)) {
-    Link::ResetLinkState(!!aNotify);
+    if (aNameSpaceID == kNameSpaceID_XLink) {
+      WarnDeprecated(MOZ_UTF16("xlink:href"),
+                     MOZ_UTF16("href"), OwnerDoc());
+    }
+    Link::ResetLinkState(!!aNotify, true);
   }
 
   return rv;
@@ -813,8 +1104,16 @@ nsMathMLElement::UnsetAttr(int32_t aNameSpaceID, nsIAtom* aAttr,
   if (aAttr == nsGkAtoms::href &&
       (aNameSpaceID == kNameSpaceID_None ||
        aNameSpaceID == kNameSpaceID_XLink)) {
-    Link::ResetLinkState(!!aNotify);
+    // Note: just because we removed a single href attr doesn't mean there's no href,
+    // since there are 2 possible namespaces.
+    Link::ResetLinkState(!!aNotify, Link::ElementHasHref());
   }
 
   return rv;
 }
+
+JSObject*
+nsMathMLElement::WrapNode(JSContext *aCx, JS::Handle<JSObject*> aScope)
+{
+  return ElementBinding::Wrap(aCx, aScope, this);
+}
