@@ -6,36 +6,21 @@
 #include "mozilla/dom/SVGSetElement.h"
 #include "mozilla/dom/SVGSetElementBinding.h"
 
-DOMCI_NODE_DATA(SVGSetElement, mozilla::dom::SVGSetElement)
-
 NS_IMPL_NS_NEW_NAMESPACED_SVG_ELEMENT(Set)
 
 namespace mozilla {
 namespace dom {
 
 JSObject*
-SVGSetElement::WrapNode(JSContext *aCx, JSObject *aScope, bool *aTriedToWrap)
+SVGSetElement::WrapNode(JSContext *aCx, JS::Handle<JSObject*> aScope)
 {
-  return SVGSetElementBinding::Wrap(aCx, aScope, this, aTriedToWrap);
+  return SVGSetElementBinding::Wrap(aCx, aScope, this);
 }
 
-//----------------------------------------------------------------------
-// nsISupports methods
-
-NS_IMPL_ADDREF_INHERITED(SVGSetElement, SVGAnimationElement)
-NS_IMPL_RELEASE_INHERITED(SVGSetElement, SVGAnimationElement)
-
-NS_INTERFACE_TABLE_HEAD(SVGSetElement)
-  NS_NODE_INTERFACE_TABLE5(SVGSetElement, nsIDOMNode, nsIDOMElement,
-                           nsIDOMSVGElement, nsIDOMSVGAnimationElement,
-                           nsIDOMSVGSetElement)
-  NS_DOM_INTERFACE_MAP_ENTRY_CLASSINFO(SVGSetElement)
-NS_INTERFACE_MAP_END_INHERITING(SVGAnimationElement)
-
 //----------------------------------------------------------------------
 // Implementation
 
-SVGSetElement::SVGSetElement(already_AddRefed<nsINodeInfo> aNodeInfo)
+SVGSetElement::SVGSetElement(already_AddRefed<nsINodeInfo>& aNodeInfo)
   : SVGAnimationElement(aNodeInfo)
 {
 }
@@ -46,7 +31,6 @@ SVGSetElement::SVGSetElement(already_AddRefed<nsINodeInfo> aNodeInfo)
 NS_IMPL_ELEMENT_CLONE_WITH_INIT(SVGSetElement)
 
 //----------------------------------------------------------------------
-// nsISMILAnimationElement methods
 
 nsSMILAnimationFunction&
 SVGSetElement::AnimationFunction()
