@@ -5,20 +5,21 @@
 
 
 #include "nsXULPrototypeDocument.h"
-#include "nsXULDocument.h"
+#include "XULDocument.h"
 
 #include "nsAString.h"
 #include "nsIObjectInputStream.h"
 #include "nsIObjectOutputStream.h"
 #include "nsIPrincipal.h"
+#include "nsJSPrincipals.h"
 #include "nsIScriptGlobalObject.h"
 #include "nsIScriptObjectPrincipal.h"
 #include "nsIScriptSecurityManager.h"
-#include "nsIScriptRuntime.h"
 #include "nsIServiceManager.h"
 #include "nsIArray.h"
 #include "nsIURI.h"
 #include "jsapi.h"
+#include "jsfriendapi.h"
 #include "nsString.h"
 #include "nsIConsoleService.h"
 #include "nsIScriptError.h"
@@ -26,54 +27,46 @@
 #include "nsDOMCID.h"
 #include "nsNodeInfoManager.h"
 #include "nsContentUtils.h"
+#include "nsCxPusher.h"
 #include "nsCCUncollectableMarker.h"
 #include "nsDOMJSUtils.h" // for GetScriptContextFromJSContext
 #include "xpcpublic.h"
 #include "mozilla/dom/BindingUtils.h"
 
-using mozilla::dom::DestroyProtoOrIfaceCache;
+using mozilla::dom::DestroyProtoAndIfaceCache;
+using mozilla::AutoPushJSContext;
+using mozilla::AutoSafeJSContext;
+using mozilla::dom::XULDocument;
 
-static NS_DEFINE_CID(kDOMScriptObjectFactoryCID,
-                     NS_DOM_SCRIPT_OBJECT_FACTORY_CID);
-
-
-class nsXULPDGlobalObject : public nsIScriptGlobalObject,
-                            public nsIScriptObjectPrincipal
+class nsXULPDGlobalObject : public nsISupports
 {
 public:
     nsXULPDGlobalObject(nsXULPrototypeDocument* owner);
 
     // nsISupports interface
     NS_DECL_CYCLE_COLLECTING_ISUPPORTS
+    NS_DECL_CYCLE_COLLECTION_SCRIPT_HOLDER_CLASS(nsXULPDGlobalObject)
 
-    // nsIScriptGlobalObject methods
-    virtual void OnFinalize(JSObject* aObject);
-    virtual void SetScriptsEnabled(bool aEnabled, bool aFireTimeouts);
-
-    virtual JSObject* GetGlobalJSObject();
-    virtual nsresult EnsureScriptEnvironment();
-
-    virtual nsIScriptContext *GetScriptContext();
-
-    // nsIScriptObjectPrincipal methods
-    virtual nsIPrincipal* GetPrincipal();
-
-    NS_DECL_CYCLE_COLLECTION_CLASS_AMBIGUOUS(nsXULPDGlobalObject,
-                                             nsIScriptGlobalObject)
-
+    JSObject* GetCompilationGlobal();
+    void UnmarkCompilationGlobal()
+    {
+        if (mJSObject) {
+            JS::ExposeObjectToActiveJS(mJSObject);
+        }
+    }
+    void Destroy();
+    nsIPrincipal* GetPrincipal();
     void ClearGlobalObjectOwner();
 
 protected:
     virtual ~nsXULPDGlobalObject();
 
-    nsXULPrototypeDocument* mGlobalObjectOwner; // weak reference
-
-    nsCOMPtr<nsIScriptContext> mContext;
-    JSObject* mJSObject;
-
     nsCOMPtr<nsIPrincipal> mCachedPrincipal;
+    nsXULPrototypeDocument* mGlobalObjectOwner; // weak reference
+    JS::Heap<JSObject*> mJSObject;
+    bool mDestroyed; // Probably not necessary, but let's be safe.
 
-    static JSClass gSharedGlobalClass;
+    static const JSClass gSharedGlobalClass;
 };
 
 nsIPrincipal* nsXULPrototypeDocument::gSystemPrincipal;
@@ -84,37 +77,30 @@ uint32_t nsXULPrototypeDocument::gRefCnt;
 void
 nsXULPDGlobalObject_finalize(JSFreeOp *fop, JSObject *obj)
 {
-    nsISupports *nativeThis = (nsISupports*)JS_GetPrivate(obj);
-
-    nsCOMPtr<nsIScriptGlobalObject> sgo(do_QueryInterface(nativeThis));
-
-    if (sgo) {
-        sgo->OnFinalize(obj);
-    }
+    nsXULPDGlobalObject* nativeThis = static_cast<nsXULPDGlobalObject*>(JS_GetPrivate(obj));
+    nativeThis->Destroy();
 
     // The addref was part of JSObject construction
-    NS_RELEASE(nativeThis);
-
-    DestroyProtoOrIfaceCache(obj);
+    nsContentUtils::DeferredFinalize(nativeThis);
 }
 
 
-JSBool
-nsXULPDGlobalObject_resolve(JSContext *cx, JSHandleObject obj, JSHandleId id)
+bool
+nsXULPDGlobalObject_resolve(JSContext *cx, JS::Handle<JSObject*> obj, JS::Handle<jsid> id)
 {
-    JSBool did_resolve = JS_FALSE;
+    bool did_resolve = false;
 
     return JS_ResolveStandardClass(cx, obj, id, &did_resolve);
 }
 
 
-JSClass nsXULPDGlobalObject::gSharedGlobalClass = {
+const JSClass nsXULPDGlobalObject::gSharedGlobalClass = {
     "nsXULPrototypeScript compilation scope",
-    XPCONNECT_GLOBAL_FLAGS,
-    JS_PropertyStub,  JS_PropertyStub, JS_PropertyStub, JS_StrictPropertyStub,
+    JSCLASS_HAS_PRIVATE | JSCLASS_PRIVATE_IS_NSISUPPORTS |
+    JSCLASS_IMPLEMENTS_BARRIERS | JSCLASS_GLOBAL_FLAGS_WITH_SLOTS(0),
+    JS_PropertyStub,  JS_DeletePropertyStub, JS_PropertyStub, JS_StrictPropertyStub,
     JS_EnumerateStub, nsXULPDGlobalObject_resolve,  JS_ConvertStub,
-    nsXULPDGlobalObject_finalize, NULL, NULL, NULL, NULL,
-    TraceXPCGlobal
+    nsXULPDGlobalObject_finalize, nullptr, nullptr, nullptr, nullptr
 };
 
 
@@ -127,7 +113,8 @@ JSClass nsXULPDGlobalObject::gSharedGlobalClass = {
 nsXULPrototypeDocument::nsXULPrototypeDocument()
     : mRoot(nullptr),
       mLoaded(false),
-      mCCGeneration(0)
+      mCCGeneration(0),
+      mGCNumber(0)
 {
     ++gRefCnt;
 }
@@ -159,6 +146,7 @@ nsXULPrototypeDocument::~nsXULPrototypeDocument()
 }
 
 NS_IMPL_CYCLE_COLLECTION_CLASS(nsXULPrototypeDocument)
+
 NS_IMPL_CYCLE_COLLECTION_UNLINK_BEGIN(nsXULPrototypeDocument)
     tmp->mPrototypeWaiters.Clear();
 NS_IMPL_CYCLE_COLLECTION_UNLINK_END
@@ -166,12 +154,9 @@ NS_IMPL_CYCLE_COLLECTION_TRAVERSE_BEGIN(nsXULPrototypeDocument)
     if (nsCCUncollectableMarker::InGeneration(cb, tmp->mCCGeneration)) {
         return NS_SUCCESS_INTERRUPTED_TRAVERSE;
     }
-    NS_IMPL_CYCLE_COLLECTION_TRAVERSE_NATIVE_MEMBER(mRoot,
-                                                    nsXULPrototypeElement)
-    NS_CYCLE_COLLECTION_NOTE_EDGE_NAME(cb, "mGlobalObject");
-    cb.NoteXPCOMChild(static_cast<nsIScriptGlobalObject*>(tmp->mGlobalObject));
-    NS_IMPL_CYCLE_COLLECTION_TRAVERSE_NATIVE_MEMBER(mNodeInfoManager,
-                                                    nsNodeInfoManager)
+    NS_IMPL_CYCLE_COLLECTION_TRAVERSE(mRoot)
+    NS_IMPL_CYCLE_COLLECTION_TRAVERSE(mGlobalObject)
+    NS_IMPL_CYCLE_COLLECTION_TRAVERSE(mNodeInfoManager)
     for (uint32_t i = 0; i < tmp->mPrototypeWaiters.Length(); ++i) {
         NS_CYCLE_COLLECTION_NOTE_EDGE_NAME(cb, "mPrototypeWaiters[i]");
         cb.NoteXPCOMChild(static_cast<nsINode*>(tmp->mPrototypeWaiters[i].get()));
@@ -179,9 +164,8 @@ NS_IMPL_CYCLE_COLLECTION_TRAVERSE_BEGIN(nsXULPrototypeDocument)
 NS_IMPL_CYCLE_COLLECTION_TRAVERSE_END
 
 NS_INTERFACE_MAP_BEGIN_CYCLE_COLLECTION(nsXULPrototypeDocument)
-    NS_INTERFACE_MAP_ENTRY(nsIScriptGlobalObjectOwner)
     NS_INTERFACE_MAP_ENTRY(nsISerializable)
-    NS_INTERFACE_MAP_ENTRY_AMBIGUOUS(nsISupports, nsIScriptGlobalObjectOwner)
+    NS_INTERFACE_MAP_ENTRY(nsISupports)
 NS_INTERFACE_MAP_END
 
 NS_IMPL_CYCLE_COLLECTING_ADDREF(nsXULPrototypeDocument)
@@ -208,8 +192,7 @@ NS_NewXULPrototypeDocument(nsXULPrototypeDocument** aResult)
 
 // Helper method that shares a system global among all prototype documents
 // that have the system principal as their security principal.   Called by
-// nsXULPrototypeDocument::Read and
-// nsXULPrototypeDocument::GetScriptGlobalObject.
+// nsXULPrototypeDocument::Read and nsXULPrototypeDocument::GetCompilationGlobal.
 // This method greatly reduces the number of nsXULPDGlobalObjects and their
 // nsIScriptContexts in apps that load many XUL documents via chrome: URLs.
 
@@ -246,7 +229,9 @@ nsXULPrototypeDocument::Read(nsIObjectInputStream* aStream)
 {
     nsresult rv;
 
-    rv = aStream->ReadObject(true, getter_AddRefs(mURI));
+    nsCOMPtr<nsISupports> supports;
+    rv = aStream->ReadObject(true, getter_AddRefs(supports));
+    mURI = do_QueryInterface(supports);
 
     uint32_t count, i;
     nsCOMPtr<nsIURI> styleOverlayURI;
@@ -260,17 +245,19 @@ nsXULPrototypeDocument::Read(nsIObjectInputStream* aStream)
     }
 
     for (i = 0; i < count; ++i) {
-        tmp = aStream->ReadObject(true, getter_AddRefs(styleOverlayURI));
+        tmp = aStream->ReadObject(true, getter_AddRefs(supports));
         if (NS_FAILED(tmp)) {
           rv = tmp;
         }
+        styleOverlayURI = do_QueryInterface(supports);
         mStyleSheetReferences.AppendObject(styleOverlayURI);
     }
 
 
     // nsIPrincipal mNodeInfoManager->mPrincipal
     nsCOMPtr<nsIPrincipal> principal;
-    tmp = aStream->ReadObject(true, getter_AddRefs(principal));
+    tmp = aStream->ReadObject(true, getter_AddRefs(supports));
+    principal = do_QueryInterface(supports);
     if (NS_FAILED(tmp)) {
       rv = tmp;
     }
@@ -349,7 +336,7 @@ nsXULPrototypeDocument::Read(nsIObjectInputStream* aStream)
                break;
             }
 
-            tmp = pi->Deserialize(aStream, mGlobalObject, mURI, &nodeInfos);
+            tmp = pi->Deserialize(aStream, this, mURI, &nodeInfos);
             if (NS_FAILED(tmp)) {
               rv = tmp;
             }
@@ -358,7 +345,7 @@ nsXULPrototypeDocument::Read(nsIObjectInputStream* aStream)
               rv = tmp;
             }
         } else if ((nsXULPrototypeNode::Type)type == nsXULPrototypeNode::eType_Element) {
-            tmp = mRoot->Deserialize(aStream, mGlobalObject, mURI, &nodeInfos);
+            tmp = mRoot->Deserialize(aStream, this, mURI, &nodeInfos);
             if (NS_FAILED(tmp)) {
               rv = tmp;
             }
@@ -397,7 +384,6 @@ GetNodeInfos(nsXULPrototypeElement* aPrototype,
             ni = aPrototype->mNodeInfo->NodeInfoManager()->
                 GetNodeInfo(name->Atom(), nullptr, kNameSpaceID_None,
                             nsIDOMNode::ATTRIBUTE_NODE);
-            NS_ENSURE_TRUE(ni, NS_ERROR_OUT_OF_MEMORY);
         }
         else {
             ni = name->NodeInfo();
@@ -480,10 +466,7 @@ nsXULPrototypeDocument::Write(nsIObjectOutputStream* aStream)
         NS_ENSURE_TRUE(nodeInfo, NS_ERROR_FAILURE);
 
         nsAutoString namespaceURI;
-        tmp = nodeInfo->GetNamespaceURI(namespaceURI);
-        if (NS_FAILED(tmp)) {
-          rv = tmp;
-        }
+        nodeInfo->GetNamespaceURI(namespaceURI);
         tmp = aStream->WriteWStringZ(namespaceURI.get());
         if (NS_FAILED(tmp)) {
           rv = tmp;
@@ -512,20 +495,17 @@ nsXULPrototypeDocument::Write(nsIObjectOutputStream* aStream)
     }
 
     // Now serialize the document contents
-    nsIScriptGlobalObject* globalObject = GetScriptGlobalObject();
-    NS_ENSURE_TRUE(globalObject, NS_ERROR_UNEXPECTED);
-
     count = mProcessingInstructions.Length();
     for (i = 0; i < count; ++i) {
         nsXULPrototypePI* pi = mProcessingInstructions[i];
-        tmp = pi->Serialize(aStream, globalObject, &nodeInfos);
+        tmp = pi->Serialize(aStream, this, &nodeInfos);
         if (NS_FAILED(tmp)) {
           rv = tmp;
         }
     }
 
     if (mRoot) {
-      tmp = mRoot->Serialize(aStream, globalObject, &nodeInfos);
+      tmp = mRoot->Serialize(aStream, this, &nodeInfos);
       if (NS_FAILED(tmp)) {
         rv = tmp;
       }
@@ -633,6 +613,24 @@ nsXULPrototypeDocument::SetDocumentPrincipal(nsIPrincipal* aPrincipal)
     mNodeInfoManager->SetDocumentPrincipal(aPrincipal);
 }
 
+JSObject*
+nsXULPrototypeDocument::GetCompilationGlobal()
+{
+  if (!mGlobalObject) {
+      mGlobalObject = NewXULPDGlobalObject();
+  }
+  return mGlobalObject->GetCompilationGlobal();
+}
+
+void
+nsXULPrototypeDocument::MarkInCCGeneration(uint32_t aCCGeneration)
+{
+    mCCGeneration = aCCGeneration;
+    if (mGlobalObject) {
+        mGlobalObject->UnmarkCompilationGlobal();
+    }
+}
+
 nsNodeInfoManager*
 nsXULPrototypeDocument::GetNodeInfoManager()
 {
@@ -641,7 +639,7 @@ nsXULPrototypeDocument::GetNodeInfoManager()
 
 
 nsresult
-nsXULPrototypeDocument::AwaitLoadDone(nsXULDocument* aDocument, bool* aResult)
+nsXULPrototypeDocument::AwaitLoadDone(XULDocument* aDocument, bool* aResult)
 {
     nsresult rv = NS_OK;
 
@@ -680,18 +678,18 @@ nsXULPrototypeDocument::NotifyLoadDone()
     return rv;
 }
 
-//----------------------------------------------------------------------
-//
-// nsIScriptGlobalObjectOwner methods
-//
-
-nsIScriptGlobalObject*
-nsXULPrototypeDocument::GetScriptGlobalObject()
+void
+nsXULPrototypeDocument::TraceProtos(JSTracer* aTrc, uint32_t aGCNumber)
 {
-    if (!mGlobalObject)
-        mGlobalObject = NewXULPDGlobalObject();
+  // Only trace the protos once per GC.
+  if (mGCNumber == aGCNumber) {
+    return;
+  }
 
-    return mGlobalObject;
+  mGCNumber = aGCNumber;
+  if (mRoot) {
+    mRoot->TraceAllScripts(aTrc);
+  }
 }
 
 //----------------------------------------------------------------------
@@ -701,115 +699,75 @@ nsXULPrototypeDocument::GetScriptGlobalObject()
 
 nsXULPDGlobalObject::nsXULPDGlobalObject(nsXULPrototypeDocument* owner)
   : mGlobalObjectOwner(owner)
-  , mJSObject(NULL)
+  , mJSObject(nullptr)
+  , mDestroyed(false)
 {
 }
 
 
 nsXULPDGlobalObject::~nsXULPDGlobalObject()
 {
+  MOZ_ASSERT(!mJSObject);
 }
 
 NS_IMPL_CYCLE_COLLECTION_CLASS(nsXULPDGlobalObject)
-NS_IMPL_CYCLE_COLLECTION_UNLINK_0(nsXULPDGlobalObject)
+
+NS_IMPL_CYCLE_COLLECTION_TRACE_BEGIN(nsXULPDGlobalObject)
+  NS_IMPL_CYCLE_COLLECTION_TRACE_JS_MEMBER_CALLBACK(mJSObject)
+NS_IMPL_CYCLE_COLLECTION_TRACE_END
+
 NS_IMPL_CYCLE_COLLECTION_TRAVERSE_BEGIN(nsXULPDGlobalObject)
-  NS_IMPL_CYCLE_COLLECTION_TRAVERSE_NSCOMPTR(mContext)
+  NS_IMPL_CYCLE_COLLECTION_TRAVERSE_SCRIPT_OBJECTS
 NS_IMPL_CYCLE_COLLECTION_TRAVERSE_END
 
+NS_IMPL_CYCLE_COLLECTION_UNLINK_BEGIN(nsXULPDGlobalObject)
+  tmp->Destroy();
+NS_IMPL_CYCLE_COLLECTION_UNLINK_END
+
 NS_INTERFACE_MAP_BEGIN_CYCLE_COLLECTION(nsXULPDGlobalObject)
-  NS_INTERFACE_MAP_ENTRY(nsIScriptGlobalObject)
-  NS_INTERFACE_MAP_ENTRY(nsIScriptObjectPrincipal)
-  NS_INTERFACE_MAP_ENTRY_AMBIGUOUS(nsISupports, nsIScriptGlobalObject)
+  NS_INTERFACE_MAP_ENTRY(nsISupports)
 NS_INTERFACE_MAP_END
 
 NS_IMPL_CYCLE_COLLECTING_ADDREF(nsXULPDGlobalObject)
 NS_IMPL_CYCLE_COLLECTING_RELEASE(nsXULPDGlobalObject)
 
-//----------------------------------------------------------------------
-//
-// nsIScriptGlobalObject methods
-//
-
-nsresult
-nsXULPDGlobalObject::EnsureScriptEnvironment()
+JSObject *
+nsXULPDGlobalObject::GetCompilationGlobal()
 {
-  if (mContext) {
-    return NS_OK;
-  }
-  NS_ASSERTION(!mJSObject, "Have global without context?");
-
-  nsCOMPtr<nsIScriptRuntime> languageRuntime;
-  nsresult rv = NS_GetJSRuntime(getter_AddRefs(languageRuntime));
-  NS_ENSURE_SUCCESS(rv, NS_OK);
-
-  nsCOMPtr<nsIScriptContext> ctxNew = languageRuntime->CreateContext();
-  MOZ_ASSERT(ctxNew);
-
-  // We have to setup a special global object.  We do this then
-  // attach it as the global for this context.  Then, we
-  // will re-fetch the global and set it up in our language globals array.
-  {
-    JSContext *cx = ctxNew->GetNativeContext();
-    JSAutoRequest ar(cx);
-
-    nsIPrincipal *principal = GetPrincipal();
-    JSObject *newGlob;
-    JSCompartment *compartment;
-
-    rv = xpc::CreateGlobalObject(cx, &gSharedGlobalClass, principal, false,
-                                 &newGlob, &compartment);
-    NS_ENSURE_SUCCESS(rv, NS_OK);
-
-    ::JS_SetGlobalObject(cx, newGlob);
-
-    // Add an owning reference from JS back to us. This'll be
-    // released when the JSObject is finalized.
-    ::JS_SetPrivate(newGlob, this);
-    NS_ADDREF(this);
+  if (mJSObject) {
+    // We've been initialized before. This is what we get.
+    JS::ExposeObjectToActiveJS(mJSObject);
+    return mJSObject;
   }
 
-  // should probably assert the context is clean???
-  ctxNew->WillInitializeContext();
-  rv = ctxNew->InitContext();
-  NS_ENSURE_SUCCESS(rv, NS_OK);
+  if (mDestroyed) {
+    return nullptr;
+  }
 
-  ctxNew->SetGCOnDestruction(false);
-  ctxNew->DidInitializeContext();
+  AutoSafeJSContext cx;
+  JS::CompartmentOptions options;
+  options.setZone(JS::SystemZone)
+         .setInvisibleToDebugger(true);
+  mJSObject = JS_NewGlobalObject(cx, &gSharedGlobalClass,
+                                 nsJSPrincipals::get(GetPrincipal()),
+                                 JS::DontFireOnNewGlobalHook, options);
+  NS_ENSURE_TRUE(mJSObject, nullptr);
 
-  JSObject* global = ctxNew->GetNativeGlobal();
-  NS_ASSERTION(global, "GetNativeGlobal returned NULL!");
+  mozilla::HoldJSObjects(this);
 
-  mContext = ctxNew;
-  mJSObject = global;
+  // Add an owning reference from JS back to us. This'll be
+  // released when the JSObject is finalized.
+  JS_SetPrivate(mJSObject, this);
+  NS_ADDREF(this);
 
   // Set the location information for the new global, so that tools like
   // about:memory may use that information
   nsIURI *ownerURI = mGlobalObjectOwner->GetURI();
   xpc::SetLocationForGlobal(mJSObject, ownerURI);
 
-  return NS_OK;
+  return mJSObject;
 }
 
-nsIScriptContext*
-nsXULPDGlobalObject::GetScriptContext()
-{
-  // This global object creates a context on demand - do that now.
-  nsresult rv = EnsureScriptEnvironment();
-  if (NS_FAILED(rv)) {
-    NS_ERROR("Failed to setup script language");
-    return NULL;
-  }
-
-  return mContext;
-}
-
-JSObject*
-nsXULPDGlobalObject::GetGlobalJSObject()
-{
-  return xpc_UnmarkGrayObject(mJSObject);
-}
-
-
 void
 nsXULPDGlobalObject::ClearGlobalObjectOwner()
 {
@@ -819,28 +777,20 @@ nsXULPDGlobalObject::ClearGlobalObjectOwner()
   if (this != nsXULPrototypeDocument::gSystemGlobal)
     mCachedPrincipal = mGlobalObjectOwner->DocumentPrincipal();
 
-  mContext = NULL;
-  mGlobalObjectOwner = NULL;
-}
-
-
-void
-nsXULPDGlobalObject::OnFinalize(JSObject* aObject)
-{
-  mJSObject = NULL;
+  mGlobalObjectOwner = nullptr;
 }
 
 void
-nsXULPDGlobalObject::SetScriptsEnabled(bool aEnabled, bool aFireTimeouts)
+nsXULPDGlobalObject::Destroy()
 {
-  // We don't care...
+  mDestroyed = true;
+  if (!mJSObject) {
+    return;
+  }
+  mJSObject = nullptr;
+  mozilla::DropJSObjects(this);
 }
 
-//----------------------------------------------------------------------
-//
-// nsIScriptObjectPrincipal methods
-//
-
 nsIPrincipal*
 nsXULPDGlobalObject::GetPrincipal()
 {
