@@ -10,8 +10,6 @@
 
 #include "mozilla/storage.h"
 #include "nsComponentManagerUtils.h"
-#include "nsContentUtils.h"
-#include "nsDOMClassInfoID.h"
 #include "nsEventDispatcher.h"
 #include "nsJSUtils.h"
 #include "nsThreadUtils.h"
@@ -21,19 +19,25 @@
 #include "IDBIndex.h"
 #include "IDBObjectStore.h"
 #include "IDBTransaction.h"
+#include "ProfilerHelpers.h"
+#include "ReportInternalError.h"
 #include "TransactionThreadPool.h"
 
 #include "ipc/IndexedDBChild.h"
 #include "ipc/IndexedDBParent.h"
 
 #include "IndexedDatabaseInlines.h"
+#include "mozilla/dom/BindingDeclarations.h"
+#include "mozilla/dom/UnionTypes.h"
 
 USING_INDEXEDDB_NAMESPACE
 using namespace mozilla::dom::indexedDB::ipc;
+using mozilla::dom::Optional;
+using mozilla::dom::OwningIDBObjectStoreOrIDBIndex;
+using mozilla::ErrorResult;
 
-MOZ_STATIC_ASSERT(sizeof(size_t) >= sizeof(IDBCursor::Direction),
-                  "Relying on conversion between size_t and "
-                  "IDBCursor::Direction");
+static_assert(sizeof(size_t) >= sizeof(IDBCursor::Direction),
+              "Relying on conversion between size_t and IDBCursor::Direction");
 
 namespace {
 
@@ -58,6 +62,9 @@ public:
   UnpackResponseFromParentProcess(const ResponseValue& aResponseValue) = 0;
 
 protected:
+  virtual ~CursorHelper()
+  { }
+
   nsRefPtr<IDBCursor> mCursor;
 
 private:
@@ -75,19 +82,17 @@ public:
                  int32_t aCount)
   : CursorHelper(aCursor), mCount(aCount)
   {
-    NS_ASSERTION(aCount > 0, "Must have a count!");
-  }
-
-  ~ContinueHelper()
-  {
-    IDBObjectStore::ClearCloneReadInfo(mCloneReadInfo);
+    MOZ_ASSERT(NS_IsMainThread());
+    MOZ_ASSERT(aCursor);
+    MOZ_ASSERT(aCount > 0);
   }
 
   virtual nsresult DoDatabaseWork(mozIStorageConnection* aConnection)
                                   MOZ_OVERRIDE;
 
   virtual nsresult GetSuccessResult(JSContext* aCx,
-                                    jsval* aVal) MOZ_OVERRIDE;
+                                    JS::MutableHandle<JS::Value> aVal)
+                                    MOZ_OVERRIDE;
 
   virtual void ReleaseMainThreadObjects() MOZ_OVERRIDE;
 
@@ -102,6 +107,11 @@ public:
                                   MOZ_OVERRIDE;
 
 protected:
+  virtual ~ContinueHelper()
+  {
+    IDBObjectStore::ClearCloneReadInfo(mCloneReadInfo);
+  }
+
   virtual nsresult
   BindArgumentsToStatement(mozIStorageStatement* aStatement) = 0;
 
@@ -120,17 +130,17 @@ protected:
 
     if (mKey.IsUnset()) {
       mCursor->mHaveValue = false;
-    }
-    else {
-      NS_ASSERTION(mCursor->mType == IDBCursor::OBJECTSTORE ||
-                   !mObjectKey.IsUnset(), "Bad key!");
+    } else {
+      MOZ_ASSERT(mCursor->mType == IDBCursor::OBJECTSTORE ||
+                 mCursor->mType == IDBCursor::OBJECTSTOREKEY ||
+                 !mObjectKey.IsUnset());
 
       // Set new values.
       mCursor->mKey = mKey;
       mCursor->mObjectKey = mObjectKey;
       mCursor->mContinueToKey.Unset();
 
-      mCursor->mCloneReadInfo.Swap(mCloneReadInfo);
+      mCursor->mCloneReadInfo = Move(mCloneReadInfo);
       mCloneReadInfo.mCloneBuffer.clear();
     }
   }
@@ -149,11 +159,31 @@ public:
   : ContinueHelper(aCursor, aCount)
   { }
 
+protected:
+  virtual ~ContinueObjectStoreHelper()
+  { }
+
 private:
   nsresult BindArgumentsToStatement(mozIStorageStatement* aStatement);
   nsresult GatherResultsFromStatement(mozIStorageStatement* aStatement);
 };
 
+class ContinueObjectStoreKeyHelper : public ContinueObjectStoreHelper
+{
+public:
+  ContinueObjectStoreKeyHelper(IDBCursor* aCursor,
+                               uint32_t aCount)
+  : ContinueObjectStoreHelper(aCursor, aCount)
+  { }
+
+private:
+  virtual ~ContinueObjectStoreKeyHelper()
+  { }
+
+  virtual nsresult
+  GatherResultsFromStatement(mozIStorageStatement* aStatement) MOZ_OVERRIDE;
+};
+
 class ContinueIndexHelper : public ContinueHelper
 {
 public:
@@ -162,6 +192,10 @@ public:
   : ContinueHelper(aCursor, aCount)
   { }
 
+protected:
+  virtual ~ContinueIndexHelper()
+  { }
+
 private:
   nsresult BindArgumentsToStatement(mozIStorageStatement* aStatement);
   nsresult GatherResultsFromStatement(mozIStorageStatement* aStatement);
@@ -176,6 +210,9 @@ public:
   { }
 
 private:
+  virtual ~ContinueIndexObjectHelper()
+  { }
+
   nsresult GatherResultsFromStatement(mozIStorageStatement* aStatement);
 };
 
@@ -191,7 +228,7 @@ IDBCursor::Create(IDBRequest* aRequest,
                   const nsACString& aContinueQuery,
                   const nsACString& aContinueToQuery,
                   const Key& aKey,
-                  StructuredCloneReadInfo& aCloneReadInfo)
+                  StructuredCloneReadInfo&& aCloneReadInfo)
 {
   NS_ASSERTION(aObjectStore, "Null pointer!");
   NS_ASSERTION(!aKey.IsUnset(), "Bad key!");
@@ -204,7 +241,33 @@ IDBCursor::Create(IDBRequest* aRequest,
   cursor->mObjectStore = aObjectStore;
   cursor->mType = OBJECTSTORE;
   cursor->mKey = aKey;
-  cursor->mCloneReadInfo.Swap(aCloneReadInfo);
+  cursor->mCloneReadInfo = Move(aCloneReadInfo);
+
+  return cursor.forget();
+}
+
+// static
+already_AddRefed<IDBCursor>
+IDBCursor::Create(IDBRequest* aRequest,
+                  IDBTransaction* aTransaction,
+                  IDBObjectStore* aObjectStore,
+                  Direction aDirection,
+                  const Key& aRangeKey,
+                  const nsACString& aContinueQuery,
+                  const nsACString& aContinueToQuery,
+                  const Key& aKey)
+{
+  MOZ_ASSERT(aObjectStore);
+  MOZ_ASSERT(!aKey.IsUnset());
+
+  nsRefPtr<IDBCursor> cursor =
+    IDBCursor::CreateCommon(aRequest, aTransaction, aObjectStore, aDirection,
+                            aRangeKey, aContinueQuery, aContinueToQuery);
+  NS_ASSERTION(cursor, "This shouldn't fail!");
+
+  cursor->mObjectStore = aObjectStore;
+  cursor->mType = OBJECTSTOREKEY;
+  cursor->mKey = aKey;
 
   return cursor.forget();
 }
@@ -250,7 +313,7 @@ IDBCursor::Create(IDBRequest* aRequest,
                   const nsACString& aContinueToQuery,
                   const Key& aKey,
                   const Key& aObjectKey,
-                  StructuredCloneReadInfo& aCloneReadInfo)
+                  StructuredCloneReadInfo&& aCloneReadInfo)
 {
   NS_ASSERTION(aIndex, "Null pointer!");
   NS_ASSERTION(!aKey.IsUnset(), "Bad key!");
@@ -266,32 +329,31 @@ IDBCursor::Create(IDBRequest* aRequest,
   cursor->mType = INDEXOBJECT;
   cursor->mKey = aKey;
   cursor->mObjectKey = aObjectKey;
-  cursor->mCloneReadInfo.Swap(aCloneReadInfo);
+  cursor->mCloneReadInfo = Move(aCloneReadInfo);
 
   return cursor.forget();
 }
 
 // static
-nsresult
-IDBCursor::ParseDirection(const nsAString& aDirection, Direction* aResult)
+IDBCursor::Direction
+IDBCursor::ConvertDirection(mozilla::dom::IDBCursorDirection aDirection)
 {
-  if (aDirection.EqualsLiteral("next")) {
-    *aResult = NEXT;
+  switch (aDirection) {
+    case mozilla::dom::IDBCursorDirection::Next:
+      return NEXT;
+
+    case mozilla::dom::IDBCursorDirection::Nextunique:
+      return NEXT_UNIQUE;
+
+    case mozilla::dom::IDBCursorDirection::Prev:
+      return PREV;
+
+    case mozilla::dom::IDBCursorDirection::Prevunique:
+      return PREV_UNIQUE;
+
+    default:
+      MOZ_ASSUME_UNREACHABLE("Unknown direction!");
   }
-  else if (aDirection.EqualsLiteral("nextunique")) {
-    *aResult = NEXT_UNIQUE;
-  }
-  else if (aDirection.EqualsLiteral("prev")) {
-    *aResult = PREV;
-  }
-  else if (aDirection.EqualsLiteral("prevunique")) {
-    *aResult = PREV_UNIQUE;
-  }
-  else {
-    return NS_ERROR_TYPE_ERR;
-  }
-  
-  return NS_OK;
 }
 
 // static
@@ -321,10 +383,7 @@ IDBCursor::CreateCommon(IDBRequest* aRequest,
   cursor->mScriptOwner = database->GetScriptOwner();
 
   if (cursor->mScriptOwner) {
-    if (NS_FAILED(NS_HOLD_JS_OBJECTS(cursor, IDBCursor))) {
-      return nullptr;
-    }
-
+    mozilla::HoldJSObjects(cursor.get());
     cursor->mRooted = true;
   }
 
@@ -356,6 +415,8 @@ IDBCursor::IDBCursor()
   mHaveValue(true)
 {
   NS_ASSERTION(NS_IsMainThread(), "Wrong thread!");
+
+  SetIsDOMBinding();
 }
 
 IDBCursor::~IDBCursor()
@@ -369,38 +430,47 @@ IDBCursor::~IDBCursor()
     NS_ASSERTION(!mActorChild, "Should have cleared in Send__delete__!");
   }
 
-  if (mRooted) {
-    NS_DROP_JS_OBJECTS(this, IDBCursor);
-  }
+  DropJSObjects();
   IDBObjectStore::ClearCloneReadInfo(mCloneReadInfo);
 }
 
-nsresult
-IDBCursor::ContinueInternal(const Key& aKey,
-                            int32_t aCount)
+void
+IDBCursor::DropJSObjects()
 {
-  NS_ASSERTION(NS_IsMainThread(), "Wrong thread!");
-  NS_ASSERTION(aCount > 0, "Must have a count!");
+  if (!mRooted) {
+    return;
+  }
+  mScriptOwner = nullptr;
+  mCachedKey = JSVAL_VOID;
+  mCachedPrimaryKey = JSVAL_VOID;
+  mCachedValue = JSVAL_VOID;
+  mHaveCachedKey = false;
+  mHaveCachedPrimaryKey = false;
+  mHaveCachedValue = false;
+  mRooted = false;
+  mHaveValue = false;
+  mozilla::DropJSObjects(this);
+}
+
+void
+IDBCursor::ContinueInternal(const Key& aKey, int32_t aCount, ErrorResult& aRv)
+{
+  MOZ_ASSERT(NS_IsMainThread());
+  MOZ_ASSERT(aCount > 0);
 
   if (!mTransaction->IsOpen()) {
-    return NS_ERROR_DOM_INDEXEDDB_TRANSACTION_INACTIVE_ERR;
+    aRv.Throw(NS_ERROR_DOM_INDEXEDDB_TRANSACTION_INACTIVE_ERR);
+    return;
   }
 
   if (!mHaveValue || mContinueCalled) {
-    return NS_ERROR_DOM_INDEXEDDB_NOT_ALLOWED_ERR;
+    aRv.Throw(NS_ERROR_DOM_INDEXEDDB_NOT_ALLOWED_ERR);
+    return;
   }
 
   mContinueToKey = aKey;
 
-#ifdef DEBUG
-  {
-    nsAutoString readyState;
-    if (NS_FAILED(mRequest->GetReadyState(readyState))) {
-      NS_ERROR("This should never fail!");
-    }
-    NS_ASSERTION(readyState.EqualsLiteral("done"), "Should be DONE!");
-  }
-#endif
+  MOZ_ASSERT(mRequest->ReadyState() == IDBRequestReadyState::Done);
 
   mRequest->Reset();
 
@@ -410,6 +480,10 @@ IDBCursor::ContinueInternal(const Key& aKey,
       helper = new ContinueObjectStoreHelper(this, aCount);
       break;
 
+    case OBJECTSTOREKEY:
+      helper = new ContinueObjectStoreKeyHelper(this, aCount);
+      break;
+
     case INDEXKEY:
       helper = new ContinueIndexHelper(this, aCount);
       break;
@@ -419,14 +493,17 @@ IDBCursor::ContinueInternal(const Key& aKey,
       break;
 
     default:
-      NS_NOTREACHED("Unknown cursor type!");
+      MOZ_ASSUME_UNREACHABLE("Unknown cursor type!");
   }
 
   nsresult rv = helper->DispatchToTransactionPool();
-  NS_ENSURE_SUCCESS(rv, NS_ERROR_DOM_INDEXEDDB_UNKNOWN_ERR);
+  if (NS_FAILED(rv)) {
+    IDB_WARNING("Failed to dispatch!");
+    aRv.Throw(NS_ERROR_DOM_INDEXEDDB_UNKNOWN_ERR);
+    return;
+  }
 
   mContinueCalled = true;
-  return NS_OK;
 }
 
 NS_IMPL_CYCLE_COLLECTION_CLASS(IDBCursor)
@@ -440,6 +517,7 @@ NS_IMPL_CYCLE_COLLECTION_TRAVERSE_BEGIN(IDBCursor)
 NS_IMPL_CYCLE_COLLECTION_TRAVERSE_END
 
 NS_IMPL_CYCLE_COLLECTION_TRACE_BEGIN(IDBCursor)
+  NS_IMPL_CYCLE_COLLECTION_TRACE_PRESERVED_WRAPPER
   NS_ASSERTION(tmp->mHaveCachedKey || JSVAL_IS_VOID(tmp->mCachedKey),
                "Should have a cached key");
   NS_ASSERTION(tmp->mHaveCachedPrimaryKey ||
@@ -454,162 +532,158 @@ NS_IMPL_CYCLE_COLLECTION_TRACE_BEGIN(IDBCursor)
 NS_IMPL_CYCLE_COLLECTION_TRACE_END
 
 NS_IMPL_CYCLE_COLLECTION_UNLINK_BEGIN(IDBCursor)
+  NS_IMPL_CYCLE_COLLECTION_UNLINK_PRESERVED_WRAPPER
   // Don't unlink mObjectStore, mIndex, or mTransaction!
-  if (tmp->mRooted) {
-    NS_DROP_JS_OBJECTS(tmp, IDBCursor);
-    tmp->mScriptOwner = nullptr;
-    tmp->mCachedKey = JSVAL_VOID;
-    tmp->mCachedPrimaryKey = JSVAL_VOID;
-    tmp->mCachedValue = JSVAL_VOID;
-    tmp->mHaveCachedKey = false;
-    tmp->mHaveCachedPrimaryKey = false;
-    tmp->mHaveCachedValue = false;
-    tmp->mRooted = false;
-    tmp->mHaveValue = false;
-  }
+  tmp->DropJSObjects();
   NS_IMPL_CYCLE_COLLECTION_UNLINK(mRequest)
 NS_IMPL_CYCLE_COLLECTION_UNLINK_END
 
 NS_INTERFACE_MAP_BEGIN_CYCLE_COLLECTION(IDBCursor)
-  NS_INTERFACE_MAP_ENTRY(nsIIDBCursor)
-  NS_INTERFACE_MAP_ENTRY_CONDITIONAL(nsIIDBCursorWithValue, mType != INDEXKEY)
-  NS_DOM_INTERFACE_MAP_ENTRY_CLASSINFO_CONDITIONAL(IDBCursorWithValue,
-                                                   mType != INDEXKEY)
-  NS_DOM_INTERFACE_MAP_ENTRY_CLASSINFO_CONDITIONAL(IDBCursor,
-                                                   mType == INDEXKEY)
+  NS_WRAPPERCACHE_INTERFACE_MAP_ENTRY
   NS_INTERFACE_MAP_ENTRY(nsISupports)
 NS_INTERFACE_MAP_END
 
 NS_IMPL_CYCLE_COLLECTING_ADDREF(IDBCursor)
 NS_IMPL_CYCLE_COLLECTING_RELEASE(IDBCursor)
 
-DOMCI_DATA(IDBCursor, IDBCursor)
-DOMCI_DATA(IDBCursorWithValue, IDBCursor)
-
-NS_IMETHODIMP
-IDBCursor::GetDirection(nsAString& aDirection)
+JSObject*
+IDBCursor::WrapObject(JSContext* aCx, JS::Handle<JSObject*> aScope)
 {
-  NS_ASSERTION(NS_IsMainThread(), "Wrong thread!");
+  MOZ_ASSERT(NS_IsMainThread());
+
+  switch (mType) {
+    case OBJECTSTORE:
+    case INDEXOBJECT:
+      return IDBCursorWithValueBinding::Wrap(aCx, aScope, this);
+
+    case OBJECTSTOREKEY:
+    case INDEXKEY:
+      return IDBCursorBinding::Wrap(aCx, aScope, this);
+
+    default:
+      MOZ_ASSUME_UNREACHABLE("Bad type!");
+  }
+}
+
+mozilla::dom::IDBCursorDirection
+IDBCursor::GetDirection() const
+{
+  MOZ_ASSERT(NS_IsMainThread());
 
   switch (mDirection) {
     case NEXT:
-      aDirection.AssignLiteral("next");
-      break;
+      return mozilla::dom::IDBCursorDirection::Next;
+
     case NEXT_UNIQUE:
-      aDirection.AssignLiteral("nextunique");
-      break;
+      return mozilla::dom::IDBCursorDirection::Nextunique;
+
     case PREV:
-      aDirection.AssignLiteral("prev");
-      break;
+      return mozilla::dom::IDBCursorDirection::Prev;
+
     case PREV_UNIQUE:
-      aDirection.AssignLiteral("prevunique");
+      return mozilla::dom::IDBCursorDirection::Prevunique;
+
+    default:
+      MOZ_ASSUME_UNREACHABLE("Bad direction!");
+  }
+}
+
+void
+IDBCursor::GetSource(OwningIDBObjectStoreOrIDBIndex& aSource) const
+{
+  MOZ_ASSERT(NS_IsMainThread());
+
+  switch (mType) {
+    case OBJECTSTORE:
+    case OBJECTSTOREKEY:
+      MOZ_ASSERT(mObjectStore);
+      aSource.SetAsIDBObjectStore() = mObjectStore;
+      break;
+
+    case INDEXKEY:
+    case INDEXOBJECT:
+      MOZ_ASSERT(mIndex);
+      aSource.SetAsIDBIndex() = mIndex;
       break;
 
-    case DIRECTION_INVALID:
     default:
-      NS_NOTREACHED("Bad direction value!");
-      return NS_ERROR_UNEXPECTED;
+      MOZ_ASSUME_UNREACHABLE("Bad type!");
   }
-
-  return NS_OK;
 }
 
-NS_IMETHODIMP
-IDBCursor::GetSource(nsISupports** aSource)
+JS::Value
+IDBCursor::GetKey(JSContext* aCx, ErrorResult& aRv)
 {
-  NS_ASSERTION(NS_IsMainThread(), "Wrong thread!");
-
-  return mType == OBJECTSTORE ?
-         CallQueryInterface(mObjectStore, aSource) :
-         CallQueryInterface(mIndex, aSource);
-}
-
-NS_IMETHODIMP
-IDBCursor::GetKey(JSContext* aCx,
-                  jsval* aKey)
-{
-  NS_ASSERTION(NS_IsMainThread(), "Wrong thread!");
-
-  NS_ASSERTION(!mKey.IsUnset() || !mHaveValue, "Bad key!");
+  MOZ_ASSERT(NS_IsMainThread());
+  MOZ_ASSERT(!mKey.IsUnset() || !mHaveValue);
 
   if (!mHaveValue) {
-    *aKey = JSVAL_VOID;
-    return NS_OK;
+    return JSVAL_VOID;
   }
 
   if (!mHaveCachedKey) {
     if (!mRooted) {
-      NS_HOLD_JS_OBJECTS(this, IDBCursor);
+      mozilla::HoldJSObjects(this);
       mRooted = true;
     }
 
-    JSAutoRequest ar(aCx);
-
-    nsresult rv = mKey.ToJSVal(aCx, &mCachedKey);
-    NS_ENSURE_SUCCESS(rv, rv);
+    aRv = mKey.ToJSVal(aCx, mCachedKey);
+    ENSURE_SUCCESS(aRv, JSVAL_VOID);
 
     mHaveCachedKey = true;
   }
 
-  *aKey = mCachedKey;
-  return NS_OK;
+  return mCachedKey;
 }
 
-NS_IMETHODIMP
-IDBCursor::GetPrimaryKey(JSContext* aCx,
-                         jsval* aValue)
+JS::Value
+IDBCursor::GetPrimaryKey(JSContext* aCx, ErrorResult& aRv)
 {
-  NS_ASSERTION(NS_IsMainThread(), "Wrong thread!");
+  MOZ_ASSERT(NS_IsMainThread());
 
   if (!mHaveValue) {
-    *aValue = JSVAL_VOID;
-    return NS_OK;
+    return JSVAL_VOID;
   }
 
   if (!mHaveCachedPrimaryKey) {
     if (!mRooted) {
-      NS_HOLD_JS_OBJECTS(this, IDBCursor);
+      mozilla::HoldJSObjects(this);
       mRooted = true;
     }
 
-    JSAutoRequest ar(aCx);
+    const Key& key =
+      (mType == OBJECTSTORE || mType == OBJECTSTOREKEY) ? mKey : mObjectKey;
+    MOZ_ASSERT(!key.IsUnset());
 
-    NS_ASSERTION(mType == OBJECTSTORE ? !mKey.IsUnset() :
-                                        !mObjectKey.IsUnset(), "Bad key!");
-
-    const Key& key = mType == OBJECTSTORE ? mKey : mObjectKey;
-
-    nsresult rv = key.ToJSVal(aCx, &mCachedPrimaryKey);
-    NS_ENSURE_SUCCESS(rv, rv);
+    aRv = key.ToJSVal(aCx, mCachedPrimaryKey);
+    ENSURE_SUCCESS(aRv, JSVAL_VOID);
 
     mHaveCachedPrimaryKey = true;
   }
 
-  *aValue = mCachedPrimaryKey;
-  return NS_OK;
+  return mCachedPrimaryKey;
 }
 
-NS_IMETHODIMP
-IDBCursor::GetValue(JSContext* aCx,
-                    jsval* aValue)
+JS::Value
+IDBCursor::GetValue(JSContext* aCx, ErrorResult& aRv)
 {
-  NS_ASSERTION(NS_IsMainThread(), "Wrong thread!");
-  NS_ASSERTION(mType != INDEXKEY, "GetValue shouldn't exist on index keys");
+  MOZ_ASSERT(NS_IsMainThread());
+  MOZ_ASSERT(mType == OBJECTSTORE || mType == INDEXOBJECT);
 
   if (!mHaveValue) {
-    *aValue = JSVAL_VOID;
-    return NS_OK;
+    return JSVAL_VOID;
   }
 
   if (!mHaveCachedValue) {
     if (!mRooted) {
-      NS_HOLD_JS_OBJECTS(this, IDBCursor);
+      mozilla::HoldJSObjects(this);
       mRooted = true;
     }
 
-    jsval val;
+    JS::Rooted<JS::Value> val(aCx);
     if (!IDBObjectStore::DeserializeValue(aCx, mCloneReadInfo, &val)) {
-      return NS_ERROR_DOM_DATA_CLONE_ERR;
+      aRv.Throw(NS_ERROR_DOM_DATA_CLONE_ERR);
+      return JSVAL_VOID;
     }
 
     mCloneReadInfo.mCloneBuffer.clear();
@@ -618,138 +692,284 @@ IDBCursor::GetValue(JSContext* aCx,
     mHaveCachedValue = true;
   }
 
-  *aValue = mCachedValue;
-  return NS_OK;
+  return mCachedValue;
 }
 
-NS_IMETHODIMP
-IDBCursor::Continue(const jsval &aKey,
-                    JSContext* aCx)
+void
+IDBCursor::Continue(JSContext* aCx,
+                    JS::Handle<JS::Value> aKey,
+                    ErrorResult &aRv)
 {
-  NS_ASSERTION(NS_IsMainThread(), "Wrong thread!");
+  MOZ_ASSERT(NS_IsMainThread());
 
   Key key;
-  nsresult rv = key.SetFromJSVal(aCx, aKey);
-  NS_ENSURE_SUCCESS(rv, rv);
+  aRv = key.SetFromJSVal(aCx, aKey);
+  ENSURE_SUCCESS_VOID(aRv);
 
   if (!key.IsUnset()) {
     switch (mDirection) {
       case IDBCursor::NEXT:
       case IDBCursor::NEXT_UNIQUE:
         if (key <= mKey) {
-          return NS_ERROR_DOM_INDEXEDDB_DATA_ERR;
+          aRv.Throw(NS_ERROR_DOM_INDEXEDDB_DATA_ERR);
+          return;
         }
         break;
 
       case IDBCursor::PREV:
       case IDBCursor::PREV_UNIQUE:
         if (key >= mKey) {
-          return NS_ERROR_DOM_INDEXEDDB_DATA_ERR;
+          aRv.Throw(NS_ERROR_DOM_INDEXEDDB_DATA_ERR);
+          return;
         }
         break;
 
       default:
-        NS_NOTREACHED("Unknown direction type!");
+        MOZ_ASSUME_UNREACHABLE("Unknown direction type!");
     }
   }
 
-  return ContinueInternal(key, 1);
+  ContinueInternal(key, 1, aRv);
+  if (aRv.Failed()) {
+    return;
+  }
+
+#ifdef IDB_PROFILER_USE_MARKS
+  if (mType == OBJECTSTORE || mType == OBJECTSTOREKEY) {
+    IDB_PROFILER_MARK("IndexedDB Request %llu: "
+                      "database(%s).transaction(%s).objectStore(%s).cursor(%s)."
+                      "continue(%s)",
+                      "IDBRequest[%llu] MT IDBCursor.continue()",
+                      Request()->GetSerialNumber(),
+                      IDB_PROFILER_STRING(Transaction()->Database()),
+                      IDB_PROFILER_STRING(Transaction()),
+                      IDB_PROFILER_STRING(mObjectStore),
+                      IDB_PROFILER_STRING(mDirection),
+                      key.IsUnset() ? "" : IDB_PROFILER_STRING(key));
+  }
+  else {
+    IDB_PROFILER_MARK("IndexedDB Request %llu: "
+                      "database(%s).transaction(%s).objectStore(%s).index(%s)."
+                      "cursor(%s).continue(%s)",
+                      "IDBRequest[%llu] MT IDBCursor.continue()",
+                      Request()->GetSerialNumber(),
+                      IDB_PROFILER_STRING(Transaction()->Database()),
+                      IDB_PROFILER_STRING(Transaction()),
+                      IDB_PROFILER_STRING(mObjectStore),
+                      IDB_PROFILER_STRING(mIndex),
+                      IDB_PROFILER_STRING(mDirection),
+                      key.IsUnset() ? "" : IDB_PROFILER_STRING(key));
+  }
+#endif
 }
 
-NS_IMETHODIMP
-IDBCursor::Update(const jsval& aValue,
-                  JSContext* aCx,
-                  nsIIDBRequest** _retval)
+already_AddRefed<IDBRequest>
+IDBCursor::Update(JSContext* aCx, JS::Handle<JS::Value> aValue,
+                  ErrorResult& aRv)
 {
-  NS_ASSERTION(NS_IsMainThread(), "Wrong thread!");
+  MOZ_ASSERT(NS_IsMainThread());
 
   if (!mTransaction->IsOpen()) {
-    return NS_ERROR_DOM_INDEXEDDB_TRANSACTION_INACTIVE_ERR;
+    aRv.Throw(NS_ERROR_DOM_INDEXEDDB_TRANSACTION_INACTIVE_ERR);
+    return nullptr;
   }
 
   if (!mTransaction->IsWriteAllowed()) {
-    return NS_ERROR_DOM_INDEXEDDB_READ_ONLY_ERR;
+    aRv.Throw(NS_ERROR_DOM_INDEXEDDB_READ_ONLY_ERR);
+    return nullptr;
   }
 
-  if (!mHaveValue || mType == INDEXKEY) {
-    return NS_ERROR_DOM_INDEXEDDB_NOT_ALLOWED_ERR;
+  if (!mHaveValue || mType == OBJECTSTOREKEY || mType == INDEXKEY) {
+    aRv.Throw(NS_ERROR_DOM_INDEXEDDB_NOT_ALLOWED_ERR);
+    return nullptr;
   }
 
-  NS_ASSERTION(mObjectStore, "This cannot be null!");
-  NS_ASSERTION(!mKey.IsUnset() , "Bad key!");
-  NS_ASSERTION(mType != INDEXOBJECT || !mObjectKey.IsUnset(), "Bad key!");
+  MOZ_ASSERT(mObjectStore);
+  MOZ_ASSERT(!mKey.IsUnset());
+  MOZ_ASSERT(mType == OBJECTSTORE || mType == INDEXOBJECT);
+  MOZ_ASSERT_IF(mType == INDEXOBJECT, !mObjectKey.IsUnset());
 
-  nsresult rv;
-
-  JSAutoRequest ar(aCx);
-
-  Key& objectKey = (mType == OBJECTSTORE) ? mKey : mObjectKey;
+  const Key& objectKey = (mType == OBJECTSTORE) ? mKey : mObjectKey;
 
+  nsRefPtr<IDBRequest> request;
   if (mObjectStore->HasValidKeyPath()) {
     // Make sure the object given has the correct keyPath value set on it.
     const KeyPath& keyPath = mObjectStore->GetKeyPath();
     Key key;
 
-    rv = keyPath.ExtractKey(aCx, aValue, key);
-    if (NS_FAILED(rv)) {
-      return rv;
+    aRv = keyPath.ExtractKey(aCx, aValue, key);
+    if (aRv.Failed()) {
+      return nullptr;
     }
 
     if (key != objectKey) {
-      return NS_ERROR_DOM_INDEXEDDB_DATA_ERR;
+      aRv.Throw(NS_ERROR_DOM_INDEXEDDB_DATA_ERR);
+      return nullptr;
     }
 
-    return mObjectStore->Put(aValue, JSVAL_VOID, aCx, 0, _retval);
+    request = mObjectStore->Put(aCx, aValue, JS::UndefinedHandleValue, aRv);
+    if (aRv.Failed()) {
+      return nullptr;
+    }
+  }
+  else {
+    JS::Rooted<JS::Value> keyVal(aCx);
+    aRv = objectKey.ToJSVal(aCx, &keyVal);
+    ENSURE_SUCCESS(aRv, nullptr);
+
+    request = mObjectStore->Put(aCx, aValue, keyVal, aRv);
+    if (aRv.Failed()) {
+      return nullptr;
+    }
   }
 
-  jsval keyVal;
-  rv = objectKey.ToJSVal(aCx, &keyVal);
-  NS_ENSURE_SUCCESS(rv, rv);
+#ifdef IDB_PROFILER_USE_MARKS
+  {
+    uint64_t requestSerial =
+      static_cast<IDBRequest*>(request.get())->GetSerialNumber();
+    if (mType == OBJECTSTORE) {
+      IDB_PROFILER_MARK("IndexedDB Request %llu: "
+                        "database(%s).transaction(%s).objectStore(%s)."
+                        "cursor(%s).update(%s)",
+                        "IDBRequest[%llu] MT IDBCursor.update()",
+                        requestSerial,
+                        IDB_PROFILER_STRING(mTransaction->Database()),
+                        IDB_PROFILER_STRING(mTransaction),
+                        IDB_PROFILER_STRING(mObjectStore),
+                        IDB_PROFILER_STRING(mDirection),
+                        mObjectStore->HasValidKeyPath() ? "" :
+                          IDB_PROFILER_STRING(objectKey));
+    }
+    else {
+      IDB_PROFILER_MARK("IndexedDB Request %llu: "
+                        "database(%s).transaction(%s).objectStore(%s)."
+                        "index(%s).cursor(%s).update(%s)",
+                        "IDBRequest[%llu] MT IDBCursor.update()",
+                        requestSerial,
+                        IDB_PROFILER_STRING(mTransaction->Database()),
+                        IDB_PROFILER_STRING(mTransaction),
+                        IDB_PROFILER_STRING(mObjectStore),
+                        IDB_PROFILER_STRING(mIndex),
+                        IDB_PROFILER_STRING(mDirection),
+                        mObjectStore->HasValidKeyPath() ? "" :
+                          IDB_PROFILER_STRING(objectKey));
+    }
+  }
+#endif
 
-  return mObjectStore->Put(aValue, keyVal, aCx, 1, _retval);
+  return request.forget();
 }
 
-NS_IMETHODIMP
-IDBCursor::Delete(JSContext* aCx,
-                  nsIIDBRequest** _retval)
+already_AddRefed<IDBRequest>
+IDBCursor::Delete(JSContext* aCx, ErrorResult& aRv)
 {
-  NS_ASSERTION(NS_IsMainThread(), "Wrong thread!");
+  MOZ_ASSERT(NS_IsMainThread());
 
   if (!mTransaction->IsOpen()) {
-    return NS_ERROR_DOM_INDEXEDDB_TRANSACTION_INACTIVE_ERR;
+    aRv.Throw(NS_ERROR_DOM_INDEXEDDB_TRANSACTION_INACTIVE_ERR);
+    return nullptr;
   }
 
   if (!mTransaction->IsWriteAllowed()) {
-    return NS_ERROR_DOM_INDEXEDDB_READ_ONLY_ERR;
+    aRv.Throw(NS_ERROR_DOM_INDEXEDDB_READ_ONLY_ERR);
+    return nullptr;
   }
 
-  if (!mHaveValue || mType == INDEXKEY) {
-    return NS_ERROR_DOM_INDEXEDDB_NOT_ALLOWED_ERR;
+  if (!mHaveValue || mType == OBJECTSTOREKEY || mType == INDEXKEY) {
+    aRv.Throw(NS_ERROR_DOM_INDEXEDDB_NOT_ALLOWED_ERR);
+    return nullptr;
   }
 
-  NS_ASSERTION(mObjectStore, "This cannot be null!");
-  NS_ASSERTION(!mKey.IsUnset() , "Bad key!");
+  MOZ_ASSERT(mObjectStore);
+  MOZ_ASSERT(mType == OBJECTSTORE || mType == INDEXOBJECT);
+  MOZ_ASSERT(!mKey.IsUnset());
 
-  Key& objectKey = (mType == OBJECTSTORE) ? mKey : mObjectKey;
+  const Key& objectKey = (mType == OBJECTSTORE) ? mKey : mObjectKey;
 
-  jsval key;
-  nsresult rv = objectKey.ToJSVal(aCx, &key);
-  NS_ENSURE_SUCCESS(rv, rv);
+  JS::Rooted<JS::Value> key(aCx);
+  aRv = objectKey.ToJSVal(aCx, &key);
+  ENSURE_SUCCESS(aRv, nullptr);
 
-  return mObjectStore->Delete(key, aCx, _retval);
+  nsRefPtr<IDBRequest> request = mObjectStore->Delete(aCx, key, aRv);
+  ENSURE_SUCCESS(aRv, nullptr);
+
+#ifdef IDB_PROFILER_USE_MARKS
+  {
+    uint64_t requestSerial = request->GetSerialNumber();
+    if (mType == OBJECTSTORE) {
+      IDB_PROFILER_MARK("IndexedDB Request %llu: "
+                        "database(%s).transaction(%s).objectStore(%s)."
+                        "cursor(%s).delete(%s)",
+                        "IDBRequest[%llu] MT IDBCursor.delete()",
+                        requestSerial,
+                        IDB_PROFILER_STRING(mTransaction->Database()),
+                        IDB_PROFILER_STRING(mTransaction),
+                        IDB_PROFILER_STRING(mObjectStore),
+                        IDB_PROFILER_STRING(mDirection),
+                        mObjectStore->HasValidKeyPath() ? "" :
+                          IDB_PROFILER_STRING(objectKey));
+    }
+    else {
+      IDB_PROFILER_MARK("IndexedDB Request %llu: "
+                        "database(%s).transaction(%s).objectStore(%s)."
+                        "index(%s).cursor(%s).delete(%s)",
+                        "IDBRequest[%llu] MT IDBCursor.delete()",
+                        requestSerial,
+                        IDB_PROFILER_STRING(mTransaction->Database()),
+                        IDB_PROFILER_STRING(mTransaction),
+                        IDB_PROFILER_STRING(mObjectStore),
+                        IDB_PROFILER_STRING(mIndex),
+                        IDB_PROFILER_STRING(mDirection),
+                        mObjectStore->HasValidKeyPath() ? "" :
+                          IDB_PROFILER_STRING(objectKey));
+    }
+  }
+#endif
+
+  return request.forget();
 }
 
-NS_IMETHODIMP
-IDBCursor::Advance(int64_t aCount)
+void
+IDBCursor::Advance(uint32_t aCount, ErrorResult &aRv)
 {
-  NS_ASSERTION(NS_IsMainThread(), "Wrong thread!");
+  MOZ_ASSERT(NS_IsMainThread());
 
-  if (aCount < 1 || aCount > UINT32_MAX) {
-    return NS_ERROR_TYPE_ERR;
+  if (aCount < 1) {
+    aRv.ThrowTypeError(MSG_INVALID_ADVANCE_COUNT);
+    return;
   }
 
   Key key;
-  return ContinueInternal(key, int32_t(aCount));
+  ContinueInternal(key, int32_t(aCount), aRv);
+  ENSURE_SUCCESS_VOID(aRv);
+
+#ifdef IDB_PROFILER_USE_MARKS
+  {
+    if (mType == OBJECTSTORE || mType == OBJECTSTOREKEY) {
+      IDB_PROFILER_MARK("IndexedDB Request %llu: "
+                        "database(%s).transaction(%s).objectStore(%s)."
+                        "cursor(%s).advance(%ld)",
+                        "IDBRequest[%llu] MT IDBCursor.advance()",
+                        Request()->GetSerialNumber(),
+                        IDB_PROFILER_STRING(Transaction()->Database()),
+                        IDB_PROFILER_STRING(Transaction()),
+                        IDB_PROFILER_STRING(mObjectStore),
+                        IDB_PROFILER_STRING(mDirection), aCount);
+    }
+    else {
+      IDB_PROFILER_MARK("IndexedDB Request %llu: "
+                        "database(%s).transaction(%s).objectStore(%s)."
+                        "index(%s).cursor(%s).advance(%ld)",
+                        "IDBRequest[%llu] MT IDBCursor.advance()",
+                        Request()->GetSerialNumber(),
+                        IDB_PROFILER_STRING(Transaction()->Database()),
+                        IDB_PROFILER_STRING(Transaction()),
+                        IDB_PROFILER_STRING(mObjectStore),
+                        IDB_PROFILER_STRING(mIndex),
+                        IDB_PROFILER_STRING(mDirection), aCount);
+    }
+  }
+#endif
 }
 
 void
@@ -762,6 +982,10 @@ CursorHelper::ReleaseMainThreadObjects()
 nsresult
 CursorHelper::Dispatch(nsIEventTarget* aDatabaseThread)
 {
+  NS_ASSERTION(NS_IsMainThread(), "Wrong thread!");
+
+  PROFILER_MAIN_THREAD_LABEL("IndexedDB", "CursorHelper::Dispatch");
+
   if (IndexedDatabaseManager::IsMainProcess()) {
     return AsyncConnectionHelper::Dispatch(aDatabaseThread);
   }
@@ -769,6 +993,7 @@ CursorHelper::Dispatch(nsIEventTarget* aDatabaseThread)
   // If we've been invalidated then there's no point sending anything to the
   // parent process.
   if (mCursor->Transaction()->Database()->IsInvalidated()) {
+    IDB_REPORT_INTERNAL_ERR();
     return NS_ERROR_DOM_INDEXEDDB_UNKNOWN_ERR;
   }
 
@@ -777,11 +1002,11 @@ CursorHelper::Dispatch(nsIEventTarget* aDatabaseThread)
 
   CursorRequestParams params;
   nsresult rv = PackArgumentsForParentProcess(params);
-  NS_ENSURE_SUCCESS(rv, NS_ERROR_DOM_INDEXEDDB_UNKNOWN_ERR);
+  IDB_ENSURE_SUCCESS(rv, NS_ERROR_DOM_INDEXEDDB_UNKNOWN_ERR);
 
   NoDispatchEventTarget target;
   rv = AsyncConnectionHelper::Dispatch(&target);
-  NS_ENSURE_SUCCESS(rv, NS_ERROR_DOM_INDEXEDDB_UNKNOWN_ERR);
+  IDB_ENSURE_SUCCESS(rv, NS_ERROR_DOM_INDEXEDDB_UNKNOWN_ERR);
 
   mActor = new IndexedDBCursorRequestChild(this, mCursor, params.type());
   cursorActor->SendPIndexedDBRequestConstructor(mActor, params);
@@ -792,6 +1017,11 @@ CursorHelper::Dispatch(nsIEventTarget* aDatabaseThread)
 nsresult
 ContinueHelper::DoDatabaseWork(mozIStorageConnection* aConnection)
 {
+  NS_ASSERTION(!NS_IsMainThread(), "Wrong thread!");
+  NS_ASSERTION(IndexedDatabaseManager::IsMainProcess(), "Wrong process!");
+
+  PROFILER_LABEL("IndexedDB", "ContinueHelper::DoDatabaseWork");
+
   // We need to pick a query based on whether or not the cursor's mContinueToKey
   // is set. If it is unset then othing was passed to continue so we'll grab the
   // next item in the database that is greater than (less than, if we're running
@@ -812,19 +1042,19 @@ ContinueHelper::DoDatabaseWork(mozIStorageConnection* aConnection)
   query.AppendInt(mCount);
 
   nsCOMPtr<mozIStorageStatement> stmt = mTransaction->GetCachedStatement(query);
-  NS_ENSURE_TRUE(stmt, NS_ERROR_DOM_INDEXEDDB_UNKNOWN_ERR);
+  IDB_ENSURE_TRUE(stmt, NS_ERROR_DOM_INDEXEDDB_UNKNOWN_ERR);
 
   mozStorageStatementScoper scoper(stmt);
 
   nsresult rv = BindArgumentsToStatement(stmt);
-  NS_ENSURE_SUCCESS(rv, NS_ERROR_DOM_INDEXEDDB_UNKNOWN_ERR);
+  IDB_ENSURE_SUCCESS(rv, NS_ERROR_DOM_INDEXEDDB_UNKNOWN_ERR);
 
   NS_ASSERTION(mCount > 0, "Not ok!");
 
   bool hasResult;
   for (int32_t index = 0; index < mCount; index++) {
     rv = stmt->ExecuteStep(&hasResult);
-    NS_ENSURE_SUCCESS(rv, NS_ERROR_DOM_INDEXEDDB_UNKNOWN_ERR);
+    IDB_ENSURE_SUCCESS(rv, NS_ERROR_DOM_INDEXEDDB_UNKNOWN_ERR);
 
     if (!hasResult) {
       break;
@@ -833,7 +1063,7 @@ ContinueHelper::DoDatabaseWork(mozIStorageConnection* aConnection)
 
   if (hasResult) {
     rv = GatherResultsFromStatement(stmt);
-    NS_ENSURE_SUCCESS(rv, NS_ERROR_DOM_INDEXEDDB_UNKNOWN_ERR);
+    IDB_ENSURE_SUCCESS(rv, NS_ERROR_DOM_INDEXEDDB_UNKNOWN_ERR);
   }
   else {
     mKey.Unset();
@@ -844,12 +1074,12 @@ ContinueHelper::DoDatabaseWork(mozIStorageConnection* aConnection)
 
 nsresult
 ContinueHelper::GetSuccessResult(JSContext* aCx,
-                                 jsval* aVal)
+                                 JS::MutableHandle<JS::Value> aVal)
 {
   UpdateCursorState();
 
   if (mKey.IsUnset()) {
-    *aVal = JSVAL_NULL;
+    aVal.setNull();
   }
   else {
     nsresult rv = WrapNative(aCx, mCursor, aVal);
@@ -869,6 +1099,12 @@ ContinueHelper::ReleaseMainThreadObjects()
 nsresult
 ContinueHelper::PackArgumentsForParentProcess(CursorRequestParams& aParams)
 {
+  NS_ASSERTION(NS_IsMainThread(), "Wrong thread!");
+  NS_ASSERTION(!IndexedDatabaseManager::IsMainProcess(), "Wrong process!");
+
+  PROFILER_MAIN_THREAD_LABEL("IndexedDB",
+                             "ContinueHelper::PackArgumentsForParentProcess");
+
   ContinueParams params;
 
   params.key() = mCursor->mContinueToKey;
@@ -884,6 +1120,9 @@ ContinueHelper::SendResponseToChildProcess(nsresult aResultCode)
   NS_ASSERTION(NS_IsMainThread(), "Wrong thread!");
   NS_ASSERTION(IndexedDatabaseManager::IsMainProcess(), "Wrong process!");
 
+  PROFILER_MAIN_THREAD_LABEL("IndexedDB",
+                             "ContinueHelper::SendResponseToChildProcess");
+
   IndexedDBRequestParentBase* actor = mRequest->GetActorParent();
   NS_ASSERTION(actor, "How did we get this far without an actor?");
 
@@ -950,7 +1189,7 @@ ContinueHelper::UnpackResponseFromParentProcess(
                "Inconsistent clone info!");
 
   if (!mCloneReadInfo.SetFromSerialized(cloneInfo)) {
-    NS_WARNING("Failed to copy clone buffer!");
+    IDB_WARNING("Failed to copy clone buffer!");
     return NS_ERROR_DOM_INDEXEDDB_UNKNOWN_ERR;
   }
 
@@ -963,10 +1202,13 @@ nsresult
 ContinueObjectStoreHelper::BindArgumentsToStatement(
                                                mozIStorageStatement* aStatement)
 {
+  MOZ_ASSERT(!NS_IsMainThread());
+  MOZ_ASSERT(aStatement);
+
   // Bind object store id.
   nsresult rv = aStatement->BindInt64ByName(NS_LITERAL_CSTRING("id"),
                                             mCursor->mObjectStore->Id());
-  NS_ENSURE_SUCCESS(rv, NS_ERROR_DOM_INDEXEDDB_UNKNOWN_ERR);
+  IDB_ENSURE_SUCCESS(rv, NS_ERROR_DOM_INDEXEDDB_UNKNOWN_ERR);
 
   NS_NAMED_LITERAL_CSTRING(currentKeyName, "current_key");
   NS_NAMED_LITERAL_CSTRING(rangeKeyName, "range_key");
@@ -994,12 +1236,29 @@ nsresult
 ContinueObjectStoreHelper::GatherResultsFromStatement(
                                                mozIStorageStatement* aStatement)
 {
+  MOZ_ASSERT(!NS_IsMainThread());
+  MOZ_ASSERT(aStatement);
+
   // Figure out what kind of key we have next.
   nsresult rv = mKey.SetFromStatement(aStatement, 0);
   NS_ENSURE_SUCCESS(rv, rv);
 
   rv = IDBObjectStore::GetStructuredCloneReadInfoFromStatement(aStatement, 1, 2,
-    mDatabase, mCloneReadInfo);
+                                                               mDatabase,
+                                                               mCloneReadInfo);
+  NS_ENSURE_SUCCESS(rv, rv);
+
+  return NS_OK;
+}
+
+nsresult
+ContinueObjectStoreKeyHelper::GatherResultsFromStatement(
+                                               mozIStorageStatement* aStatement)
+{
+  MOZ_ASSERT(!NS_IsMainThread());
+  MOZ_ASSERT(aStatement);
+
+  nsresult rv = mKey.SetFromStatement(aStatement, 0);
   NS_ENSURE_SUCCESS(rv, rv);
 
   return NS_OK;
@@ -1011,7 +1270,7 @@ ContinueIndexHelper::BindArgumentsToStatement(mozIStorageStatement* aStatement)
   // Bind index id.
   nsresult rv = aStatement->BindInt64ByName(NS_LITERAL_CSTRING("id"),
                                             mCursor->mIndex->Id());
-  NS_ENSURE_SUCCESS(rv, NS_ERROR_DOM_INDEXEDDB_UNKNOWN_ERR);
+  IDB_ENSURE_SUCCESS(rv, NS_ERROR_DOM_INDEXEDDB_UNKNOWN_ERR);
 
   NS_NAMED_LITERAL_CSTRING(currentKeyName, "current_key");
 
