@@ -1084,6 +1084,10 @@ MacFontInfo::LoadFontFamilyData(const nsAString& aFamilyName)
         }
         CTFontRef fontRef = CTFontCreateWithFontDescriptor(faceDesc,
                                                            0.0, nullptr);
+        if (!fontRef) {
+            NS_WARNING("failed to create a CTFontRef");
+            continue;
+        }
 
         if (mLoadCmaps) {
             // face name
