@@ -10,11 +10,12 @@
 #include <stdlib.h>
 
 #include "ImageLogging.h"
-#include "EndianMacros.h"
+#include "mozilla/Endian.h"
 #include "nsBMPDecoder.h"
 
 #include "nsIInputStream.h"
 #include "RasterImage.h"
+#include <algorithm>
 
 namespace mozilla {
 namespace image {
@@ -34,12 +35,11 @@ GetBMPLog()
 #define LINE(row) ((mBIH.height < 0) ? (-mBIH.height - (row)) : ((row) - 1))
 #define PIXEL_OFFSET(row, col) (LINE(row) * mBIH.width + col)
 
-nsBMPDecoder::nsBMPDecoder(RasterImage &aImage, imgDecoderObserver* aObserver)
- : Decoder(aImage, aObserver)
+nsBMPDecoder::nsBMPDecoder(RasterImage &aImage)
+ : Decoder(aImage)
 {
   mColors = nullptr;
   mRow = nullptr;
-  mImageData = nullptr;
   mCurPos = mPos = mNumColors = mRowBytes = 0;
   mOldLine = mCurLine = 1; // Otherwise decoder will never start
   mState = eRLEStateInitial;
@@ -88,7 +88,7 @@ nsBMPDecoder::GetHeight() const
 uint32_t* 
 nsBMPDecoder::GetImageData() 
 {
-  return mImageData;
+  return reinterpret_cast<uint32_t*>(mImageData);
 }
 
 // Obtains the size of the compressed image resource
@@ -133,13 +133,17 @@ nsBMPDecoder::FinishInternal()
     NS_ABORT_IF_FALSE(GetFrameCount() <= 1, "Multiple BMP frames?");
 
     // Send notifications if appropriate
-    if (!IsSizeDecode() && (GetFrameCount() == 1)) {
+    if (!IsSizeDecode() && HasSize()) {
 
         // Invalidate
         nsIntRect r(0, 0, mBIH.width, GetHeight());
         PostInvalidation(r);
 
-        PostFrameStop();
+        if (mUseAlphaData) {
+          PostFrameStop(FrameBlender::kFrameHasAlpha);
+        } else {
+          PostFrameStop(FrameBlender::kFrameOpaque);
+        }
         PostDecodeDone();
     }
 }
@@ -185,7 +189,7 @@ NS_METHOD nsBMPDecoder::CalcBitShift()
 }
 
 void
-nsBMPDecoder::WriteInternal(const char* aBuffer, uint32_t aCount)
+nsBMPDecoder::WriteInternal(const char* aBuffer, uint32_t aCount, DecodeStrategy)
 {
     NS_ABORT_IF_FALSE(!HasError(), "Shouldn't call WriteInternal after error!");
 
@@ -193,7 +197,6 @@ nsBMPDecoder::WriteInternal(const char* aBuffer, uint32_t aCount)
     if (!aCount || !mCurLine)
         return;
 
-    nsresult rv;
     if (mPos < BFH_INTERNAL_LENGTH) { /* In BITMAPFILEHEADER */
         uint32_t toCopy = BFH_INTERNAL_LENGTH - mPos;
         if (toCopy > aCount)
@@ -222,10 +225,10 @@ nsBMPDecoder::WriteInternal(const char* aBuffer, uint32_t aCount)
         aBuffer += toCopy;
     }
 
-    // GetNumFrames is called to ensure that if at this point mPos == mLOH but
+    // HasSize is called to ensure that if at this point mPos == mLOH but
     // we have no data left to process, the next time WriteInternal is called
     // we won't enter this condition again.
-    if (mPos == mLOH && GetFrameCount() == 0) {
+    if (mPos == mLOH && !HasSize()) {
         ProcessInfoHeader();
         PR_LOG(GetBMPLog(), PR_LOG_DEBUG, ("BMP is %lix%lix%lu. compression=%lu\n",
                mBIH.width, mBIH.height, mBIH.bpp, mBIH.compression));
@@ -244,6 +247,11 @@ nsBMPDecoder::WriteInternal(const char* aBuffer, uint32_t aCount)
             return;
         }
 
+        if (mBIH.height == INT_MIN) {
+            PostDataError();
+            return;
+        }
+
         uint32_t real_height = GetHeight();
 
         // Post our size to the superclass
@@ -308,34 +316,19 @@ nsBMPDecoder::WriteInternal(const char* aBuffer, uint32_t aCount)
           return;
         }
 
-        uint32_t imageLength;
-        if (mBIH.compression == BI_RLE8 || mBIH.compression == BI_RLE4 || 
-            mBIH.compression == BI_ALPHABITFIELDS) {
-            rv = mImage.EnsureFrame(0, 0, 0, mBIH.width, real_height, 
-                                    gfxASurface::ImageFormatARGB32,
-                                    (uint8_t**)&mImageData, &imageLength);
-        } else {
+        if (mBIH.compression != BI_RLE8 && mBIH.compression != BI_RLE4 &&
+            mBIH.compression != BI_ALPHABITFIELDS) {
             // mRow is not used for RLE encoded images
             mRow = (uint8_t*)moz_malloc((mBIH.width * mBIH.bpp) / 8 + 4);
             // + 4 because the line is padded to a 4 bit boundary, but I don't want
             // to make exact calculations here, that's unnecessary.
             // Also, it compensates rounding error.
             if (!mRow) {
-                PostDecoderError(NS_ERROR_OUT_OF_MEMORY);
-                return;
-            }
-
-            if (mUseAlphaData) {
-              rv = mImage.EnsureFrame(0, 0, 0, mBIH.width, real_height, 
-                                      gfxASurface::ImageFormatARGB32,
-                                      (uint8_t**)&mImageData, &imageLength);
-            } else {
-              rv = mImage.EnsureFrame(0, 0, 0, mBIH.width, real_height, 
-                                      gfxASurface::ImageFormatRGB24,
-                                      (uint8_t**)&mImageData, &imageLength);
+              PostDataError();
+              return;
             }
         }
-        if (NS_FAILED(rv) || !mImageData) {
+        if (!mImageData) {
             PostDecoderError(NS_ERROR_FAILURE);
             return;
         }
@@ -343,11 +336,8 @@ nsBMPDecoder::WriteInternal(const char* aBuffer, uint32_t aCount)
         // Prepare for transparency
         if ((mBIH.compression == BI_RLE8) || (mBIH.compression == BI_RLE4)) {
             // Clear the image, as the RLE may jump over areas
-            memset(mImageData, 0, imageLength);
+            memset(mImageData, 0, mImageDataLength);
         }
-
-        // Tell the superclass we're starting a frame
-        PostFrameStart();
     }
 
     if (mColors && mPos >= mLOH) {
@@ -398,9 +388,9 @@ nsBMPDecoder::WriteInternal(const char* aBuffer, uint32_t aCount)
     }
     if (mPos == WIN_V3_HEADER_LENGTH + BITFIELD_LENGTH && 
         mBIH.compression == BI_BITFIELDS) {
-        mBitFields.red = LITTLE_TO_NATIVE32(*(uint32_t*)mRawBuf);
-        mBitFields.green = LITTLE_TO_NATIVE32(*(uint32_t*)(mRawBuf + 4));
-        mBitFields.blue = LITTLE_TO_NATIVE32(*(uint32_t*)(mRawBuf + 8));
+        mBitFields.red = LittleEndian::readUint32(reinterpret_cast<uint32_t*>(mRawBuf));
+        mBitFields.green = LittleEndian::readUint32(reinterpret_cast<uint32_t*>(mRawBuf + 4));
+        mBitFields.blue = LittleEndian::readUint32(reinterpret_cast<uint32_t*>(mRawBuf + 8));
         CalcBitShift();
     }
     while (aCount && (mPos < mBFH.dataoffset)) { // Skip whatever is between header and data
@@ -428,7 +418,7 @@ nsBMPDecoder::WriteInternal(const char* aBuffer, uint32_t aCount)
                 if (rowSize == mRowBytes) {
                     // Collected a whole row into mRow, process it
                     uint8_t* p = mRow;
-                    uint32_t* d = mImageData + PIXEL_OFFSET(mCurLine, 0);
+                    uint32_t* d = reinterpret_cast<uint32_t*>(mImageData) + PIXEL_OFFSET(mCurLine, 0);
                     uint32_t lpos = mBIH.width;
                     switch (mBIH.bpp) {
                       case 1:
@@ -458,7 +448,7 @@ nsBMPDecoder::WriteInternal(const char* aBuffer, uint32_t aCount)
                         break;
                       case 16:
                         while (lpos > 0) {
-                          uint16_t val = LITTLE_TO_NATIVE16(*(uint16_t*)p);
+                          uint16_t val = LittleEndian::readUint16(reinterpret_cast<uint16_t*>(p));
                           SetPixel(d,
                                   (val & mBitFields.red) >> mBitFields.redRightShift << mBitFields.redLeftShift,
                                   (val & mBitFields.green) >> mBitFields.greenRightShift << mBitFields.greenLeftShift,
@@ -486,7 +476,7 @@ nsBMPDecoder::WriteInternal(const char* aBuffer, uint32_t aCount)
                               // 4 has been right all along.  And we know it
                               // has been set to 0 the whole time, so that 
                               // means that everything is transparent so far.
-                              uint32_t* start = mImageData + GetWidth() * (mCurLine - 1);
+                              uint32_t* start = reinterpret_cast<uint32_t*>(mImageData) + GetWidth() * (mCurLine - 1);
                               uint32_t heightDifference = GetHeight() - mCurLine + 1;
                               uint32_t pixelCount = GetWidth() * heightDifference;
 
@@ -544,9 +534,9 @@ nsBMPDecoder::WriteInternal(const char* aBuffer, uint32_t aCount)
                             // the second byte
                             // Work around bitmaps that specify too many pixels
                             mState = eRLEStateInitial;
-                            uint32_t pixelsNeeded = NS_MIN<uint32_t>(mBIH.width - mCurPos, mStateData);
+                            uint32_t pixelsNeeded = std::min<uint32_t>(mBIH.width - mCurPos, mStateData);
                             if (pixelsNeeded) {
-                                uint32_t* d = mImageData + PIXEL_OFFSET(mCurLine, mCurPos);
+                                uint32_t* d = reinterpret_cast<uint32_t*>(mImageData) + PIXEL_OFFSET(mCurLine, mCurPos);
                                 mCurPos += pixelsNeeded;
                                 if (mBIH.compression == BI_RLE8) {
                                     do {
@@ -624,7 +614,7 @@ nsBMPDecoder::WriteInternal(const char* aBuffer, uint32_t aCount)
                         byte = *aBuffer++;
                         aCount--;
                         mState = eRLEStateInitial;
-                        mCurLine -= NS_MIN<int32_t>(byte, mCurLine);
+                        mCurLine -= std::min<int32_t>(byte, mCurLine);
                         break;
 
                     case eRLEStateAbsoluteMode: // Absolute Mode
@@ -634,7 +624,7 @@ nsBMPDecoder::WriteInternal(const char* aBuffer, uint32_t aCount)
                             // represents the number of pixels 
                             // that follow, each of which contains 
                             // the color index of a single pixel.
-                            uint32_t* d = mImageData + PIXEL_OFFSET(mCurLine, mCurPos);
+                            uint32_t* d = reinterpret_cast<uint32_t*>(mImageData) + PIXEL_OFFSET(mCurLine, mCurPos);
                             uint32_t* oldPos = d;
                             if (mBIH.compression == BI_RLE8) {
                                 while (aCount > 0 && mStateData > 0) {
@@ -708,9 +698,9 @@ void nsBMPDecoder::ProcessFileHeader()
     memcpy(&mBFH.bihsize, mRawBuf + 14, sizeof(mBFH.bihsize));
 
     // Now correct the endianness of the header
-    mBFH.filesize = LITTLE_TO_NATIVE32(mBFH.filesize);
-    mBFH.dataoffset = LITTLE_TO_NATIVE32(mBFH.dataoffset);
-    mBFH.bihsize = LITTLE_TO_NATIVE32(mBFH.bihsize);
+    mBFH.filesize = LittleEndian::readUint32(&mBFH.filesize);
+    mBFH.dataoffset = LittleEndian::readUint32(&mBFH.dataoffset);
+    mBFH.bihsize = LittleEndian::readUint32(&mBFH.bihsize);
 }
 
 void nsBMPDecoder::ProcessInfoHeader()
@@ -736,17 +726,17 @@ void nsBMPDecoder::ProcessInfoHeader()
     }
 
     // Convert endianness
-    mBIH.width = LITTLE_TO_NATIVE32(mBIH.width);
-    mBIH.height = LITTLE_TO_NATIVE32(mBIH.height);
-    mBIH.planes = LITTLE_TO_NATIVE16(mBIH.planes);
-    mBIH.bpp = LITTLE_TO_NATIVE16(mBIH.bpp);
+    mBIH.width = LittleEndian::readUint32(&mBIH.width);
+    mBIH.height = LittleEndian::readUint32(&mBIH.height);
+    mBIH.planes = LittleEndian::readUint16(&mBIH.planes);
+    mBIH.bpp = LittleEndian::readUint16(&mBIH.bpp);
 
-    mBIH.compression = LITTLE_TO_NATIVE32(mBIH.compression);
-    mBIH.image_size = LITTLE_TO_NATIVE32(mBIH.image_size);
-    mBIH.xppm = LITTLE_TO_NATIVE32(mBIH.xppm);
-    mBIH.yppm = LITTLE_TO_NATIVE32(mBIH.yppm);
-    mBIH.colors = LITTLE_TO_NATIVE32(mBIH.colors);
-    mBIH.important_colors = LITTLE_TO_NATIVE32(mBIH.important_colors);
+    mBIH.compression = LittleEndian::readUint32(&mBIH.compression);
+    mBIH.image_size = LittleEndian::readUint32(&mBIH.image_size);
+    mBIH.xppm = LittleEndian::readUint32(&mBIH.xppm);
+    mBIH.yppm = LittleEndian::readUint32(&mBIH.yppm);
+    mBIH.colors = LittleEndian::readUint32(&mBIH.colors);
+    mBIH.important_colors = LittleEndian::readUint32(&mBIH.important_colors);
 }
 
 } // namespace image
