@@ -107,138 +107,6 @@ gc::GetPageFaultCount()
     return pmc.PageFaultCount;
 }
 
-#elif defined(XP_OS2)
-
-#define INCL_DOSMEMMGR
-#include <os2.h>
-
-#define JS_GC_HAS_MAP_ALIGN 1
-#define OS2_MAX_RECURSIONS  16
-
-void
-gc::InitMemorySubsystem(JSRuntime *rt)
-{
-    rt->gcSystemPageSize = rt->gcSystemAllocGranularity = ArenaSize;
-}
-
-void
-gc::UnmapPages(JSRuntime *rt, void *addr, size_t size)
-{
-    if (!DosFreeMem(addr))
-        return;
-
-    /*
-     * If DosFreeMem() failed, 'addr' is probably part of an "expensive"
-     * allocation, so calculate the base address and try again.
-     */
-    unsigned long cb = 2 * size;
-    unsigned long flags;
-    if (DosQueryMem(addr, &cb, &flags) || cb < size)
-        return;
-
-    uintptr_t base = reinterpret_cast<uintptr_t>(addr) - ((2 * size) - cb);
-    DosFreeMem(reinterpret_cast<void*>(base));
-
-    return;
-}
-
-static void *
-MapAlignedPagesRecursively(JSRuntime *rt, size_t size, size_t alignment, int& recursions)
-{
-    if (++recursions >= OS2_MAX_RECURSIONS)
-        return nullptr;
-
-    void *tmp;
-    if (DosAllocMem(&tmp, size,
-                    OBJ_ANY | PAG_COMMIT | PAG_READ | PAG_WRITE)) {
-        JS_ALWAYS_TRUE(DosAllocMem(&tmp, size,
-                                   PAG_COMMIT | PAG_READ | PAG_WRITE) == 0);
-    }
-    size_t offset = reinterpret_cast<uintptr_t>(tmp) & (alignment - 1);
-    if (!offset)
-        return tmp;
-
-    /*
-     * If there are 'filler' bytes of free space above 'tmp', free 'tmp',
-     * then reallocate it as a 'filler'-sized block;  assuming we're not
-     * in a race with another thread, the next recursion should succeed.
-     */
-    size_t filler = size + alignment - offset;
-    unsigned long cb = filler;
-    unsigned long flags = 0;
-    unsigned long rc = DosQueryMem(&(static_cast<char*>(tmp))[size],
-                                   &cb, &flags);
-    if (!rc && (flags & PAG_FREE) && cb >= filler) {
-        UnmapPages(rt, tmp, 0);
-        if (DosAllocMem(&tmp, filler,
-                        OBJ_ANY | PAG_COMMIT | PAG_READ | PAG_WRITE)) {
-            JS_ALWAYS_TRUE(DosAllocMem(&tmp, filler,
-                                       PAG_COMMIT | PAG_READ | PAG_WRITE) == 0);
-        }
-    }
-
-    void *p = MapAlignedPagesRecursively(rt, size, alignment, recursions);
-    UnmapPages(rt, tmp, 0);
-
-    return p;
-}
-
-void *
-gc::MapAlignedPages(JSRuntime *rt, size_t size, size_t alignment)
-{
-    JS_ASSERT(size >= alignment);
-    JS_ASSERT(size % alignment == 0);
-    JS_ASSERT(size % rt->gcSystemPageSize == 0);
-    JS_ASSERT(alignment % rt->gcSystemAllocGranularity == 0);
-
-    int recursions = -1;
-
-    /*
-     * Make up to OS2_MAX_RECURSIONS attempts to get an aligned block
-     * of the right size by recursively allocating blocks of unaligned
-     * free memory until only an aligned allocation is possible.
-     */
-    void *p = MapAlignedPagesRecursively(rt, size, alignment, recursions);
-    if (p)
-        return p;
-
-    /*
-     * If memory is heavily fragmented, the recursive strategy may fail;
-     * instead, use the "expensive" strategy:  allocate twice as much
-     * as requested and return an aligned address within this block.
-     */
-    if (DosAllocMem(&p, 2 * size,
-                    OBJ_ANY | PAG_COMMIT | PAG_READ | PAG_WRITE)) {
-        JS_ALWAYS_TRUE(DosAllocMem(&p, 2 * size,
-                                   PAG_COMMIT | PAG_READ | PAG_WRITE) == 0);
-    }
-
-    uintptr_t addr = reinterpret_cast<uintptr_t>(p);
-    addr = (addr + (alignment - 1)) & ~(alignment - 1);
-
-    return reinterpret_cast<void *>(addr);
-}
-
-bool
-gc::MarkPagesUnused(JSRuntime *rt, void *p, size_t size)
-{
-    JS_ASSERT(uintptr_t(p) % rt->gcSystemPageSize == 0);
-    return true;
-}
-
-bool
-gc::MarkPagesInUse(JSRuntime *rt, void *p, size_t size)
-{
-    JS_ASSERT(uintptr_t(p) % rt->gcSystemPageSize == 0);
-    return true;
-}
-
-size_t
-gc::GetPageFaultCount()
-{
-    return 0;
-}
-
 #elif defined(SOLARIS)
 
 #include <sys/mman.h>
@@ -297,7 +165,7 @@ gc::GetPageFaultCount()
     return 0;
 }
 
-#elif defined(XP_UNIX) || defined(XP_MACOSX) || defined(DARWIN)
+#elif defined(XP_UNIX)
 
 #include <sys/mman.h>
 #include <sys/resource.h>
