@@ -4,8 +4,8 @@
  * file, You can obtain one at http://mozilla.org/MPL/2.0/. */
 
 //-----------------------------------------------------------------------------
-var BUGNUMBER = 355052;
-var summary = 'Do not crash with valueOf:gc and __iterator__';
+var BUGNUMBER = 435497;
+var summary = 'Do not assert op2 == JSOP_INITELEM';
 var actual = 'No Crash';
 var expect = 'No Crash';
 
@@ -19,19 +19,16 @@ function test()
   enterFunc ('test');
   printBugNumber(BUGNUMBER);
   printStatus (summary);
-
-  expect = /TypeError: NaN is not a function/;
-  actual = 'No Error';
+ 
   try
   {
-    ( {valueOf: gc} - [].a )();
+    eval('(function() { x, x setter = 0, y; const x; })();');
   }
   catch(ex)
   {
-    actual = ex + '';
   }
 
-  reportMatch(expect, actual, summary);
+  reportCompare(expect, actual, summary);
 
   exitFunc ('test');
 }
