@@ -4,8 +4,8 @@
  * file, You can obtain one at http://mozilla.org/MPL/2.0/. */
 
 //-----------------------------------------------------------------------------
-var BUGNUMBER = 452498;
-var summary = 'TM: upvar2 regression tests';
+var BUGNUMBER = 465460;
+var summary = 'TM: valueOf in a loop: do not assert';
 var actual = '';
 var expect = '';
 
@@ -19,15 +19,14 @@ function test()
   enterFunc ('test');
   printBugNumber(BUGNUMBER);
   printStatus (summary);
+ 
+  jit(true);
 
-// ------- Comment #108 From Jesse Ruderman
+  for each (let _ in [{}, {}, null, null, null, null]) { }
 
-  function p(){p}
+  jit(false);
 
-  expect = 'function p(){p;}';
-  actual = p + '';
-
-  compareSource(expect, actual, summary);
+  reportCompare(expect, actual, summary);
 
   exitFunc ('test');
 }
