@@ -38,8 +38,10 @@ XPCJSContextStack::Pop()
     JSContext *cx = mStack[idx].cx;
 
     mStack.RemoveElementAt(idx);
-    if (idx == 0)
+    if (idx == 0) {
+        js::Debug_SetActiveJSContext(mRuntime->Runtime(), nullptr);
         return cx;
+    }
 
     --idx; // Advance to new top of the stack
 
@@ -50,12 +52,14 @@ XPCJSContextStack::Pop()
         JS_RestoreFrameChain(e.cx);
         e.savedFrameChain = false;
     }
+    js::Debug_SetActiveJSContext(mRuntime->Runtime(), e.cx);
     return cx;
 }
 
 bool
 XPCJSContextStack::Push(JSContext *cx)
 {
+    js::Debug_SetActiveJSContext(mRuntime->Runtime(), cx);
     if (mStack.Length() == 0) {
         mStack.AppendElement(cx);
         return true;
@@ -128,14 +132,20 @@ const JSClass xpc::SafeJSContextGlobalClass = {
     XPCONNECT_GLOBAL_FLAGS,
     JS_PropertyStub, JS_DeletePropertyStub, JS_PropertyStub, JS_StrictPropertyStub,
     JS_EnumerateStub, SafeGlobalResolve, JS_ConvertStub, SafeFinalize,
-    nullptr, nullptr, nullptr, nullptr, TraceXPCGlobal
+    nullptr, nullptr, nullptr, TraceXPCGlobal
 };
 
 JSContext*
 XPCJSContextStack::GetSafeJSContext()
 {
-    if (mSafeJSContext)
-        return mSafeJSContext;
+    MOZ_ASSERT(mSafeJSContext);
+    return mSafeJSContext;
+}
+
+JSContext*
+XPCJSContextStack::InitSafeJSContext()
+{
+    MOZ_ASSERT(!mSafeJSContext);
 
     // Start by getting the principal holder and principal for this
     // context.  If we can't manage that, don't bother with the rest.
@@ -170,7 +180,7 @@ XPCJSContextStack::GetSafeJSContext()
     // Note: make sure to set the private before calling
     // InitClasses
     nsRefPtr<SandboxPrivate> sp = new SandboxPrivate(principal, glob);
-    JS_SetPrivate(glob, sp.forget().get());
+    JS_SetPrivate(glob, sp.forget().take());
 
     // After this point either glob is null and the
     // nsIScriptObjectPrincipal ownership is either handled by the
