@@ -110,23 +110,20 @@ NS_IMPL_CYCLE_COLLECTION_UNLINK_BEGIN(XPCVariant)
 NS_IMPL_CYCLE_COLLECTION_UNLINK_END
 
 // static
-XPCVariant* XPCVariant::newVariant(JSContext* cx, jsval aJSVal)
+already_AddRefed<XPCVariant>
+XPCVariant::newVariant(JSContext* cx, jsval aJSVal)
 {
-    XPCVariant* variant;
+    nsRefPtr<XPCVariant> variant;
 
     if (!JSVAL_IS_TRACEABLE(aJSVal))
         variant = new XPCVariant(cx, aJSVal);
     else
         variant = new XPCTraceableVariant(cx, aJSVal);
 
-    if (!variant)
-        return nullptr;
-    NS_ADDREF(variant);
-
     if (!variant->InitializeData(cx))
-        NS_RELEASE(variant);     // Also sets variant to nullptr.
+        return nullptr;
 
-    return variant;
+    return variant.forget();
 }
 
 // Helper class to give us a namespace for the table based code below.
@@ -644,7 +641,7 @@ VARIANT_DONE:
         }
         case nsIDataType::VTYPE_EMPTY_ARRAY:
         {
-            JSObject* array = JS_NewArrayObject(cx, 0, nullptr);
+            JSObject* array = JS_NewArrayObject(cx, 0);
             if (!array)
                 return false;
             pJSVal.setObject(*array);
