@@ -5,34 +5,79 @@
 #ifndef nsDOMQS_h__
 #define nsDOMQS_h__
 
-#include "mozilla/dom/ImageData.h"
 #include "nsDOMClassInfoID.h"
 #include "nsGenericHTMLElement.h"
-#include "nsHTMLCanvasElement.h"
-#include "nsHTMLImageElement.h"
-#include "nsHTMLOptionElement.h"
-#include "nsHTMLOptGroupElement.h"
-#include "nsHTMLVideoElement.h"
+#include "mozilla/dom/HTMLCanvasElement.h"
+#include "mozilla/dom/HTMLFormElement.h"
+#include "mozilla/dom/HTMLImageElement.h"
+#include "mozilla/dom/HTMLOptionElement.h"
+#include "HTMLOptGroupElement.h"
+#include "mozilla/dom/HTMLVideoElement.h"
 #include "nsHTMLDocument.h"
 #include "nsICSSDeclaration.h"
-#include "nsSVGStylableElement.h"
+#include "nsSVGElement.h"
+#include "mozilla/dom/Event.h"
+#include "mozilla/dom/UIEvent.h"
+#include "mozilla/dom/MouseEvent.h"
+#include "mozilla/dom/EventTargetBinding.h"
+#include "mozilla/dom/NodeBinding.h"
+#include "mozilla/dom/ElementBinding.h"
+#include "mozilla/dom/HTMLElementBinding.h"
+#include "mozilla/dom/DocumentBinding.h"
+#include "mozilla/dom/SVGElementBinding.h"
+#include "mozilla/dom/HTMLDocumentBinding.h"
+#include "XPCQuickStubs.h"
+#include "nsGlobalWindow.h"
+
+template<class T>
+struct ProtoIDAndDepth
+{
+    enum {
+        PrototypeID = mozilla::dom::prototypes::id::_ID_Count,
+        Depth = -1
+    };
+};
+
+#define NEW_BINDING(_native, _id)                                             \
+template<>                                                                    \
+struct ProtoIDAndDepth<_native>                                               \
+{                                                                             \
+    enum {                                                                    \
+        PrototypeID = mozilla::dom::prototypes::id::_id,                      \
+        Depth = mozilla::dom::PrototypeTraits<                                \
+            static_cast<mozilla::dom::prototypes::ID>(PrototypeID)>::Depth    \
+    };                                                                        \
+}
+
+NEW_BINDING(mozilla::dom::EventTarget, EventTarget);
+NEW_BINDING(nsINode, Node);
+NEW_BINDING(mozilla::dom::Element, Element);
+NEW_BINDING(nsGenericHTMLElement, HTMLElement);
+NEW_BINDING(nsIDocument, Document);
+NEW_BINDING(nsDocument, Document);
+NEW_BINDING(nsHTMLDocument, HTMLDocument);
+NEW_BINDING(nsSVGElement, SVGElement);
+NEW_BINDING(mozilla::dom::Event, Event);
+NEW_BINDING(mozilla::dom::UIEvent, UIEvent);
+NEW_BINDING(mozilla::dom::MouseEvent, MouseEvent);
 
 #define DEFINE_UNWRAP_CAST(_interface, _base, _bit)                           \
 template <>                                                                   \
-MOZ_ALWAYS_INLINE JSBool                                                      \
+MOZ_ALWAYS_INLINE bool                                                        \
 xpc_qsUnwrapThis<_interface>(JSContext *cx,                                   \
-                             JSObject *obj,                                   \
+                             JS::HandleObject obj,                            \
                              _interface **ppThis,                             \
                              nsISupports **pThisRef,                          \
-                             jsval *pThisVal,                                 \
-                             XPCLazyCallContext *lccx,                        \
+                             JS::MutableHandleValue pThisVal,                 \
                              bool failureFatal)                               \
 {                                                                             \
     nsresult rv;                                                              \
-    nsISupports *native = castNativeFromWrapper(cx, obj, _bit,                \
-                                                pThisRef, pThisVal, lccx,     \
-                                                &rv);                         \
-    *ppThis = NULL;  /* avoids uninitialized warnings in callers */           \
+    nsISupports *native =                                                     \
+        castNativeFromWrapper(cx, obj, _bit,                                  \
+                              ProtoIDAndDepth<_interface>::PrototypeID,       \
+                              ProtoIDAndDepth<_interface>::Depth,             \
+                              pThisRef, pThisVal, &rv);                       \
+    *ppThis = nullptr;  /* avoids uninitialized warnings in callers */        \
     if (failureFatal && !native)                                              \
         return xpc_qsThrow(cx, rv);                                           \
     *ppThis = static_cast<_interface*>(static_cast<_base*>(native));          \
@@ -40,20 +85,65 @@ xpc_qsUnwrapThis<_interface>(JSContext *cx,                                   \
 }                                                                             \
                                                                               \
 template <>                                                                   \
-inline nsresult                                                               \
+MOZ_ALWAYS_INLINE nsresult                                                    \
 xpc_qsUnwrapArg<_interface>(JSContext *cx,                                    \
-                            jsval v,                                          \
+                            JS::HandleValue v,                                \
                             _interface **ppArg,                               \
                             nsISupports **ppArgRef,                           \
-                            jsval *vp)                                        \
+                            JS::MutableHandleValue vp)                        \
 {                                                                             \
     nsresult rv;                                                              \
-    nsISupports *native = castNativeArgFromWrapper(cx, v, _bit, ppArgRef, vp, \
-                                                   &rv);                      \
+    nsISupports *native =                                                     \
+        castNativeArgFromWrapper(cx, v, _bit,                                 \
+                                 ProtoIDAndDepth<_interface>::PrototypeID,    \
+                                 ProtoIDAndDepth<_interface>::Depth,          \
+                                 ppArgRef, vp, &rv);                          \
     if (NS_SUCCEEDED(rv))                                                     \
         *ppArg = static_cast<_interface*>(static_cast<_base*>(native));       \
     return rv;                                                                \
-}
+}                                                                             \
+                                                                              \
+template <>                                                                   \
+inline nsresult                                                               \
+xpc_qsUnwrapArg<_interface>(JSContext *cx,                                    \
+                            JS::HandleValue v,                                \
+                            _interface **ppArg,                               \
+                            _interface **ppArgRef,                            \
+                            JS::MutableHandleValue vp)                        \
+{                                                                             \
+    nsISupports* argRef = static_cast<_base*>(*ppArgRef);                     \
+    nsresult rv = xpc_qsUnwrapArg<_interface>(cx, v, ppArg, &argRef, vp);     \
+    *ppArgRef = static_cast<_interface*>(static_cast<_base*>(argRef));        \
+    return rv;                                                                \
+}                                                                             \
+                                                                              \
+namespace mozilla {                                                           \
+namespace dom {                                                               \
+                                                                              \
+template <>                                                                   \
+MOZ_ALWAYS_INLINE nsresult                                                    \
+UnwrapArg<_interface>(JSContext *cx,                                          \
+                      JS::HandleValue v,                                      \
+                      _interface **ppArg,                                     \
+                      nsISupports **ppArgRef,                                 \
+                      JS::MutableHandleValue vp)                              \
+{                                                                             \
+  return xpc_qsUnwrapArg<_interface>(cx, v, ppArg, ppArgRef, vp);             \
+}                                                                             \
+                                                                              \
+template <>                                                                   \
+inline nsresult                                                               \
+UnwrapArg<_interface>(JSContext *cx,                                          \
+                      JS::HandleValue v,                                      \
+                      _interface **ppArg,                                     \
+                      _interface **ppArgRef,                                  \
+                      JS::MutableHandleValue vp)                              \
+{                                                                             \
+  return xpc_qsUnwrapArg<_interface>(cx, v, ppArg, ppArgRef, vp);             \
+}                                                                             \
+                                                                              \
+} /* namespace dom */                                                         \
+} /* namespace mozilla */
 
 #undef DOMCI_CASTABLE_INTERFACE
 
@@ -65,83 +155,22 @@ DOMCI_CASTABLE_INTERFACES(unused)
 
 #undef DOMCI_CASTABLE_INTERFACE
 
-// Ideally we'd just add nsGenericElement to the castable interfaces, but for
-// now nsDocumentFragment inherits from nsGenericElement (even though it's not
-// an Element) so we have to special-case nsGenericElement and use
-// nsIContent::IsElement().
-// FIXME: bug 563659.
-inline JSBool
-castToElement(nsIContent *content, jsval val, nsGenericElement **ppInterface,
-              jsval *pVal)
-{
-    if (!content->IsElement())
-        return false;
-    *ppInterface = static_cast<nsGenericElement*>(content->AsElement());
-    *pVal = val;
-    return true;
-}
-
-template <>
-inline JSBool
-xpc_qsUnwrapThis<nsGenericElement>(JSContext *cx,
-                                   JSObject *obj,
-                                   nsGenericElement **ppThis,
-                                   nsISupports **pThisRef,
-                                   jsval *pThisVal,
-                                   XPCLazyCallContext *lccx,
-                                   bool failureFatal)
-{
-    nsIContent *content;
-    jsval val;
-    JSBool ok = xpc_qsUnwrapThis<nsIContent>(cx, obj, &content,
-                                             pThisRef, &val, lccx,
-                                             failureFatal);
-    if (ok) {
-        if (failureFatal || content)
-          ok = castToElement(content, val, ppThis, pThisVal);
-        if (failureFatal && !ok)
-            xpc_qsThrow(cx, NS_ERROR_XPC_BAD_OP_ON_WN_PROTO);
-    }
-
-    if (!failureFatal && (!ok || !content)) {
-      ok = true;
-      *ppThis = nullptr;
-    }
-
-    return ok;
-}
-
-template <>
-inline nsresult
-xpc_qsUnwrapArg<nsGenericElement>(JSContext *cx,
-                                  jsval v,
-                                  nsGenericElement **ppArg,
-                                  nsISupports **ppArgRef,
-                                  jsval *vp)
-{
-    nsIContent *content;
-    jsval val;
-    nsresult rv = xpc_qsUnwrapArg<nsIContent>(cx, v, &content, ppArgRef, &val);
-    if (NS_SUCCEEDED(rv) && !castToElement(content, val, ppArg, vp))
-        rv = NS_ERROR_XPC_BAD_CONVERT_JS;
-    return rv;
-}
-
 inline nsresult
 xpc_qsUnwrapArg_HTMLElement(JSContext *cx,
-                            jsval v,
+                            JS::HandleValue v,
                             nsIAtom *aTag,
                             nsIContent **ppArg,
                             nsISupports **ppArgRef,
-                            jsval *vp)
+                            JS::MutableHandleValue vp)
 {
-    nsIContent *elem;
-    jsval val;
-    nsresult rv = xpc_qsUnwrapArg<nsIContent>(cx, v, &elem, ppArgRef, &val);
+    nsGenericHTMLElement *elem;
+    JS::RootedValue val(cx);
+    nsresult rv =
+        xpc_qsUnwrapArg<nsGenericHTMLElement>(cx, v, &elem, ppArgRef, &val);
     if (NS_SUCCEEDED(rv)) {
         if (elem->IsHTML(aTag)) {
             *ppArg = elem;
-            *vp = val;
+            vp.set(val);
         } else {
             rv = NS_ERROR_XPC_BAD_CONVERT_JS;
         }
@@ -153,10 +182,10 @@ xpc_qsUnwrapArg_HTMLElement(JSContext *cx,
 template <>                                                                   \
 inline nsresult                                                               \
 xpc_qsUnwrapArg<_clazz>(JSContext *cx,                                        \
-                        jsval v,                                              \
+                        JS::HandleValue v,                                    \
                         _clazz **ppArg,                                       \
                         nsISupports **ppArgRef,                               \
-                        jsval *vp)                                            \
+                        JS::MutableHandleValue vp)                            \
 {                                                                             \
     nsIContent *elem;                                                         \
     nsresult rv = xpc_qsUnwrapArg_HTMLElement(cx, v, nsGkAtoms::_tag, &elem,  \
@@ -168,37 +197,46 @@ xpc_qsUnwrapArg<_clazz>(JSContext *cx,                                        \
                                                                               \
 template <>                                                                   \
 inline nsresult                                                               \
-xpc_qsUnwrapArg<_clazz>(JSContext *cx, jsval v, _clazz **ppArg,               \
-                        _clazz **ppArgRef, jsval *vp)                         \
+xpc_qsUnwrapArg<_clazz>(JSContext *cx, JS::HandleValue v, _clazz **ppArg,     \
+                        _clazz **ppArgRef, JS::MutableHandleValue vp)         \
 {                                                                             \
     nsISupports* argRef = static_cast<nsIContent*>(*ppArgRef);                \
     nsresult rv = xpc_qsUnwrapArg<_clazz>(cx, v, ppArg, &argRef, vp);         \
     *ppArgRef = static_cast<_clazz*>(static_cast<nsIContent*>(argRef));       \
     return rv;                                                                \
-}
+}                                                                             \
+                                                                              \
+namespace mozilla {                                                           \
+namespace dom {                                                               \
+                                                                              \
+template <>                                                                   \
+inline nsresult                                                               \
+UnwrapArg<_clazz>(JSContext *cx,                                              \
+                  JS::HandleValue v,                                          \
+                  _clazz **ppArg,                                             \
+                  nsISupports **ppArgRef,                                     \
+                  JS::MutableHandleValue vp)                                  \
+{                                                                             \
+    return xpc_qsUnwrapArg<_clazz>(cx, v, ppArg, ppArgRef, vp);               \
+}                                                                             \
+                                                                              \
+template <>                                                                   \
+inline nsresult                                                               \
+UnwrapArg<_clazz>(JSContext *cx, JS::HandleValue v, _clazz **ppArg,           \
+                  _clazz **ppArgRef, JS::MutableHandleValue vp)               \
+{                                                                             \
+    return xpc_qsUnwrapArg<_clazz>(cx, v, ppArg, ppArgRef, vp);               \
+}                                                                             \
+                                                                              \
+} /* namespace dom */                                                         \
+} /* namespace mozilla */
 
-DEFINE_UNWRAP_CAST_HTML(canvas, nsHTMLCanvasElement)
-DEFINE_UNWRAP_CAST_HTML(img, nsHTMLImageElement)
-DEFINE_UNWRAP_CAST_HTML(optgroup, nsHTMLOptGroupElement)
-DEFINE_UNWRAP_CAST_HTML(option, nsHTMLOptionElement)
-DEFINE_UNWRAP_CAST_HTML(video, nsHTMLVideoElement)
-
-template <>
-inline nsresult
-xpc_qsUnwrapArg<mozilla::dom::ImageData>(JSContext *cx, jsval v,
-                                         mozilla::dom::ImageData **ppArg,
-                                         mozilla::dom::ImageData **ppArgRef,
-                                         jsval *vp)
-{
-    nsIDOMImageData* arg;
-    nsIDOMImageData* argRef;
-    nsresult rv = xpc_qsUnwrapArg<nsIDOMImageData>(cx, v, &arg, &argRef, vp);
-    if (NS_SUCCEEDED(rv)) {
-        *ppArg = static_cast<mozilla::dom::ImageData*>(arg);
-        *ppArgRef = static_cast<mozilla::dom::ImageData*>(argRef);
-    }
-    return rv;
-}
+DEFINE_UNWRAP_CAST_HTML(canvas, mozilla::dom::HTMLCanvasElement)
+DEFINE_UNWRAP_CAST_HTML(form, mozilla::dom::HTMLFormElement)
+DEFINE_UNWRAP_CAST_HTML(img, mozilla::dom::HTMLImageElement)
+DEFINE_UNWRAP_CAST_HTML(optgroup, mozilla::dom::HTMLOptGroupElement)
+DEFINE_UNWRAP_CAST_HTML(option, mozilla::dom::HTMLOptionElement)
+DEFINE_UNWRAP_CAST_HTML(video, mozilla::dom::HTMLVideoElement)
 
 inline nsISupports*
 ToSupports(nsContentList *p)
@@ -206,18 +244,6 @@ ToSupports(nsContentList *p)
     return static_cast<nsINodeList*>(p);
 }
 
-inline nsISupports*
-ToCanonicalSupports(nsINode* p)
-{
-    return p;
-}
-
-inline nsISupports*
-ToSupports(nsINode* p)
-{
-  return p;
-}
-
 inline nsISupports*
 ToCanonicalSupports(nsContentList *p)
 {
