@@ -11,25 +11,27 @@
 #include "nsTArray.h"
 #include "nsRegion.h"
 #include "nsIFrame.h"
-#include "nsDisplayListInvalidation.h"
-#include "LayerTreeInvalidation.h"
 #include "ImageLayers.h"
+#include "DisplayItemClip.h"
+#include "mozilla/layers/LayersTypes.h"
 
 class nsDisplayListBuilder;
 class nsDisplayList;
 class nsDisplayItem;
 class gfxContext;
-class nsRootPresContext;
+class nsDisplayItemGeometry;
 
 namespace mozilla {
 namespace layers {
 class ContainerLayer;
 class LayerManager;
+class BasicLayerManager;
 class ThebesLayer;
 }
 
 class FrameLayerBuilder;
 class LayerManagerData;
+class ThebesLayerData;
 
 enum LayerState {
   LAYER_NONE,
@@ -46,11 +48,59 @@ enum LayerState {
 
 class RefCountedRegion : public RefCounted<RefCountedRegion> {
 public:
+  MOZ_DECLARE_REFCOUNTED_TYPENAME(RefCountedRegion)
   RefCountedRegion() : mIsInfinite(false) {}
   nsRegion mRegion;
   bool mIsInfinite;
 };
 
+struct ContainerLayerParameters {
+  ContainerLayerParameters() :
+    mXScale(1), mYScale(1), mAncestorClipRect(nullptr),
+    mInTransformedSubtree(false), mInActiveTransformedSubtree(false),
+    mDisableSubpixelAntialiasingInDescendants(false)
+  {}
+  ContainerLayerParameters(float aXScale, float aYScale) :
+    mXScale(aXScale), mYScale(aYScale), mAncestorClipRect(nullptr),
+    mInTransformedSubtree(false), mInActiveTransformedSubtree(false),
+    mDisableSubpixelAntialiasingInDescendants(false)
+  {}
+  ContainerLayerParameters(float aXScale, float aYScale,
+                           const nsIntPoint& aOffset,
+                           const ContainerLayerParameters& aParent) :
+    mXScale(aXScale), mYScale(aYScale), mAncestorClipRect(nullptr),
+    mOffset(aOffset),
+    mInTransformedSubtree(aParent.mInTransformedSubtree),
+    mInActiveTransformedSubtree(aParent.mInActiveTransformedSubtree),
+    mDisableSubpixelAntialiasingInDescendants(aParent.mDisableSubpixelAntialiasingInDescendants)
+  {}
+  float mXScale, mYScale;
+  /**
+   * An ancestor clip rect that can be applied to restrict the visibility
+   * of this container. Null if none available.
+   */
+  const nsIntRect* mAncestorClipRect;
+  /**
+   * An offset to append to the transform set on all child layers created.
+   */
+  nsIntPoint mOffset;
+
+  bool mInTransformedSubtree;
+  bool mInActiveTransformedSubtree;
+  bool mDisableSubpixelAntialiasingInDescendants;
+  /**
+   * When this is false, ThebesLayer coordinates are drawn to with an integer
+   * translation and the scale in mXScale/mYScale.
+   */
+  bool AllowResidualTranslation()
+  {
+    // If we're in a transformed subtree, but no ancestor transform is actively
+    // changing, we'll use the residual translation when drawing into the
+    // ThebesLayer to ensure that snapping exactly matches the ideal transform.
+    return mInTransformedSubtree && !mInActiveTransformedSubtree;
+  }
+};
+
 /**
  * The FrameLayerBuilder is responsible for converting display lists 
  * into layer trees. Every LayerManager needs a unique FrameLayerBuilder
@@ -76,7 +126,7 @@ public:
  * return it as a candidate for recycling.
  * 
  * FrameLayerBuilder sets up ThebesLayers so that 0,0 in the Thebes layer
- * corresponds to the (pixel-snapped) top-left of the aActiveScrolledRoot.
+ * corresponds to the (pixel-snapped) top-left of the aAnimatedGeometryRoot.
  * It sets up ContainerLayers so that 0,0 in the container layer
  * corresponds to the snapped top-left of the display item reference frame.
  *
@@ -96,16 +146,18 @@ public:
   typedef layers::ThebesLayer ThebesLayer;
   typedef layers::ImageLayer ImageLayer;
   typedef layers::LayerManager LayerManager;
+  typedef layers::BasicLayerManager BasicLayerManager;
+  typedef layers::EventRegions EventRegions;
 
   FrameLayerBuilder() :
     mRetainingManager(nullptr),
     mDetectedDOMModification(false),
     mInvalidateAllLayers(false),
+    mInLayerTreeCompressionMode(false),
     mContainerLayerGeneration(0),
     mMaxContainerLayerGeneration(0)
   {
     MOZ_COUNT_CTOR(FrameLayerBuilder);
-    mThebesLayerItems.Init();
   }
   ~FrameLayerBuilder()
   {
@@ -114,7 +166,8 @@ public:
 
   static void Shutdown();
 
-  void Init(nsDisplayListBuilder* aBuilder, LayerManager* aManager);
+  void Init(nsDisplayListBuilder* aBuilder, LayerManager* aManager,
+            ThebesLayerData* aLayerData = nullptr);
 
   /**
    * Call this to notify that we have just started a transaction on the
@@ -132,47 +185,8 @@ public:
    */
   void DidEndTransaction();
 
-  struct ContainerParameters {
-    ContainerParameters() :
-      mXScale(1), mYScale(1),
-      mInTransformedSubtree(false), mInActiveTransformedSubtree(false),
-      mDisableSubpixelAntialiasingInDescendants(false)
-    {}
-    ContainerParameters(float aXScale, float aYScale) :
-      mXScale(aXScale), mYScale(aYScale),
-      mInTransformedSubtree(false), mInActiveTransformedSubtree(false),
-      mDisableSubpixelAntialiasingInDescendants(false)
-    {}
-    ContainerParameters(float aXScale, float aYScale,
-                        const nsIntPoint& aOffset,
-                        const ContainerParameters& aParent) :
-      mXScale(aXScale), mYScale(aYScale),
-      mOffset(aOffset),
-      mInTransformedSubtree(aParent.mInTransformedSubtree),
-      mInActiveTransformedSubtree(aParent.mInActiveTransformedSubtree),
-      mDisableSubpixelAntialiasingInDescendants(aParent.mDisableSubpixelAntialiasingInDescendants)
-    {}
-    float mXScale, mYScale;
-
-    /**
-     * An offset to append to the transform set on all child layers created.
-     */
-    nsIntPoint mOffset;
-
-    bool mInTransformedSubtree;
-    bool mInActiveTransformedSubtree;
-    bool mDisableSubpixelAntialiasingInDescendants;
-    /**
-     * When this is false, ThebesLayer coordinates are drawn to with an integer
-     * translation and the scale in mXScale/mYScale.
-     */
-    bool AllowResidualTranslation()
-    {
-      // If we're in a transformed subtree, but no ancestor transform is actively
-      // changing, we'll use the residual translation when drawing into the
-      // ThebesLayer to ensure that snapping exactly matches the ideal transform.
-      return mInTransformedSubtree && !mInActiveTransformedSubtree;
-    }
+  enum {
+    CONTAINER_NOT_CLIPPED_BY_ANCESTORS = 0x01
   };
   /**
    * Build a container layer for a display item that contains a child
@@ -197,8 +211,9 @@ public:
                          nsIFrame* aContainerFrame,
                          nsDisplayItem* aContainerItem,
                          const nsDisplayList& aChildren,
-                         const ContainerParameters& aContainerParameters,
-                         const gfx3DMatrix* aTransform);
+                         const ContainerLayerParameters& aContainerParameters,
+                         const gfx3DMatrix* aTransform,
+                         uint32_t aFlags = 0);
 
   /**
    * Get a retained layer for a display item that needs to create its own
@@ -236,6 +251,7 @@ public:
   static void DrawThebesLayer(ThebesLayer* aLayer,
                               gfxContext* aContext,
                               const nsIntRegion& aRegionToDraw,
+                              mozilla::layers::DrawRegionClip aClip,
                               const nsIntRegion& aRegionToInvalidate,
                               void* aCallbackData);
 
@@ -262,13 +278,12 @@ public:
    * @param aManager If the layer is in the LAYER_INACTIVE state,
    * then this is the temporary layer manager to draw with.
    */
-  struct Clip;
   void AddLayerDisplayItem(Layer* aLayer,
                            nsDisplayItem* aItem,
-                           const Clip& aClip,
+                           const DisplayItemClip& aClip,
                            LayerState aLayerState,
                            const nsPoint& aTopLeft,
-                           LayerManager* aManager,
+                           BasicLayerManager* aManager,
                            nsAutoPtr<nsDisplayItemGeometry> aGeometry);
 
   /**
@@ -278,9 +293,9 @@ public:
    * aItem must have an underlying frame.
    * @param aTopLeft offset from active scrolled root to reference frame
    */
-  void AddThebesDisplayItem(ThebesLayer* aLayer,
+  void AddThebesDisplayItem(ThebesLayerData* aLayer,
                             nsDisplayItem* aItem,
-                            const Clip& aClip,
+                            const DisplayItemClip& aClip,
                             nsIFrame* aContainerLayerFrame,
                             LayerState aLayerState,
                             const nsPoint& aTopLeft,
@@ -299,19 +314,12 @@ public:
    */
   Layer* GetOldLayerFor(nsDisplayItem* aItem, 
                         nsDisplayItemGeometry** aOldGeometry = nullptr, 
-                        Clip** aOldClip = nullptr,
-                        nsTArray<nsIFrame*>* aChangedFrames = nullptr);
+                        DisplayItemClip** aOldClip = nullptr,
+                        nsTArray<nsIFrame*>* aChangedFrames = nullptr,
+                        bool *aIsInvalid = nullptr);
 
   static Layer* GetDebugOldLayerFor(nsIFrame* aFrame, uint32_t aDisplayItemKey);
 
-  /**
-   * Try to determine whether the ThebesLayer aLayer paints an opaque
-   * single color everywhere it's visible in aRect.
-   * If successful, return that color, otherwise return NS_RGBA(0,0,0,0).
-   */
-  nscolor FindOpaqueColorCovering(nsDisplayListBuilder* aBuilder,
-                                  ThebesLayer* aLayer, const nsRect& aRect);
-
   /**
    * Destroy any stored LayerManagerDataProperty and the associated data for
    * aFrame.
@@ -326,6 +334,11 @@ public:
    */
   static bool HasRetainedDataFor(nsIFrame* aFrame, uint32_t aDisplayItemKey);
 
+  class DisplayItemData;
+  typedef void (*DisplayItemDataCallback)(nsIFrame *aFrame, DisplayItemData* aItem);
+
+  static void IterateRetainedDataFor(nsIFrame* aFrame, DisplayItemDataCallback aCallback);
+
   /**
    * Save transform that was in aLayer when we last painted, and the position
    * of the active scrolled root frame. It must be an integer
@@ -355,104 +368,10 @@ public:
    * DisplayItemData so we can retrieve the layer from within layout.
    */
   void StoreOptimizedLayerForFrame(nsDisplayItem* aItem, Layer* aLayer);
-
-  /**
-   * Clip represents the intersection of an optional rectangle with a
-   * list of rounded rectangles.
-   */
-  struct Clip {
-    struct RoundedRect {
-      nsRect mRect;
-      // Indices into mRadii are the NS_CORNER_* constants in nsStyleConsts.h
-      nscoord mRadii[8];
-
-      RoundedRect operator+(const nsPoint& aOffset) const {
-        RoundedRect r = *this;
-        r.mRect += aOffset;
-        return r;
-      }
-      bool operator==(const RoundedRect& aOther) const {
-        if (!mRect.IsEqualInterior(aOther.mRect)) {
-          return false;
-        }
-
-        NS_FOR_CSS_HALF_CORNERS(corner) {
-          if (mRadii[corner] != aOther.mRadii[corner]) {
-            return false;
-          }
-        }
-        return true;
-      }
-      bool operator!=(const RoundedRect& aOther) const {
-        return !(*this == aOther);
-      }
-    };
-    nsRect mClipRect;
-    nsTArray<RoundedRect> mRoundedClipRects;
-    bool mHaveClipRect;
-
-    Clip() : mHaveClipRect(false) {}
-
-    // Construct as the intersection of aOther and aClipItem.
-    Clip(const Clip& aOther, nsDisplayItem* aClipItem);
-
-    // Apply this |Clip| to the given gfxContext.  Any saving of state
-    // or clearing of other clips must be done by the caller.
-    // See aBegin/aEnd note on ApplyRoundedRectsTo.
-    void ApplyTo(gfxContext* aContext, nsPresContext* aPresContext,
-                 uint32_t aBegin = 0, uint32_t aEnd = UINT32_MAX);
-
-    void ApplyRectTo(gfxContext* aContext, int32_t A2D) const;
-    // Applies the rounded rects in this Clip to aContext
-    // Will only apply rounded rects from aBegin (inclusive) to aEnd
-    // (exclusive) or the number of rounded rects, whichever is smaller.
-    void ApplyRoundedRectsTo(gfxContext* aContext, int32_t A2DPRInt32,
-                             uint32_t aBegin, uint32_t aEnd) const;
-
-    // Draw (fill) the rounded rects in this clip to aContext
-    void DrawRoundedRectsTo(gfxContext* aContext, int32_t A2D,
-                            uint32_t aBegin, uint32_t aEnd) const;
-    // 'Draw' (create as a path, does not stroke or fill) aRoundRect to aContext
-    void AddRoundedRectPathTo(gfxContext* aContext, int32_t A2D,
-                              const RoundedRect &aRoundRect) const;
-
-    // Return a rectangle contained in the intersection of aRect with this
-    // clip region. Tries to return the largest possible rectangle, but may
-    // not succeed.
-    nsRect ApproximateIntersect(const nsRect& aRect) const;
-
-    // Returns false if aRect is definitely not clipped by a rounded corner in
-    // this clip. Returns true if aRect is clipped by a rounded corner in this
-    // clip or it can not be quickly determined that it is not clipped by a
-    // rounded corner in this clip.
-    bool IsRectClippedByRoundedCorner(const nsRect& aRect) const;
-
-    // Intersection of all rects in this clip ignoring any rounded corners.
-    nsRect NonRoundedIntersection() const;
-
-    // Gets rid of any rounded corners in this clip.
-    void RemoveRoundedCorners();
-
-    // Adds the difference between Intersect(*this + aPoint, aBounds) and
-    // Intersect(aOther, aOtherBounds) to aDifference.
-    void AddOffsetAndComputeDifference(const nsPoint& aPoint, const nsRect& aBounds,
-                                       const Clip& aOther, const nsRect& aOtherBounds,
-                                       nsRegion* aDifference);
-
-    bool operator==(const Clip& aOther) const {
-      return mHaveClipRect == aOther.mHaveClipRect &&
-             (!mHaveClipRect || mClipRect.IsEqualInterior(aOther.mClipRect)) &&
-             mRoundedClipRects == aOther.mRoundedClipRects;
-    }
-    bool operator!=(const Clip& aOther) const {
-      return !(*this == aOther);
-    }
-  };
   
   NS_DECLARE_FRAME_PROPERTY_WITH_FRAME_IN_DTOR(LayerManagerDataProperty,
                                                RemoveFrameFromLayerManager)
 
-protected:
   /**
    * Retained data storage:
    *
@@ -471,6 +390,12 @@ protected:
    */
   class DisplayItemData {
   public:
+    friend class FrameLayerBuilder;
+
+    uint32_t GetDisplayItemKey() { return mDisplayItemKey; }
+    Layer* GetLayer() { return mLayer; }
+    void Invalidate() { mIsInvalid = true; }
+  protected:
 
     DisplayItemData(LayerManagerData* aParent, uint32_t aKey, Layer* aLayer, LayerState aLayerState, uint32_t aGeneration);
     DisplayItemData(DisplayItemData &toCopy);
@@ -483,6 +408,7 @@ protected:
 
     NS_INLINE_DECL_REFCOUNTING(DisplayItemData)
 
+
     /**
      * Associates this DisplayItemData with a frame, and adds it
      * to the LayerManagerDataProperty list on the frame.
@@ -504,10 +430,10 @@ protected:
     LayerManagerData* mParent;
     nsRefPtr<Layer> mLayer;
     nsRefPtr<Layer> mOptLayer;
-    nsRefPtr<LayerManager> mInactiveManager;
+    nsRefPtr<BasicLayerManager> mInactiveManager;
     nsAutoTArray<nsIFrame*, 1> mFrameList;
     nsAutoPtr<nsDisplayItemGeometry> mGeometry;
-    Clip            mClip;
+    DisplayItemClip mClip;
     uint32_t        mDisplayItemKey;
     uint32_t        mContainerLayerGeneration;
     LayerState      mLayerState;
@@ -517,8 +443,11 @@ protected:
      * paint) has been updated in the current paint.
      */
     bool            mUsed;
+    bool            mIsInvalid;
   };
 
+protected:
+
   friend class LayerManagerData;
 
   static void RemoveFrameFromLayerManager(nsIFrame* aFrame, void* aPropertyValue);
@@ -578,11 +507,10 @@ protected:
    * These are only stored during the paint process, so that the
    * DrawThebesLayer callback can figure out which items to draw for the
    * ThebesLayer.
-   * mItem always has an underlying frame.
    */
   struct ClippedDisplayItem {
-    ClippedDisplayItem(nsDisplayItem* aItem, const Clip& aClip, uint32_t aGeneration)
-      : mItem(aItem), mClip(aClip), mContainerLayerGeneration(aGeneration)
+    ClippedDisplayItem(nsDisplayItem* aItem, uint32_t aGeneration)
+      : mItem(aItem), mContainerLayerGeneration(aGeneration)
     {
     }
 
@@ -597,10 +525,27 @@ protected:
      */
     nsRefPtr<LayerManager> mInactiveLayerManager;
 
-    Clip mClip;
     uint32_t mContainerLayerGeneration;
   };
 
+  static void RecomputeVisibilityForItems(nsTArray<ClippedDisplayItem>& aItems,
+                                          nsDisplayListBuilder* aBuilder,
+                                          const nsIntRegion& aRegionToDraw,
+                                          const nsIntPoint& aOffset,
+                                          int32_t aAppUnitsPerDevPixel,
+                                          float aXScale,
+                                          float aYScale);
+
+  void PaintItems(nsTArray<ClippedDisplayItem>& aItems,
+                  const nsIntRect& aRect,
+                  gfxContext* aContext,
+                  nsRenderingContext* aRC,
+                  nsDisplayListBuilder* aBuilder,
+                  nsPresContext* aPresContext,
+                  const nsIntPoint& aOffset,
+                  float aXScale, float aYScale,
+                  int32_t aCommonClipCount);
+
   /**
    * We accumulate ClippedDisplayItem elements in a hashtable during
    * the paint process. This is the hashentry for that hashtable.
@@ -643,15 +588,25 @@ public:
     return mThebesLayerItems.GetEntry(aLayer);
   }
 
-  static PLDHashOperator ProcessRemovedDisplayItems(nsRefPtrHashKey<DisplayItemData>* aEntry,
-                                                    void* aUserArg);
+  ThebesLayerData* GetContainingThebesLayerData()
+  {
+    return mContainingThebesLayer;
+  }
+
+  /**
+   * Attempt to build the most compressed layer tree possible, even if it means
+   * throwing away existing retained buffers.
+   */
+  void SetLayerTreeCompressionMode() { mInLayerTreeCompressionMode = true; }
+  bool CheckInLayerTreeCompressionMode();
+
 protected:
   void RemoveThebesItemsAndOwnerDataForLayerSubtree(Layer* aLayer,
                                                     bool aRemoveThebesItems,
                                                     bool aRemoveOwnerData);
 
-  static PLDHashOperator UpdateDisplayItemDataForFrame(nsRefPtrHashKey<DisplayItemData>* aEntry,
-                                                       void* aUserArg);
+  static PLDHashOperator ProcessRemovedDisplayItems(nsRefPtrHashKey<DisplayItemData>* aEntry,
+                                                    void* aUserArg);
   static PLDHashOperator RestoreDisplayItemData(nsRefPtrHashKey<DisplayItemData>* aEntry,
                                                 void *aUserArg);
 
@@ -684,6 +639,13 @@ protected:
    * clipping data) to be rendered in the layer.
    */
   nsTHashtable<ThebesLayerItemsEntry> mThebesLayerItems;
+
+  /**
+   * When building layers for an inactive layer, this is where the
+   * inactive layer will be placed.
+   */
+  ThebesLayerData*                    mContainingThebesLayer;
+
   /**
    * Saved generation counter so we can detect DOM changes.
    */
@@ -699,6 +661,8 @@ protected:
    */
   bool                                mInvalidateAllLayers;
 
+  bool                                mInLayerTreeCompressionMode;
+
   uint32_t                            mContainerLayerGeneration;
   uint32_t                            mMaxContainerLayerGeneration;
 };
