@@ -3,18 +3,15 @@
  * License, v. 2.0. If a copy of the MPL was not distributed with this
  * file, You can obtain one at http://mozilla.org/MPL/2.0/. */
 
+#include "nsMathMLOperators.h"
 #include "nsCOMPtr.h"
-#include "nsString.h"
 #include "nsHashtable.h"
 #include "nsTArray.h"
 
-#include "nsIComponentManager.h"
 #include "nsIPersistentProperties2.h"
 #include "nsNetUtil.h"
 #include "nsCRT.h"
 
-#include "nsMathMLOperators.h"
-
 // operator dictionary entry
 struct OperatorData {
   OperatorData(void)
@@ -31,33 +28,14 @@ struct OperatorData {
   float           mTrailingSpace;  // unit is em
 };
 
-static PRInt32         gTableRefCount = 0;
-static PRUint32        gOperatorCount = 0;
-static OperatorData*   gOperatorArray = nsnull;
-static nsHashtable*    gOperatorTable = nsnull;
-static bool            gInitialized   = false;
-static nsTArray<nsString>*      gInvariantCharArray    = nsnull;
+static int32_t         gTableRefCount = 0;
+static uint32_t        gOperatorCount = 0;
+static OperatorData*   gOperatorArray = nullptr;
+static nsHashtable*    gOperatorTable = nullptr;
+static bool            gGlobalsInitialized   = false;
 
-static const PRUnichar kNullCh  = PRUnichar('\0');
-static const PRUnichar kDashCh  = PRUnichar('#');
-static const PRUnichar kColonCh = PRUnichar(':');
-
-static const char* const kMathVariant_name[] = {
-  "normal",
-  "bold",
-  "italic",
-  "bold-italic",
-  "sans-serif",
-  "bold-sans-serif",
-  "sans-serif-italic",
-  "sans-serif-bold-italic",
-  "monospace",
-  "script",
-  "bold-script",
-  "fraktur",
-  "bold-fraktur",
-  "double-struck"
-};
+static const char16_t kDashCh  = char16_t('#');
+static const char16_t kColonCh = char16_t(':');
 
 static void
 SetBooleanProperty(OperatorData* aOperatorData,
@@ -114,9 +92,9 @@ SetProperty(OperatorData* aOperatorData,
     else return;  // input is not applicable
 
     // aValue is assumed to be a digit from 0 to 7
-    PRInt32 error = 0;
+    nsresult error = NS_OK;
     float space = aValue.ToFloat(&error) / 18.0;
-    if (error) return;
+    if (NS_FAILED(error)) return;
 
     if (isLeadingSpace)
       aOperatorData->mLeadingSpace = space;
@@ -132,14 +110,16 @@ SetOperator(OperatorData*   aOperatorData,
             nsString&        aAttributes)
 
 {
+  static const char16_t kNullCh = char16_t('\0');
+
   // aOperator is in the expanded format \uNNNN\uNNNN ...
   // First compress these Unicode points to the internal nsString format
-  PRInt32 i = 0;
+  int32_t i = 0;
   nsAutoString name, value;
-  PRInt32 len = aOperator.Length();
-  PRUnichar c = aOperator[i++];
-  PRUint32 state  = 0;
-  PRUnichar uchar = 0;
+  int32_t len = aOperator.Length();
+  char16_t c = aOperator[i++];
+  uint32_t state  = 0;
+  char16_t uchar = 0;
   while (i <= len) {
     if (0 == state) {
       if (c != '\\')
@@ -192,8 +172,8 @@ SetOperator(OperatorData*   aOperatorData,
 #endif
   // Loop over the space-delimited list of attributes to get the name:value pairs
   aAttributes.Append(kNullCh);  // put an extra null at the end
-  PRUnichar* start = aAttributes.BeginWriting();
-  PRUnichar* end   = start;
+  char16_t* start = aAttributes.BeginWriting();
+  char16_t* end   = start;
   while ((kNullCh != *start) && (kDashCh != *start)) {
     name.SetLength(0);
     value.SetLength(0);
@@ -245,36 +225,29 @@ InitOperators(void)
        NS_LITERAL_CSTRING("resource://gre/res/fonts/mathfont.properties"));
   if (NS_FAILED(rv)) return rv;
 
-  // Get the list of invariant chars
-  for (PRInt32 i = 0; i < eMATHVARIANT_COUNT; ++i) {
-    nsCAutoString key(NS_LITERAL_CSTRING("mathvariant."));
-    key.Append(kMathVariant_name[i]);
-    nsAutoString value;
-    mathfontProp->GetStringProperty(key, value);
-    gInvariantCharArray->AppendElement(value); // i.e., gInvariantCharArray[i] holds this list
-  }
-
   // Parse the Operator Dictionary in two passes.
   // The first pass is to count the number of operators; the second pass is to
   // allocate the necessary space for them and to add them in the hash table.
-  for (PRInt32 pass = 1; pass <= 2; pass++) {
+  for (int32_t pass = 1; pass <= 2; pass++) {
     OperatorData dummyData;
     OperatorData* operatorData = &dummyData;
     nsCOMPtr<nsISimpleEnumerator> iterator;
     if (NS_SUCCEEDED(mathfontProp->Enumerate(getter_AddRefs(iterator)))) {
       bool more;
-      PRUint32 index = 0;
-      nsCAutoString name;
+      uint32_t index = 0;
+      nsAutoCString name;
       nsAutoString attributes;
       while ((NS_SUCCEEDED(iterator->HasMoreElements(&more))) && more) {
+        nsCOMPtr<nsISupports> supports;
         nsCOMPtr<nsIPropertyElement> element;
-        if (NS_SUCCEEDED(iterator->GetNext(getter_AddRefs(element)))) {
+        if (NS_SUCCEEDED(iterator->GetNext(getter_AddRefs(supports)))) {
+          element = do_QueryInterface(supports);
           if (NS_SUCCEEDED(element->GetKey(name)) &&
               NS_SUCCEEDED(element->GetValue(attributes))) {
             // expected key: operator.\uNNNN.{infix,postfix,prefix}
             if ((21 <= name.Length()) && (0 == name.Find("operator.\\u"))) {
               name.Cut(0, 9); // 9 is the length of "operator.";
-              PRInt32 len = name.Length();
+              int32_t len = name.Length();
               nsOperatorFlags form = 0;
               if (kNotFound != name.RFind(".infix")) {
                 form = NS_MATHML_OPERATOR_FORM_INFIX;
@@ -318,14 +291,11 @@ InitOperators(void)
 static nsresult
 InitGlobals()
 {
-  gInitialized = true;
+  gGlobalsInitialized = true;
   nsresult rv = NS_ERROR_OUT_OF_MEMORY;
-  gInvariantCharArray = new nsTArray<nsString>();
-  if (gInvariantCharArray) {
-    gOperatorTable = new nsHashtable();
-    if (gOperatorTable) {
-      rv = InitOperators();
-    }
+  gOperatorTable = new nsHashtable();
+  if (gOperatorTable) {
+    rv = InitOperators();
   }
   if (NS_FAILED(rv))
     nsMathMLOperators::CleanUp();
@@ -335,17 +305,13 @@ InitGlobals()
 void
 nsMathMLOperators::CleanUp()
 {
-  if (gInvariantCharArray) {
-    delete gInvariantCharArray;
-    gInvariantCharArray = nsnull;
-  }
   if (gOperatorArray) {
     delete[] gOperatorArray;
-    gOperatorArray = nsnull;
+    gOperatorArray = nullptr;
   }
   if (gOperatorTable) {
     delete gOperatorTable;
-    gOperatorTable = nsnull;
+    gOperatorTable = nullptr;
   }
 }
 
@@ -379,7 +345,7 @@ nsMathMLOperators::LookupOperator(const nsString&       aOperator,
                                   float*                aLeadingSpace,
                                   float*                aTrailingSpace)
 {
-  if (!gInitialized) {
+  if (!gGlobalsInitialized) {
     InitGlobals();
   }
   if (gOperatorTable) {
@@ -392,7 +358,7 @@ nsMathMLOperators::LookupOperator(const nsString&       aOperator,
     // order of preference: infix, postfix, prefix.
 
     OperatorData* found;
-    PRInt32 form = NS_MATHML_OPERATOR_GET_FORM(aForm);
+    int32_t form = NS_MATHML_OPERATOR_GET_FORM(aForm);
     if (!(found = GetOperatorData(aOperator, form))) {
       if (form == NS_MATHML_OPERATOR_FORM_INFIX ||
           !(found =
@@ -424,7 +390,7 @@ nsMathMLOperators::LookupOperators(const nsString&       aOperator,
                                    float*                aLeadingSpace,
                                    float*                aTrailingSpace)
 {
-  if (!gInitialized) {
+  if (!gGlobalsInitialized) {
     InitGlobals();
   }
 
@@ -466,7 +432,7 @@ nsMathMLOperators::LookupOperators(const nsString&       aOperator,
 bool
 nsMathMLOperators::IsMutableOperator(const nsString& aOperator)
 {
-  if (!gInitialized) {
+  if (!gGlobalsInitialized) {
     InitGlobals();
   }
   // lookup all the variants of the operator and return true if there
@@ -516,47 +482,3 @@ nsMathMLOperators::GetStretchyDirection(const nsString& aOperator)
     return NS_STRETCH_DIRECTION_UNSUPPORTED;
   }
 }
-
-/* static */ eMATHVARIANT
-nsMathMLOperators::LookupInvariantChar(const nsAString& aChar)
-{
-  if (!gInitialized) {
-    InitGlobals();
-  }
-  if (gInvariantCharArray) {
-    for (PRInt32 i = gInvariantCharArray->Length()-1; i >= 0; --i) {
-      const nsString& list = gInvariantCharArray->ElementAt(i);
-      nsString::const_iterator start, end;
-      list.BeginReading(start);
-      list.EndReading(end);
-      // Style-invariant characters are at offset 3*j + 1.
-      if (FindInReadable(aChar, start, end) &&
-          start.size_backward() % 3 == 1) {
-        return eMATHVARIANT(i);
-      }
-    }
-  }
-  return eMATHVARIANT_NONE;
-}
-
-/* static */ const nsDependentSubstring
-nsMathMLOperators::TransformVariantChar(const PRUnichar& aChar,
-                                        eMATHVARIANT aVariant)
-{
-  if (!gInitialized) {
-    InitGlobals();
-  }
-  if (gInvariantCharArray) {
-    nsString list = gInvariantCharArray->ElementAt(aVariant);
-    PRInt32 index = list.FindChar(aChar);
-    // BMP characters are at offset 3*j
-    if (index != kNotFound && index % 3 == 0 && list.Length() - index >= 2 ) {
-      // The style-invariant character is the next character
-      // (and list should contain padding if the next character is in the BMP).
-      ++index;
-      PRUint32 len = NS_IS_HIGH_SURROGATE(list.CharAt(index)) ? 2 : 1;
-      return nsDependentSubstring(list, index, len);
-    }
-  }
-  return nsDependentSubstring(&aChar, &aChar + 1);  
-}
