@@ -10,28 +10,20 @@
  */
 
 #include "mozilla/css/StyleRule.h"
+
+#include "mozilla/MemoryReporting.h"
 #include "mozilla/css/GroupRule.h"
 #include "mozilla/css/Declaration.h"
 #include "nsCSSStyleSheet.h"
-#include "mozilla/css/Loader.h"
-#include "nsIURL.h"
 #include "nsIDocument.h"
 #include "nsIAtom.h"
-#include "nsCRT.h"
 #include "nsString.h"
-#include "nsStyleConsts.h"
 #include "nsStyleUtil.h"
-#include "nsIDOMCSSStyleSheet.h"
 #include "nsICSSStyleRuleDOMWrapper.h"
-#include "nsIDOMCSSStyleDeclaration.h"
 #include "nsDOMCSSDeclaration.h"
-#include "nsINameSpaceManager.h"
+#include "nsNameSpaceManager.h"
 #include "nsXMLNameSpaceMap.h"
-#include "nsRuleNode.h"
-#include "nsUnicharUtils.h"
 #include "nsCSSPseudoElements.h"
-#include "nsIPrincipal.h"
-#include "nsComponentManagerUtils.h"
 #include "nsCSSPseudoClasses.h"
 #include "nsCSSAnonBoxes.h"
 #include "nsTArray.h"
@@ -40,9 +32,10 @@
 #include "nsError.h"
 #include "mozAutoDocUpdate.h"
 
-#include "prlog.h"
+class nsIDOMCSSStyleDeclaration;
+class nsIDOMCSSStyleSheet;
 
-namespace css = mozilla::css;
+using namespace mozilla;
 
 #define NS_IF_CLONE(member_)                                                  \
   PR_BEGIN_MACRO                                                              \
@@ -91,7 +84,7 @@ nsAtomList::Clone(bool aDeep) const
 }
 
 size_t
-nsAtomList::SizeOfIncludingThis(nsMallocSizeOfFun aMallocSizeOf) const
+nsAtomList::SizeOfIncludingThis(mozilla::MallocSizeOf aMallocSizeOf) const
 {
   size_t n = 0;
   const nsAtomList* a = this;
@@ -124,7 +117,7 @@ nsPseudoClassList::nsPseudoClassList(nsCSSPseudoClasses::Type aType)
 }
 
 nsPseudoClassList::nsPseudoClassList(nsCSSPseudoClasses::Type aType,
-                                     const PRUnichar* aString)
+                                     const char16_t* aString)
   : mType(aType),
     mNext(nullptr)
 {
@@ -186,7 +179,7 @@ nsPseudoClassList::Clone(bool aDeep) const
 }
 
 size_t
-nsPseudoClassList::SizeOfIncludingThis(nsMallocSizeOfFun aMallocSizeOf) const
+nsPseudoClassList::SizeOfIncludingThis(mozilla::MallocSizeOf aMallocSizeOf) const
 {
   size_t n = 0;
   const nsPseudoClassList* p = this;
@@ -309,8 +302,8 @@ nsCSSSelector::nsCSSSelector(void)
     mPseudoType(nsCSSPseudoElements::ePseudo_NotPseudoElement)
 {
   MOZ_COUNT_CTOR(nsCSSSelector);
-  MOZ_STATIC_ASSERT(nsCSSPseudoElements::ePseudo_MAX < INT16_MAX,
-                    "nsCSSPseudoElements::Type values overflow mPseudoType");
+  static_assert(nsCSSPseudoElements::ePseudo_MAX < INT16_MAX,
+                "nsCSSPseudoElements::Type values overflow mPseudoType");
 }
 
 nsCSSSelector*
@@ -371,7 +364,7 @@ void nsCSSSelector::Reset(void)
   NS_ASSERTION(!mNegations || !mNegations->mNext,
                "mNegations can't have non-null mNext");
   NS_CSS_DELETE_LIST_MEMBER(nsCSSSelector, this, mNegations);
-  mOperator = PRUnichar(0);
+  mOperator = char16_t(0);
 }
 
 void nsCSSSelector::SetNameSpace(int32_t aNameSpace)
@@ -421,7 +414,7 @@ void nsCSSSelector::AddPseudoClass(nsCSSPseudoClasses::Type aType)
 }
 
 void nsCSSSelector::AddPseudoClass(nsCSSPseudoClasses::Type aType,
-                                   const PRUnichar* aString)
+                                   const char16_t* aString)
 {
   AddPseudoClassInternal(new nsPseudoClassList(aType, aString));
 }
@@ -471,7 +464,7 @@ void nsCSSSelector::AddAttribute(int32_t aNameSpace, const nsString& aAttr, uint
   }
 }
 
-void nsCSSSelector::SetOperator(PRUnichar aOperator)
+void nsCSSSelector::SetOperator(char16_t aOperator)
 {
   mOperator = aOperator;
 }
@@ -480,7 +473,13 @@ int32_t nsCSSSelector::CalcWeightWithoutNegations() const
 {
   int32_t weight = 0;
 
-  if (nullptr != mLowercaseTag) {
+  MOZ_ASSERT(!IsPseudoElement() ||
+             mPseudoType >= nsCSSPseudoElements::ePseudo_PseudoElementCount ||
+             (!mIDList && !mClassList && !mAttrList),
+             "if pseudo-elements can have ID, class or attribute selectors "
+             "after them, specificity calculation must be updated");
+
+  if (nullptr != mCasedTag) {
     weight += 0x000001;
   }
   nsAtomList* list = mIDList;
@@ -549,18 +548,18 @@ nsCSSSelector::ToString(nsAString& aString, nsCSSStyleSheet* aSheet,
     // Append the combinator, if needed.
     if (!stack.IsEmpty()) {
       const nsCSSSelector *next = stack.ElementAt(index - 1);
-      PRUnichar oper = s->mOperator;
+      char16_t oper = s->mOperator;
       if (next->IsPseudoElement()) {
-        NS_ASSERTION(oper == PRUnichar('>'),
+        NS_ASSERTION(oper == char16_t(':'),
                      "improperly chained pseudo element");
       } else {
-        NS_ASSERTION(oper != PRUnichar(0),
+        NS_ASSERTION(oper != char16_t(0),
                      "compound selector without combinator");
 
-        aString.Append(PRUnichar(' '));
-        if (oper != PRUnichar(' ')) {
+        aString.Append(char16_t(' '));
+        if (oper != char16_t(' ')) {
           aString.Append(oper);
-          aString.Append(PRUnichar(' '));
+          aString.Append(char16_t(' '));
         }
       }
     }
@@ -578,7 +577,7 @@ nsCSSSelector::AppendToStringWithoutCombinators
     aString.AppendLiteral(":not(");
     negation->AppendToStringWithoutCombinatorsOrNegations(aString, aSheet,
                                                           true);
-    aString.Append(PRUnichar(')'));
+    aString.Append(char16_t(')'));
   }
 }
 
@@ -606,7 +605,7 @@ nsCSSSelector::AppendToStringWithoutCombinatorsOrNegations
                    mNameSpace == kNameSpaceID_None,
                    "How did we get this namespace?");
       if (mNameSpace == kNameSpaceID_None) {
-        aString.Append(PRUnichar('|'));
+        aString.Append(char16_t('|'));
         wroteNamespace = true;
       }
     } else if (sheetNS->FindNameSpaceID(nullptr) == mNameSpace) {
@@ -618,7 +617,7 @@ nsCSSSelector::AppendToStringWithoutCombinatorsOrNegations
     } else if (mNameSpace == kNameSpaceID_None) {
       NS_ASSERTION(CanBeNamespaced(aIsNegated),
                    "How did we end up with this namespace?");
-      aString.Append(PRUnichar('|'));
+      aString.Append(char16_t('|'));
       wroteNamespace = true;
     } else if (mNameSpace != kNameSpaceID_Unknown) {
       NS_ASSERTION(CanBeNamespaced(aIsNegated),
@@ -628,7 +627,7 @@ nsCSSSelector::AppendToStringWithoutCombinatorsOrNegations
                    "without a prefix?");
       nsStyleUtil::AppendEscapedCSSIdent(nsDependentAtomString(prefixAtom),
                                          aString);
-      aString.Append(PRUnichar('|'));
+      aString.Append(char16_t('|'));
       wroteNamespace = true;
     } else {
       // A selector for an element in any namespace, while the default
@@ -650,7 +649,7 @@ nsCSSSelector::AppendToStringWithoutCombinatorsOrNegations
     if (wroteNamespace ||
         (!mIDList && !mClassList && !mPseudoClassList && !mAttrList &&
          (aIsNegated || !mNegations))) {
-      aString.Append(PRUnichar('*'));
+      aString.Append(char16_t('*'));
     }
   } else {
     // Append the tag name
@@ -660,10 +659,10 @@ nsCSSSelector::AppendToStringWithoutCombinatorsOrNegations
       if (!mNext) {
         // Lone pseudo-element selector -- toss in a wildcard type selector
         // XXXldb Why?
-        aString.Append(PRUnichar('*'));
+        aString.Append(char16_t('*'));
       }
       if (!nsCSSPseudoElements::IsCSS2PseudoElement(mLowercaseTag)) {
-        aString.Append(PRUnichar(':'));
+        aString.Append(char16_t(':'));
       }
       // This should not be escaped since (a) the pseudo-element string
       // has a ":" that can't be escaped and (b) all pseudo-elements at
@@ -680,7 +679,7 @@ nsCSSSelector::AppendToStringWithoutCombinatorsOrNegations
     nsAtomList* list = mIDList;
     while (list != nullptr) {
       list->mAtom->ToString(temp);
-      aString.Append(PRUnichar('#'));
+      aString.Append(char16_t('#'));
       nsStyleUtil::AppendEscapedCSSIdent(temp, aString);
       list = list->mNext;
     }
@@ -693,13 +692,13 @@ nsCSSSelector::AppendToStringWithoutCombinatorsOrNegations
       NS_ABORT_IF_FALSE(nsCSSAnonBoxes::IsTreePseudoElement(mLowercaseTag),
                         "must be tree pseudo-element");
 
-      aString.Append(PRUnichar('('));
+      aString.Append(char16_t('('));
       for (nsAtomList* list = mClassList; list; list = list->mNext) {
         nsStyleUtil::AppendEscapedCSSIdent(nsDependentAtomString(list->mAtom), aString);
-        aString.Append(PRUnichar(','));
+        aString.Append(char16_t(','));
       }
       // replace the final comma with a close-paren
-      aString.Replace(aString.Length() - 1, 1, PRUnichar(')'));
+      aString.Replace(aString.Length() - 1, 1, char16_t(')'));
 #else
       NS_ERROR("Can't happen");
 #endif
@@ -707,7 +706,7 @@ nsCSSSelector::AppendToStringWithoutCombinatorsOrNegations
       nsAtomList* list = mClassList;
       while (list != nullptr) {
         list->mAtom->ToString(temp);
-        aString.Append(PRUnichar('.'));
+        aString.Append(char16_t('.'));
         nsStyleUtil::AppendEscapedCSSIdent(temp, aString);
         list = list->mNext;
       }
@@ -718,11 +717,11 @@ nsCSSSelector::AppendToStringWithoutCombinatorsOrNegations
   if (mAttrList) {
     nsAttrSelector* list = mAttrList;
     while (list != nullptr) {
-      aString.Append(PRUnichar('['));
+      aString.Append(char16_t('['));
       // Append the namespace prefix
       if (list->mNameSpace == kNameSpaceID_Unknown) {
-        aString.Append(PRUnichar('*'));
-        aString.Append(PRUnichar('|'));
+        aString.Append(char16_t('*'));
+        aString.Append(char16_t('|'));
       } else if (list->mNameSpace != kNameSpaceID_None) {
         if (aSheet) {
           nsXMLNameSpaceMap *sheetNS = aSheet->GetNameSpaceMap();
@@ -735,7 +734,7 @@ nsCSSSelector::AppendToStringWithoutCombinatorsOrNegations
           nsAutoString prefix;
           prefixAtom->ToString(prefix);
           nsStyleUtil::AppendEscapedCSSIdent(prefix, aString);
-          aString.Append(PRUnichar('|'));
+          aString.Append(char16_t('|'));
         }
       }
       // Append the attribute name
@@ -745,23 +744,23 @@ nsCSSSelector::AppendToStringWithoutCombinatorsOrNegations
       if (list->mFunction != NS_ATTR_FUNC_SET) {
         // Append the function
         if (list->mFunction == NS_ATTR_FUNC_INCLUDES)
-          aString.Append(PRUnichar('~'));
+          aString.Append(char16_t('~'));
         else if (list->mFunction == NS_ATTR_FUNC_DASHMATCH)
-          aString.Append(PRUnichar('|'));
+          aString.Append(char16_t('|'));
         else if (list->mFunction == NS_ATTR_FUNC_BEGINSMATCH)
-          aString.Append(PRUnichar('^'));
+          aString.Append(char16_t('^'));
         else if (list->mFunction == NS_ATTR_FUNC_ENDSMATCH)
-          aString.Append(PRUnichar('$'));
+          aString.Append(char16_t('$'));
         else if (list->mFunction == NS_ATTR_FUNC_CONTAINSMATCH)
-          aString.Append(PRUnichar('*'));
+          aString.Append(char16_t('*'));
 
-        aString.Append(PRUnichar('='));
+        aString.Append(char16_t('='));
       
         // Append the value
         nsStyleUtil::AppendEscapedCSSString(list->mValue, aString);
       }
 
-      aString.Append(PRUnichar(']'));
+      aString.Append(char16_t(']'));
       
       list = list->mNext;
     }
@@ -776,7 +775,7 @@ nsCSSSelector::AppendToStringWithoutCombinatorsOrNegations
     // escaping.
     aString.Append(temp);
     if (list->u.mMemory) {
-      aString.Append(PRUnichar('('));
+      aString.Append(char16_t('('));
       if (nsCSSPseudoClasses::HasStringArg(list->mType)) {
         nsStyleUtil::AppendEscapedCSSIdent(
           nsDependentString(list->u.mString), aString);
@@ -786,15 +785,15 @@ nsCSSSelector::AppendToStringWithoutCombinatorsOrNegations
         temp.Truncate();
         if (a != 0) {
           if (a == -1) {
-            temp.Append(PRUnichar('-'));
+            temp.Append(char16_t('-'));
           } else if (a != 1) {
             temp.AppendInt(a);
           }
-          temp.Append(PRUnichar('n'));
+          temp.Append(char16_t('n'));
         }
         if (b != 0 || a == 0) {
           if (b >= 0 && a != 0) // check a != 0 for whether we printed above
-            temp.Append(PRUnichar('+'));
+            temp.Append(char16_t('+'));
           temp.AppendInt(b);
         }
         aString.Append(temp);
@@ -805,7 +804,7 @@ nsCSSSelector::AppendToStringWithoutCombinatorsOrNegations
         list->u.mSelectors->ToString(tmp, aSheet);
         aString.Append(tmp);
       }
-      aString.Append(PRUnichar(')'));
+      aString.Append(char16_t(')'));
     }
   }
 }
@@ -818,7 +817,7 @@ nsCSSSelector::CanBeNamespaced(bool aIsNegated) const
 }
 
 size_t
-nsCSSSelector::SizeOfIncludingThis(nsMallocSizeOfFun aMallocSizeOf) const
+nsCSSSelector::SizeOfIncludingThis(mozilla::MallocSizeOf aMallocSizeOf) const
 {
   size_t n = 0;
   const nsCSSSelector* s = this;
@@ -863,15 +862,15 @@ nsCSSSelectorList::~nsCSSSelectorList()
 }
 
 nsCSSSelector*
-nsCSSSelectorList::AddSelector(PRUnichar aOperator)
+nsCSSSelectorList::AddSelector(char16_t aOperator)
 {
   nsCSSSelector* newSel = new nsCSSSelector();
 
   if (mSelectors) {
-    NS_ASSERTION(aOperator != PRUnichar(0), "chaining without combinator");
+    NS_ASSERTION(aOperator != char16_t(0), "chaining without combinator");
     mSelectors->SetOperator(aOperator);
   } else {
-    NS_ASSERTION(aOperator == PRUnichar(0), "combinator without chaining");
+    NS_ASSERTION(aOperator == char16_t(0), "combinator without chaining");
   }
 
   newSel->mNext = mSelectors;
@@ -908,7 +907,7 @@ nsCSSSelectorList::Clone(bool aDeep) const
 }
 
 size_t
-nsCSSSelectorList::SizeOfIncludingThis(nsMallocSizeOfFun aMallocSizeOf) const
+nsCSSSelectorList::SizeOfIncludingThis(mozilla::MallocSizeOf aMallocSizeOf) const
 {
   size_t n = 0;
   const nsCSSSelectorList* s = this;
@@ -971,19 +970,19 @@ public:
   DOMCSSDeclarationImpl(css::StyleRule *aRule);
   virtual ~DOMCSSDeclarationImpl(void);
 
-  NS_IMETHOD GetParentRule(nsIDOMCSSRule **aParent);
+  NS_IMETHOD GetParentRule(nsIDOMCSSRule **aParent) MOZ_OVERRIDE;
   void DropReference(void);
-  virtual css::Declaration* GetCSSDeclaration(bool aAllocate);
-  virtual nsresult SetCSSDeclaration(css::Declaration* aDecl);
-  virtual void GetCSSParsingEnvironment(CSSParsingEnvironment& aCSSParseEnv);
-  virtual nsIDocument* DocToUpdate();
+  virtual css::Declaration* GetCSSDeclaration(bool aAllocate) MOZ_OVERRIDE;
+  virtual nsresult SetCSSDeclaration(css::Declaration* aDecl) MOZ_OVERRIDE;
+  virtual void GetCSSParsingEnvironment(CSSParsingEnvironment& aCSSParseEnv) MOZ_OVERRIDE;
+  virtual nsIDocument* DocToUpdate() MOZ_OVERRIDE;
 
   // Override |AddRef| and |Release| for being a member of
   // |DOMCSSStyleRule|.  Also, we need to forward QI for cycle
   // collection things to DOMCSSStyleRule.
   NS_DECL_ISUPPORTS_INHERITED
 
-  virtual nsINode *GetParentObject()
+  virtual nsINode *GetParentObject() MOZ_OVERRIDE
   {
     return mRule ? mRule->GetDocument() : nullptr;
   }
@@ -1121,7 +1120,7 @@ DOMCSSDeclarationImpl::SetCSSDeclaration(css::Declaration* aDecl)
   mozAutoDocUpdate updateBatch(owningDoc, UPDATE_STYLE, true);
 
   nsRefPtr<css::StyleRule> oldRule = mRule;
-  mRule = oldRule->DeclarationChanged(aDecl, true).get();
+  mRule = oldRule->DeclarationChanged(aDecl, true).take();
   if (!mRule)
     return NS_ERROR_OUT_OF_MEMORY;
   nsrefcnt cnt = mRule->Release();
@@ -1175,14 +1174,14 @@ NS_IMPL_CYCLE_COLLECTION_TRACE_BEGIN(DOMCSSStyleRule)
   // Trace the wrapper for our declaration.  This just expands out
   // NS_IMPL_CYCLE_COLLECTION_TRACE_PRESERVED_WRAPPER which we can't use
   // directly because the wrapper is on the declaration, not on us.
-  nsContentUtils::TraceWrapper(tmp->DOMDeclaration(), aCallback, aClosure);
+  tmp->DOMDeclaration()->TraceWrapper(aCallbacks, aClosure);
 NS_IMPL_CYCLE_COLLECTION_TRACE_END
 
 NS_IMPL_CYCLE_COLLECTION_UNLINK_BEGIN(DOMCSSStyleRule)
   // Unlink the wrapper for our declaraton.  This just expands out
   // NS_IMPL_CYCLE_COLLECTION_UNLINK_PRESERVED_WRAPPER which we can't use
   // directly because the wrapper is on the declaration, not on us.
-  nsContentUtils::ReleaseWrapper(static_cast<nsISupports*>(p), tmp->DOMDeclaration());
+  tmp->DOMDeclaration()->ReleaseWrapper(static_cast<nsISupports*>(p));
 NS_IMPL_CYCLE_COLLECTION_UNLINK_END
 
 NS_IMPL_CYCLE_COLLECTION_TRAVERSE_BEGIN(DOMCSSStyleRule)
@@ -1292,6 +1291,7 @@ StyleRule::StyleRule(nsCSSSelectorList* aSelector,
     mImportantRule(nullptr),
     mDOMRule(nullptr),
     mLineNumber(0),
+    mColumnNumber(0),
     mWasMatched(false)
 {
   NS_PRECONDITION(aDeclaration, "must have a declaration");
@@ -1305,6 +1305,7 @@ StyleRule::StyleRule(const StyleRule& aCopy)
     mImportantRule(nullptr),
     mDOMRule(nullptr),
     mLineNumber(aCopy.mLineNumber),
+    mColumnNumber(aCopy.mColumnNumber),
     mWasMatched(false)
 {
   // rest is constructed lazily on existing data
@@ -1319,6 +1320,7 @@ StyleRule::StyleRule(StyleRule& aCopy,
     mImportantRule(nullptr),
     mDOMRule(aCopy.mDOMRule),
     mLineNumber(aCopy.mLineNumber),
+    mColumnNumber(aCopy.mColumnNumber),
     mWasMatched(false)
 {
   // The DOM rule is replacing |aCopy| with |this|, so transfer
@@ -1394,12 +1396,13 @@ StyleRule::Clone() const
 /* virtual */ nsIDOMCSSRule*
 StyleRule::GetDOMRule()
 {
-  if (!GetStyleSheet()) {
-    // inline style rules aren't supposed to have a DOM rule object, only
-    // a declaration.
-    return nullptr;
-  }
   if (!mDOMRule) {
+    if (!GetStyleSheet()) {
+      // Inline style rules aren't supposed to have a DOM rule object, only
+      // a declaration.  But if we do have one already, from a style sheet
+      // rule that used to be in a document, we still want to return it.
+      return nullptr;
+    }
     mDOMRule = new DOMCSSStyleRule(this);
     NS_ADDREF(mDOMRule);
   }
@@ -1416,12 +1419,7 @@ StyleRule::GetExistingDOMRule()
 StyleRule::DeclarationChanged(Declaration* aDecl,
                               bool aHandleContainer)
 {
-  StyleRule* clone = new StyleRule(*this, aDecl);
-  if (!clone) {
-    return nullptr;
-  }
-
-  NS_ADDREF(clone); // for return
+  nsRefPtr<StyleRule> clone = new StyleRule(*this, aDecl);
 
   if (aHandleContainer) {
     nsCSSStyleSheet* sheet = GetStyleSheet();
@@ -1436,7 +1434,7 @@ StyleRule::DeclarationChanged(Declaration* aDecl,
     }
   }
 
-  return clone;
+  return clone.forget();
 }
 
 /* virtual */ void
@@ -1475,18 +1473,18 @@ StyleRule::GetCssText(nsAString& aCssText)
 {
   if (mSelector) {
     mSelector->ToString(aCssText, GetStyleSheet());
-    aCssText.Append(PRUnichar(' '));
+    aCssText.Append(char16_t(' '));
   }
-  aCssText.Append(PRUnichar('{'));
-  aCssText.Append(PRUnichar(' '));
+  aCssText.Append(char16_t('{'));
+  aCssText.Append(char16_t(' '));
   if (mDeclaration)
   {
     nsAutoString   tempString;
     mDeclaration->ToString( tempString );
     aCssText.Append( tempString );
   }
-  aCssText.Append(PRUnichar(' '));
-  aCssText.Append(PRUnichar('}'));
+  aCssText.Append(char16_t(' '));
+  aCssText.Append(char16_t('}'));
 }
 
 void
@@ -1513,7 +1511,7 @@ StyleRule::SetSelectorText(const nsAString& aSelectorText)
 }
 
 /* virtual */ size_t
-StyleRule::SizeOfIncludingThis(nsMallocSizeOfFun aMallocSizeOf) const
+StyleRule::SizeOfIncludingThis(mozilla::MallocSizeOf aMallocSizeOf) const
 {
   size_t n = aMallocSizeOf(this);
   n += mSelector ? mSelector->SizeOfIncludingThis(aMallocSizeOf) : 0;
