@@ -13,7 +13,7 @@
 #include "nsIPresShell.h"
 #include "nsStyleContext.h"
 #include "nsCSSRendering.h"
-#include "nsINameSpaceManager.h"
+#include "nsNameSpaceManager.h"
 #include "nsMenuPopupFrame.h"
 #include "nsMenuBarFrame.h"
 #include "nsIDocument.h"
@@ -50,7 +50,7 @@ using namespace mozilla;
 
 #define NS_MENU_POPUP_LIST_INDEX 0
 
-#if defined(XP_WIN) || defined(XP_OS2)
+#if defined(XP_WIN)
 #define NSCONTEXTMENUISMOUSEUP 1
 #endif
 
@@ -76,7 +76,7 @@ public:
   {
   }
 
-  NS_IMETHOD Run()
+  NS_IMETHOD Run() MOZ_OVERRIDE
   {
     nsAutoString domEventToFire;
 
@@ -123,7 +123,7 @@ public:
   {
   }
 
-  NS_IMETHOD Run()
+  NS_IMETHOD Run() MOZ_OVERRIDE
   {
     nsMenuFrame* frame = static_cast<nsMenuFrame*>(mFrame.GetFrame());
     NS_ENSURE_STATE(frame);
@@ -220,7 +220,7 @@ public:
   {
   }
 
-  virtual bool ReflowFinished()
+  virtual bool ReflowFinished() MOZ_OVERRIDE
   {
     bool shouldFlush = false;
     nsMenuFrame* menu = do_QueryFrame(mWeakFrame.GetFrame());
@@ -232,7 +232,7 @@ public:
     return shouldFlush;
   }
 
-  virtual void ReflowCallbackCanceled()
+  virtual void ReflowCallbackCanceled() MOZ_OVERRIDE
   {
     delete this;
   }
@@ -327,7 +327,7 @@ nsMenuFrame::SetPopupFrame(nsFrameList& aFrameList)
   }
 }
 
-NS_IMETHODIMP
+nsresult
 nsMenuFrame::SetInitialChildList(ChildListID     aListID,
                                  nsFrameList&    aChildList)
 {
@@ -389,7 +389,7 @@ nsMenuFrame::BuildDisplayListForChildren(nsDisplayListBuilder*   aBuilder,
   WrapListsInRedirector(aBuilder, set, aLists);
 }
 
-NS_IMETHODIMP
+nsresult
 nsMenuFrame::HandleEvent(nsPresContext* aPresContext,
                          WidgetGUIEvent* aEvent,
                          nsEventStatus* aEventStatus)
@@ -645,7 +645,7 @@ nsMenuFrame::SelectMenu(bool aActivateFlag)
   return NS_OK;
 }
 
-NS_IMETHODIMP
+nsresult
 nsMenuFrame::AttributeChanged(int32_t aNameSpaceID,
                               nsIAtom* aAttribute,
                               int32_t aModType)
@@ -753,7 +753,7 @@ nsMenuFrame::DoLayout(nsBoxLayoutState& aState)
 }
 
 #ifdef DEBUG_LAYOUT
-NS_IMETHODIMP
+nsresult
 nsMenuFrame::SetDebug(nsBoxLayoutState& aState, bool aDebug)
 {
   // see if our state matches the given debug state
@@ -1284,7 +1284,7 @@ nsMenuFrame::PassMenuCommandEventToPopupManager()
   mDelayedMenuCommandEvent = nullptr;
 }
 
-NS_IMETHODIMP
+nsresult
 nsMenuFrame::RemoveFrame(ChildListID     aListID,
                          nsIFrame*       aOldFrame)
 {
@@ -1301,7 +1301,7 @@ nsMenuFrame::RemoveFrame(ChildListID     aListID,
   return nsBoxFrame::RemoveFrame(aListID, aOldFrame);
 }
 
-NS_IMETHODIMP
+nsresult
 nsMenuFrame::InsertFrames(ChildListID     aListID,
                           nsIFrame*       aPrevFrame,
                           nsFrameList&    aFrameList)
@@ -1330,7 +1330,7 @@ nsMenuFrame::InsertFrames(ChildListID     aListID,
   return nsBoxFrame::InsertFrames(aListID, aPrevFrame, aFrameList);
 }
 
-NS_IMETHODIMP
+nsresult
 nsMenuFrame::AppendFrames(ChildListID     aListID,
                           nsFrameList&    aFrameList)
 {
