@@ -18,6 +18,7 @@
 #include "VideoUtils.h"
 #endif
 #include "MediaConduitInterface.h"
+#include "MediaPipelineFilter.h"
 #include "AudioSegment.h"
 #include "mozilla/ReentrantMonitor.h"
 #include "SrtpFlow.h"
@@ -29,6 +30,8 @@
 #include "VideoSegment.h"
 #endif
 
+#include "webrtc/modules/rtp_rtcp/interface/rtp_header_parser.h"
+
 namespace mozilla {
 
 // A class that represents the pipeline of audio and video
@@ -71,37 +74,33 @@ class MediaPipeline : public sigslot::has_slots<> {
                 nsCOMPtr<nsIEventTarget> sts_thread,
                 MediaStream *stream,
                 TrackID track_id,
+                int level,
                 RefPtr<MediaSessionConduit> conduit,
                 RefPtr<TransportFlow> rtp_transport,
                 RefPtr<TransportFlow> rtcp_transport)
       : direction_(direction),
         stream_(stream),
         track_id_(track_id),
+        level_(level),
         conduit_(conduit),
-        rtp_transport_(rtp_transport),
-        rtp_state_(MP_CONNECTING),
-        rtcp_transport_(rtcp_transport),
-        rtcp_state_(MP_CONNECTING),
+        rtp_(rtp_transport, rtcp_transport ? RTP : MUX),
+        rtcp_(rtcp_transport ? rtcp_transport : rtp_transport,
+              rtcp_transport ? RTCP : MUX),
         main_thread_(main_thread),
         sts_thread_(sts_thread),
-        rtp_send_srtp_(),
-        rtcp_send_srtp_(),
-        rtp_recv_srtp_(),
-        rtcp_recv_srtp_(),
         rtp_packets_sent_(0),
         rtcp_packets_sent_(0),
         rtp_packets_received_(0),
         rtcp_packets_received_(0),
+        rtp_bytes_sent_(0),
+        rtp_bytes_received_(0),
         pc_(pc),
         description_() {
-      // To indicate rtcp-mux rtcp_transport should be NULL.
+      // To indicate rtcp-mux rtcp_transport should be nullptr.
       // Therefore it's an error to send in the same flow for
       // both rtp and rtcp.
-      MOZ_ASSERT(rtp_transport_ != rtcp_transport_);
+      MOZ_ASSERT(rtp_transport != rtcp_transport);
 
-      if (!rtcp_transport_) {
-        rtcp_transport_ = rtp_transport;
-      }
       // PipelineTransport() will access this->sts_thread_; moved here for safety
       transport_ = new PipelineTransport(this);
   }
@@ -122,22 +121,43 @@ class MediaPipeline : public sigslot::has_slots<> {
 
   virtual nsresult Init();
 
+  // When we have offered bundle, the MediaPipelines are created in an
+  // indeterminate state; we do not know whether the answerer will take us
+  // up on our offer. In the meantime, we need to behave in a manner that
+  // errs on the side of packet loss when it is unclear whether an arriving
+  // packet is meant for us. We want to get out of this indeterminate state
+  // ASAP, which is what this function can be used for.
+  void SetUsingBundle_s(bool decision);
+  MediaPipelineFilter* UpdateFilterFromRemoteDescription_s(
+      nsAutoPtr<MediaPipelineFilter> filter);
+
   virtual Direction direction() const { return direction_; }
+  virtual TrackID trackid() const { return track_id_; }
+  virtual int level() const { return level_; }
 
   bool IsDoingRtcpMux() const {
-    return (rtp_transport_ == rtcp_transport_);
+    return (rtp_.type_ == MUX);
   }
 
-  int rtp_packets_sent() const { return rtp_packets_sent_; }
-  int rtcp_packets_sent() const { return rtcp_packets_sent_; }
-  int rtp_packets_received() const { return rtp_packets_received_; }
-  int rtcp_packets_received() const { return rtcp_packets_received_; }
+  int32_t rtp_packets_sent() const { return rtp_packets_sent_; }
+  int64_t rtp_bytes_sent() const { return rtp_bytes_sent_; }
+  int32_t rtcp_packets_sent() const { return rtcp_packets_sent_; }
+  int32_t rtp_packets_received() const { return rtp_packets_received_; }
+  int64_t rtp_bytes_received() const { return rtp_bytes_received_; }
+  int32_t rtcp_packets_received() const { return rtcp_packets_received_; }
 
-  MediaSessionConduit *Conduit() { return conduit_; }
+  MediaSessionConduit *Conduit() const { return conduit_; }
 
   // Thread counting
   NS_INLINE_DECL_THREADSAFE_REFCOUNTING(MediaPipeline)
 
+  typedef enum {
+    RTP,
+    RTCP,
+    MUX,
+    MAX_RTP_TYPE
+  } RtpType;
+
  protected:
   virtual void DetachMediaStream() {}
 
@@ -145,11 +165,11 @@ class MediaPipeline : public sigslot::has_slots<> {
   class PipelineTransport : public TransportInterface {
    public:
     // Implement the TransportInterface functions
-    PipelineTransport(MediaPipeline *pipeline)
+    explicit PipelineTransport(MediaPipeline *pipeline)
         : pipeline_(pipeline),
           sts_thread_(pipeline->sts_thread_) {}
 
-    void Detach() { pipeline_ = NULL; }
+    void Detach() { pipeline_ = nullptr; }
     MediaPipeline *pipeline() const { return pipeline_; }
 
     virtual nsresult SendRtpPacket(const void* data, int len);
@@ -164,15 +184,40 @@ class MediaPipeline : public sigslot::has_slots<> {
   };
   friend class PipelineTransport;
 
-  virtual nsresult TransportFailed_s(TransportFlow *flow);  // The transport is down
-  virtual nsresult TransportReady_s(TransportFlow *flow);   // The transport is ready
+  class TransportInfo {
+    public:
+      TransportInfo(RefPtr<TransportFlow> flow, RtpType type) :
+        transport_(flow),
+        state_(MP_CONNECTING),
+        type_(type) {
+        MOZ_ASSERT(flow);
+      }
 
-  void increment_rtp_packets_sent();
+      RefPtr<TransportFlow> transport_;
+      State state_;
+      RefPtr<SrtpFlow> send_srtp_;
+      RefPtr<SrtpFlow> recv_srtp_;
+      RtpType type_;
+  };
+
+  // The transport is down
+  virtual nsresult TransportFailed_s(TransportInfo &info);
+  // The transport is ready
+  virtual nsresult TransportReady_s(TransportInfo &info);
+  void UpdateRtcpMuxState(TransportInfo &info);
+
+  // Unhooks from signals
+  void DisconnectTransport_s(TransportInfo &info);
+  nsresult ConnectTransport_s(TransportInfo &info);
+
+  TransportInfo* GetTransportInfo_s(TransportFlow *flow);
+
+  void increment_rtp_packets_sent(int bytes);
   void increment_rtcp_packets_sent();
-  void increment_rtp_packets_received();
+  void increment_rtp_packets_received(int bytes);
   void increment_rtcp_packets_received();
 
-  virtual nsresult SendPacket(TransportFlow *flow, const void* data, int len);
+  virtual nsresult SendPacket(TransportFlow *flow, const void *data, int len);
 
   // Process slots on transports
   void StateChange(TransportFlow *flow, TransportLayer::State);
@@ -189,14 +234,23 @@ class MediaPipeline : public sigslot::has_slots<> {
                                 // Used on STS and MediaStreamGraph threads.
   TrackID track_id_;            // The track on the stream.
                                 // Written and used as the stream_;
+  int level_; // The m-line index (starting at 1, to match convention)
   RefPtr<MediaSessionConduit> conduit_;  // Our conduit. Written on the main
                                          // thread. Read on STS thread.
 
   // The transport objects. Read/written on STS thread.
-  RefPtr<TransportFlow> rtp_transport_;
-  State rtp_state_;
-  RefPtr<TransportFlow> rtcp_transport_;
-  State rtcp_state_;
+  TransportInfo rtp_;
+  TransportInfo rtcp_;
+  // These are for bundle. We have a separate set because when we have offered
+  // bundle, we do not know whether we will receive traffic on the transport
+  // in this pipeline's m-line, or the transport in the "master" m-line for
+  // the bundle. We need to be ready for either. Once this ambiguity is
+  // resolved, the transport we know that we'll be using will be set in
+  // rtp_transport_ and rtcp_transport_, and these will be unset.
+  // TODO(bcampen@mozilla.com): I'm pretty sure this could be leveraged for
+  // re-offer with a new address on an m-line too, with a little work.
+  nsAutoPtr<TransportInfo> possible_bundle_rtp_;
+  nsAutoPtr<TransportInfo> possible_bundle_rtcp_;
 
   // Pointers to the threads we need. Initialized at creation
   // and used all over the place.
@@ -207,24 +261,23 @@ class MediaPipeline : public sigslot::has_slots<> {
   // destroyed on the STS thread.
   RefPtr<PipelineTransport> transport_;
 
-  // Used only on STS thread.
-  RefPtr<SrtpFlow> rtp_send_srtp_;
-  RefPtr<SrtpFlow> rtcp_send_srtp_;
-  RefPtr<SrtpFlow> rtp_recv_srtp_;
-  RefPtr<SrtpFlow> rtcp_recv_srtp_;
-
-  // Written only on STS thread. May be read on other
-  // threads but since there is no mutex, the values
-  // will only be approximate.
-  int rtp_packets_sent_;
-  int rtcp_packets_sent_;
-  int rtp_packets_received_;
-  int rtcp_packets_received_;
+  // Only safe to access from STS thread.
+  // Build into TransportInfo?
+  int32_t rtp_packets_sent_;
+  int32_t rtcp_packets_sent_;
+  int32_t rtp_packets_received_;
+  int32_t rtcp_packets_received_;
+  int64_t rtp_bytes_sent_;
+  int64_t rtp_bytes_received_;
 
   // Written on Init. Read on STS thread.
   std::string pc_;
   std::string description_;
 
+  // Written on Init, all following accesses are on the STS thread.
+  nsAutoPtr<MediaPipelineFilter> filter_;
+  nsAutoPtr<webrtc::RtpHeaderParser> rtp_parser_;
+
  private:
   nsresult Init_s();
 
@@ -302,18 +355,19 @@ private:
 // and transmitting to the network.
 class MediaPipelineTransmit : public MediaPipeline {
  public:
-  // Set rtcp_transport to NULL to use rtcp-mux
+  // Set rtcp_transport to nullptr to use rtcp-mux
   MediaPipelineTransmit(const std::string& pc,
                         nsCOMPtr<nsIEventTarget> main_thread,
                         nsCOMPtr<nsIEventTarget> sts_thread,
                         DOMMediaStream *domstream,
                         TrackID track_id,
+                        int level,
                         RefPtr<MediaSessionConduit> conduit,
                         RefPtr<TransportFlow> rtp_transport,
                         RefPtr<TransportFlow> rtcp_transport) :
       MediaPipeline(pc, TRANSMIT, main_thread, sts_thread,
-                    domstream->GetStream(), track_id, conduit, rtp_transport,
-                    rtcp_transport),
+                    domstream->GetStream(), track_id, level,
+                    conduit, rtp_transport, rtcp_transport),
       listener_(new PipelineListener(conduit)),
       domstream_(domstream)
   {}
@@ -332,7 +386,7 @@ class MediaPipelineTransmit : public MediaPipeline {
   }
 
   // Override MediaPipeline::TransportReady.
-  virtual nsresult TransportReady_s(TransportFlow *flow);
+  virtual nsresult TransportReady_s(TransportInfo &info);
 
   // Separate class to allow ref counting
   class PipelineListener : public MediaStreamDirectListener {
@@ -342,10 +396,14 @@ class MediaPipelineTransmit : public MediaPipeline {
       : conduit_(conduit),
         active_(false),
         direct_connect_(false),
-        last_img_(-1),
         samples_10ms_buffer_(nullptr),
         buffer_current_(0),
-        samplenum_10ms_(0) {}
+        samplenum_10ms_(0)
+#ifdef MOZILLA_INTERNAL_API
+        , last_img_(-1)
+#endif // MOZILLA_INTERNAL_API
+    {
+    }
 
     ~PipelineListener()
     {
@@ -396,8 +454,6 @@ class MediaPipelineTransmit : public MediaPipeline {
     volatile bool active_;
     bool direct_connect_;
 
-    int32_t last_img_; // serial number of last Image
-
     // These vars handle breaking audio samples into exact 10ms chunks:
     // The buffer of 10ms audio samples that we will send once full
     // (can be carried over from one call to another).
@@ -406,6 +462,10 @@ class MediaPipelineTransmit : public MediaPipeline {
     int64_t buffer_current_;
     // The number of samples in a 10ms audio chunk.
     int64_t samplenum_10ms_;
+
+#ifdef MOZILLA_INTERNAL_API
+    int32_t last_img_; // serial number of last Image
+#endif // MOZILLA_INTERNAL_API
   };
 
  private:
@@ -418,19 +478,35 @@ class MediaPipelineTransmit : public MediaPipeline {
 // rendering video.
 class MediaPipelineReceive : public MediaPipeline {
  public:
-  // Set rtcp_transport to NULL to use rtcp-mux
+  // Set rtcp_transport to nullptr to use rtcp-mux
   MediaPipelineReceive(const std::string& pc,
                        nsCOMPtr<nsIEventTarget> main_thread,
                        nsCOMPtr<nsIEventTarget> sts_thread,
                        MediaStream *stream,
                        TrackID track_id,
+                       int level,
                        RefPtr<MediaSessionConduit> conduit,
                        RefPtr<TransportFlow> rtp_transport,
-                       RefPtr<TransportFlow> rtcp_transport) :
+                       RefPtr<TransportFlow> rtcp_transport,
+                       RefPtr<TransportFlow> bundle_rtp_transport,
+                       RefPtr<TransportFlow> bundle_rtcp_transport,
+                       nsAutoPtr<MediaPipelineFilter> filter) :
       MediaPipeline(pc, RECEIVE, main_thread, sts_thread,
-                    stream, track_id, conduit, rtp_transport,
+                    stream, track_id, level, conduit, rtp_transport,
                     rtcp_transport),
       segments_added_(0) {
+    filter_ = filter;
+    rtp_parser_ = webrtc::RtpHeaderParser::Create();
+    if (bundle_rtp_transport) {
+      if (bundle_rtcp_transport) {
+        MOZ_ASSERT(bundle_rtp_transport != bundle_rtcp_transport);
+        possible_bundle_rtp_ = new TransportInfo(bundle_rtp_transport, RTP);
+        possible_bundle_rtcp_ = new TransportInfo(bundle_rtcp_transport, RTCP);
+      } else {
+        possible_bundle_rtp_ = new TransportInfo(bundle_rtp_transport, MUX);
+        possible_bundle_rtcp_ = new TransportInfo(bundle_rtp_transport, MUX);
+      }
+    }
   }
 
   int segments_added() const { return segments_added_; }
@@ -451,12 +527,17 @@ class MediaPipelineReceiveAudio : public MediaPipelineReceive {
                             nsCOMPtr<nsIEventTarget> sts_thread,
                             MediaStream *stream,
                             TrackID track_id,
+                            int level,
                             RefPtr<AudioSessionConduit> conduit,
                             RefPtr<TransportFlow> rtp_transport,
-                            RefPtr<TransportFlow> rtcp_transport) :
+                            RefPtr<TransportFlow> rtcp_transport,
+                            RefPtr<TransportFlow> bundle_rtp_transport,
+                            RefPtr<TransportFlow> bundle_rtcp_transport,
+                            nsAutoPtr<MediaPipelineFilter> filter) :
       MediaPipelineReceive(pc, main_thread, sts_thread,
-                           stream, track_id, conduit, rtp_transport,
-                           rtcp_transport),
+                           stream, track_id, level, conduit, rtp_transport,
+                           rtcp_transport, bundle_rtp_transport,
+                           bundle_rtcp_transport, filter),
       listener_(new PipelineListener(stream->AsSourceStream(),
                                      track_id, conduit)) {
   }
@@ -513,12 +594,17 @@ class MediaPipelineReceiveVideo : public MediaPipelineReceive {
                             nsCOMPtr<nsIEventTarget> sts_thread,
                             MediaStream *stream,
                             TrackID track_id,
+                            int level,
                             RefPtr<VideoSessionConduit> conduit,
                             RefPtr<TransportFlow> rtp_transport,
-                            RefPtr<TransportFlow> rtcp_transport) :
+                            RefPtr<TransportFlow> rtcp_transport,
+                            RefPtr<TransportFlow> bundle_rtp_transport,
+                            RefPtr<TransportFlow> bundle_rtcp_transport,
+                            nsAutoPtr<MediaPipelineFilter> filter) :
       MediaPipelineReceive(pc, main_thread, sts_thread,
-                           stream, track_id, conduit, rtp_transport,
-                           rtcp_transport),
+                           stream, track_id, level, conduit, rtp_transport,
+                           rtcp_transport, bundle_rtp_transport,
+                           bundle_rtcp_transport, filter),
       renderer_(new PipelineRenderer(MOZ_THIS_IN_INITIALIZER_LIST())),
       listener_(new PipelineListener(stream->AsSourceStream(), track_id)) {
   }
@@ -545,7 +631,7 @@ class MediaPipelineReceiveVideo : public MediaPipelineReceive {
     PipelineRenderer(MediaPipelineReceiveVideo *pipeline) :
       pipeline_(pipeline) {}
 
-    void Detach() { pipeline_ = NULL; }
+    void Detach() { pipeline_ = nullptr; }
 
     // Implement VideoRenderer
     virtual void FrameSizeChange(unsigned int width,
