@@ -5,46 +5,45 @@
 
 package org.mozilla.gecko;
 
-import org.mozilla.gecko.db.BrowserDB;
-import org.mozilla.gecko.gfx.Layer;
-import org.mozilla.gecko.mozglue.DirectBufferAllocator;
-import org.mozilla.gecko.util.GeckoAsyncTask;
+import java.util.ArrayList;
+import java.util.Collection;
+import java.util.HashMap;
+import java.util.regex.Matcher;
+import java.util.regex.Pattern;
 
 import org.json.JSONException;
 import org.json.JSONObject;
+import org.mozilla.gecko.db.BrowserContract.Bookmarks;
+import org.mozilla.gecko.db.BrowserDB;
+import org.mozilla.gecko.gfx.Layer;
+import org.mozilla.gecko.util.ThreadUtils;
 
 import android.content.ContentResolver;
-import android.database.ContentObserver;
+import android.content.Context;
 import android.graphics.Bitmap;
 import android.graphics.Color;
 import android.graphics.drawable.BitmapDrawable;
-import android.graphics.drawable.Drawable;
-import android.net.Uri;
 import android.os.Build;
 import android.text.TextUtils;
 import android.util.Log;
 import android.view.View;
 
-import java.nio.ByteBuffer;
-import java.util.ArrayList;
-import java.util.Collection;
-import java.util.HashMap;
-import java.util.HashSet;
-import java.util.regex.Matcher;
-import java.util.regex.Pattern;
-
-public final class Tab {
+public class Tab {
     private static final String LOGTAG = "GeckoTab";
 
     private static Pattern sColorPattern;
     private final int mId;
     private long mLastUsed;
     private String mUrl;
+    private String mBaseDomain;
+    private String mUserSearch;
     private String mTitle;
-    private Drawable mFavicon;
+    private Bitmap mFavicon;
     private String mFaviconUrl;
     private int mFaviconSize;
-    private JSONObject mIdentityData;
+    private boolean mHasFeeds;
+    private boolean mHasOpenSearch;
+    private SiteIdentity mSiteIdentity;
     private boolean mReaderEnabled;
     private BitmapDrawable mThumbnail;
     private int mHistoryIndex;
@@ -53,38 +52,60 @@ public final class Tab {
     private boolean mExternal;
     private boolean mBookmark;
     private boolean mReadingListItem;
-    private long mFaviconLoadId;
-    private String mDocumentURI;
+    private int mFaviconLoadId;
     private String mContentType;
     private boolean mHasTouchListeners;
     private ZoomConstraints mZoomConstraints;
+    private boolean mIsRTL;
     private ArrayList<View> mPluginViews;
     private HashMap<Object, Layer> mPluginLayers;
-    private ContentResolver mContentResolver;
-    private ContentObserver mContentObserver;
-    private int mCheckerboardColor = Color.WHITE;
+    private int mBackgroundColor;
     private int mState;
-    private ByteBuffer mThumbnailBuffer;
     private Bitmap mThumbnailBitmap;
     private boolean mDesktopMode;
     private boolean mEnteringReaderMode;
+    private Context mAppContext;
+    private ErrorType mErrorType = ErrorType.NONE;
+    private static final int MAX_HISTORY_LIST_SIZE = 50;
+    private volatile int mLoadProgress;
+    private String mMostRecentHomePanel;
 
     public static final int STATE_DELAYED = 0;
     public static final int STATE_LOADING = 1;
     public static final int STATE_SUCCESS = 2;
     public static final int STATE_ERROR = 3;
 
-    public Tab(int id, String url, boolean external, int parentId, String title) {
+    public static final int LOAD_PROGRESS_INIT = 10;
+    public static final int LOAD_PROGRESS_START = 20;
+    public static final int LOAD_PROGRESS_LOCATION_CHANGE = 60;
+    public static final int LOAD_PROGRESS_LOADED = 80;
+    public static final int LOAD_PROGRESS_STOP = 100;
+
+    private static final int DEFAULT_BACKGROUND_COLOR = Color.WHITE;
+
+    public enum ErrorType {
+        CERT_ERROR,  // Pages with certificate problems
+        BLOCKED,     // Pages blocked for phishing or malware warnings
+        NET_ERROR,   // All other types of error
+        NONE         // Non error pages
+    }
+
+    public Tab(Context context, int id, String url, boolean external, int parentId, String title) {
+        mAppContext = context.getApplicationContext();
         mId = id;
         mLastUsed = 0;
         mUrl = url;
+        mBaseDomain = "";
+        mUserSearch = "";
         mExternal = external;
         mParentId = parentId;
         mTitle = title == null ? "" : title;
         mFavicon = null;
         mFaviconUrl = null;
         mFaviconSize = 0;
-        mIdentityData = null;
+        mHasFeeds = false;
+        mHasOpenSearch = false;
+        mSiteIdentity = new SiteIdentity();
         mReaderEnabled = false;
         mEnteringReaderMode = false;
         mThumbnail = null;
@@ -93,23 +114,27 @@ public final class Tab {
         mBookmark = false;
         mReadingListItem = false;
         mFaviconLoadId = 0;
-        mDocumentURI = "";
         mContentType = "";
         mZoomConstraints = new ZoomConstraints(false);
         mPluginViews = new ArrayList<View>();
         mPluginLayers = new HashMap<Object, Layer>();
-        mState = GeckoApp.shouldShowProgress(url) ? STATE_SUCCESS : STATE_LOADING;
-        mContentResolver = Tabs.getInstance().getContentResolver();
-        mContentObserver = new ContentObserver(GeckoAppShell.getHandler()) {
-            public void onChange(boolean selfChange) {
-                updateBookmark();
-            }
-        };
-        BrowserDB.registerBookmarkObserver(mContentResolver, mContentObserver);
+        mState = shouldShowProgress(url) ? STATE_SUCCESS : STATE_LOADING;
+        mLoadProgress = LOAD_PROGRESS_INIT;
+
+        // At startup, the background is set to a color specified by LayerView
+        // when the LayerView is created. Shortly after, this background color
+        // will be used before the tab's content is shown.
+        mBackgroundColor = DEFAULT_BACKGROUND_COLOR;
+
+        updateBookmark();
+    }
+
+    private ContentResolver getContentResolver() {
+        return mAppContext.getContentResolver();
     }
 
     public void onDestroy() {
-        BrowserDB.unregisterContentObserver(mContentResolver, mContentObserver);
+        Tabs.getInstance().notifyListeners(this, Tabs.TabEvents.CLOSED);
     }
 
     public int getId() {
@@ -133,6 +158,11 @@ public final class Tab {
         return mUrl;
     }
 
+    // mUserSearch should never be null, but it may be an empty string
+    public synchronized String getUserSearch() {
+        return mUserSearch;
+    }
+
     // mTitle should never be null, but it may be an empty string
     public synchronized String getTitle() {
         return mTitle;
@@ -146,77 +176,66 @@ public final class Tab {
         return mUrl;
     }
 
-    public Drawable getFavicon() {
+    public String getBaseDomain() {
+        return mBaseDomain;
+    }
+
+    public Bitmap getFavicon() {
         return mFavicon;
     }
 
-    public Drawable getThumbnail() {
+    public BitmapDrawable getThumbnail() {
         return mThumbnail;
     }
 
-    synchronized public ByteBuffer getThumbnailBuffer() {
-        int capacity = getThumbnailWidth() * getThumbnailHeight() * 2 /* 16 bpp */;
-        if (mThumbnailBuffer != null && mThumbnailBuffer.capacity() == capacity)
-            return mThumbnailBuffer;
-        freeBuffer();
-        mThumbnailBuffer = DirectBufferAllocator.allocate(capacity);
-        return mThumbnailBuffer;
+    public String getMostRecentHomePanel() {
+        return mMostRecentHomePanel;
     }
 
-    public Bitmap getThumbnailBitmap() {
-        // Bug 787318 - Honeycomb has a bug with bitmap caching, we can't
-        // reuse the bitmap there.
-        if (Build.VERSION.SDK_INT < Build.VERSION_CODES.HONEYCOMB
-            || Build.VERSION.SDK_INT > Build.VERSION_CODES.HONEYCOMB_MR2) {
-            if (mThumbnailBitmap != null)
-                return mThumbnailBitmap;
-        } else {
-            if (mThumbnailBitmap != null)
-                mThumbnailBitmap.recycle();
+    public void setMostRecentHomePanel(String panelId) {
+        mMostRecentHomePanel = panelId;
+    }
+
+    public Bitmap getThumbnailBitmap(int width, int height) {
+        if (mThumbnailBitmap != null) {
+            // Bug 787318 - Honeycomb has a bug with bitmap caching, we can't
+            // reuse the bitmap there.
+            boolean honeycomb = (Build.VERSION.SDK_INT >= Build.VERSION_CODES.HONEYCOMB
+                              && Build.VERSION.SDK_INT <= Build.VERSION_CODES.HONEYCOMB_MR2);
+            boolean sizeChange = mThumbnailBitmap.getWidth() != width
+                              || mThumbnailBitmap.getHeight() != height;
+            if (honeycomb || sizeChange) {
+                mThumbnailBitmap = null;
+            }
         }
-        return mThumbnailBitmap = Bitmap.createBitmap(getThumbnailWidth(), getThumbnailHeight(), Bitmap.Config.RGB_565);
-    }
 
-    public void finalize() {
-        freeBuffer();
-    }
+        if (mThumbnailBitmap == null) {
+            Bitmap.Config config = (GeckoAppShell.getScreenDepth() == 24) ?
+                Bitmap.Config.ARGB_8888 : Bitmap.Config.RGB_565;
+            mThumbnailBitmap = Bitmap.createBitmap(width, height, config);
+        }
 
-    synchronized void freeBuffer() {
-        DirectBufferAllocator.free(mThumbnailBuffer);
-        mThumbnailBuffer = null;
-    }
-
-    int getThumbnailWidth() {
-        int desiredWidth = (int) (GeckoApp.mAppContext.getResources().getDimension(R.dimen.tab_thumbnail_width));
-        return desiredWidth & ~0x1;
-    }
-
-    int getThumbnailHeight() {
-        int desiredHeight = (int) (GeckoApp.mAppContext.getResources().getDimension(R.dimen.tab_thumbnail_height));
-        return desiredHeight & ~0x1;
+        return mThumbnailBitmap;
     }
 
     public void updateThumbnail(final Bitmap b) {
-        final Tab tab = this;
-        GeckoAppShell.getHandler().post(new Runnable() {
+        ThreadUtils.postToBackgroundThread(new Runnable() {
+            @Override
             public void run() {
                 if (b != null) {
                     try {
-                        mThumbnail = new BitmapDrawable(b);
+                        mThumbnail = new BitmapDrawable(mAppContext.getResources(), b);
                         if (mState == Tab.STATE_SUCCESS)
                             saveThumbnailToDB();
                     } catch (OutOfMemoryError oom) {
-                        Log.e(LOGTAG, "Unable to create/scale bitmap", oom);
+                        Log.w(LOGTAG, "Unable to create/scale bitmap.", oom);
                         mThumbnail = null;
                     }
                 } else {
                     mThumbnail = null;
                 }
-                GeckoApp.mAppContext.mMainHandler.post(new Runnable() {
-                    public void run() {
-                        Tabs.getInstance().notifyListeners(tab, Tabs.TabEvents.THUMBNAIL);
-                    }
-                });
+
+                Tabs.getInstance().notifyListeners(Tab.this, Tabs.TabEvents.THUMBNAIL);
             }
         });
     }
@@ -225,17 +244,16 @@ public final class Tab {
         return mFaviconUrl;
     }
 
-    public String getSecurityMode() {
-        try {
-            return mIdentityData.getString("mode");
-        } catch (Exception e) {
-            // If mIdentityData is null, or we get a JSONException
-            return SiteIdentityPopup.UNKNOWN;
-        }
+    public boolean hasFeeds() {
+        return mHasFeeds;
     }
 
-    public JSONObject getIdentityData() {
-        return mIdentityData;
+    public boolean hasOpenSearch() {
+        return mHasOpenSearch;
+    }
+
+    public SiteIdentity getSiteIdentity() {
+        return mSiteIdentity;
     }
 
     public boolean getReaderEnabled() {
@@ -257,18 +275,30 @@ public final class Tab {
     public synchronized void updateURL(String url) {
         if (url != null && url.length() > 0) {
             mUrl = url;
-            Log.d(LOGTAG, "Updated URL for tab with id: " + mId);
-            updateBookmark();
-            updateHistory(mUrl, mTitle);
         }
     }
 
-    public void setDocumentURI(String documentURI) {
-        mDocumentURI = documentURI;
+    private synchronized void updateUserSearch(String userSearch) {
+        mUserSearch = userSearch;
     }
 
-    public String getDocumentURI() {
-        return mDocumentURI;
+    public void setErrorType(String type) {
+        if ("blocked".equals(type))
+            setErrorType(ErrorType.BLOCKED);
+        else if ("certerror".equals(type))
+            setErrorType(ErrorType.CERT_ERROR);
+        else if ("neterror".equals(type))
+            setErrorType(ErrorType.NET_ERROR);
+        else
+            setErrorType(ErrorType.NONE);
+    }
+
+    public void setErrorType(ErrorType type) {
+        mErrorType = type;
+    }
+
+    public ErrorType getErrorType() {
+        return mErrorType;
     }
 
     public void setContentType(String contentType) {
@@ -280,29 +310,19 @@ public final class Tab {
     }
 
     public synchronized void updateTitle(String title) {
-        // Keep the title unchanged while entering reader mode
-        if (mEnteringReaderMode)
+        // Keep the title unchanged while entering reader mode.
+        if (mEnteringReaderMode) {
             return;
+        }
+
+        // If there was a title, but it hasn't changed, do nothing.
+        if (mTitle != null &&
+            TextUtils.equals(mTitle, title)) {
+            return;
+        }
 
         mTitle = (title == null ? "" : title);
-
-        Log.d(LOGTAG, "Updated title for tab with id: " + mId);
-        updateHistory(mUrl, mTitle);
-        final Tab tab = this;
-
-        GeckoAppShell.getMainHandler().post(new Runnable() {
-            public void run() {
-                Tabs.getInstance().notifyListeners(tab, Tabs.TabEvents.TITLE);
-            }
-        });
-    }
-
-    private void updateHistory(final String uri, final String title) {
-        GeckoAppShell.getHandler().post(new Runnable() {
-            public void run() {
-                GlobalHistory.getInstance().update(uri, title);
-            }
-        });
+        Tabs.getInstance().notifyListeners(this, Tabs.TabEvents.TITLE);
     }
 
     public void setState(int state) {
@@ -310,7 +330,7 @@ public final class Tab {
 
         if (mState != Tab.STATE_LOADING)
             mEnteringReaderMode = false;
-        }
+    }
 
     public int getState() {
         return mState;
@@ -324,6 +344,14 @@ public final class Tab {
         return mZoomConstraints;
     }
 
+    public void setIsRTL(boolean aIsRTL) {
+        mIsRTL = aIsRTL;
+    }
+
+    public boolean getIsRTL() {
+        return mIsRTL;
+    }
+
     public void setHasTouchListeners(boolean aValue) {
         mHasTouchListeners = aValue;
     }
@@ -332,17 +360,23 @@ public final class Tab {
         return mHasTouchListeners;
     }
 
-    public void setFaviconLoadId(long faviconLoadId) {
+    public void setFaviconLoadId(int faviconLoadId) {
         mFaviconLoadId = faviconLoadId;
     }
 
-    public long getFaviconLoadId() {
+    public int getFaviconLoadId() {
         return mFaviconLoadId;
     }
 
-    public void updateFavicon(Drawable favicon) {
+    /**
+     * Returns true if the favicon changed.
+     */
+    public boolean updateFavicon(Bitmap favicon) {
+        if (mFavicon == favicon) {
+            return false;
+        }
         mFavicon = favicon;
-        Log.d(LOGTAG, "Updated favicon for tab with id: " + mId);
+        return true;
     }
 
     public synchronized void updateFaviconURL(String faviconUrl, int size) {
@@ -355,7 +389,6 @@ public final class Tab {
         if (size == -1 || size >= mFaviconSize) {
             mFaviconUrl = faviconUrl;
             mFaviconSize = size;
-            Log.d(LOGTAG, "Updated favicon URL for tab with id: " + mId);
         }
     }
 
@@ -369,61 +402,66 @@ public final class Tab {
         mFaviconSize = 0;
     }
 
+    public void setHasFeeds(boolean hasFeeds) {
+        mHasFeeds = hasFeeds;
+    }
+
+    public void setHasOpenSearch(boolean hasOpenSearch) {
+        mHasOpenSearch = hasOpenSearch;
+    }
+
     public void updateIdentityData(JSONObject identityData) {
-        mIdentityData = identityData;
+        mSiteIdentity.update(identityData);
     }
 
     public void setReaderEnabled(boolean readerEnabled) {
         mReaderEnabled = readerEnabled;
-        GeckoAppShell.getMainHandler().post(new Runnable() {
+        Tabs.getInstance().notifyListeners(this, Tabs.TabEvents.MENU_UPDATED);
+    }
+
+    void updateBookmark() {
+        if (getURL() == null) {
+            return;
+        }
+
+        ThreadUtils.postToBackgroundThread(new Runnable() {
+            @Override
             public void run() {
+                final String url = getURL();
+                if (url == null) {
+                    return;
+                }
+
+                final int flags = BrowserDB.getItemFlags(getContentResolver(), url);
+                mBookmark = (flags & Bookmarks.FLAG_BOOKMARK) > 0;
+                mReadingListItem = (flags & Bookmarks.FLAG_READING) > 0;
                 Tabs.getInstance().notifyListeners(Tab.this, Tabs.TabEvents.MENU_UPDATED);
             }
         });
     }
 
-    private void updateBookmark() {
-        final String url = getURL();
-        if (url == null)
-            return;
-
-        (new GeckoAsyncTask<Void, Void, Void>(GeckoApp.mAppContext, GeckoAppShell.getHandler()) {
-            @Override
-            public Void doInBackground(Void... params) {
-                if (url.equals(getURL())) {
-                    mBookmark = BrowserDB.isBookmark(mContentResolver, url);
-                    mReadingListItem = BrowserDB.isReadingListItem(mContentResolver, url);
-                }
-                return null;
-            }
-
-            @Override
-            public void onPostExecute(Void result) {
-                Tabs.getInstance().notifyListeners(Tab.this, Tabs.TabEvents.MENU_UPDATED);
-            }
-        }).execute();
-    }
-
     public void addBookmark() {
-        GeckoAppShell.getHandler().post(new Runnable() {
+        ThreadUtils.postToBackgroundThread(new Runnable() {
+            @Override
             public void run() {
                 String url = getURL();
                 if (url == null)
                     return;
 
-                BrowserDB.addBookmark(mContentResolver, mTitle, url);
+                BrowserDB.addBookmark(getContentResolver(), mTitle, url);
             }
         });
     }
 
     public void removeBookmark() {
-        GeckoAppShell.getHandler().post(new Runnable() {
+        ThreadUtils.postToBackgroundThread(new Runnable() {
+            @Override
             public void run() {
                 String url = getURL();
                 if (url == null)
                     return;
 
-                BrowserDB.removeBookmarksWithURL(mContentResolver, url);
+                BrowserDB.removeBookmarksWithURL(getContentResolver(), url);
             }
         });
     }
@@ -432,38 +470,25 @@ public final class Tab {
         if (!mReaderEnabled)
             return;
 
-        GeckoEvent e = GeckoEvent.createBroadcastEvent("Reader:Add", String.valueOf(getId()));
+        JSONObject json = new JSONObject();
+        try {
+            json.put("tabID", String.valueOf(getId()));
+        } catch (JSONException e) {
+            Log.e(LOGTAG, "JSON error - failing to add to reading list", e);
+            return;
+        }
+
+        GeckoEvent e = GeckoEvent.createBroadcastEvent("Reader:Add", json.toString());
         GeckoAppShell.sendEventToGecko(e);
     }
 
-    public void removeFromReadingList() {
-        if (!mReaderEnabled)
-            return;
-
-        GeckoAppShell.getHandler().post(new Runnable() {
-            public void run() {
-                String url = getURL();
-                if (url == null)
-                    return;
-
-                BrowserDB.removeReadingListItemWithURL(mContentResolver, url);
-
-                GeckoApp.mAppContext.mMainHandler.post(new Runnable() {
-                    public void run() {
-                        GeckoEvent e = GeckoEvent.createBroadcastEvent("Reader:Remove", getURL());
-                        GeckoAppShell.sendEventToGecko(e);
-                    }
-                });
-            }
-        });
-    }
-
-    public void readerMode() {
-        if (!mReaderEnabled)
-            return;
-
-        mEnteringReaderMode = true;
-        GeckoApp.mAppContext.loadUrl(ReaderModeUtils.getAboutReaderForUrl(getURL(), mId, mReadingListItem));
+    public void toggleReaderMode() {
+        if (AboutPages.isAboutReader(mUrl)) {
+            Tabs.getInstance().loadUrl(ReaderModeUtils.getUrlFromAboutReader(mUrl));
+        } else if (mReaderEnabled) {
+            mEnteringReaderMode = true;
+            Tabs.getInstance().loadUrl(ReaderModeUtils.getAboutReaderForUrl(mUrl, mId));
+        }
     }
 
     public boolean isEnteringReaderMode() {
@@ -489,6 +514,53 @@ public final class Tab {
         return true;
     }
 
+    public boolean showBackHistory() {
+        if (!canDoBack())
+            return false;
+        return this.showHistory(Math.max(mHistoryIndex - MAX_HISTORY_LIST_SIZE, 0), mHistoryIndex, mHistoryIndex);
+    }
+
+    public boolean showForwardHistory() {
+        if (!canDoForward())
+            return false;
+        return this.showHistory(mHistoryIndex, Math.min(mHistorySize - 1, mHistoryIndex + MAX_HISTORY_LIST_SIZE), mHistoryIndex);
+    }
+
+    public boolean showAllHistory() {
+        if (!canDoForward() && !canDoBack())
+            return false;
+
+        int min = mHistoryIndex - MAX_HISTORY_LIST_SIZE / 2;
+        int max = mHistoryIndex + MAX_HISTORY_LIST_SIZE / 2;
+        if (min < 0) {
+            max -= min;
+        }
+        if (max > mHistorySize - 1) {
+            min -= max - (mHistorySize - 1);
+            max = mHistorySize - 1;
+        }
+        min = Math.max(min, 0);
+
+        return this.showHistory(min, max, mHistoryIndex);
+    }
+
+    /**
+     * This method will show the history starting on fromIndex until toIndex of the history.
+     */
+    public boolean showHistory(int fromIndex, int toIndex, int selIndex) {
+        JSONObject json = new JSONObject();
+        try {
+            json.put("fromIndex", fromIndex);
+            json.put("toIndex", toIndex);
+            json.put("selIndex", selIndex);
+        } catch (JSONException e) {
+            Log.e(LOGTAG, "JSON error", e);
+        }
+        GeckoEvent e = GeckoEvent.createBroadcastEvent("Session:ShowHistory", json.toString());
+        GeckoAppShell.sendEventToGecko(e);
+        return true;
+    }
+
     public void doStop() {
         GeckoEvent e = GeckoEvent.createBroadcastEvent("Session:Stop", "");
         GeckoAppShell.sendEventToGecko(e);
@@ -513,34 +585,29 @@ public final class Tab {
             final String url = message.getString("url");
             mHistoryIndex++;
             mHistorySize = mHistoryIndex + 1;
-            GeckoAppShell.getHandler().post(new Runnable() {
-                public void run() {
-                    GlobalHistory.getInstance().add(url);
-                }
-            });
         } else if (event.equals("Back")) {
             if (!canDoBack()) {
-                Log.e(LOGTAG, "Received unexpected back notification");
+                Log.w(LOGTAG, "Received unexpected back notification");
                 return;
             }
             mHistoryIndex--;
         } else if (event.equals("Forward")) {
             if (!canDoForward()) {
-                Log.e(LOGTAG, "Received unexpected forward notification");
+                Log.w(LOGTAG, "Received unexpected forward notification");
                 return;
             }
             mHistoryIndex++;
         } else if (event.equals("Goto")) {
             int index = message.getInt("index");
             if (index < 0 || index >= mHistorySize) {
-                Log.e(LOGTAG, "Received unexpected history-goto notification");
+                Log.w(LOGTAG, "Received unexpected history-goto notification");
                 return;
             }
             mHistoryIndex = index;
         } else if (event.equals("Purge")) {
             int numEntries = message.getInt("numEntries");
             if (numEntries > mHistorySize) {
-                Log.e(LOGTAG, "Received unexpectedly large number of history entries to purge");
+                Log.w(LOGTAG, "Received unexpectedly large number of history entries to purge");
                 mHistoryIndex = -1;
                 mHistorySize = 0;
                 return;
@@ -551,44 +618,101 @@ public final class Tab {
 
             // If we weren't at the last history entry, mHistoryIndex may have become too small
             if (mHistoryIndex < -1)
-                 mHistoryIndex = -1;
+                mHistoryIndex = -1;
         }
     }
 
     void handleLocationChange(JSONObject message) throws JSONException {
         final String uri = message.getString("uri");
-        mEnteringReaderMode = ReaderModeUtils.isEnteringReaderMode(mUrl, uri);
-        updateURL(uri);
+        final String oldUrl = getURL();
+        final boolean sameDocument = message.getBoolean("sameDocument");
+        mEnteringReaderMode = ReaderModeUtils.isEnteringReaderMode(oldUrl, uri);
 
-        setDocumentURI(message.getString("documentURI"));
-        if (message.getBoolean("sameDocument")) {
+        if (!TextUtils.equals(oldUrl, uri)) {
+            updateURL(uri);
+            updateBookmark();
+            if (!sameDocument) {
+                // We can unconditionally clear the favicon and title here: we
+                // already filtered both cases in which this was a (pseudo-)
+                // spurious location change, so we're definitely loading a new
+                // page.
+                clearFavicon();
+                updateTitle(null);
+            }
+        }
+
+        if (sameDocument) {
             // We can get a location change event for the same document with an anchor tag
+            // Notify listeners so that buttons like back or forward will update themselves
+            Tabs.getInstance().notifyListeners(this, Tabs.TabEvents.LOCATION_CHANGE, oldUrl);
             return;
         }
 
         setContentType(message.getString("contentType"));
-        clearFavicon();
-        updateTitle(null);
+        updateUserSearch(message.getString("userSearch"));
+        mBaseDomain = message.optString("baseDomain");
+
+        setHasFeeds(false);
+        setHasOpenSearch(false);
         updateIdentityData(null);
         setReaderEnabled(false);
         setZoomConstraints(new ZoomConstraints(true));
         setHasTouchListeners(false);
-        setCheckerboardColor(Color.WHITE);
+        setBackgroundColor(DEFAULT_BACKGROUND_COLOR);
+        setErrorType(ErrorType.NONE);
+        setLoadProgressIfLoading(LOAD_PROGRESS_LOCATION_CHANGE);
 
-        GeckoApp.mAppContext.mMainHandler.post(new Runnable() {
-            public void run() {
-                Tabs.getInstance().notifyListeners(Tab.this, Tabs.TabEvents.LOCATION_CHANGE, uri);
-            }
-        });
+        Tabs.getInstance().notifyListeners(this, Tabs.TabEvents.LOCATION_CHANGE, oldUrl);
     }
 
-    private void saveThumbnailToDB() {
+    private static boolean shouldShowProgress(final String url) {
+        return AboutPages.isAboutHome(url) ||
+               AboutPages.isAboutReader(url) ||
+               AboutPages.isAboutPrivateBrowsing(url);
+    }
+
+    void handleDocumentStart(boolean showProgress, String url) {
+        setLoadProgress(LOAD_PROGRESS_START);
+        setState(showProgress ? STATE_LOADING : STATE_SUCCESS);
+        updateIdentityData(null);
+        setReaderEnabled(false);
+    }
+
+    void handleDocumentStop(boolean success) {
+        setState(success ? STATE_SUCCESS : STATE_ERROR);
+
+        final String oldURL = getURL();
+        final Tab tab = this;
+        tab.setLoadProgress(LOAD_PROGRESS_STOP);
+        ThreadUtils.getBackgroundHandler().postDelayed(new Runnable() {
+            @Override
+            public void run() {
+                // tab.getURL() may return null
+                if (!TextUtils.equals(oldURL, getURL()))
+                    return;
+
+                ThumbnailHelper.getInstance().getAndProcessThumbnailFor(tab);
+            }
+        }, 500);
+    }
+
+    void handleContentLoaded() {
+        setLoadProgressIfLoading(LOAD_PROGRESS_LOADED);
+    }
+
+    protected void saveThumbnailToDB() {
+        final BitmapDrawable thumbnail = mThumbnail;
+        if (thumbnail == null) {
+            return;
+        }
+
         try {
             String url = getURL();
-            if (url == null)
+            if (url == null) {
                 return;
+            }
 
-            BrowserDB.updateThumbnailForUrl(mContentResolver, url, mThumbnail);
+            BrowserDB.updateThumbnailForUrl(getContentResolver(), url, thumbnail);
         } catch (Exception e) {
             // ignore
         }
@@ -630,18 +754,18 @@ public final class Tab {
         }
     }
 
-    public int getCheckerboardColor() {
-        return mCheckerboardColor;
+    public int getBackgroundColor() {
+        return mBackgroundColor;
     }
 
-    /** Sets a new color for the checkerboard. */
-    public void setCheckerboardColor(int color) {
-        mCheckerboardColor = color;
+    /** Sets a new color for the background. */
+    public void setBackgroundColor(int color) {
+        mBackgroundColor = color;
     }
 
-    /** Parses and sets a new color for the checkerboard. */
-    public void setCheckerboardColor(String newColor) {
-        setCheckerboardColor(parseColorFromGecko(newColor));
+    /** Parses and sets a new color for the background. */
+    public void setBackgroundColor(String newColor) {
+        setBackgroundColor(parseColorFromGecko(newColor));
     }
 
     // Parses a color from an RGB triple of the form "rgb([0-9]+, [0-9]+, [0-9]+)". If the color
@@ -669,4 +793,42 @@ public final class Tab {
     public boolean getDesktopMode() {
         return mDesktopMode;
     }
+
+    public boolean isPrivate() {
+        return false;
+    }
+
+    /**
+     * Sets the tab load progress to the given percentage.
+     *
+     * @param progressPercentage Percentage to set progress to (0-100)
+     */
+    void setLoadProgress(int progressPercentage) {
+        mLoadProgress = progressPercentage;
+    }
+
+    /**
+     * Sets the tab load progress to the given percentage only if the tab is
+     * currently loading.
+     *
+     * about:neterror can trigger a STOP before other page load events (bug
+     * 976426), so any post-START events should make sure the page is loading
+     * before updating progress.
+     *
+     * @param progressPercentage Percentage to set progress to (0-100)
+     */
+    void setLoadProgressIfLoading(int progressPercentage) {
+        if (getState() == STATE_LOADING) {
+            setLoadProgress(progressPercentage);
+        }
+    }
+
+    /**
+     * Gets the tab load progress percentage.
+     *
+     * @return Current progress percentage
+     */
+    public int getLoadProgress() {
+        return mLoadProgress;
+    }
 }
