@@ -16,15 +16,11 @@
 #include <unistd.h>
 #elif defined(XP_WIN)
 #include <windows.h>
-#elif defined(XP_OS2)
-#define INCL_DOSERRORS
-#include <os2.h>
 #else
 // XXX add necessary include file for ftruncate (or equivalent)
 #endif
 
 #include "prthread.h"
-#include "prbit.h"
 
 #include "private/pprio.h"
 
@@ -353,11 +349,6 @@ nsDiskCache::Truncate(PRFileDesc *  fd, uint32_t  newEOF)
         return NS_ERROR_FAILURE;
     }
 
-#elif defined(XP_OS2)
-    if (DosSetFileSize((HFILE) PR_FileDesc2NativeHandle(fd), newEOF) != NO_ERROR) {
-        NS_ERROR("DosSetFileSize failed");
-        return NS_ERROR_FAILURE;
-    }
 #else
     // add implementations for other platforms here
 #endif
@@ -369,43 +360,16 @@ nsDiskCache::Truncate(PRFileDesc *  fd, uint32_t  newEOF)
  *  nsDiskCacheDevice
  *****************************************************************************/
 
-class NetworkDiskCacheReporter MOZ_FINAL : public MemoryUniReporter
-{
-public:
-    NetworkDiskCacheReporter(nsDiskCacheDevice* aDevice)
-      : MemoryUniReporter(
-            "explicit/network/disk-cache",
-            KIND_HEAP,
-            UNITS_BYTES,
-            "Memory used by the network disk cache.")
-      , mDevice(aDevice)
-    {}
-
-private:
-    int64_t Amount()
-    {
-        nsCacheServiceAutoLock
-            lock(LOCK_TELEM(NSCACHESERVICE_DISKDEVICEHEAPSIZE));
-        return mDevice->SizeOfIncludingThis(MallocSizeOf);
-    }
-
-    nsDiskCacheDevice* mDevice;
-};
-
 nsDiskCacheDevice::nsDiskCacheDevice()
     : mCacheCapacity(0)
     , mMaxEntrySize(-1) // -1 means "no limit"
     , mInitialized(false)
     , mClearingDiskCache(false)
-    , mReporter(nullptr)
 {
-    mReporter = new NetworkDiskCacheReporter(this);
-    NS_RegisterMemoryReporter(mReporter);
 }
 
 nsDiskCacheDevice::~nsDiskCacheDevice()
 {
-    NS_UnregisterMemoryReporter(mReporter);
     Shutdown();
 }
 
