@@ -7,6 +7,12 @@
 // HttpLog.h should generally be included first
 #include "HttpLog.h"
 
+// Log on level :5, instead of default :4.
+#undef LOG
+#define LOG(args) LOG5(args)
+#undef LOG_ENABLED
+#define LOG_ENABLED() LOG5_ENABLED()
+
 #include "mozilla/Telemetry.h"
 #include "mozilla/Preferences.h"
 #include "nsHttp.h"
@@ -188,22 +194,6 @@ SpdySession31::LogIO(SpdySession31 *self, SpdyStream31 *stream, const char *labe
   }
 }
 
-typedef nsresult  (*Control_FX) (SpdySession31 *self);
-static Control_FX sControlFunctions[] =
-{
-    nullptr,
-    SpdySession31::HandleSynStream,
-    SpdySession31::HandleSynReply,
-    SpdySession31::HandleRstStream,
-    SpdySession31::HandleSettings,
-    SpdySession31::HandleNoop,
-    SpdySession31::HandlePing,
-    SpdySession31::HandleGoAway,
-    SpdySession31::HandleHeaders,
-    SpdySession31::HandleWindowUpdate,
-    SpdySession31::HandleCredential
-};
-
 bool
 SpdySession31::RoomForMoreConcurrent()
 {
@@ -227,7 +217,7 @@ SpdySession31::IdleTime()
   return PR_IntervalNow() - mLastDataReadEpoch;
 }
 
-void
+uint32_t
 SpdySession31::ReadTimeoutTick(PRIntervalTime now)
 {
   MOZ_ASSERT(PR_GetCurrentThread() == gSocketThread);
@@ -237,13 +227,15 @@ SpdySession31::ReadTimeoutTick(PRIntervalTime now)
        this, PR_IntervalToSeconds(now - mLastReadEpoch)));
 
   if (!mPingThreshold)
-    return;
+    return UINT32_MAX;
 
   if ((now - mLastReadEpoch) < mPingThreshold) {
     // recent activity means ping is not an issue
     if (mPingSentEpoch)
       mPingSentEpoch = 0;
-    return;
+
+    return PR_IntervalToSeconds(mPingThreshold) -
+      PR_IntervalToSeconds(now - mLastReadEpoch);
   }
 
   if (mPingSentEpoch) {
@@ -253,8 +245,9 @@ SpdySession31::ReadTimeoutTick(PRIntervalTime now)
            this));
       mPingSentEpoch = 0;
       Close(NS_ERROR_NET_TIMEOUT);
+      return UINT32_MAX;
     }
-    return;
+    return 1; // run the tick aggressively while ping is outstanding
   }
 
   LOG(("SpdySession31::ReadTimeoutTick %p generating ping 0x%X\n",
@@ -263,7 +256,7 @@ SpdySession31::ReadTimeoutTick(PRIntervalTime now)
   if (mNextPingID == 0xffffffff) {
     LOG(("SpdySession31::ReadTimeoutTick %p cannot form ping - ids exhausted\n",
          this));
-    return;
+    return UINT32_MAX;
   }
 
   mPingSentEpoch = PR_IntervalNow();
@@ -307,6 +300,7 @@ SpdySession31::ReadTimeoutTick(PRIntervalTime now)
          "ping ids exhausted marking goaway\n", this));
     mShouldGoAway = true;
   }
+  return 1; // run the tick aggressively while ping is outstanding
 }
 
 uint32_t
@@ -762,7 +756,7 @@ SpdySession31::GenerateSettings()
   // 2nd entry is bytes 20 to 27
   // 3rd entry is bytes 28 to 35
 
-  if (!gHttpHandler->AllowSpdyPush()) {
+  if (!gHttpHandler->AllowPush()) {
     // announcing that we accept 0 incoming streams is done to
     // disable server push
     packet[15 + 8 * numberOfEntries] = SETTINGS_TYPE_MAX_CONCURRENT;
@@ -819,7 +813,7 @@ SpdySession31::GenerateSettings()
 
   LOG3(("Session Window increase at start of session %p %u\n",
         this, PR_ntohl(sessionWindowBump)));
-  LogIO(this, nullptr, "Session Window Bump ", packet, 12);
+  LogIO(this, nullptr, "Session Window Bump ", packet, 16);
 
 generateSettings_complete:
   FlushOutputQueue();
@@ -1036,7 +1030,7 @@ SpdySession31::HandleSynStream(SpdySession31 *self)
     LOG3(("SpdySession31::HandleSynStream %p associated ID of 0 failed.\n", self));
     self->GenerateRstStream(RST_PROTOCOL_ERROR, streamID);
 
-  } else if (!gHttpHandler->AllowSpdyPush()) {
+  } else if (!gHttpHandler->AllowPush()) {
     // MAX_CONCURRENT_STREAMS of 0 in settings should have disabled push,
     // but some servers are buggy about that.. or the config could have
     // been updated after the settings frame was sent. In both cases just
@@ -1877,6 +1871,22 @@ SpdySession31::WriteSegments(nsAHttpSegmentWriter *writer,
                              uint32_t count,
                              uint32_t *countWritten)
 {
+  typedef nsresult  (*Control_FX) (SpdySession31 *self);
+  static const Control_FX sControlFunctions[] =
+  {
+      nullptr,
+      SpdySession31::HandleSynStream,
+      SpdySession31::HandleSynReply,
+      SpdySession31::HandleRstStream,
+      SpdySession31::HandleSettings,
+      SpdySession31::HandleNoop,
+      SpdySession31::HandlePing,
+      SpdySession31::HandleGoAway,
+      SpdySession31::HandleHeaders,
+      SpdySession31::HandleWindowUpdate,
+      SpdySession31::HandleCredential
+  };
+
   MOZ_ASSERT(PR_GetCurrentThread() == gSocketThread);
 
   nsresult rv;
@@ -2088,9 +2098,10 @@ SpdySession31::WriteSegments(nsAHttpSegmentWriter *writer,
 
     mLastDataReadEpoch = mLastReadEpoch;
 
-    if (rv == NS_BASE_STREAM_CLOSED) {
+    if (SoftStreamError(rv)) {
       // This will happen when the transaction figures out it is EOF, generally
-      // due to a content-length match being made
+      // due to a content-length match being made. Return OK from this function
+      // otherwise the whole session would be torn down.
       SpdyStream31 *stream = mInputFrameDataStream;
 
       // if we were doing PROCESSING_COMPLETE_HEADERS need to pop the state
@@ -2101,9 +2112,9 @@ SpdySession31::WriteSegments(nsAHttpSegmentWriter *writer,
         ResetDownstreamState();
       LOG3(("SpdySession31::WriteSegments session=%p stream=%p 0x%X "
             "needscleanup=%p. cleanup stream based on "
-            "stream->writeSegments returning BASE_STREAM_CLOSED\n",
+            "stream->writeSegments returning code %X\n",
             this, stream, stream ? stream->StreamID() : 0,
-            mNeedsCleanup));
+            mNeedsCleanup, rv));
       CleanupStream(stream, NS_OK, RST_CANCEL);
       MOZ_ASSERT(!mNeedsCleanup, "double cleanup out of data frame");
       mNeedsCleanup = nullptr;                     /* just in case */
@@ -2585,6 +2596,18 @@ SpdySession31::ConnectPushedStream(SpdyStream31 *stream)
   ForceRecv();
 }
 
+nsresult
+SpdySession31::BufferOutput(const char *buf,
+                            uint32_t count,
+                            uint32_t *countRead)
+{
+  nsAHttpSegmentReader *old = mSegmentReader;
+  mSegmentReader = nullptr;
+  nsresult rv = OnReadSegment(buf, count, countRead);
+  mSegmentReader = old;
+  return rv;
+}
+
 //-----------------------------------------------------------------------------
 // Modified methods of nsAHttpConnection
 //-----------------------------------------------------------------------------
@@ -2705,6 +2728,11 @@ SpdySession31::Caps()
   return 0;
 }
 
+void
+SpdySession31::SetDNSWasRefreshed()
+{
+}
+
 uint64_t
 SpdySession31::Available()
 {
