@@ -269,6 +269,9 @@ class TestRecursiveMakeBackend(BackendTester):
                 'CSRCS += bar.c',
                 'CSRCS += foo.c',
             ],
+            'DISABLE_STL_WRAPPING': [
+                'DISABLE_STL_WRAPPING := 1',
+            ],
             'EXTRA_COMPONENTS': [
                 'EXTRA_COMPONENTS += bar.js',
                 'EXTRA_COMPONENTS += foo.js',
@@ -321,6 +324,43 @@ class TestRecursiveMakeBackend(BackendTester):
             'VISIBILITY_FLAGS': [
                 'VISIBILITY_FLAGS :=',
             ],
+            'DELAYLOAD_LDFLAGS': [
+                'DELAYLOAD_LDFLAGS += -DELAYLOAD:foo.dll',
+                'DELAYLOAD_LDFLAGS += -DELAYLOAD:bar.dll',
+            ],
+            'USE_DELAYIMP': [
+                'USE_DELAYIMP := 1',
+            ],
+            'RCFILE': [
+                'RCFILE := foo.rc',
+            ],
+            'RESFILE': [
+                'RESFILE := bar.res',
+            ],
+            'RCINCLUDE': [
+                'RCINCLUDE := bar.rc',
+            ],
+            'DEFFILE': [
+                'DEFFILE := baz.def',
+            ],
+            'USE_STATIC_LIBS': [
+                'USE_STATIC_LIBS := 1',
+            ],
+            'MOZBUILD_CFLAGS': [
+                'MOZBUILD_CFLAGS += -fno-exceptions',
+                'MOZBUILD_CFLAGS += -w',
+            ],
+            'MOZBUILD_CXXFLAGS': [
+                'MOZBUILD_CXXFLAGS += -fcxx-exceptions',
+                'MOZBUILD_CXXFLAGS += -include foo.h',
+            ],
+            'MOZBUILD_LDFLAGS': [
+                'MOZBUILD_LDFLAGS += -framework Foo',
+                'MOZBUILD_LDFLAGS += -x',
+            ],
+            'WIN32_EXE_LDFLAGS': [
+                'WIN32_EXE_LDFLAGS += -subsystem:console',
+            ],
         }
 
         for var, val in expected.items():
@@ -340,6 +380,22 @@ class TestRecursiveMakeBackend(BackendTester):
         self.assertIn('mozilla/mozilla1.h', m)
         self.assertIn('mozilla/dom/dom2.h', m)
 
+    def test_resources(self):
+        """Ensure RESOURCE_FILES is handled properly."""
+        env = self._consume('resources', RecursiveMakeBackend)
+
+        # RESOURCE_FILES should appear in the dist_bin install manifest.
+        m = InstallManifest(path=os.path.join(env.topobjdir,
+            '_build_manifests', 'install', 'dist_bin'))
+        self.assertEqual(len(m), 10)
+        self.assertIn('res/foo.res', m)
+        self.assertIn('res/fonts/font1.ttf', m)
+        self.assertIn('res/fonts/desktop/desktop2.ttf', m)
+
+        self.assertIn('res/bar.res', m)
+        self.assertIn('res/tests/test.manifest', m)
+        self.assertIn('res/tests/extra.manifest', m)
+
     def test_test_manifests_files_written(self):
         """Ensure test manifests get turned into files."""
         env = self._consume('test-manifests-written', RecursiveMakeBackend)
@@ -474,7 +530,7 @@ class TestRecursiveMakeBackend(BackendTester):
         var = 'DEFINES'
         defines = [val for val in lines if val.startswith(var)]
 
-        expected = ['DEFINES += -DFOO -DBAZ=\'"ab\'\\\'\'cd"\' -DBAR=7 -DVALUE=\'xyz\'']
+        expected = ['DEFINES += -DFOO -DBAZ=\'"ab\'\\\'\'cd"\' -UQUX -DBAR=7 -DVALUE=\'xyz\'']
         self.assertEqual(defines, expected)
 
     def test_local_includes(self):
@@ -581,6 +637,48 @@ class TestRecursiveMakeBackend(BackendTester):
 
         self.assertIn('JAR_MANIFEST := %s/jar.mn' % env.topsrcdir, lines)
 
+    def test_test_manifests_duplicate_support_files(self):
+        """Ensure duplicate support-files in test manifests work."""
+        env = self._consume('test-manifests-duplicate-support-files',
+            RecursiveMakeBackend)
+
+        p = os.path.join(env.topobjdir, '_build_manifests', 'install', 'tests')
+        m = InstallManifest(p)
+        self.assertIn('testing/mochitest/tests/support-file.txt', m)
+
+    def test_android_eclipse(self):
+        env = self._consume('android_eclipse', RecursiveMakeBackend)
+
+        with open(mozpath.join(env.topobjdir, 'backend.mk'), 'rb') as fh:
+            lines = fh.readlines()
+
+        lines = [line.rstrip() for line in lines]
+
+        # Dependencies first.
+        self.assertIn('ANDROID_ECLIPSE_PROJECT_main1: target1 target2', lines)
+        self.assertIn('ANDROID_ECLIPSE_PROJECT_main4: target3 target4', lines)
+
+        command_template = '\t$(call py_action,process_install_manifest,' + \
+                           '--no-remove --no-remove-all-directory-symlinks ' + \
+                           '--no-remove-empty-directories %s %s.manifest)'
+        # Commands second.
+        for project_name in ['main1', 'main2', 'library1', 'library2']:
+            stem = '%s/android_eclipse/%s' % (env.topobjdir, project_name)
+            self.assertIn(command_template % (stem, stem), lines)
+
+        # Projects declared in subdirectories.
+        with open(mozpath.join(env.topobjdir, 'subdir', 'backend.mk'), 'rb') as fh:
+            lines = fh.readlines()
+
+        lines = [line.rstrip() for line in lines]
+
+        self.assertIn('ANDROID_ECLIPSE_PROJECT_submain: subtarget1 subtarget2', lines)
+
+        for project_name in ['submain', 'sublibrary']:
+            # Destination and install manifest are relative to topobjdir.
+            stem = '%s/android_eclipse/%s' % (env.topobjdir, project_name)
+            self.assertIn(command_template % (stem, stem), lines)
+
 
 if __name__ == '__main__':
     main()
