@@ -3,10 +3,13 @@
  * file, You can obtain one at http://mozilla.org/MPL/2.0/. */
 /* $Id: nsPKCS12Blob.cpp,v 1.49 2007/09/05 07:13:46 jwalden%mit.edu Exp $ */
 
+#include "nsPKCS12Blob.h"
+
+#include "insanity/pkixtypes.h"
+
 #include "prmem.h"
 #include "prprf.h"
 
-#include "nsISupportsArray.h"
 #include "nsIFile.h"
 #include "nsNetUtil.h"
 #include "nsIDirectoryService.h"
@@ -14,7 +17,6 @@
 
 #include "nsNSSComponent.h"
 #include "nsNSSHelper.h"
-#include "nsPKCS12Blob.h"
 #include "nsString.h"
 #include "nsReadableUtils.h"
 #include "nsXPIDLString.h"
@@ -26,17 +28,14 @@
 #include "nsICertificateDialogs.h"
 #include "nsNSSShutDown.h"
 #include "nsCRT.h"
-#include "pk11func.h"
+
 #include "secerr.h"
 
 #ifdef PR_LOGGING
 extern PRLogModuleInfo* gPIPNSSLog;
 #endif
 
-#include "nsNSSCleaner.h"
-NSSCleanupAutoPtrClass(CERTCertificate, CERT_DestroyCertificate)
-
-static NS_DEFINE_CID(kNSSComponentCID, NS_NSSCOMPONENT_CID);
+using namespace mozilla;
 
 #define PIP_PKCS12_TMPFILENAME   NS_LITERAL_CSTRING(".pip_p12tmp")
 #define PIP_PKCS12_BUFFER_SIZE   2048
@@ -51,7 +50,6 @@ static NS_DEFINE_CID(kNSSComponentCID, NS_NSSCOMPONENT_CID);
 // constructor
 nsPKCS12Blob::nsPKCS12Blob():mCertArray(0),
                              mTmpFile(nullptr),
-                             mTmpFilePath(nullptr),
                              mDigest(nullptr),
                              mDigestIterator(nullptr),
                              mTokenSet(false)
@@ -140,7 +138,7 @@ nsPKCS12Blob::ImportFromFileHelper(nsIFile *file,
                                    nsPKCS12Blob::RetryReason &aWantRetry)
 {
   nsNSSShutDownPreventionLock locker;
-  nsresult rv;
+  nsresult rv = NS_OK;
   SECStatus srv = SECSuccess;
   SEC_PKCS12DecoderContext *dcx = nullptr;
   SECItem unicodePw;
@@ -211,7 +209,7 @@ finish:
   // for every error possible.
   if (srv != SECSuccess) {
     if (SEC_ERROR_BAD_PASSWORD == PORT_GetError()) {
-      if (unicodePw.len == sizeof(PRUnichar))
+      if (unicodePw.len == sizeof(char16_t))
       {
         // no password chars available, 
         // unicodeToItem allocated space for the trailing zero character only.
@@ -239,46 +237,6 @@ finish:
   return NS_OK;
 }
 
-#if 0
-// nsPKCS12Blob::LoadCerts
-//
-// Given an array of certificate nicknames, load the corresponding
-// certificates into a local array.
-nsresult
-nsPKCS12Blob::LoadCerts(const PRUnichar **certNames, int numCerts)
-{
-  nsresult rv;
-  char namecpy[256];
-  /* Create the local array if needed */
-  if (!mCertArray) {
-    rv = NS_NewISupportsArray(getter_AddRefs(mCertArray));
-    if (NS_FAILED(rv)) {
-      if (!handleError())
-        return NS_ERROR_OUT_OF_MEMORY;
-    }
-  }
-  /* Add the certs */
-  for (int i=0; i<numCerts; i++) {
-    strcpy(namecpy, NS_ConvertUTF16toUTF8(certNames[i]));
-    CERTCertificate *nssCert = PK11_FindCertFromNickname(namecpy, nullptr);
-    if (!nssCert) {
-      if (!handleError())
-        return NS_ERROR_FAILURE;
-      else continue; /* user may request to keep going */
-    }
-    nsCOMPtr<nsIX509Cert> cert = nsNSSCertificate::Create(nssCert);
-    CERT_DestroyCertificate(nssCert);
-    if (!cert) {
-      if (!handleError())
-        return NS_ERROR_OUT_OF_MEMORY;
-    } else {
-      mCertArray->AppendElement(cert);
-    }
-  }
-  return NS_OK;
-}
-#endif
-
 static bool
 isExtractable(SECKEYPrivateKey *privKey)
 {
@@ -346,23 +304,10 @@ nsPKCS12Blob::ExportToFile(nsIFile *file,
   // add password integrity
   srv = SEC_PKCS12AddPasswordIntegrity(ecx, &unicodePw, SEC_OID_SHA1);
   if (srv) goto finish;
-#if 0
-  // count the number of certs to export
-  nrv = mCertArray->Count(&numCerts);
-  if (NS_FAILED(nrv)) goto finish;
-  // loop over the certs
   for (i=0; i<numCerts; i++) {
-    nsCOMPtr<nsIX509Cert> cert;
-    nrv = mCertArray->GetElementAt(i, getter_AddRefs(cert));
-    if (NS_FAILED(nrv)) goto finish;
-#endif
-  for (i=0; i<numCerts; i++) {
-//    nsNSSCertificate *cert = reinterpret_cast<nsNSSCertificate *>(certs[i]);
     nsNSSCertificate *cert = (nsNSSCertificate *)certs[i];
     // get it as a CERTCertificate XXX
-    CERTCertificate *nssCert = NULL;
-    CERTCertificateCleaner nssCertCleaner(nssCert);
-    nssCert = cert->GetCert();
+    insanity::pkix::ScopedCERTCertificate nssCert(cert->GetCert());
     if (!nssCert) {
       rv = NS_ERROR_FAILURE;
       goto finish;
@@ -375,7 +320,7 @@ nsPKCS12Blob::ExportToFile(nsIFile *file,
     if (nssCert->slot && !PK11_IsInternal(nssCert->slot)) {
       // we aren't the internal token, see if the key is extractable.
       SECKEYPrivateKey *privKey=PK11_FindKeyByDERCert(nssCert->slot,
-                                                      nssCert, this);
+                                                      nssCert.get(), this);
 
       if (privKey) {
         bool privKeyIsExtractable = isExtractable(privKey);
@@ -407,7 +352,7 @@ nsPKCS12Blob::ExportToFile(nsIFile *file,
       goto finish;
     }
     // add the cert and key to the blob
-    srv = SEC_PKCS12AddCertAndKey(ecx, certSafe, nullptr, nssCert,
+    srv = SEC_PKCS12AddCertAndKey(ecx, certSafe, nullptr, nssCert.get(),
                                   CERT_GetDefaultCertDB(), // XXX
                                   keySafe, nullptr, true, &unicodePw,
                       SEC_OID_PKCS12_V2_PBE_WITH_SHA1_AND_3KEY_TRIPLE_DES_CBC);
@@ -467,11 +412,11 @@ finish:
 // a buffer of octets.  Must handle byte order correctly.
 // TODO: Is there a mozilla way to do this?  In the string lib?
 void
-nsPKCS12Blob::unicodeToItem(const PRUnichar *uni, SECItem *item)
+nsPKCS12Blob::unicodeToItem(const char16_t *uni, SECItem *item)
 {
   int len = 0;
   while (uni[len++] != 0);
-  SECITEM_AllocItem(nullptr, item, sizeof(PRUnichar) * len);
+  SECITEM_AllocItem(nullptr, item, sizeof(char16_t) * len);
 #ifdef IS_LITTLE_ENDIAN
   int i = 0;
   for (i=0; i<len; i++) {
@@ -686,6 +631,8 @@ nsPKCS12Blob::digest_write(void *arg, unsigned char *buf, unsigned long len)
 SECItem *
 nsPKCS12Blob::nickname_collision(SECItem *oldNick, PRBool *cancel, void *wincx)
 {
+  static NS_DEFINE_CID(kNSSComponentCID, NS_NSSCOMPONENT_CID);
+
   nsNSSShutDownPreventionLock locker;
   *cancel = false;
   nsresult rv;
@@ -738,7 +685,7 @@ nsPKCS12Blob::nickname_collision(SECItem *oldNick, PRBool *cancel, void *wincx)
     return nullptr;
 
   newNick->type = siAsciiString;
-  newNick->data = (unsigned char*) nsCRT::strdup(nickname.get());
+  newNick->data = (unsigned char*) strdup(nickname.get());
   newNick->len  = strlen((char*)newNick->data);
   return newNick;
 }
@@ -773,6 +720,8 @@ pip_ucs2_ascii_conversion_fn(PRBool toUnicode,
 void
 nsPKCS12Blob::handleError(int myerr)
 {
+  static NS_DEFINE_CID(kNSSComponentCID, NS_NSSCOMPONENT_CID);
+
   if (!NS_IsMainThread()) {
     NS_ERROR("nsPKCS12Blob::handleError called off the mai nthread.");
     return;
@@ -802,15 +751,6 @@ nsPKCS12Blob::handleError(int myerr)
       /* ask to keep going?  what happens if one collision but others ok? */
       // The following errors cannot be "handled", notify the user (via an alert)
       // that the operation failed.
-#if 0
-      // XXX a boy can dream...
-      //     but the PKCS12 lib never throws this error
-      //     but then again, how would it?  anyway, convey the info below
-    case SEC_ERROR_PKCS12_PRIVACY_PASSWORD_INCORRECT:
-      msgID = "PKCS12PasswordInvalid";
-      break;
-#endif
-
     case SEC_ERROR_BAD_PASSWORD: msgID = "PK11BadPassword"; break;
 
     case SEC_ERROR_BAD_DER:
@@ -832,4 +772,3 @@ nsPKCS12Blob::handleError(int myerr)
   if (NS_SUCCEEDED(rv))
     (void) nssComponent->ShowAlertFromStringBundle(msgID);
 }
-
