@@ -1,7 +1,7 @@
 import os
 from setuptools import setup, find_packages
 
-version = '0.5.10'
+version = '0.7.6'
 
 # get documentation from the README
 try:
@@ -11,9 +11,8 @@ except (OSError, IOError):
     description = ''
 
 # dependencies
-deps = ['manifestdestiny', 'mozhttpd >= 0.3',
-        'mozprocess >= 0.6', 'mozrunner >= 5.11',
-        'mozdevice >= 0.12']
+with open('requirements.txt') as f:
+    deps = f.read().splitlines()
 
 setup(name='marionette_client',
       version=version,
