@@ -2,15 +2,20 @@
 # License, v. 2.0. If a copy of the MPL was not distributed with this
 # file, You can obtain one at http://mozilla.org/MPL/2.0/.
 
-import sys
 from setuptools import setup, find_packages
+import sys
 
-version = '0.2.40'
+version = '0.5'
 
-deps = ['pulsebuildmonitor >= 0.62', 'MozillaPulse == 0.61', 
-        'mozinfo == 0.3.1', 'mozprofile == 0.1t',
-        'mozprocess == 0.1a', 'mozrunner == 3.0a', 'mozregression == 0.3',
-        'mozautolog >= 0.2.1', 'mozautoeslib == 0.1.1']
+deps = ['httplib2 >= 0.7.3',
+        'mozfile >= 1.1',
+        'mozhttpd >= 0.7',
+        'mozinfo >= 0.7',
+        'mozinstall >= 1.9',
+        'mozprocess >= 0.18',
+        'mozprofile >= 0.21',
+        'mozrunner >= 5.35',
+       ]
 
 # we only support python 2.6+ right now
 assert sys.version_info[0] == 2
@@ -23,13 +28,10 @@ setup(name='tps',
 """,
       classifiers=[], # Get strings from http://pypi.python.org/pypi?%3Aaction=list_classifiers
       keywords='',
-      author='Jonathan Griffin',
-      author_email='jgriffin@mozilla.com',
-      url='http://hg.mozilla.org/services/services-central',
-      license='MPL',
-      dependency_links = [
-         "http://people.mozilla.org/~jgriffin/packages/"
-      ],
+      author='Mozilla Automation and Tools team',
+      author_email='tools@lists.mozilla.org',
+      url='https://developer.mozilla.org/en-US/docs/TPS',
+      license='MPL 2.0',
       packages=find_packages(exclude=['ez_setup', 'examples', 'tests']),
       include_package_data=True,
       zip_safe=False,
