@@ -13,7 +13,6 @@
 #include "nsString.h"
 #include "nsReadableUtils.h"
 #include "nsCRT.h"
-#include "prmem.h"
 #include "nsPrintfCString.h"
 #include "nsIDateTimeFormat.h"
 #include "nsDateTimeFormatCID.h"
@@ -85,11 +84,15 @@ nsFileResult::nsFileResult(const nsAString& aSearchString,
       nextFile->GetLeafName(fileName);
       if (StringBeginsWith(fileName, prefix)) {
         fileName.Insert(parent, 0);
-        mValues.AppendElement(fileName);
         if (mSearchResult == RESULT_NOMATCH && fileName.Equals(mSearchString))
           mSearchResult = RESULT_IGNORED;
         else
           mSearchResult = RESULT_SUCCESS;
+        bool isDirectory = false;
+        nextFile->IsDirectory(&isDirectory);
+        if (isDirectory)
+          fileName.Append('/');
+        mValues.AppendElement(fileName);
       }
     }
     mValues.Sort();
@@ -139,6 +142,8 @@ NS_IMETHODIMP nsFileResult::GetTypeAheadResult(bool *aTypeAheadResult)
 NS_IMETHODIMP nsFileResult::GetValueAt(int32_t index, nsAString & aValue)
 {
   aValue = mValues[index];
+  if (aValue.Last() == '/')
+    aValue.Truncate(aValue.Length() - 1);
   return NS_OK;
 }
 
@@ -155,7 +160,10 @@ NS_IMETHODIMP nsFileResult::GetCommentAt(int32_t index, nsAString & aComment)
 
 NS_IMETHODIMP nsFileResult::GetStyleAt(int32_t index, nsAString & aStyle)
 {
-  aStyle.Truncate();
+  if (mValues[index].Last() == '/')
+    aStyle.AssignLiteral("directory");
+  else
+    aStyle.AssignLiteral("file");
   return NS_OK;
 }
 
@@ -215,25 +223,23 @@ protected:
   virtual ~nsFileView();
   
   void FilterFiles();
-  void ReverseArray(nsISupportsArray* aArray);
-  void SortArray(nsISupportsArray* aArray);
+  void ReverseArray(nsTArray<nsCOMPtr<nsIFile> >& aArray);
+  void SortArray(nsTArray<nsCOMPtr<nsIFile> >& aArray);
   void SortInternal();
 
-  nsCOMPtr<nsISupportsArray> mFileList;
-  nsCOMPtr<nsISupportsArray> mDirList;
-  nsCOMPtr<nsISupportsArray> mFilteredFiles;
+  nsTArray<nsCOMPtr<nsIFile> > mFileList;
+  nsTArray<nsCOMPtr<nsIFile> > mDirList;
+  nsTArray<nsCOMPtr<nsIFile> > mFilteredFiles;
 
   nsCOMPtr<nsIFile> mDirectoryPath;
   nsCOMPtr<nsITreeBoxObject> mTree;
   nsCOMPtr<nsITreeSelection> mSelection;
-  nsCOMPtr<nsIAtom> mDirectoryAtom;
-  nsCOMPtr<nsIAtom> mFileAtom;
   nsCOMPtr<nsIDateTimeFormat> mDateFormatter;
 
   int16_t mSortType;
   int32_t mTotalRows;
 
-  nsTArray<PRUnichar*> mCurrentFilters;
+  nsTArray<char16_t*> mCurrentFilters;
 
   bool mShowHiddenFiles;
   bool mDirectoryFilter;
@@ -247,15 +253,15 @@ NS_DEFINE_NAMED_CID(NS_FILECOMPLETE_CID);
 NS_DEFINE_NAMED_CID(NS_FILEVIEW_CID);
 
 static const mozilla::Module::CIDEntry kFileViewCIDs[] = {
-  { &kNS_FILECOMPLETE_CID, false, NULL, nsFileCompleteConstructor },
-  { &kNS_FILEVIEW_CID, false, NULL, nsFileViewConstructor },
-  { NULL }
+  { &kNS_FILECOMPLETE_CID, false, nullptr, nsFileCompleteConstructor },
+  { &kNS_FILEVIEW_CID, false, nullptr, nsFileViewConstructor },
+  { nullptr }
 };
 
 static const mozilla::Module::ContractIDEntry kFileViewContracts[] = {
   { NS_FILECOMPLETE_CONTRACTID, &kNS_FILECOMPLETE_CID },
   { NS_FILEVIEW_CONTRACTID, &kNS_FILEVIEW_CID },
-  { NULL }
+  { nullptr }
 };
 
 static const mozilla::Module kFileViewModule = {
@@ -285,26 +291,6 @@ nsFileView::~nsFileView()
 nsresult
 nsFileView::Init()
 {
-  mDirectoryAtom = do_GetAtom("directory");
-  if (!mDirectoryAtom)
-    return NS_ERROR_OUT_OF_MEMORY;
-
-  mFileAtom = do_GetAtom("file");
-  if (!mFileAtom)
-    return NS_ERROR_OUT_OF_MEMORY;
-
-  NS_NewISupportsArray(getter_AddRefs(mFileList));
-  if (!mFileList)
-    return NS_ERROR_OUT_OF_MEMORY;
-
-  NS_NewISupportsArray(getter_AddRefs(mDirList));
-  if (!mDirList)
-    return NS_ERROR_OUT_OF_MEMORY;
-
-  NS_NewISupportsArray(getter_AddRefs(mFilteredFiles));
-  if (!mFilteredFiles)
-    return NS_ERROR_OUT_OF_MEMORY;
-
   mDateFormatter = do_CreateInstance(NS_DATETIMEFORMAT_CONTRACTID);
   if (!mDateFormatter)
     return NS_ERROR_OUT_OF_MEMORY;
@@ -346,12 +332,11 @@ nsFileView::SetShowOnlyDirectories(bool aOnlyDirs)
     return NS_OK;
 
   mDirectoryFilter = aOnlyDirs;
-  uint32_t dirCount;
-  mDirList->Count(&dirCount);
+  uint32_t dirCount = mDirList.Length();
   if (mDirectoryFilter) {
     int32_t rowDiff = mTotalRows - dirCount;
 
-    mFilteredFiles->Clear();
+    mFilteredFiles.Clear();
     mTotalRows = dirCount;
     if (mTree)
       mTree->RowCountChanged(mTotalRows, -rowDiff);
@@ -428,8 +413,8 @@ nsFileView::SetDirectory(nsIFile* aDirectory)
   }
 
   mDirectoryPath = aDirectory;
-  mFileList->Clear();
-  mDirList->Clear();
+  mFileList.Clear();
+  mDirList.Clear();
 
   bool hasMore = false;
 
@@ -446,11 +431,11 @@ nsFileView::SetDirectory(nsIFile* aDirectory)
         bool isHidden;
         theFile->IsHidden(&isHidden);
         if (mShowHiddenFiles || !isHidden) {
-          mDirList->AppendElement(theFile);
+          mDirList.AppendElement(theFile);
         }
       }
       else {
-        mFileList->AppendElement(theFile);
+        mFileList.AppendElement(theFile);
       }
     }
   }
@@ -500,7 +485,7 @@ nsFileView::SetFilter(const nsAString& aFilterString)
     while (iter != end && (*iter != ';' && *iter != ' '))
       ++iter;
 
-    PRUnichar* filter = ToNewUnicode(Substring(start, iter));
+    char16_t* filter = ToNewUnicode(Substring(start, iter));
     if (!filter)
       return NS_ERROR_OUT_OF_MEMORY;
 
@@ -517,12 +502,11 @@ nsFileView::SetFilter(const nsAString& aFilterString)
 
   if (mTree) {
     mTree->BeginUpdateBatch();
-    uint32_t count;
-    mDirList->Count(&count);
+    uint32_t count = mDirList.Length();
     mTree->RowCountChanged(count, count - mTotalRows);
   }
 
-  mFilteredFiles->Clear();
+  mFilteredFiles.Clear();
 
   FilterFiles();
 
@@ -546,9 +530,7 @@ nsFileView::GetSelectedFiles(nsIArray** aFiles)
   int32_t numRanges;
   mSelection->GetRangeCount(&numRanges);
 
-  uint32_t dirCount;
-  mDirList->Count(&dirCount);
-
+  uint32_t dirCount = mDirList.Length();
   nsCOMPtr<nsIMutableArray> fileArray =
     do_CreateInstance(NS_ARRAY_CONTRACTID);
   NS_ENSURE_STATE(fileArray);
@@ -558,13 +540,13 @@ nsFileView::GetSelectedFiles(nsIArray** aFiles)
     mSelection->GetRangeAt(range, &rangeBegin, &rangeEnd);
 
     for (int32_t itemIndex = rangeBegin; itemIndex <= rangeEnd; ++itemIndex) {
-      nsCOMPtr<nsIFile> curFile;
+      nsIFile* curFile = nullptr;
 
       if (itemIndex < (int32_t) dirCount)
-        curFile = do_QueryElementAt(mDirList, itemIndex);
+        curFile = mDirList[itemIndex];
       else {
         if (itemIndex < mTotalRows)
-          curFile = do_QueryElementAt(mFilteredFiles, itemIndex - dirCount);
+          curFile = mFilteredFiles[itemIndex - dirCount];
       }
 
       if (curFile)
@@ -602,30 +584,27 @@ nsFileView::SetSelection(nsITreeSelection* aSelection)
 }
 
 NS_IMETHODIMP
-nsFileView::GetRowProperties(int32_t aIndex,
-                             nsISupportsArray* aProperties)
+nsFileView::GetRowProperties(int32_t aIndex, nsAString& aProps)
 {
   return NS_OK;
 }
 
 NS_IMETHODIMP
 nsFileView::GetCellProperties(int32_t aRow, nsITreeColumn* aCol,
-                              nsISupportsArray* aProperties)
+                              nsAString& aProps)
 {
-  uint32_t dirCount;
-  mDirList->Count(&dirCount);
+  uint32_t dirCount = mDirList.Length();
 
   if (aRow < (int32_t) dirCount)
-    aProperties->AppendElement(mDirectoryAtom);
+    aProps.AppendLiteral("directory");
   else if (aRow < mTotalRows)
-    aProperties->AppendElement(mFileAtom);
+    aProps.AppendLiteral("file");
 
   return NS_OK;
 }
 
 NS_IMETHODIMP
-nsFileView::GetColumnProperties(nsITreeColumn* aCol,
-                                nsISupportsArray* aProperties)
+nsFileView::GetColumnProperties(nsITreeColumn* aCol, nsAString& aProps)
 {
   return NS_OK;
 }
@@ -726,26 +705,23 @@ NS_IMETHODIMP
 nsFileView::GetCellText(int32_t aRow, nsITreeColumn* aCol,
                         nsAString& aCellText)
 {
-  uint32_t dirCount, fileCount;
-  mDirList->Count(&dirCount);
-  mFilteredFiles->Count(&fileCount);
-
+  uint32_t dirCount = mDirList.Length();
   bool isDirectory;
-  nsCOMPtr<nsIFile> curFile;
+  nsIFile* curFile = nullptr;
 
   if (aRow < (int32_t) dirCount) {
     isDirectory = true;
-    curFile = do_QueryElementAt(mDirList, aRow);
+    curFile = mDirList[aRow];
   } else if (aRow < mTotalRows) {
     isDirectory = false;
-    curFile = do_QueryElementAt(mFilteredFiles, aRow - dirCount);
+    curFile = mFilteredFiles[aRow - dirCount];
   } else {
     // invalid row
     aCellText.SetCapacity(0);
     return NS_OK;
   }
 
-  const PRUnichar* colID;
+  const char16_t* colID;
   aCol->GetIdConst(&colID);
   if (NS_LITERAL_STRING("FilenameColumn").Equals(colID)) {
     curFile->GetLeafName(aCellText);
@@ -833,19 +809,19 @@ nsFileView::SetCellText(int32_t aRow, nsITreeColumn* aCol,
 }
 
 NS_IMETHODIMP
-nsFileView::PerformAction(const PRUnichar* aAction)
+nsFileView::PerformAction(const char16_t* aAction)
 {
   return NS_OK;
 }
 
 NS_IMETHODIMP
-nsFileView::PerformActionOnRow(const PRUnichar* aAction, int32_t aRow)
+nsFileView::PerformActionOnRow(const char16_t* aAction, int32_t aRow)
 {
   return NS_OK;
 }
 
 NS_IMETHODIMP
-nsFileView::PerformActionOnCell(const PRUnichar* aAction, int32_t aRow,
+nsFileView::PerformActionOnCell(const char16_t* aAction, int32_t aRow,
                                 nsITreeColumn* aCol)
 {
   return NS_OK;
@@ -856,16 +832,14 @@ nsFileView::PerformActionOnCell(const PRUnichar* aAction, int32_t aRow,
 void
 nsFileView::FilterFiles()
 {
-  uint32_t count = 0;
-  mDirList->Count(&count);
+  uint32_t count = mDirList.Length();
   mTotalRows = count;
-  mFileList->Count(&count);
-  mFilteredFiles->Clear();
+  count = mFileList.Length();
+  mFilteredFiles.Clear();
   uint32_t filterCount = mCurrentFilters.Length();
 
-  nsCOMPtr<nsIFile> file;
   for (uint32_t i = 0; i < count; ++i) {
-    file = do_QueryElementAt(mFileList, i);
+    nsIFile* file = mFileList[i];
     bool isHidden = false;
     if (!mShowHiddenFiles)
       file->IsHidden(&isHidden);
@@ -880,7 +854,7 @@ nsFileView::FilterFiles()
       for (uint32_t j = 0; j < filterCount; ++j) {
         bool matched = false;
         if (!nsCRT::strcmp(mCurrentFilters.ElementAt(j),
-                           NS_LITERAL_STRING("..apps").get()))
+                           MOZ_UTF16("..apps")))
         {
           file->IsExecutable(&matched);
         } else
@@ -889,7 +863,7 @@ nsFileView::FilterFiles()
                                       true) == MATCH);
 
         if (matched) {
-          mFilteredFiles->AppendElement(file);
+          mFilteredFiles.AppendElement(file);
           ++mTotalRows;
           break;
         }
@@ -899,15 +873,15 @@ nsFileView::FilterFiles()
 }
 
 void
-nsFileView::ReverseArray(nsISupportsArray* aArray)
+nsFileView::ReverseArray(nsTArray<nsCOMPtr<nsIFile> >& aArray)
 {
-  uint32_t count;
-  aArray->Count(&count);
+  uint32_t count = aArray.Length();
   for (uint32_t i = 0; i < count/2; ++i) {
-    nsCOMPtr<nsISupports> element = dont_AddRef(aArray->ElementAt(i));
-    nsCOMPtr<nsISupports> element2 = dont_AddRef(aArray->ElementAt(count-i-1));
-    aArray->ReplaceElementAt(element2, i);
-    aArray->ReplaceElementAt(element, count-i-1);
+    // If we get references to the COMPtrs in the array, and then .swap() them
+    // we avoid AdRef() / Release() calls.
+    nsCOMPtr<nsIFile>& element = aArray[i];
+    nsCOMPtr<nsIFile>& element2 = aArray[count - i - 1];
+    element.swap(element2);
   }
 }
 
@@ -957,7 +931,7 @@ SortDateCallback(const void* aElement1, const void* aElement2, void* aContext)
 }
 
 void
-nsFileView::SortArray(nsISupportsArray* aArray)
+nsFileView::SortArray(nsTArray<nsCOMPtr<nsIFile> >& aArray)
 {
   // We assume the array to be in filesystem order, which
   // for our purposes, is completely unordered.
@@ -978,21 +952,18 @@ nsFileView::SortArray(nsISupportsArray* aArray)
     return;
   }
 
-  uint32_t count;
-  aArray->Count(&count);
+  uint32_t count = aArray.Length();
 
-  // each item will have an additional refcount while
-  // the array is alive.
   nsIFile** array = new nsIFile*[count];
-  uint32_t i;
-  for (i = 0; i < count; ++i)
-    aArray->QueryElementAt(i, NS_GET_IID(nsIFile), (void**)&(array[i]));
+  for (uint32_t i = 0; i < count; ++i) {
+    array[i] = aArray[i];
+  }
 
   NS_QuickSort(array, count, sizeof(nsIFile*), compareFunc, nullptr);
 
-  for (i = 0; i < count; ++i) {
-    aArray->ReplaceElementAt(array[i], i);
-    NS_RELEASE(array[i]);
+  for (uint32_t i = 0; i < count; ++i) {
+    // Use swap() to avoid refcounting.
+    aArray[i].swap(array[i]);
   }
 
   delete[] array;
