@@ -2,6 +2,10 @@
  * License, v. 2.0. If a copy of the MPL was not distributed with this
  * file, You can obtain one at http://mozilla.org/MPL/2.0/. */
 
+#include "mozilla/ArrayUtils.h"
+#include "mozilla/Attributes.h"
+#include "mozilla/DebugOnly.h"
+
 #include "Database.h"
 
 #include "nsINavBookmarksService.h"
@@ -20,13 +24,12 @@
 #include "nsDirectoryServiceUtils.h"
 #include "prsystem.h"
 #include "nsPrintfCString.h"
-#include "mozilla/Util.h"
 #include "mozilla/Preferences.h"
 #include "mozilla/Services.h"
-#include "mozilla/Attributes.h"
+#include "prtime.h"
 
 // Time between corrupt database backups.
-#define RECENT_BACKUP_TIME_MICROSEC (PRInt64)86400 * PR_USEC_PER_SEC // 24H
+#define RECENT_BACKUP_TIME_MICROSEC (int64_t)86400 * PR_USEC_PER_SEC // 24H
 
 // Filename of the database.
 #define DATABASE_FILENAME NS_LITERAL_STRING("places.sqlite")
@@ -36,12 +39,15 @@
 // Set when the database file was found corrupt by a previous maintenance.
 #define PREF_FORCE_DATABASE_REPLACEMENT "places.database.replaceOnStartup"
 
+// Set to specify the size of the places database growth increments in kibibytes
+#define PREF_GROWTH_INCREMENT_KIB "places.database.growthIncrementKiB"
+
 // Maximum size for the WAL file.  It should be small enough since in case of
 // crashes we could lose all the transactions in the file.  But a too small
 // file could hurt performance.
 #define DATABASE_MAX_WAL_SIZE_IN_KIBIBYTES 512
 
-#define BYTES_PER_MEBIBYTE 1048576
+#define BYTES_PER_KIBIBYTE 1024
 
 // Old Sync GUID annotation.
 #define SYNCGUID_ANNO NS_LITERAL_CSTRING("sync/guid")
@@ -90,7 +96,7 @@ hasRecentCorruptDB()
     if (NS_SUCCEEDED(currFile->GetLeafName(leafName)) &&
         leafName.Length() >= DATABASE_CORRUPT_FILENAME.Length() &&
         leafName.Find(".corrupt", DATABASE_FILENAME.Length()) != -1) {
-      PRInt64 lastMod = 0;
+      PRTime lastMod = 0;
       currFile->GetLastModifiedTime(&lastMod);
       NS_ENSURE_TRUE(lastMod > 0, false);
       return (PR_Now() - lastMod) > RECENT_BACKUP_TIME_MICROSEC;
@@ -140,7 +146,7 @@ updateSQLiteStatistics(mozIStorageConnection* aDBConn)
   };
 
   nsCOMPtr<mozIStoragePendingStatement> ps;
-  (void)aDBConn->ExecuteAsync(stmts, ArrayLength(stmts), nsnull,
+  (void)aDBConn->ExecuteAsync(stmts, ArrayLength(stmts), nullptr,
                               getter_AddRefs(ps));
   return NS_OK;
 }
@@ -161,10 +167,10 @@ SetJournalMode(nsCOMPtr<mozIStorageConnection>& aDBConn,
                              enum JournalMode aJournalMode)
 {
   MOZ_ASSERT(NS_IsMainThread());
-  nsCAutoString journalMode;
+  nsAutoCString journalMode;
   switch (aJournalMode) {
     default:
-      MOZ_ASSERT("Trying to set an unknown journal mode.");
+      MOZ_ASSERT(false, "Trying to set an unknown journal mode.");
       // Fall through to the default DELETE journal.
     case JOURNAL_DELETE:
       journalMode.AssignLiteral("delete");
@@ -181,7 +187,7 @@ SetJournalMode(nsCOMPtr<mozIStorageConnection>& aDBConn,
   }
 
   nsCOMPtr<mozIStorageStatement> statement;
-  nsCAutoString query(MOZ_STORAGE_UNIQUIFY_QUERY_STR
+  nsAutoCString query(MOZ_STORAGE_UNIQUIFY_QUERY_STR
 		      "PRAGMA journal_mode = ");
   query.Append(journalMode);
   aDBConn->CreateStatement(query, getter_AddRefs(statement));
@@ -213,23 +219,23 @@ class BlockingConnectionCloseCallback MOZ_FINAL : public mozIStorageCompletionCa
   bool mDone;
 
 public:
-  NS_DECL_ISUPPORTS
+  NS_DECL_THREADSAFE_ISUPPORTS
   NS_DECL_MOZISTORAGECOMPLETIONCALLBACK
   BlockingConnectionCloseCallback();
   void Spin();
 };
 
 NS_IMETHODIMP
-BlockingConnectionCloseCallback::Complete()
+BlockingConnectionCloseCallback::Complete(nsresult, nsISupports*)
 {
   mDone = true;
   nsCOMPtr<nsIObserverService> os = mozilla::services::GetObserverService();
   MOZ_ASSERT(os);
   if (!os)
     return NS_OK;
-  DebugOnly<nsresult> rv = os->NotifyObservers(nsnull,
+  DebugOnly<nsresult> rv = os->NotifyObservers(nullptr,
                                                TOPIC_PLACES_CONNECTION_CLOSED,
-                                               nsnull);
+                                               nullptr);
   MOZ_ASSERT(NS_SUCCEEDED(rv));
   return NS_OK;
 }
@@ -247,7 +253,7 @@ void BlockingConnectionCloseCallback::Spin() {
   }
 }
 
-NS_IMPL_THREADSAFE_ISUPPORTS1(
+NS_IMPL_ISUPPORTS1(
   BlockingConnectionCloseCallback
 , mozIStorageCompletionCallback
 )
@@ -260,7 +266,7 @@ CreateRoot(nsCOMPtr<mozIStorageConnection>& aDBConn,
   MOZ_ASSERT(NS_IsMainThread());
 
   // The position of the new item in its folder.
-  static PRInt32 itemPosition = 0;
+  static int32_t itemPosition = 0;
 
   // A single creation timestamp for all roots so that the root folder's
   // last modification time isn't earlier than its childrens' creation time.
@@ -330,7 +336,7 @@ CreateRoot(nsCOMPtr<mozIStorageConnection>& aDBConn,
 
 PLACES_FACTORY_SINGLETON_IMPLEMENTATION(Database, gDatabase)
 
-NS_IMPL_THREADSAFE_ISUPPORTS2(Database
+NS_IMPL_ISUPPORTS2(Database
 , nsIObserver
 , nsISupportsWeakReference
 )
@@ -340,9 +346,9 @@ Database::Database()
   , mMainThreadAsyncStatements(mMainConn)
   , mAsyncThreadStatements(mMainConn)
   , mDBPageSize(0)
-  , mCurrentJournalMode(JOURNAL_DELETE)
   , mDatabaseStatus(nsINavHistoryService::DATABASE_STATUS_OK)
   , mShuttingDown(false)
+  , mClosed(false)
 {
   // Attempting to create two instances of the service?
   MOZ_ASSERT(!gDatabase);
@@ -357,19 +363,13 @@ Database::~Database()
 
   // Remove the static reference to the service.
   if (gDatabase == this) {
-    gDatabase = nsnull;
+    gDatabase = nullptr;
   }
 }
 
 nsresult
 Database::Init()
 {
-#ifdef MOZ_ANDROID_HISTORY
-  // Currently places has deeply weaved it way throughout the gecko codebase.
-  // Here we disable all database creation and loading of places.
-  return NS_ERROR_NOT_IMPLEMENTED;
-#endif
-
   MOZ_ASSERT(NS_IsMainThread());
 
   nsCOMPtr<mozIStorageService> storage =
@@ -560,14 +560,14 @@ Database::InitSchema(bool* aDatabaseMigrated)
 
   // Be sure to set journal mode after page_size.  WAL would prevent the change
   // otherwise.
-  if (NS_SUCCEEDED(SetJournalMode(mMainConn, JOURNAL_WAL))) {
+  if (JOURNAL_WAL == SetJournalMode(mMainConn, JOURNAL_WAL)) {
     // Set the WAL journal size limit.  We want it to be small, since in
     // synchronous = NORMAL mode a crash could cause loss of all the
     // transactions in the journal.  For added safety we will also force
     // checkpointing at strategic moments.
-    PRInt32 checkpointPages =
-      static_cast<PRInt32>(DATABASE_MAX_WAL_SIZE_IN_KIBIBYTES * 1024 / mDBPageSize);
-    nsCAutoString checkpointPragma("PRAGMA wal_autocheckpoint = ");
+    int32_t checkpointPages =
+      static_cast<int32_t>(DATABASE_MAX_WAL_SIZE_IN_KIBIBYTES * 1024 / mDBPageSize);
+    nsAutoCString checkpointPragma("PRAGMA wal_autocheckpoint = ");
     checkpointPragma.AppendInt(checkpointPages);
     rv = mMainConn->ExecuteSimpleSQL(checkpointPragma);
     NS_ENSURE_SUCCESS(rv, rv);
@@ -591,19 +591,24 @@ Database::InitSchema(bool* aDatabaseMigrated)
   // Since exceeding the limit will cause a truncate, allow a slightly
   // larger limit than DATABASE_MAX_WAL_SIZE_IN_KIBIBYTES to reduce the number
   // of times it is needed.
-  nsCAutoString journalSizePragma("PRAGMA journal_size_limit = ");
+  nsAutoCString journalSizePragma("PRAGMA journal_size_limit = ");
   journalSizePragma.AppendInt(DATABASE_MAX_WAL_SIZE_IN_KIBIBYTES * 3);
   (void)mMainConn->ExecuteSimpleSQL(journalSizePragma);
 
-  // Grow places in 10MiB increments to limit fragmentation on disk.
-  (void)mMainConn->SetGrowthIncrement(10 * BYTES_PER_MEBIBYTE, EmptyCString());
+  // Grow places in |growthIncrementKiB| increments to limit fragmentation on disk.
+  // By default, it's 10 MB.
+  int32_t growthIncrementKiB =
+    Preferences::GetInt(PREF_GROWTH_INCREMENT_KIB, 10 * BYTES_PER_KIBIBYTE);
+  if (growthIncrementKiB > 0) {
+    (void)mMainConn->SetGrowthIncrement(growthIncrementKiB * BYTES_PER_KIBIBYTE, EmptyCString());
+  }
 
   // We use our functions during migration, so initialize them now.
   rv = InitFunctions();
   NS_ENSURE_SUCCESS(rv, rv);
 
   // Get the database schema version.
-  PRInt32 currentSchemaVersion;
+  int32_t currentSchemaVersion;
   rv = mMainConn->GetSchemaVersion(&currentSchemaVersion);
   NS_ENSURE_SUCCESS(rv, rv);
   bool databaseInitialized = currentSchemaVersion > 0;
@@ -726,6 +731,20 @@ Database::InitSchema(bool* aDatabaseMigrated)
 
       // Firefox 14 uses schema version 21.
 
+      if (currentSchemaVersion < 22) {
+        rv = MigrateV22Up();
+        NS_ENSURE_SUCCESS(rv, rv);
+      }
+
+      // Firefox 22 uses schema version 22.
+
+      if (currentSchemaVersion < 23) {
+        rv = MigrateV23Up();
+        NS_ENSURE_SUCCESS(rv, rv);
+      }
+
+      // Firefox 24 uses schema version 23.
+
       // Schema Upgrades must add migration code here.
 
       rv = UpdateBookmarkRootTitles();
@@ -853,25 +872,25 @@ Database::CreateBookmarkRoots()
   if (NS_FAILED(rv)) return rv;
 
   // Fetch the internationalized folder name from the string bundle.
-  rv = bundle->GetStringFromName(NS_LITERAL_STRING("BookmarksMenuFolderTitle").get(),
+  rv = bundle->GetStringFromName(MOZ_UTF16("BookmarksMenuFolderTitle"),
                                  getter_Copies(rootTitle));
   if (NS_FAILED(rv)) return rv;
   rv = CreateRoot(mMainConn, NS_LITERAL_CSTRING("menu"), rootTitle);
   if (NS_FAILED(rv)) return rv;
 
-  rv = bundle->GetStringFromName(NS_LITERAL_STRING("BookmarksToolbarFolderTitle").get(),
+  rv = bundle->GetStringFromName(MOZ_UTF16("BookmarksToolbarFolderTitle"),
                                  getter_Copies(rootTitle));
   if (NS_FAILED(rv)) return rv;
   rv = CreateRoot(mMainConn, NS_LITERAL_CSTRING("toolbar"), rootTitle);
   if (NS_FAILED(rv)) return rv;
 
-  rv = bundle->GetStringFromName(NS_LITERAL_STRING("TagsFolderTitle").get(),
+  rv = bundle->GetStringFromName(MOZ_UTF16("TagsFolderTitle"),
                                  getter_Copies(rootTitle));
   if (NS_FAILED(rv)) return rv;
   rv = CreateRoot(mMainConn, NS_LITERAL_CSTRING("tags"), rootTitle);
   if (NS_FAILED(rv)) return rv;
 
-  rv = bundle->GetStringFromName(NS_LITERAL_STRING("UnsortedBookmarksFolderTitle").get(),
+  rv = bundle->GetStringFromName(MOZ_UTF16("UnsortedBookmarksFolderTitle"),
                                  getter_Copies(rootTitle));
   if (NS_FAILED(rv)) return rv;
   rv = CreateRoot(mMainConn, NS_LITERAL_CSTRING("unfiled"), rootTitle);
@@ -892,13 +911,13 @@ Database::CreateBookmarkRoots()
   rv = stmt->ExecuteStep(&hasResult);
   if (NS_FAILED(rv)) return rv;
   MOZ_ASSERT(hasResult);
-  PRInt32 bookmarkCount = 0;
+  int32_t bookmarkCount = 0;
   rv = stmt->GetInt32(0, &bookmarkCount);
   if (NS_FAILED(rv)) return rv;
-  PRInt32 rootCount = 0;
+  int32_t rootCount = 0;
   rv = stmt->GetInt32(1, &rootCount);
   if (NS_FAILED(rv)) return rv;
-  PRInt32 positionSum = 0;
+  int32_t positionSum = 0;
   rv = stmt->GetInt32(2, &positionSum);
   if (NS_FAILED(rv)) return rv;
   MOZ_ASSERT(bookmarkCount == 5 && rootCount == 5 && positionSum == 6);
@@ -979,7 +998,7 @@ Database::UpdateBookmarkRootTitles()
     "TagsFolderTitle", "UnsortedBookmarksFolderTitle"
   };
 
-  for (PRUint32 i = 0; i < ArrayLength(rootNames); ++i) {
+  for (uint32_t i = 0; i < ArrayLength(rootNames); ++i) {
     nsXPIDLString title;
     rv = bundle->GetStringFromName(NS_ConvertASCIItoUTF16(titleStringIDs[i]).get(),
                                    getter_Copies(title));
@@ -1001,7 +1020,7 @@ Database::UpdateBookmarkRootTitles()
   rv = stmt->BindParameters(paramsArray);
   if (NS_FAILED(rv)) return rv;
   nsCOMPtr<mozIStoragePendingStatement> pendingStmt;
-  rv = stmt->ExecuteAsync(nsnull, getter_AddRefs(pendingStmt));
+  rv = stmt->ExecuteAsync(nullptr, getter_AddRefs(pendingStmt));
   if (NS_FAILED(rv)) return rv;
 
   return NS_OK;
@@ -1035,10 +1054,10 @@ Database::CheckAndUpdateGUIDs()
 
   bool hasResult;
   while (NS_SUCCEEDED(stmt->ExecuteStep(&hasResult)) && hasResult) {
-    PRInt64 itemId;
+    int64_t itemId;
     rv = stmt->GetInt64(0, &itemId);
     NS_ENSURE_SUCCESS(rv, rv);
-    nsCAutoString guid;
+    nsAutoCString guid;
     rv = stmt->GetUTF8String(1, guid);
     NS_ENSURE_SUCCESS(rv, rv);
 
@@ -1109,10 +1128,10 @@ Database::CheckAndUpdateGUIDs()
   NS_ENSURE_SUCCESS(rv, rv);
 
   while (NS_SUCCEEDED(stmt->ExecuteStep(&hasResult)) && hasResult) {
-    PRInt64 placeId;
+    int64_t placeId;
     rv = stmt->GetInt64(0, &placeId);
     NS_ENSURE_SUCCESS(rv, rv);
-    nsCAutoString guid;
+    nsAutoCString guid;
     rv = stmt->GetUTF8String(1, guid);
     NS_ENSURE_SUCCESS(rv, rv);
 
@@ -1246,7 +1265,7 @@ Database::MigrateV7Up()
     );
     NS_ENSURE_STATE(stmt);
     nsCOMPtr<mozIStoragePendingStatement> ps;
-    (void)stmt->ExecuteAsync(nsnull, getter_AddRefs(ps));
+    (void)stmt->ExecuteAsync(nullptr, getter_AddRefs(ps));
   }
 
   // Temporary migration code for bug 396300
@@ -1549,7 +1568,7 @@ Database::MigrateV13Up()
   );
   NS_ENSURE_SUCCESS(rv, rv);
   nsCOMPtr<mozIStoragePendingStatement> ps;
-  rv = deleteDynContainersStmt->ExecuteAsync(nsnull, getter_AddRefs(ps));
+  rv = deleteDynContainersStmt->ExecuteAsync(nullptr, getter_AddRefs(ps));
   NS_ENSURE_SUCCESS(rv, rv);
 
   return NS_OK;
@@ -1676,7 +1695,7 @@ Database::MigrateV17Up()
   NS_ENSURE_SUCCESS(rv, rv);
 
   nsCOMPtr<mozIStoragePendingStatement> ps;
-  rv = fillHostsStmt->ExecuteAsync(nsnull, getter_AddRefs(ps));
+  rv = fillHostsStmt->ExecuteAsync(nullptr, getter_AddRefs(ps));
   NS_ENSURE_SUCCESS(rv, rv);
 
   return NS_OK;
@@ -1720,7 +1739,7 @@ Database::MigrateV18Up()
   NS_ENSURE_SUCCESS(rv, rv);
 
   nsCOMPtr<mozIStoragePendingStatement> ps;
-  rv = updateTypedStmt->ExecuteAsync(nsnull, getter_AddRefs(ps));
+  rv = updateTypedStmt->ExecuteAsync(nullptr, getter_AddRefs(ps));
   NS_ENSURE_SUCCESS(rv, rv);
 
   return NS_OK;
@@ -1862,17 +1881,39 @@ Database::MigrateV21Up()
     NS_ENSURE_SUCCESS(rv, rv);
   }
 
-  // Update prefixes.
+  return NS_OK;
+}
+
+nsresult
+Database::MigrateV22Up()
+{
+  MOZ_ASSERT(NS_IsMainThread());
+
+  // Reset all session IDs to 0 since we don't support them anymore.
+  // We don't set them to NULL to avoid breaking downgrades.
+  nsresult rv = mMainConn->ExecuteSimpleSQL(NS_LITERAL_CSTRING(
+    "UPDATE moz_historyvisits SET session = 0"
+  ));
+  NS_ENSURE_SUCCESS(rv, rv);
+
+  return NS_OK;
+}
+
+
+nsresult
+Database::MigrateV23Up()
+{
+  MOZ_ASSERT(NS_IsMainThread());
+
+  // Recalculate hosts prefixes.
   nsCOMPtr<mozIStorageAsyncStatement> updatePrefixesStmt;
-  rv = mMainConn->CreateAsyncStatement(NS_LITERAL_CSTRING(
-    "UPDATE moz_hosts SET prefix = ( "
-      HOSTS_PREFIX_PRIORITY_FRAGMENT
-    ") "
+  nsresult rv = mMainConn->CreateAsyncStatement(NS_LITERAL_CSTRING(
+    "UPDATE moz_hosts SET prefix = ( " HOSTS_PREFIX_PRIORITY_FRAGMENT ") "
   ), getter_AddRefs(updatePrefixesStmt));
   NS_ENSURE_SUCCESS(rv, rv);
 
   nsCOMPtr<mozIStoragePendingStatement> ps;
-  rv = updatePrefixesStmt->ExecuteAsync(nsnull, getter_AddRefs(ps));
+  rv = updatePrefixesStmt->ExecuteAsync(nullptr, getter_AddRefs(ps));
   NS_ENSURE_SUCCESS(rv, rv);
 
   return NS_OK;
@@ -1883,6 +1924,9 @@ Database::Shutdown()
 {
   MOZ_ASSERT(NS_IsMainThread());
   MOZ_ASSERT(!mShuttingDown);
+  MOZ_ASSERT(!mClosed);
+
+  mShuttingDown = true;
 
   mMainThreadStatements.FinalizeStatements();
   mMainThreadAsyncStatements.FinalizeStatements();
@@ -1898,9 +1942,7 @@ Database::Shutdown()
   (void)mMainConn->AsyncClose(closeListener);
   closeListener->Spin();
 
-  // Don't set this earlier, otherwise some internal helper used on shutdown
-  // may bail out.
-  mShuttingDown = true;
+  mClosed = true;
 }
 
 ////////////////////////////////////////////////////////////////////////////////
@@ -1909,7 +1951,7 @@ Database::Shutdown()
 NS_IMETHODIMP
 Database::Observe(nsISupports *aSubject,
                   const char *aTopic,
-                  const PRUnichar *aData)
+                  const char16_t *aData)
 {
   MOZ_ASSERT(NS_IsMainThread());
  
@@ -1932,15 +1974,16 @@ Database::Observe(nsISupports *aSubject,
                      getter_AddRefs(e))) && e) {
       bool hasMore = false;
       while (NS_SUCCEEDED(e->HasMoreElements(&hasMore)) && hasMore) {
-        nsCOMPtr<nsIObserver> observer;
-        if (NS_SUCCEEDED(e->GetNext(getter_AddRefs(observer)))) {
-          (void)observer->Observe(observer, TOPIC_PLACES_INIT_COMPLETE, nsnull);
+	nsCOMPtr<nsISupports> supports;
+        if (NS_SUCCEEDED(e->GetNext(getter_AddRefs(supports)))) {
+          nsCOMPtr<nsIObserver> observer = do_QueryInterface(supports);
+          (void)observer->Observe(observer, TOPIC_PLACES_INIT_COMPLETE, nullptr);
         }
       }
     }
 
     // Notify all Places users that we are about to shutdown.
-    (void)os->NotifyObservers(nsnull, TOPIC_PLACES_SHUTDOWN, nsnull);
+    (void)os->NotifyObservers(nullptr, TOPIC_PLACES_SHUTDOWN, nullptr);
   }
 
   else if (strcmp(aTopic, TOPIC_PROFILE_BEFORE_CHANGE) == 0) {
@@ -1952,7 +1995,7 @@ Database::Observe(nsISupports *aSubject,
     // Fire internal shutdown notifications.
     nsCOMPtr<nsIObserverService> os = services::GetObserverService();
     if (os) {
-      (void)os->NotifyObservers(nsnull, TOPIC_PLACES_WILL_CLOSE_CONNECTION, nsnull);
+      (void)os->NotifyObservers(nullptr, TOPIC_PLACES_WILL_CLOSE_CONNECTION, nullptr);
     }
 
 #ifdef DEBUG
