@@ -41,10 +41,6 @@
 # include "nsWindowsHelpers.h"
 # define getcwd(path, size) _getcwd(path, size)
 # define getpid() GetCurrentProcessId()
-#elif defined(XP_OS2)
-# include <unistd.h>
-# define INCL_DOSFILEMGR
-# include <os2.h>
 #elif defined(XP_UNIX)
 # include <unistd.h>
 #endif
@@ -123,10 +119,7 @@ GetCurrentWorkingDir(char *buf, size_t size)
   // Cannot use NS_GetSpecialDirectory because XPCOM is not yet initialized.
   // This code is duplicated from xpcom/io/SpecialSystemDirectory.cpp:
 
-#if defined(XP_OS2)
-  if (DosQueryPathInfo( ".", FIL_QUERYFULLNAME, buf, size))
-    return NS_ERROR_FAILURE;
-#elif defined(XP_WIN)
+#if defined(XP_WIN)
   wchar_t wpath[MAX_PATH];
   if (!_wgetcwd(wpath, size))
     return NS_ERROR_FAILURE;
@@ -300,6 +293,19 @@ IsOlderVersion(nsIFile *versionFile, const char *appVersion)
   return false;
 }
 
+#if defined(XP_WIN) && defined(MOZ_METRO)
+static bool
+IsWindowsMetroUpdateRequest(int appArgc, char **appArgv)
+{
+  for (int index = 0; index < appArgc; index++) {
+    if (!strcmp(appArgv[index], "--metro-update")) {
+      return true;
+    }
+  }
+  return false;
+}
+#endif
+
 static bool
 CopyFileIntoUpdateDir(nsIFile *parentDir, const char *leafName, nsIFile *updateDir)
 {
@@ -368,8 +374,8 @@ CopyUpdaterIntoUpdateDir(nsIFile *greDir, nsIFile *appDir, nsIFile *updateDir,
 }
 
 /**
- * Switch an existing application directory to an updated version which has been
- * previously constructed in the background.
+ * Switch an existing application directory to an updated version that has been
+ * staged.
  *
  * @param greDir the GRE dir
  * @param updateDir the update root dir
@@ -535,8 +541,10 @@ SwitchToUpdatedApp(nsIFile *greDir, nsIFile *updateDir, nsIFile *statusFile,
   pid.AppendLiteral("/replace");
 
   int immersiveArgc = 0;
-#ifdef XP_WIN
-  if (IsRunningInWindowsMetro()) {
+#if defined(XP_WIN) && defined(MOZ_METRO)
+  // If this is desktop doing an update for metro, or if we're the metro browser
+  // we want to launch the metro browser after we're finished.
+  if (IsWindowsMetroUpdateRequest(appArgc, appArgv) || IsRunningInWindowsMetro()) {
     immersiveArgc = 1;
   }
 #endif
@@ -606,9 +614,12 @@ GetOSApplyToDir(nsACString& applyToDir)
   NS_ASSERTION(ds, "Can't get directory service");
 
   nsCOMPtr<nsIFile> osApplyToDir;
-  DebugOnly<nsresult> rv = ds->Get(XRE_OS_UPDATE_APPLY_TO_DIR, NS_GET_IID(nsIFile),
+  nsresult rv = ds->Get(XRE_OS_UPDATE_APPLY_TO_DIR, NS_GET_IID(nsIFile),
                                    getter_AddRefs(osApplyToDir));
-  NS_ASSERTION(NS_SUCCEEDED(rv), "Can't get the OS applyTo dir");
+  if (NS_FAILED(rv)) {
+    LOG(("Can't get the OS applyTo dir"));
+    return rv;
+  }
 
   return osApplyToDir->GetNativePath(applyToDir);
 }
@@ -640,7 +651,7 @@ SetOSApplyToDir(nsIUpdate* update, const nsACString& osApplyToDir)
 #endif
 
 /**
- * Apply an update, possibly in the background.
+ * Apply an update. This applies to both normal and staged updates.
  *
  * @param greDir the GRE dir
  * @param updateDir the update root dir
@@ -649,10 +660,10 @@ SetOSApplyToDir(nsIUpdate* update, const nsACString& osApplyToDir)
  * @param appArgc the number of args to the application
  * @param appArgv the args to the application, used for restarting if needed
  * @param restart if true, apply the update in the foreground and restart the
- *                application when done.  otherwise, apply the update in the
- *                background and don't restart the application.
+ *                application when done.  otherwise, stage the update and don't
+ *                restart the application.
  * @param outpid out parameter holding the handle to the updater application for
- *               background updates.
+ *               staging updates.
  */
 static void
 ApplyUpdate(nsIFile *greDir, nsIFile *updateDir, nsIFile *statusFile,
@@ -731,8 +742,7 @@ ApplyUpdate(nsIFile *greDir, nsIFile *updateDir, nsIFile *statusFile,
     if (NS_FAILED(rv))
       return;
     if (restart) {
-      // Use the correct directory if we're not applying the update in the
-      // background.
+      // Use the correct directory if we're not staging the update.
       rv = parentDir2->GetNativePath(applyToDir);
     } else {
       if (!GetFile(parentDir2, NS_LITERAL_CSTRING("Updated.app"), updatedDir))
@@ -742,8 +752,7 @@ ApplyUpdate(nsIFile *greDir, nsIFile *updateDir, nsIFile *statusFile,
   }
 #else
   if (restart) {
-    // Use the correct directory if we're not applying the update in the
-    // background.
+    // Use the correct directory if we're not staging the update.
     updatedDir = do_QueryInterface(appDir);
   } else if (!GetFile(appDir, NS_LITERAL_CSTRING("updated"), updatedDir)) {
     return;
@@ -807,8 +816,7 @@ ApplyUpdate(nsIFile *greDir, nsIFile *updateDir, nsIFile *statusFile,
   // we pass "0" which is then ignored by the updater.
   nsAutoCString pid;
   if (!restart) {
-    // Signal the updater application that it should apply the update in the
-    // background.
+    // Signal the updater application that it should stage the update.
     pid.AssignASCII("-1");
   } else {
 #if defined(USE_EXECV)
@@ -819,8 +827,10 @@ ApplyUpdate(nsIFile *greDir, nsIFile *updateDir, nsIFile *statusFile,
   }
 
   int immersiveArgc = 0;
-#ifdef XP_WIN
-  if (IsRunningInWindowsMetro()) {
+#if defined(XP_WIN) && defined(MOZ_METRO)
+  // If this is desktop doing an update for metro, or if we're the metro browser
+  // we want to launch the metro browser after we're finished.
+  if (IsWindowsMetroUpdateRequest(appArgc, appArgv) || IsRunningInWindowsMetro()) {
     immersiveArgc = 1;
   }
 #endif
@@ -876,7 +886,7 @@ ApplyUpdate(nsIFile *greDir, nsIFile *updateDir, nsIFile *statusFile,
   LOG(("spawning updater process [%s]\n", updaterPath.get()));
 
 #if defined(USE_EXECV)
-  // Don't use execv for background updates.
+  // Don't use execv when staging updates.
   if (restart) {
     execv(updaterPath.get(), argv);
   } else {
@@ -950,34 +960,12 @@ ProcessUpdates(nsIFile *greDir, nsIFile *appDir, nsIFile *updRootDir,
   if (NS_FAILED(rv))
     return rv;
  
-  ProcessType dummyPID; // this will only be used for MOZ_UPDATE_BACKGROUND
+  ProcessType dummyPID; // this will only be used for MOZ_UPDATE_STAGING
   const char *processingUpdates = PR_GetEnv("MOZ_PROCESS_UPDATES");
   if (processingUpdates && *processingUpdates) {
-    // Enable the tests to request us to use a different update root directory
-    const char *updRootOverride = PR_GetEnv("MOZ_UPDATE_ROOT_OVERRIDE");
-    if (updRootOverride && *updRootOverride) {
-      nsCOMPtr<nsIFile> overrideDir;
-      nsAutoCString path(updRootOverride);
-      rv = NS_NewNativeLocalFile(path, false, getter_AddRefs(overrideDir));
-      if (NS_FAILED(rv)) {
-        return rv;
-      }
-      updatesDir = do_QueryInterface(overrideDir);
-    }
-    // Enable the tests to request us to use a different app directory
-    const char *appDirOverride = PR_GetEnv("MOZ_UPDATE_APPDIR_OVERRIDE");
-    if (appDirOverride && *appDirOverride) {
-      nsCOMPtr<nsIFile> overrideDir;
-      nsAutoCString path(appDirOverride);
-      rv = NS_NewNativeLocalFile(path, false, getter_AddRefs(overrideDir));
-      if (NS_FAILED(rv)) {
-        return rv;
-      }
-      NS_ADDREF(appDir = overrideDir);
-    }
-    // Enable the tests to request us to perform a background update
-    const char *backgroundUpdate = PR_GetEnv("MOZ_UPDATE_BACKGROUND");
-    if (backgroundUpdate && *backgroundUpdate) {
+    // Enable the tests to request an update to be staged.
+    const char *stagingUpdate = PR_GetEnv("MOZ_UPDATE_STAGING");
+    if (stagingUpdate && *stagingUpdate) {
       restart = false;
       pid = &dummyPID;
     }
@@ -1003,8 +991,8 @@ ProcessUpdates(nsIFile *greDir, nsIFile *appDir, nsIFile *updRootDir,
   }
   case eAppliedUpdate:
   case eAppliedService:
-    // An update was applied in the background, so we need to switch to using
-    // it now.
+    // An update was staged and needs to be switched so the updated application
+    // is used.
     SwitchToUpdatedApp(greDir, updatesDir, statusFile,
                        appDir, argc, argv);
     break;
@@ -1108,7 +1096,7 @@ nsUpdateProcessor::ProcessUpdate(nsIUpdate* aUpdate)
     binary->GetNativePath(binPath);
   }
 
-  // Copy the parameters to the BackgroundUpdateInfo structure shared with the
+  // Copy the parameters to the StagedUpdateInfo structure shared with the
   // watcher thread.
   mInfo.mGREDir = greDir;
   mInfo.mAppDir = appDir;
@@ -1140,14 +1128,20 @@ nsUpdateProcessor::ProcessUpdate(nsIUpdate* aUpdate)
     // This needs to be done on the main thread, so we pass it along in
     // BackgroundThreadInfo
     nsresult rv = GetOSApplyToDir(osApplyToDir);
-    NS_ASSERTION(NS_SUCCEEDED(rv), "Can't get the OS apply to dir");
+    if (NS_FAILED(rv)) {
+      LOG(("Can't get the OS apply to dir"));
+      return rv;
+    }
 
     SetOSApplyToDir(aUpdate, osApplyToDir);
 
     mInfo.mIsOSUpdate = true;
     rv = NS_NewNativeLocalFile(osApplyToDir, false,
                                getter_AddRefs(mInfo.mOSApplyToDir));
-    NS_ASSERTION(NS_SUCCEEDED(rv), "Can't create nsIFile for OS apply to dir");
+    if (NS_FAILED(rv)) {
+      LOG(("Can't create nsIFile for OS apply to dir"));
+      return rv;
+    }
   }
 #endif
 
@@ -1155,13 +1149,13 @@ nsUpdateProcessor::ProcessUpdate(nsIUpdate* aUpdate)
 
   NS_ABORT_IF_FALSE(NS_IsMainThread(), "not main thread");
   return NS_NewThread(getter_AddRefs(mProcessWatcher),
-                      NS_NewRunnableMethod(this, &nsUpdateProcessor::StartBackgroundUpdate));
+                      NS_NewRunnableMethod(this, &nsUpdateProcessor::StartStagedUpdate));
 }
 
 
 
 void
-nsUpdateProcessor::StartBackgroundUpdate()
+nsUpdateProcessor::StartStagedUpdate()
 {
   NS_ABORT_IF_FALSE(!NS_IsMainThread(), "main thread");
 
@@ -1178,11 +1172,11 @@ nsUpdateProcessor::StartBackgroundUpdate()
   NS_ENSURE_SUCCESS_VOID(rv);
 
   if (mUpdaterPID) {
-    // Track the state of the background updater process
+    // Track the state of the updater process while it is staging an update.
     rv = NS_DispatchToCurrentThread(NS_NewRunnableMethod(this, &nsUpdateProcessor::WaitForProcess));
     NS_ENSURE_SUCCESS_VOID(rv);
   } else {
-    // Failed to launch the background updater process for some reason.
+    // Failed to launch the updater process for some reason.
     // We need to shutdown the current thread as there isn't anything more for
     // us to do...
     rv = NS_DispatchToMainThread(NS_NewRunnableMethod(this, &nsUpdateProcessor::ShutdownWatcherThread));
