@@ -11,12 +11,35 @@
 #include "nsUXThemeConstants.h"
 #include "gfxFont.h"
 #include "gfxWindowsPlatform.h"
-#include "WinUtils.h"
 #include "mozilla/Telemetry.h"
+#include "mozilla/WindowsVersion.h"
 #include "gfxFontConstants.h"
 
+using namespace mozilla;
 using namespace mozilla::widget;
-using mozilla::LookAndFeel;
+
+//static
+LookAndFeel::OperatingSystemVersion
+nsLookAndFeel::GetOperatingSystemVersion()
+{
+  static OperatingSystemVersion version = eOperatingSystemVersion_Unknown;
+
+  if (version != eOperatingSystemVersion_Unknown) {
+    return version;
+  }
+
+  if (IsWin8OrLater()) {
+    version = eOperatingSystemVersion_Windows8;
+  } else if (IsWin7OrLater()) {
+    version = eOperatingSystemVersion_Windows7;
+  } else if (IsVistaOrLater()) {
+    version = eOperatingSystemVersion_WindowsVista;
+  } else {
+    version = eOperatingSystemVersion_WindowsXP;
+  }
+
+  return version;
+}
 
 static nsresult GetColorFromTheme(nsUXThemeClass cls,
                            int32_t aPart,
@@ -42,7 +65,7 @@ static int32_t GetSystemParam(long flag, int32_t def)
 
 namespace mozilla {
 namespace widget {
-// This is in use here and in nsDOMTouchEvent.cpp
+// This is in use here and in dom/events/TouchEvent.cpp
 int32_t IsTouchDeviceSupportPresent()
 {
   int32_t touchCapabilities;
@@ -168,8 +191,7 @@ nsLookAndFeel::NativeGetColor(ColorID aID, nscolor &aColor)
       idx = COLOR_HIGHLIGHT;
       break;
     case eColorID__moz_menubarhovertext:
-      if (WinUtils::GetWindowsVersion() < WinUtils::VISTA_VERSION ||
-          !IsAppThemed())
+      if (!IsVistaOrLater() || !IsAppThemed())
       {
         idx = nsUXThemeData::sFlatMenus ?
                 COLOR_HIGHLIGHTTEXT :
@@ -178,8 +200,7 @@ nsLookAndFeel::NativeGetColor(ColorID aID, nscolor &aColor)
       }
       // Fall through
     case eColorID__moz_menuhovertext:
-      if (WinUtils::GetWindowsVersion() >= WinUtils::VISTA_VERSION &&
-          IsAppThemed())
+      if (IsVistaOrLater() && IsAppThemed())
       {
         res = ::GetColorFromTheme(eUXMenu,
                                   MENU_POPUPITEM, MPI_HOT, TMT_TEXTCOLOR, aColor);
@@ -255,8 +276,7 @@ nsLookAndFeel::NativeGetColor(ColorID aID, nscolor &aColor)
       idx = COLOR_3DFACE;
       break;
     case eColorID__moz_win_mediatext:
-      if (WinUtils::GetWindowsVersion() >= WinUtils::VISTA_VERSION &&
-          IsAppThemed()) {
+      if (IsVistaOrLater() && IsAppThemed()) {
         res = ::GetColorFromTheme(eUXMediaToolbar,
                                   TP_BUTTON, TS_NORMAL, TMT_TEXTCOLOR, aColor);
         if (NS_SUCCEEDED(res))
@@ -266,8 +286,7 @@ nsLookAndFeel::NativeGetColor(ColorID aID, nscolor &aColor)
       idx = COLOR_WINDOWTEXT;
       break;
     case eColorID__moz_win_communicationstext:
-      if (WinUtils::GetWindowsVersion() >= WinUtils::VISTA_VERSION &&
-          IsAppThemed())
+      if (IsVistaOrLater() && IsAppThemed())
       {
         res = ::GetColorFromTheme(eUXCommunicationsToolbar,
                                   TP_BUTTON, TS_NORMAL, TMT_TEXTCOLOR, aColor);
@@ -350,11 +369,7 @@ nsLookAndFeel::GetIntImpl(IntID aID, int32_t &aResult)
         // High contrast is a misnomer under Win32 -- any theme can be used with it, 
         // e.g. normal contrast with large fonts, low contrast, etc.
         // The high contrast flag really means -- use this theme and don't override it.
-        HIGHCONTRAST contrastThemeInfo;
-        contrastThemeInfo.cbSize = sizeof(contrastThemeInfo);
-        ::SystemParametersInfo(SPI_GETHIGHCONTRAST, 0, &contrastThemeInfo, 0);
-
-        aResult = ((contrastThemeInfo.dwFlags & HCF_HIGHCONTRASTON) != 0);
+        aResult = nsUXThemeData::IsHighContrastOn();
         break;
     case eIntID_ScrollArrowStyle:
         aResult = eScrollArrowStyle_Single;
@@ -392,24 +407,7 @@ nsLookAndFeel::GetIntImpl(IntID aID, int32_t &aResult)
 
     case eIntID_OperatingSystemVersionIdentifier:
     {
-        switch(WinUtils::GetWindowsVersion()) {
-            case WinUtils::WINXP_VERSION:
-            case WinUtils::WIN2K3_VERSION:
-                aResult = LookAndFeel::eOperatingSystemVersion_WindowsXP;
-                break;
-            case WinUtils::VISTA_VERSION:
-                aResult = LookAndFeel::eOperatingSystemVersion_WindowsVista;
-                break;
-            case WinUtils::WIN7_VERSION:
-                aResult = LookAndFeel::eOperatingSystemVersion_Windows7;
-                break;
-            case WinUtils::WIN8_VERSION:
-                aResult = LookAndFeel::eOperatingSystemVersion_Windows8;
-                break;
-            default:
-                aResult = LookAndFeel::eOperatingSystemVersion_Unknown;
-                break;
-        }
+        aResult = GetOperatingSystemVersion();
         break;
     }
 
@@ -423,8 +421,7 @@ nsLookAndFeel::GetIntImpl(IntID aID, int32_t &aResult)
         break;
     case eIntID_WindowsGlass:
         // Aero Glass is only available prior to Windows 8 when DWM is used.
-        aResult = (nsUXThemeData::CheckForCompositor() &&
-                   WinUtils::GetWindowsVersion() < WinUtils::WIN8_VERSION);
+        aResult = (nsUXThemeData::CheckForCompositor() && !IsWin8OrLater());
         break;
     case eIntID_AlertNotificationOrigin:
         aResult = 0;
@@ -482,6 +479,10 @@ nsLookAndFeel::GetIntImpl(IntID aID, int32_t &aResult)
     case eIntID_SwipeAnimationEnabled:
         aResult = 0;
         break;
+    case eIntID_ColorPickerAvailable:
+        // We don't have a color picker implemented on Metro yet (bug 895464)
+        aResult = (XRE_GetWindowsEnvironment() != WindowsEnvironmentType_Metro);
+        break;
     case eIntID_UseOverlayScrollbars:
         aResult = (XRE_GetWindowsEnvironment() == WindowsEnvironmentType_Metro);
         break;
@@ -535,7 +536,7 @@ GetSysFontInfo(HDC aHDC, LookAndFeel::FontID anID,
   LOGFONTW logFont;
   NONCLIENTMETRICSW ncm;
   HGDIOBJ hGDI;
-  PRUnichar name[LF_FACESIZE];
+  char16_t name[LF_FACESIZE];
 
   // Depending on which stock font we want, there are three different
   // places we might have to look it up.
@@ -649,7 +650,7 @@ GetSysFontInfo(HDC aHDC, LookAndFeel::FontID anID,
   aFontStyle.systemFont = true;
 
   name[0] = 0;
-  memcpy(name, ptrLogFont->lfFaceName, LF_FACESIZE*sizeof(PRUnichar));
+  memcpy(name, ptrLogFont->lfFaceName, LF_FACESIZE*sizeof(char16_t));
   aFontName = name;
 
   return true;
@@ -669,7 +670,7 @@ nsLookAndFeel::GetFontImpl(FontID anID, nsString &aFontName,
 }
 
 /* virtual */
-PRUnichar
+char16_t
 nsLookAndFeel::GetPasswordCharacterImpl()
 {
 #define UNICODE_BLACK_CIRCLE_CHAR 0x25cf
