@@ -96,10 +96,10 @@ hashEnumerate(PLDHashTable* table, PLDHashEntryHdr* hdr, uint32_t i, void *arg)
 {
     _HashEnumerateArgs* thunk = (_HashEnumerateArgs*)arg;
     HTEntry* entry = static_cast<HTEntry*>(hdr);
-    
+
     if (thunk->fn(entry->key, entry->value, thunk->arg))
         return PL_DHASH_NEXT;
-    return PL_DHASH_STOP;           
+    return PL_DHASH_STOP;
 }
 
 //
@@ -118,21 +118,20 @@ nsHashKey::Write(nsIObjectOutputStream* aStream) const
     return NS_ERROR_NOT_IMPLEMENTED;
 }
 
-nsHashtable::nsHashtable(uint32_t aInitSize, bool threadSafe)
+nsHashtable::nsHashtable(uint32_t aInitSize, bool aThreadSafe)
   : mLock(nullptr), mEnumerating(false)
 {
     MOZ_COUNT_CTOR(nsHashtable);
 
     bool result = PL_DHashTableInit(&mHashtable, &hashtableOps, nullptr,
-                                      sizeof(HTEntry), aInitSize);
-    
+                                    sizeof(HTEntry), aInitSize, fallible_t());
     NS_ASSERTION(result, "Hashtable failed to initialize");
 
     // make sure we detect this later
     if (!result)
         mHashtable.ops = nullptr;
-    
-    if (threadSafe) {
+
+    if (aThreadSafe) {
         mLock = PR_NewLock();
         if (mLock == nullptr) {
             // Cannot create a lock. If running on a multiprocessing system
@@ -142,7 +141,6 @@ nsHashtable::nsHashtable(uint32_t aInitSize, bool threadSafe)
     }
 }
 
-
 nsHashtable::~nsHashtable() {
     MOZ_COUNT_DTOR(nsHashtable);
     if (mHashtable.ops)
@@ -158,12 +156,12 @@ bool nsHashtable::Exists(nsHashKey *aKey)
         if (mLock) PR_Unlock(mLock);
         return false;
     }
-    
+
     PLDHashEntryHdr *entry =
         PL_DHashTableOperate(&mHashtable, aKey, PL_DHASH_LOOKUP);
-    
+
     bool exists = PL_DHASH_ENTRY_IS_BUSY(entry);
-    
+
     if (mLock) PR_Unlock(mLock);
 
     return exists;
@@ -174,16 +172,16 @@ void *nsHashtable::Put(nsHashKey *aKey, void *aData)
     void *res =  nullptr;
 
     if (!mHashtable.ops) return nullptr;
-    
+
     if (mLock) PR_Lock(mLock);
 
     // shouldn't be adding an item during enumeration
     PR_ASSERT(!mEnumerating);
-    
+
     HTEntry* entry =
         static_cast<HTEntry*>
                    (PL_DHashTableOperate(&mHashtable, aKey, PL_DHASH_ADD));
-    
+
     if (entry) {                // don't return early, or you'll be locked!
         if (entry->key) {
             // existing entry, need to boot the old value
@@ -204,14 +202,14 @@ void *nsHashtable::Put(nsHashKey *aKey, void *aData)
 void *nsHashtable::Get(nsHashKey *aKey)
 {
     if (!mHashtable.ops) return nullptr;
-    
+
     if (mLock) PR_Lock(mLock);
 
     HTEntry* entry =
         static_cast<HTEntry*>
                    (PL_DHashTableOperate(&mHashtable, aKey, PL_DHASH_LOOKUP));
     void *ret = PL_DHASH_ENTRY_IS_BUSY(entry) ? entry->value : nullptr;
-    
+
     if (mLock) PR_Unlock(mLock);
 
     return ret;
@@ -220,7 +218,7 @@ void *nsHashtable::Get(nsHashKey *aKey)
 void *nsHashtable::Remove(nsHashKey *aKey)
 {
     if (!mHashtable.ops) return nullptr;
-    
+
     if (mLock) PR_Lock(mLock);
 
     // shouldn't be adding an item during enumeration
@@ -233,7 +231,7 @@ void *nsHashtable::Remove(nsHashKey *aKey)
         static_cast<HTEntry*>
                    (PL_DHashTableOperate(&mHashtable, aKey, PL_DHASH_LOOKUP));
     void *res;
-    
+
     if (PL_DHASH_ENTRY_IS_FREE(entry)) {
         // value wasn't in the table anyway
         res = nullptr;
@@ -256,7 +254,7 @@ hashEnumerateShare(PLDHashTable *table, PLDHashEntryHdr *hdr,
 {
     nsHashtable *newHashtable = (nsHashtable *)arg;
     HTEntry * entry = static_cast<HTEntry*>(hdr);
-    
+
     newHashtable->Put(entry->key, entry->value);
     return PL_DHASH_NEXT;
 }
@@ -264,7 +262,7 @@ hashEnumerateShare(PLDHashTable *table, PLDHashEntryHdr *hdr,
 nsHashtable * nsHashtable::Clone()
 {
     if (!mHashtable.ops) return nullptr;
-    
+
     bool threadSafe = (mLock != nullptr);
     nsHashtable *newHashTable = new nsHashtable(mHashtable.entryCount, threadSafe);
 
@@ -275,7 +273,7 @@ nsHashtable * nsHashtable::Clone()
 void nsHashtable::Enumerate(nsHashtableEnumFunc aEnumFunc, void* aClosure)
 {
     if (!mHashtable.ops) return;
-    
+
     bool wasEnumerating = mEnumerating;
     mEnumerating = true;
     _HashEnumerateArgs thunk;
@@ -305,7 +303,7 @@ void nsHashtable::Reset() {
 void nsHashtable::Reset(nsHashtableEnumFunc destroyFunc, void* aClosure)
 {
     if (!mHashtable.ops) return;
-    
+
     _HashEnumerateArgs thunk, *thunkp;
     if (!destroyFunc) {
         thunkp = nullptr;
@@ -344,7 +342,8 @@ nsHashtable::nsHashtable(nsIObjectInputStream* aStream,
             if (NS_SUCCEEDED(rv)) {
                 bool status =
                     PL_DHashTableInit(&mHashtable, &hashtableOps,
-                                      nullptr, sizeof(HTEntry), count);
+                                      nullptr, sizeof(HTEntry), count,
+                                      fallible_t());
                 if (!status) {
                     mHashtable.ops = nullptr;
                     rv = NS_ERROR_OUT_OF_MEMORY;
@@ -432,7 +431,7 @@ nsISupportsKey::Write(nsIObjectOutputStream* aStream) const
 }
 
 // Copy Constructor
-// We need to free mStr if the object is passed with mOwnership as OWN. As the 
+// We need to free mStr if the object is passed with mOwnership as OWN. As the
 // destructor here is freeing mStr in that case, mStr is NOT getting leaked here.
 
 nsCStringKey::nsCStringKey(const nsCStringKey& aKey)
@@ -559,15 +558,15 @@ nsCStringKey::Write(nsIObjectOutputStream* aStream) const
 ////////////////////////////////////////////////////////////////////////////////
 
 // Copy Constructor
-// We need to free mStr if the object is passed with mOwnership as OWN. As the 
+// We need to free mStr if the object is passed with mOwnership as OWN. As the
 // destructor here is freeing mStr in that case, mStr is NOT getting leaked here.
 
 nsStringKey::nsStringKey(const nsStringKey& aKey)
     : mStr(aKey.mStr), mStrLen(aKey.mStrLen), mOwnership(aKey.mOwnership)
 {
     if (mOwnership != NEVER_OWN) {
-        uint32_t len = mStrLen * sizeof(PRUnichar);
-        PRUnichar* str = reinterpret_cast<PRUnichar*>(nsMemory::Alloc(len + sizeof(PRUnichar)));
+        uint32_t len = mStrLen * sizeof(char16_t);
+        char16_t* str = reinterpret_cast<char16_t*>(nsMemory::Alloc(len + sizeof(char16_t)));
         if (!str) {
             // Pray we don't dangle!
             mOwnership = NEVER_OWN;
@@ -586,7 +585,7 @@ nsStringKey::nsStringKey(const nsStringKey& aKey)
 }
 
 nsStringKey::nsStringKey(const nsAFlatString& str)
-    : mStr(const_cast<PRUnichar*>(str.get())),
+    : mStr((char16_t*)str.get()),
       mStrLen(str.Length()),
       mOwnership(OWN_CLONE)
 {
@@ -609,8 +608,8 @@ nsStringKey::nsStringKey(const nsAString& str)
     MOZ_COUNT_CTOR(nsStringKey);
 }
 
-nsStringKey::nsStringKey(const PRUnichar* str, int32_t strLen, Ownership own)
-    : mStr((PRUnichar*)str), mStrLen(strLen), mOwnership(own)
+nsStringKey::nsStringKey(const char16_t* str, int32_t strLen, Ownership own)
+    : mStr((char16_t*)str), mStrLen(strLen), mOwnership(own)
 {
     NS_ASSERTION(mStr, "null string key");
     if (mStrLen == uint32_t(-1))
@@ -643,7 +642,7 @@ nsStringKey::Equals(const nsHashKey* aKey) const
     NS_ASSERTION(other->mStrLen != uint32_t(-1), "never called HashCode");
     if (mStrLen != other->mStrLen)
         return false;
-    return memcmp(mStr, other->mStr, mStrLen * sizeof(PRUnichar)) == 0;
+    return memcmp(mStr, other->mStr, mStrLen * sizeof(char16_t)) == 0;
 }
 
 nsHashKey*
@@ -652,8 +651,8 @@ nsStringKey::Clone() const
     if (mOwnership == NEVER_OWN)
         return new nsStringKey(mStr, mStrLen, NEVER_OWN);
 
-    uint32_t len = (mStrLen+1) * sizeof(PRUnichar);
-    PRUnichar* str = (PRUnichar*)nsMemory::Alloc(len);
+    uint32_t len = (mStrLen+1) * sizeof(char16_t);
+    char16_t* str = (char16_t*)nsMemory::Alloc(len);
     if (str == nullptr)
         return nullptr;
     memcpy(str, mStr, len);
@@ -708,7 +707,7 @@ nsObjectHashtable::CopyElement(PLDHashTable* table,
 {
     nsObjectHashtable *newHashtable = (nsObjectHashtable *)arg;
     HTEntry *entry = static_cast<HTEntry*>(hdr);
-    
+
     void* newElement =
         newHashtable->mCloneElementFun(entry->key, entry->value,
                                        newHashtable->mCloneElementClosure);
@@ -722,7 +721,7 @@ nsHashtable*
 nsObjectHashtable::Clone()
 {
     if (!mHashtable.ops) return nullptr;
-    
+
     bool threadSafe = false;
     if (mLock)
         threadSafe = true;
@@ -749,96 +748,3 @@ nsObjectHashtable::RemoveAndDelete(nsHashKey *aKey)
         return !!(*mDestroyElementFun)(aKey, value, mDestroyElementClosure);
     return false;
 }
-
-////////////////////////////////////////////////////////////////////////////////
-// nsSupportsHashtable: an nsHashtable where the elements are nsISupports*
-
-bool
-nsSupportsHashtable::ReleaseElement(nsHashKey *aKey, void *aData, void* aClosure)
-{
-    nsISupports* element = static_cast<nsISupports*>(aData);
-    NS_IF_RELEASE(element);
-    return true;
-}
-
-nsSupportsHashtable::~nsSupportsHashtable()
-{
-    Enumerate(ReleaseElement, nullptr);
-}
-
-// Return true if we overwrote something
-
-bool
-nsSupportsHashtable::Put(nsHashKey *aKey, nsISupports* aData, nsISupports **value)
-{
-    NS_IF_ADDREF(aData);
-    void *prev = nsHashtable::Put(aKey, aData);
-    nsISupports *old = reinterpret_cast<nsISupports *>(prev);
-    if (value)  // pass own the ownership to the caller
-        *value = old;
-    else        // the caller doesn't care, we do
-        NS_IF_RELEASE(old);
-    return prev != nullptr;
-}
-
-nsISupports *
-nsSupportsHashtable::Get(nsHashKey *aKey)
-{
-    void* data = nsHashtable::Get(aKey);
-    if (!data)
-        return nullptr;
-    nsISupports* element = reinterpret_cast<nsISupports*>(data);
-    NS_IF_ADDREF(element);
-    return element;
-}
-
-// Return true if we found something (useful for checks)
-
-bool
-nsSupportsHashtable::Remove(nsHashKey *aKey, nsISupports **value)
-{
-    void* data = nsHashtable::Remove(aKey);
-    nsISupports* element = static_cast<nsISupports*>(data);
-    if (value)            // caller wants it
-        *value = element;
-    else                  // caller doesn't care, we do
-        NS_IF_RELEASE(element);
-    return data != nullptr;
-}
-
-PLDHashOperator
-nsSupportsHashtable::EnumerateCopy(PLDHashTable*,
-                                   PLDHashEntryHdr* hdr,
-                                   uint32_t i, void *arg)
-{
-    nsHashtable *newHashtable = (nsHashtable *)arg;
-    HTEntry* entry = static_cast<HTEntry*>(hdr);
-    
-    nsISupports* element = static_cast<nsISupports*>(entry->value);
-    NS_IF_ADDREF(element);
-    newHashtable->Put(entry->key, entry->value);
-    return PL_DHASH_NEXT;
-}
-
-nsHashtable*
-nsSupportsHashtable::Clone()
-{
-    if (!mHashtable.ops) return nullptr;
-    
-    bool threadSafe = (mLock != nullptr);
-    nsSupportsHashtable* newHashTable =
-        new nsSupportsHashtable(mHashtable.entryCount, threadSafe);
-
-    PL_DHashTableEnumerate(&mHashtable, EnumerateCopy, newHashTable);
-    return newHashTable;
-}
-
-void
-nsSupportsHashtable::Reset()
-{
-    Enumerate(ReleaseElement, nullptr);
-    nsHashtable::Reset();
-}
-
-////////////////////////////////////////////////////////////////////////////////
-
