@@ -5,10 +5,11 @@
 #ifndef nsIClassInfoImpl_h__
 #define nsIClassInfoImpl_h__
 
+#include "mozilla/Alignment.h"
 #include "nsIClassInfo.h"
 #include "nsISupportsImpl.h"
 
-#include NEW_H
+#include <new>
 
 /**
  * This header file provides macros which help you make your class implement
@@ -30,16 +31,16 @@
  *
  * Change this to
  *
- *   NS_IMPL_CLASSINFO(nsFooBar, NULL, 0, NS_FOOBAR_CID)
+ *   NS_IMPL_CLASSINFO(nsFooBar, nullptr, 0, NS_FOOBAR_CID)
  *   NS_IMPL_ISUPPORTS2_CI(nsFooBar, nsIFoo, nsIBar)
  *
  * If nsFooBar is threadsafe, change the 0 above to nsIClassInfo::THREADSAFE.
  * If it's a singleton, use nsIClassInfo::SINGLETON.  The full list of flags is
  * in nsIClassInfo.idl.
  *
- * The NULL parameter is there so you can pass a function for converting from
- * an XPCOM object to a scriptable helper.  Unless you're doing specialized JS
- * work, you can probably leave this as NULL.
+ * The nullptr parameter is there so you can pass a function for converting
+ * from an XPCOM object to a scriptable helper.  Unless you're doing
+ * specialized JS work, you can probably leave this as nullptr.
  *
  * This file also defines the NS_IMPL_QUERY_INTERFACE2_CI macro, which you can
  * use to replace NS_IMPL_QUERY_INTERFACE2, if you use that instead of
@@ -83,14 +84,14 @@ class NS_COM_GLUE GenericClassInfo : public nsIClassInfo
 public:
   struct ClassInfoData
   {
-    typedef NS_CALLBACK(GetInterfacesProc)(PRUint32* NS_OUTPARAM countp,
-                                           nsIID*** NS_OUTPARAM array);
-    typedef NS_CALLBACK(GetLanguageHelperProc)(PRUint32 language,
+    typedef NS_CALLBACK(GetInterfacesProc)(uint32_t* countp,
+                                           nsIID*** array);
+    typedef NS_CALLBACK(GetLanguageHelperProc)(uint32_t language,
                                                nsISupports** helper);
 
     GetInterfacesProc getinterfaces;
     GetLanguageHelperProc getlanguagehelper;
-    PRUint32 flags;
+    uint32_t flags;
     nsCID cid;
   };
 
@@ -109,7 +110,7 @@ private:
 #define NS_CI_INTERFACE_GETTER_NAME(_class) _class##_GetInterfacesHelper
 #define NS_DECL_CI_INTERFACE_GETTER(_class)                                   \
   extern NS_IMETHODIMP NS_CI_INTERFACE_GETTER_NAME(_class)                    \
-     (PRUint32 * NS_OUTPARAM, nsIID *** NS_OUTPARAM);
+     (uint32_t *, nsIID ***);
 
 #define NS_IMPL_CLASSINFO(_class, _getlanguagehelper, _flags, _cid)     \
   NS_DECL_CI_INTERFACE_GETTER(_class)                                   \
@@ -119,21 +120,20 @@ private:
     _flags | nsIClassInfo::SINGLETON_CLASSINFO,                         \
     _cid,                                                               \
   };                                                                    \
-  static char k##_class##ClassInfoDataPlace[sizeof(GenericClassInfo)];  \
-  nsIClassInfo* NS_CLASSINFO_NAME(_class) = NULL;
+  mozilla::AlignedStorage2<GenericClassInfo> k##_class##ClassInfoDataPlace;   \
+  nsIClassInfo* NS_CLASSINFO_NAME(_class) = nullptr;
 
 #define NS_IMPL_QUERY_CLASSINFO(_class)                                       \
   if ( aIID.Equals(NS_GET_IID(nsIClassInfo)) ) {                              \
     if (!NS_CLASSINFO_NAME(_class))                                           \
-      NS_CLASSINFO_NAME(_class) = new (k##_class##ClassInfoDataPlace)         \
+      NS_CLASSINFO_NAME(_class) = new (k##_class##ClassInfoDataPlace.addr())  \
         GenericClassInfo(&k##_class##ClassInfoData);                          \
     foundInterface = NS_CLASSINFO_NAME(_class);                               \
   } else
 
 #define NS_CLASSINFO_HELPER_BEGIN(_class, _c)                                 \
 NS_IMETHODIMP                                                                 \
-NS_CI_INTERFACE_GETTER_NAME(_class)(PRUint32 *count NS_OUTPARAM,              \
-                                    nsIID ***array NS_OUTPARAM)               \
+NS_CI_INTERFACE_GETTER_NAME(_class)(uint32_t *count, nsIID ***array)          \
 {                                                                             \
     *count = _c;                                                              \
     *array = (nsIID **)nsMemory::Alloc(sizeof (nsIID *) * _c);
