@@ -4,7 +4,7 @@
  * License, v. 2.0. If a copy of the MPL was not distributed with this
  * file, You can obtain one at http://mozilla.org/MPL/2.0/. */
 
-#include "mozilla/Util.h"
+#include "mozilla/ArrayUtils.h"
 
 #include "nsCOMPtr.h"
 #include "nsAutoPtr.h"
@@ -33,8 +33,6 @@
 #include <CoreServices/CoreServices.h>
 #include <Carbon/Carbon.h>
 #endif
-#elif defined(XP_OS2)
-#define MAX_PATH _MAX_PATH
 #endif
 
 #include "SpecialSystemDirectory.h"
@@ -42,8 +40,6 @@
 
 using namespace mozilla;
 
-#define COMPONENT_DIRECTORY     NS_LITERAL_CSTRING("components")
-
 // define home directory
 // For Windows platform, We are choosing Appdata folder as HOME
 #if defined (XP_WIN)
@@ -52,8 +48,6 @@ using namespace mozilla;
 #define HOME_DIR NS_OSX_HOME_DIR
 #elif defined (XP_UNIX)
 #define HOME_DIR NS_UNIX_HOME_DIR
-#elif defined (XP_OS2)
-#define HOME_DIR NS_OS2_HOME_DIR
 #endif
 
 //----------------------------------------------------------------------------------------
@@ -61,8 +55,9 @@ nsresult
 nsDirectoryService::GetCurrentProcessDirectory(nsIFile** aFile)
 //----------------------------------------------------------------------------------------
 {
-    NS_ENSURE_ARG_POINTER(aFile);
-    *aFile = nsnull;
+    if (NS_WARN_IF(!aFile))
+        return NS_ERROR_INVALID_ARG;
+    *aFile = nullptr;
     
    //  Set the component registry location:
     if (!gService)
@@ -71,9 +66,11 @@ nsDirectoryService::GetCurrentProcessDirectory(nsIFile** aFile)
     nsresult rv; 
  
     nsCOMPtr<nsIProperties> dirService;
-    rv = nsDirectoryService::Create(nsnull, 
+    rv = nsDirectoryService::Create(nullptr, 
                                     NS_GET_IID(nsIProperties), 
                                     getter_AddRefs(dirService));  // needs to be around for life of product
+    if (NS_FAILED(rv))
+        return rv;
 
     if (dirService)
     {
@@ -89,19 +86,19 @@ nsDirectoryService::GetCurrentProcessDirectory(nsIFile** aFile)
 
     nsLocalFile* localFile = new nsLocalFile;
 
-    if (localFile == nsnull)
+    if (localFile == nullptr)
         return NS_ERROR_OUT_OF_MEMORY;
     NS_ADDREF(localFile);
 
 
 
 #ifdef XP_WIN
-    PRUnichar buf[MAX_PATH + 1];
+    wchar_t buf[MAX_PATH + 1];
     SetLastError(ERROR_SUCCESS);
     if (GetModuleFileNameW(0, buf, mozilla::ArrayLength(buf)) &&
         GetLastError() != ERROR_INSUFFICIENT_BUFFER) {
         // chop off the executable name by finding the rightmost backslash
-        PRUnichar* lastSlash = wcsrchr(buf, L'\\');
+        wchar_t* lastSlash = wcsrchr(buf, L'\\');
         if (lastSlash)
             *(lastSlash + 1) = L'\0';
 
@@ -113,10 +110,10 @@ nsDirectoryService::GetCurrentProcessDirectory(nsIFile** aFile)
 #elif defined(MOZ_WIDGET_COCOA)
     // Works even if we're not bundled.
     CFBundleRef appBundle = CFBundleGetMainBundle();
-    if (appBundle != nsnull)
+    if (appBundle != nullptr)
     {
         CFURLRef bundleURL = CFBundleCopyExecutableURL(appBundle);
-        if (bundleURL != nsnull)
+        if (bundleURL != nullptr)
         {
             CFURLRef parentURL = CFURLCreateCopyDeletingLastPathComponent(kCFAllocatorDefault, bundleURL);
             if (parentURL)
@@ -196,17 +193,6 @@ nsDirectoryService::GetCurrentProcessDirectory(nsIFile** aFile)
         return NS_OK;
     }
 
-#elif defined(XP_OS2)
-    PPIB ppib;
-    PTIB ptib;
-    char buffer[CCHMAXPATH];
-    DosGetInfoBlocks( &ptib, &ppib);
-    DosQueryModuleName( ppib->pib_hmte, CCHMAXPATH, buffer);
-    *strrchr( buffer, '\\') = '\0'; // XXX DBCS misery
-    localFile->InitWithNativePath(nsDependentCString(buffer));
-    *aFile = localFile;
-    return NS_OK;
-
 #endif
     
     NS_RELEASE(localFile);
@@ -215,18 +201,20 @@ nsDirectoryService::GetCurrentProcessDirectory(nsIFile** aFile)
     return NS_ERROR_FAILURE;
 } // GetCurrentProcessDirectory()
 
-nsDirectoryService* nsDirectoryService::gService = nsnull;
+nsDirectoryService* nsDirectoryService::gService = nullptr;
 
-nsDirectoryService::nsDirectoryService() :
-    mHashtable(256, true)
+nsDirectoryService::nsDirectoryService()
+    : mHashtable(256)
 {
 }
 
 nsresult
 nsDirectoryService::Create(nsISupports *outer, REFNSIID aIID, void **aResult)
 {
-    NS_ENSURE_ARG_POINTER(aResult);
-    NS_ENSURE_NO_AGGREGATION(outer);
+    if (NS_WARN_IF(!aResult))
+        return NS_ERROR_INVALID_ARG;
+    if (NS_WARN_IF(outer))
+        return NS_ERROR_NO_AGGREGATION;
 
     if (!gService)
     {
@@ -236,7 +224,7 @@ nsDirectoryService::Create(nsISupports *outer, REFNSIID aIID, void **aResult)
     return gService->QueryInterface(aIID, aResult);
 }
 
-#define DIR_ATOM(name_, value_) nsIAtom* nsDirectoryService::name_ = nsnull;
+#define DIR_ATOM(name_, value_) nsIAtom* nsDirectoryService::name_ = nullptr;
 #include "nsDirectoryServiceAtomList.h"
 #undef DIR_ATOM
 
@@ -257,67 +245,46 @@ nsDirectoryService::Init()
     return NS_OK;
 }
 
-nsresult
+void
 nsDirectoryService::RealInit()
 {
     NS_ASSERTION(!gService, 
                  "nsDirectoryService::RealInit Mustn't initialize twice!");
 
-    nsresult rv;
-
     nsRefPtr<nsDirectoryService> self = new nsDirectoryService();
-    if (!self)
-        return NS_ERROR_OUT_OF_MEMORY;
-
-    rv = NS_NewISupportsArray(getter_AddRefs(((nsDirectoryService*) self)->mProviders));
-    if (NS_FAILED(rv))
-        return rv;
 
     NS_RegisterStaticAtoms(directory_atoms);
     
     // Let the list hold the only reference to the provider.
     nsAppFileLocationProvider *defaultProvider = new nsAppFileLocationProvider;
-    if (!defaultProvider)
-        return NS_ERROR_OUT_OF_MEMORY;
-    // AppendElement returns true for success.
-    rv = ((nsDirectoryService*) self)->mProviders->AppendElement(defaultProvider) ? NS_OK : NS_ERROR_FAILURE;
-    if (NS_FAILED(rv))
-        return rv;
+    self->mProviders.AppendElement(defaultProvider);
 
     self.swap(gService);
-    return NS_OK;
-}
-
-bool
-nsDirectoryService::ReleaseValues(nsHashKey* key, void* data, void* closure)
-{
-    nsISupports* value = (nsISupports*)data;
-    NS_IF_RELEASE(value);
-    return true;
 }
 
 nsDirectoryService::~nsDirectoryService()
 {
 }
 
-NS_IMPL_THREADSAFE_ISUPPORTS4(nsDirectoryService, nsIProperties, nsIDirectoryService, nsIDirectoryServiceProvider, nsIDirectoryServiceProvider2)
+NS_IMPL_ISUPPORTS4(nsDirectoryService, nsIProperties, nsIDirectoryService, nsIDirectoryServiceProvider, nsIDirectoryServiceProvider2)
 
 
 NS_IMETHODIMP
 nsDirectoryService::Undefine(const char* prop)
 {
-    NS_ENSURE_ARG(prop);
+    if (NS_WARN_IF(!prop))
+        return NS_ERROR_INVALID_ARG;
 
-    nsCStringKey key(prop);
-    if (!mHashtable.Exists(&key))
+    nsDependentCString key(prop);
+    if (!mHashtable.Get(key, nullptr))
         return NS_ERROR_FAILURE;
 
-    mHashtable.Remove (&key);
+    mHashtable.Remove(key);
     return NS_OK;
  }
 
 NS_IMETHODIMP
-nsDirectoryService::GetKeys(PRUint32 *count, char ***keys)
+nsDirectoryService::GetKeys(uint32_t *count, char ***keys)
 {
     return NS_ERROR_NOT_IMPLEMENTED;
 }
@@ -327,7 +294,7 @@ struct FileData
   FileData(const char* aProperty,
            const nsIID& aUUID) :
     property(aProperty),
-    data(nsnull),
+    data(nullptr),
     persistent(true),
     uuid(aUUID) {}
     
@@ -337,47 +304,43 @@ struct FileData
   const nsIID&  uuid;
 };
 
-static bool FindProviderFile(nsISupports* aElement, void *aData)
+static bool FindProviderFile(nsIDirectoryServiceProvider* aElement,
+                             FileData* aData)
 {
   nsresult rv;
-  FileData* fileData = (FileData*)aData;
-  if (fileData->uuid.Equals(NS_GET_IID(nsISimpleEnumerator)))
-  {
+  if (aData->uuid.Equals(NS_GET_IID(nsISimpleEnumerator))) {
       // Not all providers implement this iface
       nsCOMPtr<nsIDirectoryServiceProvider2> prov2 = do_QueryInterface(aElement);
       if (prov2)
       {
           nsCOMPtr<nsISimpleEnumerator> newFiles;
-          rv = prov2->GetFiles(fileData->property, getter_AddRefs(newFiles));
+          rv = prov2->GetFiles(aData->property, getter_AddRefs(newFiles));
           if (NS_SUCCEEDED(rv) && newFiles) {
-              if (fileData->data) {
+              if (aData->data) {
                   nsCOMPtr<nsISimpleEnumerator> unionFiles;
 
                   NS_NewUnionEnumerator(getter_AddRefs(unionFiles),
-                                        (nsISimpleEnumerator*) fileData->data, newFiles);
+                                        (nsISimpleEnumerator*) aData->data, newFiles);
 
                   if (unionFiles)
-                      unionFiles.swap(* (nsISimpleEnumerator**) &fileData->data);
+                      unionFiles.swap(* (nsISimpleEnumerator**) &aData->data);
               }
               else
               {
-                  NS_ADDREF(fileData->data = newFiles);
+                  NS_ADDREF(aData->data = newFiles);
               }
                   
-              fileData->persistent = false; // Enumerators can never be persistent
+              aData->persistent = false; // Enumerators can never be persistent
               return rv == NS_SUCCESS_AGGREGATE_RESULT;
           }
       }
   }
   else
   {
-      nsCOMPtr<nsIDirectoryServiceProvider> prov = do_QueryInterface(aElement);
-      if (prov)
-      {
-          rv = prov->GetFile(fileData->property, &fileData->persistent, (nsIFile **)&fileData->data);
-          if (NS_SUCCEEDED(rv) && fileData->data)
-              return false;
-      }
+      rv = aElement->GetFile(aData->property, &aData->persistent,
+                             (nsIFile **)&aData->data);
+      if (NS_SUCCEEDED(rv) && aData->data)
+          return false;
   }
 
   return true;
@@ -386,19 +349,15 @@ static bool FindProviderFile(nsISupports* aElement, void *aData)
 NS_IMETHODIMP
 nsDirectoryService::Get(const char* prop, const nsIID & uuid, void* *result)
 {
-    NS_ENSURE_ARG(prop);
+    if (NS_WARN_IF(!prop))
+        return NS_ERROR_INVALID_ARG;
 
-    nsCStringKey key(prop);
-    
-    nsCOMPtr<nsISupports> value = dont_AddRef(mHashtable.Get(&key));
-    
-    if (value)
-    {
+    nsDependentCString key(prop);
+
+    nsCOMPtr<nsIFile> cachedFile = mHashtable.Get(key);
+
+    if (cachedFile) {
         nsCOMPtr<nsIFile> cloneFile;
-        nsCOMPtr<nsIFile> cachedFile = do_QueryInterface(value);
-        NS_ASSERTION(cachedFile, 
-                     "nsDirectoryService::Get nsIFile expected");
-
         cachedFile->Clone(getter_AddRefs(cloneFile));
         return cloneFile->QueryInterface(uuid, result);
     }
@@ -406,7 +365,11 @@ nsDirectoryService::Get(const char* prop, const nsIID & uuid, void* *result)
     // it is not one of our defaults, lets check any providers
     FileData fileData(prop, uuid);
 
-    mProviders->EnumerateBackwards(FindProviderFile, &fileData);
+    for (int32_t i = mProviders.Length() - 1; i >= 0; i--) {
+        if (!FindProviderFile(mProviders[i], &fileData)) {
+            break;
+        }
+    }
     if (fileData.data)
     {
         if (fileData.persistent)
@@ -436,30 +399,31 @@ nsDirectoryService::Get(const char* prop, const nsIID & uuid, void* *result)
 NS_IMETHODIMP
 nsDirectoryService::Set(const char* prop, nsISupports* value)
 {
-    NS_ENSURE_ARG(prop);
+    if (NS_WARN_IF(!prop))
+        return NS_ERROR_INVALID_ARG;
 
-    nsCStringKey key(prop);
-    if (mHashtable.Exists(&key) || value == nsnull)
+    nsDependentCString key(prop);
+    if (mHashtable.Get(key, nullptr) || !value) {
         return NS_ERROR_FAILURE;
+    }
 
-    nsCOMPtr<nsIFile> ourFile;
-    value->QueryInterface(NS_GET_IID(nsIFile), getter_AddRefs(ourFile));
-    if (ourFile)
-    {
+    nsCOMPtr<nsIFile> ourFile = do_QueryInterface(value);
+    if (ourFile) {
       nsCOMPtr<nsIFile> cloneFile;
       ourFile->Clone (getter_AddRefs (cloneFile));
-      mHashtable.Put(&key, cloneFile);
+      mHashtable.Put(key, cloneFile);
 
       return NS_OK;
     }
 
-    return NS_ERROR_FAILURE;   
+    return NS_ERROR_FAILURE;
 }
 
 NS_IMETHODIMP
 nsDirectoryService::Has(const char *prop, bool *_retval)
 {
-    NS_ENSURE_ARG(prop);
+    if (NS_WARN_IF(!prop))
+        return NS_ERROR_INVALID_ARG;
 
     *_retval = false;
     nsCOMPtr<nsIFile> value;
@@ -478,17 +442,11 @@ nsDirectoryService::Has(const char *prop, bool *_retval)
 NS_IMETHODIMP
 nsDirectoryService::RegisterProvider(nsIDirectoryServiceProvider *prov)
 {
-    nsresult rv;
     if (!prov)
         return NS_ERROR_FAILURE;
-    if (!mProviders)
-        return NS_ERROR_NOT_INITIALIZED;
 
-    nsCOMPtr<nsISupports> supports = do_QueryInterface(prov, &rv);
-    if (NS_FAILED(rv)) return rv;
-
-    // AppendElement returns true for success.
-    return mProviders->AppendElement(supports) ? NS_OK : NS_ERROR_FAILURE;
+    mProviders.AppendElement(prov);
+    return NS_OK;
 }
 
 void
@@ -509,7 +467,7 @@ nsDirectoryService::RegisterCategoryProviders()
 
     bool more;
     while (NS_SUCCEEDED(strings->HasMore(&more)) && more) {
-        nsCAutoString entry;
+        nsAutoCString entry;
         strings->GetNext(entry);
 
         nsXPIDLCString contractID;
@@ -526,17 +484,11 @@ nsDirectoryService::RegisterCategoryProviders()
 NS_IMETHODIMP
 nsDirectoryService::UnregisterProvider(nsIDirectoryServiceProvider *prov)
 {
-    nsresult rv;
     if (!prov)
         return NS_ERROR_FAILURE;
-    if (!mProviders)
-        return NS_ERROR_NOT_INITIALIZED;
 
-    nsCOMPtr<nsISupports> supports = do_QueryInterface(prov, &rv);
-    if (NS_FAILED(rv)) return rv;
-
-    // RemoveElement returns true for success.
-    return mProviders->RemoveElement(supports) ? NS_OK : NS_ERROR_FAILURE;
+    mProviders.RemoveElement(prov);
+    return NS_OK;
 }
 
 // DO NOT ADD ANY LOCATIONS TO THIS FUNCTION UNTIL YOU TALK TO: dougt@netscape.com.
@@ -550,7 +502,7 @@ nsDirectoryService::GetFile(const char *prop, bool *persistent, nsIFile **_retva
     nsCOMPtr<nsIFile> localFile;
     nsresult rv = NS_ERROR_FAILURE;
 
-    *_retval = nsnull;
+    *_retval = nullptr;
     *persistent = true;
 
     nsCOMPtr<nsIAtom> inAtom = do_GetAtom(prop);
@@ -847,6 +799,22 @@ nsDirectoryService::GetFile(const char *prop, bool *persistent, nsIFile **_retva
     {
         rv = GetSpecialSystemDirectory(Win_Downloads, getter_AddRefs(localFile));
     }
+    else if (inAtom == nsDirectoryService::sDocs)
+    {
+        rv = GetSpecialSystemDirectory(Win_Documents, getter_AddRefs(localFile));
+    }
+    else if (inAtom == nsDirectoryService::sPictures)
+    {
+        rv = GetSpecialSystemDirectory(Win_Pictures, getter_AddRefs(localFile));
+    }
+    else if (inAtom == nsDirectoryService::sMusic)
+    {
+        rv = GetSpecialSystemDirectory(Win_Music, getter_AddRefs(localFile));
+    }
+    else if (inAtom == nsDirectoryService::sVideos)
+    {
+        rv = GetSpecialSystemDirectory(Win_Videos, getter_AddRefs(localFile));
+    }
 #elif defined (XP_UNIX)
 
     else if (inAtom == nsDirectoryService::sLocalDirectory)
@@ -903,23 +871,6 @@ nsDirectoryService::GetFile(const char *prop, bool *persistent, nsIFile **_retva
         rv = GetSpecialSystemDirectory(Unix_XDG_Videos, getter_AddRefs(localFile));
         *persistent = false;
     }
-#elif defined (XP_OS2)
-    else if (inAtom == nsDirectoryService::sSystemDirectory)
-    {
-        rv = GetSpecialSystemDirectory(OS2_SystemDirectory, getter_AddRefs(localFile)); 
-    }
-    else if (inAtom == nsDirectoryService::sOS2Directory)
-    {
-        rv = GetSpecialSystemDirectory(OS2_OS2Directory, getter_AddRefs(localFile)); 
-    }
-    else if (inAtom == nsDirectoryService::sOS_HomeDirectory)
-    {
-        rv = GetSpecialSystemDirectory(OS2_HomeDirectory, getter_AddRefs(localFile)); 
-    }
-    else if (inAtom == nsDirectoryService::sOS_DesktopDirectory)
-    {
-        rv = GetSpecialSystemDirectory(OS2_DesktopDirectory, getter_AddRefs(localFile)); 
-    }
 #endif
 
     if (NS_FAILED(rv))
@@ -934,8 +885,9 @@ nsDirectoryService::GetFile(const char *prop, bool *persistent, nsIFile **_retva
 NS_IMETHODIMP
 nsDirectoryService::GetFiles(const char *prop, nsISimpleEnumerator **_retval)
 {
-    NS_ENSURE_ARG_POINTER(_retval);
-    *_retval = nsnull;
+    if (NS_WARN_IF(!_retval))
+        return NS_ERROR_INVALID_ARG;
+    *_retval = nullptr;
         
     return NS_ERROR_FAILURE;
 }
