@@ -14,17 +14,29 @@
 #include "nsCRT.h"
 #include "prlog.h"
 #include "nsIClassInfoImpl.h"
-#include "nsAtomicRefcnt.h"
 #include "nsAlgorithm.h"
+#include "nsMemory.h"
+#include "nsIAsyncInputStream.h"
+#include "nsIAsyncOutputStream.h"
 
 using namespace mozilla;
 
+#ifdef LOG
+#undef LOG
+#endif
 #if defined(PR_LOGGING)
 //
 // set NSPR_LOG_MODULES=nsPipe:5
 //
-static PRLogModuleInfo *gPipeLog = PR_NewLogModule("nsPipe");
-#define LOG(args) PR_LOG(gPipeLog, PR_LOG_DEBUG, args)
+static PRLogModuleInfo *
+GetPipeLog()
+{
+  static PRLogModuleInfo *sLog;
+  if (!sLog)
+    sLog = PR_NewLogModule("nsPipe");
+  return sLog;
+}
+#define LOG(args) PR_LOG(GetPipeLog(), PR_LOG_DEBUG, args)
 #else
 #define LOG(args)
 #endif
@@ -121,7 +133,7 @@ private:
     nsPipe                        *mPipe;
 
     // separate refcnt so that we know when to close the consumer
-    nsrefcnt                       mReaderRefCnt;
+    mozilla::ThreadSafeAutoRefCnt  mReaderRefCnt;
     int64_t                        mLogicalOffset;
     bool                           mBlocking;
 
@@ -175,7 +187,7 @@ private:
     nsPipe                         *mPipe;
 
     // separate refcnt so that we know when to close the producer
-    nsrefcnt                        mWriterRefCnt;
+    mozilla::ThreadSafeAutoRefCnt   mWriterRefCnt;
     int64_t                         mLogicalOffset;
     bool                            mBlocking;
 
@@ -194,7 +206,7 @@ public:
     friend class nsPipeInputStream;
     friend class nsPipeOutputStream;
 
-    NS_DECL_ISUPPORTS
+    NS_DECL_THREADSAFE_ISUPPORTS
     NS_DECL_NSIPIPE
 
     // nsPipe methods:
@@ -287,8 +299,8 @@ protected:
 //-----------------------------------------------------------------------------
 
 nsPipe::nsPipe()
-    : mInput(this)
-    , mOutput(this)
+    : mInput(MOZ_THIS_IN_INITIALIZER_LIST())
+    , mOutput(MOZ_THIS_IN_INITIALIZER_LIST())
     , mReentrantMonitor("nsPipe.mReentrantMonitor")
     , mReadCursor(nullptr)
     , mReadLimit(nullptr)
@@ -304,14 +316,13 @@ nsPipe::~nsPipe()
 {
 }
 
-NS_IMPL_THREADSAFE_ISUPPORTS1(nsPipe, nsIPipe)
+NS_IMPL_ISUPPORTS1(nsPipe, nsIPipe)
 
 NS_IMETHODIMP
 nsPipe::Init(bool nonBlockingIn,
              bool nonBlockingOut,
              uint32_t segmentSize,
-             uint32_t segmentCount,
-             nsIMemory *segmentAlloc)
+             uint32_t segmentCount)
 {
     mInited = true;
 
@@ -325,7 +336,7 @@ nsPipe::Init(bool nonBlockingIn,
     if (segmentCount > maxCount)
         segmentCount = maxCount;
 
-    nsresult rv = mBuffer.Init(segmentSize, segmentSize * segmentCount, segmentAlloc);
+    nsresult rv = mBuffer.Init(segmentSize, segmentSize * segmentCount);
     if (NS_FAILED(rv))
         return rv;
 
@@ -344,7 +355,8 @@ nsPipe::GetInputStream(nsIAsyncInputStream **aInputStream)
 NS_IMETHODIMP
 nsPipe::GetOutputStream(nsIAsyncOutputStream **aOutputStream)
 {
-    NS_ENSURE_TRUE(mInited, NS_ERROR_NOT_INITIALIZED);
+    if (NS_WARN_IF(!mInited))
+	return NS_ERROR_NOT_INITIALIZED;
     NS_ADDREF(*aOutputStream = &mOutput);
     return NS_OK;
 }
@@ -673,14 +685,14 @@ nsPipeInputStream::OnInputException(nsresult reason, nsPipeEvents &events)
 NS_IMETHODIMP_(nsrefcnt)
 nsPipeInputStream::AddRef(void)
 {
-    NS_AtomicIncrementRefcnt(mReaderRefCnt);
+    ++mReaderRefCnt;
     return mPipe->AddRef();
 }
 
 NS_IMETHODIMP_(nsrefcnt)
 nsPipeInputStream::Release(void)
 {
-    if (NS_AtomicDecrementRefcnt(mReaderRefCnt) == 0)
+    if (--mReaderRefCnt == 0)
         Close();
     return mPipe->Release();
 }
@@ -824,9 +836,7 @@ nsPipeInputStream::AsyncWait(nsIInputStreamCallback *callback,
 
         nsCOMPtr<nsIInputStreamCallback> proxy;
         if (target) {
-            nsresult rv = NS_NewInputStreamReadyEvent(getter_AddRefs(proxy),
-                                                      callback, target);
-            if (NS_FAILED(rv)) return rv;
+            proxy = NS_NewInputStreamReadyEvent(callback, target);
             callback = proxy;
         }
 
@@ -928,7 +938,7 @@ nsPipeInputStream::Search(const char *forString,
         len2 = limit2 - cursor2;
 
         // check if the string is straddling the next buffer segment
-        uint32_t lim = NS_MIN(strLen, len2 + 1);
+        uint32_t lim = XPCOM_MIN(strLen, len2 + 1);
         for (i = 0; i < lim; ++i) {
             uint32_t strPart1Len = strLen - i - 1;
             uint32_t strPart2Len = strLen - strPart1Len;
@@ -1030,14 +1040,14 @@ nsPipeOutputStream::OnOutputException(nsresult reason, nsPipeEvents &events)
 NS_IMETHODIMP_(nsrefcnt)
 nsPipeOutputStream::AddRef()
 {
-    NS_AtomicIncrementRefcnt(mWriterRefCnt);
+    ++mWriterRefCnt;
     return mPipe->AddRef();
 }
 
 NS_IMETHODIMP_(nsrefcnt)
 nsPipeOutputStream::Release()
 {
-    if (NS_AtomicDecrementRefcnt(mWriterRefCnt) == 0)
+    if (--mWriterRefCnt == 0)
         Close();
     return mPipe->Release();
 }
@@ -1205,9 +1215,7 @@ nsPipeOutputStream::AsyncWait(nsIOutputStreamCallback *callback,
 
         nsCOMPtr<nsIOutputStreamCallback> proxy;
         if (target) {
-            nsresult rv = NS_NewOutputStreamReadyEvent(getter_AddRefs(proxy),
-                                                       callback, target);
-            if (NS_FAILED(rv)) return rv;
+            proxy = NS_NewOutputStreamReadyEvent(callback, target);
             callback = proxy;
         }
 
@@ -1233,8 +1241,7 @@ NS_NewPipe(nsIInputStream **pipeIn,
            uint32_t segmentSize,
            uint32_t maxSize,
            bool nonBlockingInput,
-           bool nonBlockingOutput,
-           nsIMemory *segmentAlloc)
+           bool nonBlockingOutput)
 {
     if (segmentSize == 0)
         segmentSize = DEFAULT_SEGMENT_SIZE;
@@ -1249,7 +1256,7 @@ NS_NewPipe(nsIInputStream **pipeIn,
     nsIAsyncInputStream *in;
     nsIAsyncOutputStream *out;
     nsresult rv = NS_NewPipe2(&in, &out, nonBlockingInput, nonBlockingOutput,
-                              segmentSize, segmentCount, segmentAlloc);
+                              segmentSize, segmentCount);
     if (NS_FAILED(rv)) return rv;
 
     *pipeIn = in;
@@ -1263,8 +1270,7 @@ NS_NewPipe2(nsIAsyncInputStream **pipeIn,
             bool nonBlockingInput,
             bool nonBlockingOutput,
             uint32_t segmentSize,
-            uint32_t segmentCount,
-            nsIMemory *segmentAlloc)
+            uint32_t segmentCount)
 {
     nsresult rv;
 
@@ -1275,8 +1281,7 @@ NS_NewPipe2(nsIAsyncInputStream **pipeIn,
     rv = pipe->Init(nonBlockingInput,
                     nonBlockingOutput,
                     segmentSize,
-                    segmentCount,
-                    segmentAlloc);
+                    segmentCount);
     if (NS_FAILED(rv)) {
         NS_ADDREF(pipe);
         NS_RELEASE(pipe);
