@@ -11,13 +11,12 @@
  *****************************************************************************
  */
 
-#include "mozilla/Util.h"
+#include "mozilla/ArrayUtils.h"
 
 #include "nsCOMPtr.h"
 #include "nsAutoPtr.h"
 #include "nsMemory.h"
 #include "nsProcess.h"
-#include "prtypes.h"
 #include "prio.h"
 #include "prenv.h"
 #include "nsCRT.h"
@@ -59,21 +58,21 @@ cpu_type_t pref_cpu_types[2] = {
 //-------------------------------------------------------------------//
 // nsIProcess implementation
 //-------------------------------------------------------------------//
-NS_IMPL_THREADSAFE_ISUPPORTS2(nsProcess, nsIProcess,
-                                         nsIObserver)
+NS_IMPL_ISUPPORTS2(nsProcess, nsIProcess,
+                   nsIObserver)
 
 //Constructor
 nsProcess::nsProcess()
-    : mThread(nsnull)
+    : mThread(nullptr)
     , mLock("nsProcess.mLock")
     , mShutdown(false)
     , mBlocking(false)
     , mPid(-1)
-    , mObserver(nsnull)
-    , mWeakObserver(nsnull)
+    , mObserver(nullptr)
+    , mWeakObserver(nullptr)
     , mExitValue(-1)
 #if !defined(XP_MACOSX)
-    , mProcess(nsnull)
+    , mProcess(nullptr)
 #endif
 {
 }
@@ -89,7 +88,8 @@ nsProcess::Init(nsIFile* executable)
     if (mExecutable)
         return NS_ERROR_ALREADY_INITIALIZED;
 
-    NS_ENSURE_ARG_POINTER(executable);
+    if (NS_WARN_IF(!executable))
+        return NS_ERROR_INVALID_ARG;
     bool isFile;
 
     //First make sure the file exists
@@ -113,7 +113,7 @@ nsProcess::Init(nsIFile* executable)
 
 #if defined(XP_WIN)
 // Out param `wideCmdLine` must be PR_Freed by the caller.
-static int assembleCmdLine(char *const *argv, PRUnichar **wideCmdLine,
+static int assembleCmdLine(char *const *argv, wchar_t **wideCmdLine,
                            UINT codePage)
 {
     char *const *arg;
@@ -140,7 +140,7 @@ static int assembleCmdLine(char *const *argv, PRUnichar **wideCmdLine,
                 + 1;                     /* space in between, or final null */
     }
     p = cmdLine = (char *) PR_MALLOC(cmdLineSize*sizeof(char));
-    if (p == NULL) {
+    if (p == nullptr) {
         return -1;
     }
 
@@ -213,15 +213,15 @@ static int assembleCmdLine(char *const *argv, PRUnichar **wideCmdLine,
     } 
 
     *p = '\0';
-    PRInt32 numChars = MultiByteToWideChar(codePage, 0, cmdLine, -1, NULL, 0); 
-    *wideCmdLine = (PRUnichar *) PR_MALLOC(numChars*sizeof(PRUnichar));
+    int32_t numChars = MultiByteToWideChar(codePage, 0, cmdLine, -1, nullptr, 0);
+    *wideCmdLine = (wchar_t *) PR_MALLOC(numChars*sizeof(wchar_t));
     MultiByteToWideChar(codePage, 0, cmdLine, -1, *wideCmdLine, numChars); 
     PR_Free(cmdLine);
     return 0;
 }
 #endif
 
-void PR_CALLBACK nsProcess::Monitor(void *arg)
+void nsProcess::Monitor(void *arg)
 {
     nsRefPtr<nsProcess> process = dont_AddRef(static_cast<nsProcess*>(arg));
 
@@ -242,7 +242,7 @@ void PR_CALLBACK nsProcess::Monitor(void *arg)
     {
         MutexAutoLock lock(process->mLock);
         CloseHandle(process->mProcess);
-        process->mProcess = NULL;
+        process->mProcess = nullptr;
         process->mExitValue = exitCode;
         if (process->mShutdown)
             return;
@@ -260,7 +260,7 @@ void PR_CALLBACK nsProcess::Monitor(void *arg)
         }
     }
 #else
-    PRInt32 exitCode = -1;
+    int32_t exitCode = -1;
     if (PR_WaitProcess(process->mProcess, &exitCode) != PR_SUCCESS)
         exitCode = -1;
 #endif
@@ -269,7 +269,7 @@ void PR_CALLBACK nsProcess::Monitor(void *arg)
     {
         MutexAutoLock lock(process->mLock);
 #if !defined(XP_MACOSX)
-        process->mProcess = nsnull;
+        process->mProcess = nullptr;
 #endif
         process->mExitValue = exitCode;
         if (process->mShutdown)
@@ -297,7 +297,7 @@ void nsProcess::ProcessComplete()
         if (os)
             os->RemoveObserver(this, "xpcom-shutdown");
         PR_JoinThread(mThread);
-        mThread = nsnull;
+        mThread = nullptr;
     }
 
     const char* topic;
@@ -312,23 +312,23 @@ void nsProcess::ProcessComplete()
         observer = do_QueryReferent(mWeakObserver);
     else if (mObserver)
         observer = mObserver;
-    mObserver = nsnull;
-    mWeakObserver = nsnull;
+    mObserver = nullptr;
+    mWeakObserver = nullptr;
 
     if (observer)
-        observer->Observe(NS_ISUPPORTS_CAST(nsIProcess*, this), topic, nsnull);
+        observer->Observe(NS_ISUPPORTS_CAST(nsIProcess*, this), topic, nullptr);
 }
 
 // XXXldb |args| has the wrong const-ness
 NS_IMETHODIMP  
-nsProcess::Run(bool blocking, const char **args, PRUint32 count)
+nsProcess::Run(bool blocking, const char **args, uint32_t count)
 {
-    return CopyArgsAndRunProcess(blocking, args, count, nsnull, false);
+    return CopyArgsAndRunProcess(blocking, args, count, nullptr, false);
 }
 
 // XXXldb |args| has the wrong const-ness
 NS_IMETHODIMP  
-nsProcess::RunAsync(const char **args, PRUint32 count,
+nsProcess::RunAsync(const char **args, uint32_t count,
                     nsIObserver* observer, bool holdWeak)
 {
     return CopyArgsAndRunProcess(false, args, count, observer, holdWeak);
@@ -336,11 +336,11 @@ nsProcess::RunAsync(const char **args, PRUint32 count,
 
 nsresult
 nsProcess::CopyArgsAndRunProcess(bool blocking, const char** args,
-                                 PRUint32 count, nsIObserver* observer,
+                                 uint32_t count, nsIObserver* observer,
                                  bool holdWeak)
 {
-    // Add one to the count for the program name and one for NULL termination.
-    char **my_argv = NULL;
+    // Add one to the count for the program name and one for null termination.
+    char **my_argv = nullptr;
     my_argv = (char**)NS_Alloc(sizeof(char*) * (count + 2));
     if (!my_argv) {
         return NS_ERROR_OUT_OF_MEMORY;
@@ -348,11 +348,11 @@ nsProcess::CopyArgsAndRunProcess(bool blocking, const char** args,
 
     my_argv[0] = ToNewUTF8String(mTargetPath);
 
-    for (PRUint32 i = 0; i < count; i++) {
+    for (uint32_t i = 0; i < count; i++) {
         my_argv[i + 1] = const_cast<char*>(args[i]);
     }
 
-    my_argv[count + 1] = NULL;
+    my_argv[count + 1] = nullptr;
 
     nsresult rv = RunProcess(blocking, my_argv, observer, holdWeak, false);
 
@@ -363,26 +363,26 @@ nsProcess::CopyArgsAndRunProcess(bool blocking, const char** args,
 
 // XXXldb |args| has the wrong const-ness
 NS_IMETHODIMP  
-nsProcess::Runw(bool blocking, const PRUnichar **args, PRUint32 count)
+nsProcess::Runw(bool blocking, const char16_t **args, uint32_t count)
 {
-    return CopyArgsAndRunProcessw(blocking, args, count, nsnull, false);
+    return CopyArgsAndRunProcessw(blocking, args, count, nullptr, false);
 }
 
 // XXXldb |args| has the wrong const-ness
 NS_IMETHODIMP  
-nsProcess::RunwAsync(const PRUnichar **args, PRUint32 count,
+nsProcess::RunwAsync(const char16_t **args, uint32_t count,
                     nsIObserver* observer, bool holdWeak)
 {
     return CopyArgsAndRunProcessw(false, args, count, observer, holdWeak);
 }
 
 nsresult
-nsProcess::CopyArgsAndRunProcessw(bool blocking, const PRUnichar** args,
-                                  PRUint32 count, nsIObserver* observer,
+nsProcess::CopyArgsAndRunProcessw(bool blocking, const char16_t** args,
+                                  uint32_t count, nsIObserver* observer,
                                   bool holdWeak)
 {
-    // Add one to the count for the program name and one for NULL termination.
-    char **my_argv = NULL;
+    // Add one to the count for the program name and one for null termination.
+    char **my_argv = nullptr;
     my_argv = (char**)NS_Alloc(sizeof(char*) * (count + 2));
     if (!my_argv) {
         return NS_ERROR_OUT_OF_MEMORY;
@@ -390,15 +390,15 @@ nsProcess::CopyArgsAndRunProcessw(bool blocking, const PRUnichar** args,
 
     my_argv[0] = ToNewUTF8String(mTargetPath);
 
-    for (PRUint32 i = 0; i < count; i++) {
+    for (uint32_t i = 0; i < count; i++) {
         my_argv[i + 1] = ToNewUTF8String(nsDependentString(args[i]));
     }
 
-    my_argv[count + 1] = NULL;
+    my_argv[count + 1] = nullptr;
 
     nsresult rv = RunProcess(blocking, my_argv, observer, holdWeak, true);
 
-    for (PRUint32 i = 0; i <= count; i++) {
+    for (uint32_t i = 0; i <= count; i++) {
         NS_Free(my_argv[i]);
     }
     NS_Free(my_argv);
@@ -409,8 +409,10 @@ nsresult
 nsProcess::RunProcess(bool blocking, char **my_argv, nsIObserver* observer,
                       bool holdWeak, bool argsUTF8)
 {
-    NS_ENSURE_TRUE(mExecutable, NS_ERROR_NOT_INITIALIZED);
-    NS_ENSURE_FALSE(mThread, NS_ERROR_ALREADY_INITIALIZED);
+    if (NS_WARN_IF(!mExecutable))
+        return NS_ERROR_NOT_INITIALIZED;
+    if (NS_WARN_IF(mThread))
+        return NS_ERROR_ALREADY_INITIALIZED;
 
     if (observer) {
         if (holdWeak) {
@@ -428,11 +430,11 @@ nsProcess::RunProcess(bool blocking, char **my_argv, nsIObserver* observer,
 
 #if defined(PROCESSMODEL_WINAPI)
     BOOL retVal;
-    PRUnichar *cmdLine = NULL;
+    wchar_t *cmdLine = nullptr;
 
     // The 'argv' array is null-terminated and always starts with the program path.
     // If the second slot is non-null then arguments are being passed.
-    if (my_argv[1] != NULL &&
+    if (my_argv[1] != nullptr &&
         assembleCmdLine(my_argv + 1, &cmdLine, argsUTF8 ? CP_UTF8 : CP_ACP) == -1) {
         return NS_ERROR_FILE_EXECUTION_FAILED;    
     }
@@ -443,15 +445,13 @@ nsProcess::RunProcess(bool blocking, char **my_argv, nsIObserver* observer,
      */
 
     // The program name in my_argv[0] is always UTF-8
-    PRInt32 numChars = MultiByteToWideChar(CP_UTF8, 0, my_argv[0], -1, NULL, 0); 
-    PRUnichar* wideFile = (PRUnichar *) PR_MALLOC(numChars * sizeof(PRUnichar));
-    MultiByteToWideChar(CP_UTF8, 0, my_argv[0], -1, wideFile, numChars); 
+    NS_ConvertUTF8toUTF16 wideFile(my_argv[0]);
 
     SHELLEXECUTEINFOW sinfo;
     memset(&sinfo, 0, sizeof(SHELLEXECUTEINFOW));
     sinfo.cbSize = sizeof(SHELLEXECUTEINFOW);
-    sinfo.hwnd   = NULL;
-    sinfo.lpFile = wideFile;
+    sinfo.hwnd   = nullptr;
+    sinfo.lpFile = wideFile.get();
     sinfo.nShow  = SW_SHOWNORMAL;
     sinfo.fMask  = SEE_MASK_FLAG_DDEWAIT |
                    SEE_MASK_NO_CONSOLE |
@@ -467,7 +467,6 @@ nsProcess::RunProcess(bool blocking, char **my_argv, nsIObserver* observer,
 
     mProcess = sinfo.hProcess;
 
-    PR_Free(wideFile);
     if (cmdLine)
         PR_Free(cmdLine);
 
@@ -490,8 +489,8 @@ nsProcess::RunProcess(bool blocking, char **my_argv, nsIObserver* observer,
 
     // Note that the 'argv' array is already null-terminated, which 'posix_spawnp' requires.
     pid_t newPid = 0;
-    int result = posix_spawnp(&newPid, my_argv[0], NULL, &spawnattr, my_argv, *_NSGetEnviron());
-    mPid = static_cast<PRInt32>(newPid);
+    int result = posix_spawnp(&newPid, my_argv[0], nullptr, &spawnattr, my_argv, *_NSGetEnviron());
+    mPid = static_cast<int32_t>(newPid);
 
     posix_spawnattr_destroy(&spawnattr);
 
@@ -499,11 +498,11 @@ nsProcess::RunProcess(bool blocking, char **my_argv, nsIObserver* observer,
         return NS_ERROR_FAILURE;
     }
 #else
-    mProcess = PR_CreateProcess(my_argv[0], my_argv, NULL, NULL);
+    mProcess = PR_CreateProcess(my_argv[0], my_argv, nullptr, nullptr);
     if (!mProcess)
         return NS_ERROR_FAILURE;
     struct MYProcess {
-        PRUint32 pid;
+        uint32_t pid;
     };
     MYProcess* ptrProc = (MYProcess *) mProcess;
     mPid = ptrProc->pid;
@@ -546,7 +545,7 @@ NS_IMETHODIMP nsProcess::GetIsRunning(bool *aIsRunning)
 }
 
 NS_IMETHODIMP
-nsProcess::GetPid(PRUint32 *aPid)
+nsProcess::GetPid(uint32_t *aPid)
 {
     if (!mThread)
         return NS_ERROR_FAILURE;
@@ -565,7 +564,7 @@ nsProcess::Kill()
     {
         MutexAutoLock lock(mLock);
 #if defined(PROCESSMODEL_WINAPI)
-        if (TerminateProcess(mProcess, NULL) == 0)
+        if (TerminateProcess(mProcess, 0) == 0)
             return NS_ERROR_FAILURE;
 #elif defined(XP_MACOSX)
         if (kill(mPid, SIGKILL) != 0)
@@ -582,13 +581,13 @@ nsProcess::Kill()
     if (os)
         os->RemoveObserver(this, "xpcom-shutdown");
     PR_JoinThread(mThread);
-    mThread = nsnull;
+    mThread = nullptr;
 
     return NS_OK;
 }
 
 NS_IMETHODIMP
-nsProcess::GetExitValue(PRInt32 *aExitValue)
+nsProcess::GetExitValue(int32_t *aExitValue)
 {
     MutexAutoLock lock(mLock);
 
@@ -598,7 +597,7 @@ nsProcess::GetExitValue(PRInt32 *aExitValue)
 }
 
 NS_IMETHODIMP
-nsProcess::Observe(nsISupports* subject, const char* topic, const PRUnichar* data)
+nsProcess::Observe(nsISupports* subject, const char* topic, const char16_t* data)
 {
     // Shutting down, drop all references
     if (mThread) {
@@ -606,11 +605,11 @@ nsProcess::Observe(nsISupports* subject, const char* topic, const PRUnichar* dat
           mozilla::services::GetObserverService();
         if (os)
             os->RemoveObserver(this, "xpcom-shutdown");
-        mThread = nsnull;
+        mThread = nullptr;
     }
 
-    mObserver = nsnull;
-    mWeakObserver = nsnull;
+    mObserver = nullptr;
+    mWeakObserver = nullptr;
 
     MutexAutoLock lock(mLock);
     mShutdown = true;
